@@ -2,9 +2,18 @@
  *
  * Core functionality of easel: errors, memory allocations, constants,
  * and configuration for portability.
- *
- * SRE, Wed Jul  7 09:43:28 2004 [St. Louis]
- * SVN $Id: easel.h 255 2008-05-30 17:49:15Z eddys $
+ * 
+ * Contents:
+ *   1. Macros implementing Easel error handling conventions
+ *   2. Macros implementing Easel memory allocation conventions
+ *   3. Macros implementing Easel function argument conventions
+ *   4. Macros implementing Easel debugging output conventions
+ *   5. Defined constants
+ *   6. Basic support for Easel digitized biosequences
+ *   7. Miscellaneous
+ *   8. Void declarations of missing augmentations
+ *   9. API declarations of easel.c
+ *  10. Copyright and license.
  */
 #ifndef eslEASEL_INCLUDED
 #define eslEASEL_INCLUDED
@@ -14,6 +23,7 @@
 #include <stdlib.h>
 #include <stdio.h>		/* for FILE */
 #include <stdarg.h>		/* for va_list */
+#include <math.h>               /* for HUGE_VAL */
 #ifdef HAVE_STDINT_H
 #include <stdint.h>		/* for uint32_t and the like (C99) */
 #endif
@@ -22,7 +32,7 @@
 #endif
 
 /*****************************************************************
- * Macros implementing Easel's error handling conventions
+ * 1. Macros implementing Easel's error handling conventions
  *****************************************************************/
 /* Many objects contain a fixed length "errbuf" for failure
  * diagnostics: ESL_FAIL() and ESL_XFAIL() fill this buffer.
@@ -43,27 +53,39 @@
  * macro expansion.
  */
 /*::cexcerpt::error_macros::begin::*/
-#define ESL_FAIL(code, errbuf, ...) do {\
-     if (errbuf != NULL) snprintf(errbuf, eslERRBUFSIZE, __VA_ARGS__);\
-     return code; }\
-     while (0)
+#define ESL_FAIL(code, errbuf, ...) do {				\
+    if (errbuf != NULL) snprintf(errbuf, eslERRBUFSIZE, __VA_ARGS__);	\
+    return code; }							\
+  while (0)
 
-#define ESL_XFAIL(code, errbuf, ...) do {\
-     status = code;\
-     if (errbuf != NULL) snprintf(errbuf, eslERRBUFSIZE, __VA_ARGS__);\
-     goto ERROR; }\
-     while (0)
+#define ESL_XFAIL(code, errbuf, ...) do {				\
+    status = code;							\
+    if (errbuf != NULL) snprintf(errbuf, eslERRBUFSIZE, __VA_ARGS__);	\
+    goto ERROR; }							\
+  while (0)
 
-#define ESL_EXCEPTION(code, ...)  do {\
-     esl_exception(code, __FILE__, __LINE__, __VA_ARGS__);\
-     return code; }\
-     while (0)
+#define ESL_EXCEPTION(code, ...)  do {					\
+    esl_exception(code, FALSE, __FILE__, __LINE__, __VA_ARGS__);	\
+    return code; }							\
+  while (0)
+
+#define ESL_XEXCEPTION(code, ...)  do {					\
+    status = code;							\
+    esl_exception(code, FALSE, __FILE__, __LINE__, __VA_ARGS__);	\
+    goto ERROR; }							\
+  while (0)
+
+#define ESL_EXCEPTION_SYS(code, ...) do {				\
+    esl_exception(code, TRUE, __FILE__, __LINE__, __VA_ARGS__);		\
+    return code; }							\
+  while (0)
+
+#define ESL_XEXCEPTION_SYS(code, ...)  do {				\
+    status = code;							\
+    esl_exception(code, TRUE, __FILE__, __LINE__, __VA_ARGS__);	\
+    goto ERROR; }							\
+  while (0)
 
-#define ESL_XEXCEPTION(code, ...)  do {\
-     status = code;\
-     esl_exception(code, __FILE__, __LINE__, __VA_ARGS__);\
-     goto ERROR; }\
-     while (0)
 /*::cexcerpt::error_macros::end::*/
 
 
@@ -90,8 +112,14 @@
 #define eslEDUP           17    /* saw a duplicate of something */
 #define eslENOHALT        18    /* a failure to converge        */      
 #define eslENORESULT      19    /* no result was obtained       */
-#define eslENODATA        20	/* no data provided, file empty */
-#define eslETYPE          21	/* invalid type of argument     */
+#define eslENODATA        20    /* no data provided, file empty */
+#define eslETYPE          21    /* invalid type of argument     */
+#define eslEOVERWRITE     22    /* attempted to overwrite data  */
+#define eslENOSPACE       23    /* ran out of some resource     */
+#define eslEUNIMPLEMENTED 24    /* feature is unimplemented     */
+#define eslENOFORMAT      25	/* couldn't guess file format   */
+#define eslENOALPHABET    26	/* couldn't guess seq alphabet  */
+#define eslEWRITE         27   	/* write failed (fprintf, etc)  */
 /*::cexcerpt::statuscodes::end::*/
 
 
@@ -100,19 +128,23 @@
 
 
 /*****************************************************************
- * Macros implementing Easel's memory allocation conventions
+ * 2. Macros implementing Easel's memory allocation conventions
  *****************************************************************/
 /* ESL_ALLOC(), ESL_RALLOC():
  * 
  * Allocation and reallocation wrappers.
  * Both require <int status> in scope, and <ERROR:> goto target.
  * ESL_RALLOC() also requires <void *> ptr to be provided as <tmp>.
+ *
+ * ESL_REALLOC() is a newer version of ESL_RALLOC() which doesn't
+ * need a tmp ptr. All ESL_RALLOC() calls can be safely converted
+ * to ESL_REALLOC() calls.
  */
 /*::cexcerpt::alloc_macros::begin::*/
 #define ESL_ALLOC(p, size) do {\
-     if (((p) = malloc(size)) == NULL) {\
+    if (((p) = malloc(size)) == NULL && (size)) {	\
        status = eslEMEM;\
-       esl_exception(eslEMEM, __FILE__, __LINE__, "malloc of size %d failed", size);\
+       esl_exception(eslEMEM, FALSE, __FILE__, __LINE__, "malloc of size %d failed", size); \
        goto ERROR;\
      }} while (0)
 
@@ -122,16 +154,47 @@
      if ((tmp) != NULL) (p) = (tmp);\
      else {\
        status = eslEMEM;\
-       esl_exception(eslEMEM, __FILE__, __LINE__, "realloc for size %d failed", newsize);\
+       esl_exception(eslEMEM, FALSE, __FILE__, __LINE__, "realloc for size %d failed", newsize);	\
+       goto ERROR;\
+     }} while (0)
+
+#define ESL_REALLOC(p, newsize) do {\
+     void *esltmpp;\
+     if ((p) == NULL) { (esltmpp) = malloc(newsize);         }\
+     else             { (esltmpp) = realloc((p), (newsize)); }\
+     if ((esltmpp) != NULL) (p) = (esltmpp);\
+     else {\
+       status = eslEMEM;\
+       esl_exception(eslEMEM, FALSE, __FILE__, __LINE__, "realloc for size %d failed", newsize); \
        goto ERROR;\
      }} while (0)
 /*::cexcerpt::alloc_macros::end::*/
 
-     
+/* Convert MB,GB,TB to bytes, using binary definitions (2^20, 2^30, 2^40)
+ * More pedantically: mebibytes (MiB), gibibytes (GiB), tebibytes (TiB).
+ */
+#define ESL_MBYTES(x) ((x) * 1048576) 
+#define ESL_GBYTES(x) ((x) * 1024 * 1048576) 
+#define ESL_TBYTES(x) ((x) * 1024 * 1024 * 1048576)
+
+/* Round integer <n> up to the nearest multiple of <m>. 
+ * Particularly useful when dealing w/ memory alignment issues.
+ */
+#define ESL_UPROUND(n, m)  ( ((n) + (m)-1) / (m) * (m))
 
 
 /*****************************************************************
- * Macros implementing Easel's debugging output conventions
+ * 3. Macros implementing Easel's function argument conventions
+ *****************************************************************/
+
+#define esl_byp_IsInternal(p) ((p) == NULL)
+#define esl_byp_IsReturned(p) ((p) != NULL && (*p) == NULL)
+#define esl_byp_IsProvided(p) ((p) != NULL && (*p) != NULL)
+
+
+
+/*****************************************************************
+ * 4. Macros implementing Easel's debugging output conventions
  *****************************************************************/
 /* Debugging hooks, w/ three levels (1-3).
  */
@@ -165,8 +228,9 @@
 
 
 /*****************************************************************
- * Defined constants
+ * 5. Defined constants
  *****************************************************************/
+
 /* Making sure TRUE/FALSE are defined, for convenience */
 #ifndef TRUE
 #define TRUE 1
@@ -187,21 +251,24 @@
 #define eslCONST_LOG2  0.69314718055994529
 #define eslCONST_LOG2R 1.44269504088896341
 
-/* Define <eslINFINITY>, <eslNaN> portably. Harder than it looks. 
- * We assume we're in an IEEE 754 environment.
- * We assume that HUGE_VAL in a IEEE754 environment is infinity.
- * If we don't have HUGE_VAL set, we assume we can get infinity
- * by division by zero. (But if we don't have HUGE_VAL, we probably
- * have other problems; HUGE_VAL is required by ANSI C spec.)
- * We can't portably get infinity by overflow (e.g. 1e9999);
- * some compilers (Microsoft) will complain.
+/* Define <eslINFINITY>, <eslNaN> portably.  
+ * ANSI C99 makes it easy (at last!). The failovers to other pre-C99
+ * methods are legacy now that we require a C99 compiler - but no harm
+ * leaving them in Just In Case.
  */
-#ifdef HUGE_VAL
+#if defined (INFINITY)
+#define eslINFINITY    INFINITY  /* C99 */
+#elif defined (HUGE_VAL)
 #define eslINFINITY    HUGE_VAL	 /* assume IEEE754 HUGE_VAL = infinity. ok? */
 #else
 #define eslINFINITY    (1.0/0.0) /* portable? */
 #endif
+
+#if defined (NAN)
+#define eslNaN         NAN	/* C99 */
+#else
 #define eslNaN         (eslINFINITY/eslINFINITY) /* portably make a IEEE754 NaN */
+#endif
 
 /* Define crossovers for numerical approximations.
  */
@@ -211,10 +278,8 @@
 #define eslSMALLX1    5e-9
 
 
-
-
 /*****************************************************************
- * Basic support for Easel's digitized biosequences.
+ * 6. Basic support for Easel's digitized biosequences.
  *****************************************************************/
 
 /* Most of this support is in the alphabet module, but we externalize 
@@ -249,13 +314,13 @@ typedef uint8_t ESL_DSQ;
 
 
 /*****************************************************************
- * Miscellaneous.
+ * 7. Miscellaneous.
  *****************************************************************/
 /* A placeholder for helping w/ portability of filenames/paths.
  * I think, but have not tested, that:
- *   VMS:    #define DIRSLASH ']'
- *   MacOS:  #define DIRSLASH ':'
- *   DOS:    #define DIRSLASH '\\'
+ *   VMS:            #define DIRSLASH ']'
+ *   ancient MacOS:  #define DIRSLASH ':'
+ *   DOS:            #define DIRSLASH '\\'
  * Setting DIRSLASH correctly is probably not the only thing
  * that would need to be done to port to other OS's, but it's
  * probably a start.
@@ -273,36 +338,67 @@ typedef uint8_t ESL_DSQ;
 #define ESL_MIN(a,b)          (((a)<(b))?(a):(b))
 #define ESL_MAX(a,b)          (((a)>(b))?(a):(b))
 
+static inline float esl_logf(float x) { return (x == 0.0 ? -eslINFINITY : logf(x)); }
 
+/* Typedef: <esl_pos_t> 
+ * 
+ * <esl_pos_t> is a signed integer type suitable for safe casting
+ * to EITHER an <off_t> or <size_t> on this system (i.e. as a position
+ * in memory or in a file on disk), where we may use a -1 as a flag
+ * (or even other negative numbers).
+ *
+ * <esl_pos_t> is for use for anything having to do with positions in
+ * large buffers, strings, or files, where we want strict control
+ * of integer range limits.
+ *
+ * Note that POSIX requires size_t to be unsigned, and off_t to be
+ * signed.
+ */
+typedef int64_t esl_pos_t;
 
 /*****************************************************************
- * The API declarations for easel.c
+ * 8. Void declarations of missing augmentations
+ *****************************************************************/
+#ifndef eslAUGMENT_ALPHABET
+typedef void ESL_ALPHABET;
+#endif
+#ifndef eslAUGMENT_KEYHASH
+typedef void ESL_KEYHASH;
+#endif
+
+/*****************************************************************
+ * 9. The API declarations for easel.c
  *****************************************************************/
 
 /* 1. Error handling. */
-typedef void (*esl_exception_handler_f)(int code, char *file, int line,
-					char *format, va_list argp);
-extern void esl_exception(int code, char *file, int line, char *format, ...);
+typedef void (*esl_exception_handler_f)(int errcode, int use_errno, char *sourcefile, int sourceline, char *format, va_list argp);
+extern void esl_exception(int errcode, int use_errno, char *sourcefile, int sourceline, char *format, ...);
 extern void esl_exception_SetHandler(esl_exception_handler_f);
 extern void esl_exception_ResetDefaultHandler(void);
-extern void esl_fatal(char *format, ...);
-extern void esl_nonfatal_handler(int code, char *file, int line, char *format, va_list argp);
+extern void esl_nonfatal_handler(int errcode, int use_errno, char *sourcefile, int sourceline, char *format, va_list argp);
+extern void esl_fatal(const char *format, ...);
 
 /* 2. Memory allocation/deallocation conventions. */
 extern void esl_Free2D(void  **p, int dim1);
 extern void esl_Free3D(void ***p, int dim1, int dim2);
 
 /* 3. Standard banner for Easel miniapplications. */
-extern void esl_banner(FILE *fp, char *progname, char *banner);
-extern void esl_usage (FILE *fp, char *progname, char *usage);
+extern int  esl_banner(FILE *fp, char *progname, char *banner);
+extern int  esl_usage (FILE *fp, char *progname, char *usage);
 
-/* 4. Replacements, additions to C library functions */
+/* 4. Improved replacements for some C library functions */
+extern int  esl_fgets(char **buf, int *n, FILE *fp);
 extern int  esl_strdup(const char *s, int64_t n, char **ret_dup);
 extern int  esl_strcat(char **dest, int64_t ldest, const char *src, int64_t lsrc);
-extern int  esl_fgets(char **buf, int *n, FILE *fp);
-extern int  esl_strtok(char **s, char *delim, char **ret_tok, int *ret_toklen);
-extern int  esl_strchop(char *s, int64_t n);
-extern int  esl_strdealign(char *s, const char *aseq, const char *gapchars, int64_t *opt_rlen);
+extern int  esl_strmapcat        (const ESL_DSQ *inmap, char **dest, int64_t *ldest, const char *src, esl_pos_t lsrc);
+extern int  esl_strmapcat_noalloc(const ESL_DSQ *inmap,  char *dest, int64_t *ldest, const char *src, esl_pos_t lsrc);
+extern int  esl_strtok    (char **s, char *delim, char **ret_tok);
+extern int  esl_strtok_adv(char **s, char *delim, char **ret_tok, int *opt_toklen, char *opt_endchar);
+extern int  esl_sprintf (char **ret_s, const char *format, ...);
+extern int  esl_vsprintf(char **ret_s, const char *format, va_list *ap);
+extern int  esl_strcmp(const char *s1, const char *s2);
+
+/* 5.  Portable drop-in replacements for non-standard C functions */
 #ifndef HAVE_STRCASECMP
 #ifdef _MSC_VER
 #define strcasecmp stricmp
@@ -312,25 +408,52 @@ extern int  esl_strcasecmp(const char *s1, const char *s2);
 #endif
 #endif
 
-/* 5. File path/name manipulation functions, including tmpfiles */
+/* 6. Additional string functions, esl_str*() */
+extern int     esl_strchop(char *s, int64_t n);
+extern int     esl_strdealign(char *s, const char *aseq, const char *gapchars, int64_t *opt_rlen);
+extern int     esl_str_IsBlank(char *s);
+extern int     esl_str_IsInteger(char *s);
+extern int     esl_str_IsReal(char *s);
+extern int64_t esl_str_GetMaxWidth(char **s, int n);
+
+/* 7. File path/name manipulation functions, including tmpfiles */
 extern int  esl_FileExists(const char *filename);
 extern int  esl_FileTail(const char *path, int nosuffix, char **ret_file);
+extern int  esl_file_Extension(char *filename, esl_pos_t n_ignore, char **ret_sfx, esl_pos_t *ret_n);
 extern int  esl_FileConcat(const char *dir, const char *file, char **ret_path);
 extern int  esl_FileNewSuffix(const char *filename, const char *sfx, char **ret_newpath);
 extern int  esl_FileEnvOpen(const char *fname, const char *env,
 			    FILE **ret_fp, char **ret_path);
 extern int  esl_tmpfile(char *basename6X, FILE **ret_fp);
 extern int  esl_tmpfile_named(char *basename6X, FILE **ret_fp);
+extern int  esl_getcwd(char **ret_cwd);
 
-/* 6. Typed comparison routines. */
-extern int  esl_DCompare(double a, double b, double tol);
-extern int  esl_FCompare(float  a, float  b, float  tol);
+/* 8. Typed comparison routines. */
+extern int  esl_DCompare   (double a, double b, double tol);
+extern int  esl_FCompare   (float  a, float  b, float  tol);
+extern int  esl_DCompareAbs(double a, double b, double tol);
+extern int  esl_FCompareAbs(float  a, float  b, float  tol);
 extern int  esl_CCompare(char *s1, char *s2);
 
-/* 7. Commonly used background composition (iid) frequencies. */
+/* 9. Commonly used background composition (iid) frequencies. */
 extern int  esl_composition_BL62(double *f);
-extern int  esl_composition_WAG(double *f);
+extern int  esl_composition_WAG (double *f);
 extern int  esl_composition_SW34(double *f);
+extern int  esl_composition_SW50(double *f);
 
 
 #endif /*eslEASEL_INCLUDED*/
+
+
+/*****************************************************************
+ * Easel - a library of C functions for biological sequence analysis
+ * Version i1.1.1; July 2014
+ * Copyright (C) 2014 HHMI Janelia Farm Research Campus
+ * Other copyrights also apply. See the COPYRIGHT file for a full list.
+ * 
+ * Easel is distributed under the Janelia Farm Software License, a BSD
+ * license. See the LICENSE file for more details.
+ * 
+ * SVN $Id: easel.h 748 2012-02-14 21:23:06Z eddys $
+ * SVN $URL: https://svn.janelia.org/eddylab/eddys/easel/branches/infernal/1.1/easel.h $
+ *****************************************************************/
