@@ -1,33 +1,32 @@
-/* 
- * Digital representation of biosequence symbols in Easel.
- * SVN $Id: esl_alphabet.h 255 2008-05-30 17:49:15Z eddys $
- * SRE, Tue Nov 23 19:44:01 2004 [St. Louis]
+/* Digital representation of biosequence symbols in Easel.
  */
-#ifndef ESL_ALPHABET_INCLUDED
-#define ESL_ALPHABET_INCLUDED
+#ifndef eslALPHABET_INCLUDED
+#define eslALPHABET_INCLUDED
 
 #include <ctype.h>		/* isascii() */
 #include "easel.h"
 
 /* Flags for alphabet types.
  */
-#define eslUNKNOWN     0 /* 0=unknown is easel-wide convention; don't change */
+#define eslUNKNOWN     0        /* 0=unknown is easel-wide convention; don't change */
 #define eslRNA         1
 #define eslDNA         2		
 #define eslAMINO       3		
-#define eslNONSTANDARD 4
+#define eslCOINS       4	/* for toy examples      */
+#define eslDICE        5	/* also for toy examples */
+#define eslNONSTANDARD 6
 
 /* Structure: ESL_ALPHABET
  */
 typedef struct {
-  int      type;	     /* eslDNA, eslRNA, eslAMINO, or eslNONSTANDARD     */
-  int      K;		     /* uniq alphabet size: 4 or 20                     */
-  int      Kp;		     /* total size: alphabet + degen + gap + missing    */
-  char    *sym;              /* "ACGT-RYMKSWHBVDN~", for instance    [0..Kp-1]  */
-  ESL_DSQ  inmap[128];       /* inmap['A'] = 0, etc: dsq[] index for a symbol   */
-  char   **degen;            /* 1/0, which syms inc which res [0..Kp-1][0..K-1] */
-  int     *ndegen;	     /* # of degenerate residues per code  [0..Kp-1]    */
-  ESL_DSQ *complement;       /* map a digital symbol to its complement [0..Kp-1]*/
+  int      type;	     /* eslDNA, eslRNA, eslAMINO, eslNONSTANDARD, etc.                 */
+  int      K;		     /* uniq alphabet size: 4 or 20                                    */
+  int      Kp;		     /* total size: alphabet + degen + gap + missing                   */
+  char    *sym;              /* "ACGT-RYMKSWHBVDN~", for instance    [0..Kp-1]                 */
+  ESL_DSQ  inmap[128];       /* inmap['A'] = 0, etc: dsq[] index for a symbol                  */
+  char   **degen;            /* 1/0, which syms inc which res [0..Kp-1][0..K-1]                */
+  int     *ndegen;	     /* # of degenerate residues per code  [0..Kp-1]                   */
+  ESL_DSQ *complement;       /* maps sym to complements, [0..Kp-1]; NULL if <type> not DNA/RNA */
 } ESL_ALPHABET;
 
 
@@ -41,6 +40,7 @@ extern int           esl_alphabet_SetEquiv(ESL_ALPHABET *a, char sym, char c);
 extern int           esl_alphabet_SetCaseInsensitive(ESL_ALPHABET *a);
 extern int           esl_alphabet_SetDegeneracy(ESL_ALPHABET *a, char c, char *ds);
 extern int           esl_alphabet_SetIgnored(ESL_ALPHABET *a, const char *ignoredchars);
+extern size_t        esl_alphabet_Sizeof(ESL_ALPHABET *a);
 extern void          esl_alphabet_Destroy(ESL_ALPHABET *a);
 
 /* 2. Digitized sequences.
@@ -51,12 +51,13 @@ extern int     esl_abc_Textize  (const ESL_ALPHABET *a, const ESL_DSQ *dsq,  int
 extern int     esl_abc_TextizeN (const ESL_ALPHABET *a, const ESL_DSQ *dptr, int64_t L, char   *buf);
 extern int     esl_abc_dsqcpy(const ESL_DSQ *dsq, int64_t L, ESL_DSQ *dcopy);
 extern int     esl_abc_dsqdup(const ESL_DSQ *dsq, int64_t L, ESL_DSQ **ret_dup);
-extern int     esl_abc_dsqcat(const ESL_ALPHABET *a, ESL_DSQ **dsq, int64_t *L, const char *s, int64_t n);
+extern int     esl_abc_dsqcat        (const ESL_DSQ *inmap, ESL_DSQ **dsq, int64_t *L, const char *s, esl_pos_t n);
+extern int     esl_abc_dsqcat_noalloc(const ESL_DSQ *inmap, ESL_DSQ  *dsq, int64_t *L, const char *s, esl_pos_t n);
 extern int64_t esl_abc_dsqlen(const ESL_DSQ *dsq);
 extern int64_t esl_abc_dsqrlen(const ESL_ALPHABET *a, const ESL_DSQ *dsq);
 extern int     esl_abc_CDealign(const ESL_ALPHABET *abc, char    *s, const ESL_DSQ *ref_ax, int64_t *opt_rlen);
 extern int     esl_abc_XDealign(const ESL_ALPHABET *abc, ESL_DSQ *x, const ESL_DSQ *ref_ax, int64_t *opt_rlen);
-
+extern int     esl_abc_ConvertDegen2X(const ESL_ALPHABET *abc, ESL_DSQ *dsq);
 
 /* 3. Other routines in the API.
  */
@@ -77,46 +78,53 @@ extern int    esl_abc_FExpectScVec(const ESL_ALPHABET *a, float  *sc, const floa
 extern int    esl_abc_DExpectScVec(const ESL_ALPHABET *a, double *sc, const double *p);
 extern int    esl_abc_FCount      (const ESL_ALPHABET *a, float  *ct, ESL_DSQ x, float  wt);
 extern int    esl_abc_DCount      (const ESL_ALPHABET *a, double *ct, ESL_DSQ x, double wt);
-extern char  *esl_abc_DescribeType(int type);
+extern int    esl_abc_EncodeType  (char *typestring);
+extern char  *esl_abc_DecodeType  (int type);
 extern int    esl_abc_ValidateSeq(const ESL_ALPHABET *a, const char *seq, int64_t L, char *errbuf);
 
 /* In the tests below, remember the rules of order in internal alphabets:
- *   Canonical alphabet   Gap   Degeneracies  (X/N)  Missing data
- *        0..K-1           K      K+1..Kp-2   (Kp-2)   Kp-1
- *         ACGT            -     RYMKSWHBVD     N      ~           DNA: K=4  Kp=17
- *  ACDEFGHIKLMNPQRSTVWY   -        BJZOU       X      ~       protein: K=20 Kp=28
+ *   Canonical alphabet   Gap   Degeneracies   Any    None    Missing 
+ *        0..K-1           K      K+1..Kp-4   (Kp-3)  (Kp-2)   (Kp-1)
+ *         ACGT            -     RYMKSWHBVD     N       *        ~           DNA: K=4  Kp=18
+ *  ACDEFGHIKLMNPQRSTVWY   -        BJZOU       X       *        ~       protein: K=20 Kp=29
  *                           
  * ESL_DSQ is an unsigned 8-bit type, so don't test for >= 0 or compilers will complain.
  */
 #define esl_abc_DigitizeSymbol(a, c) ((a)->inmap[(int)c])
 #define esl_abc_XIsValid(a, x)       ((x) < (a)->Kp)
-#define esl_abc_XIsResidue(a, x)     ((x) < (a)->K || ((x) > (a)->K && (x) < (a)->Kp-1))
+#define esl_abc_XIsResidue(a, x)     ((x) < (a)->K || ((x) > (a)->K && (x) < (a)->Kp-2))
 #define esl_abc_XIsCanonical(a, x)   ((x) < (a)->K)
 #define esl_abc_XIsGap(a, x)         ((x) == (a)->K)
-#define esl_abc_XIsDegenerate(a, x)  ((x) >  (a)->K && (x) < (a)->Kp-1)
-#define esl_abc_XIsUnknown(a, x)     ((x) == (a)->Kp-2)
+#define esl_abc_XIsDegenerate(a, x)  ((x) >  (a)->K && (x) < (a)->Kp-2)
+#define esl_abc_XIsUnknown(a, x)     ((x) == (a)->Kp-3)
+#define esl_abc_XIsNonresidue(a, x)  ((x) == (a)->Kp-2)
 #define esl_abc_XIsMissing(a, x)     ((x) == (a)->Kp-1)
 #define esl_abc_XGetGap(a)           ((a)->K)
-#define esl_abc_XGetUnknown(a)       ((a)->Kp-2)
+#define esl_abc_XGetUnknown(a)       ((a)->Kp-3)
+#define esl_abc_XGetNonresidue(a)    ((a)->Kp-2)
 #define esl_abc_XGetMissing(a)       ((a)->Kp-1)
 
 
 #define esl_abc_CIsValid(a, c)       (isascii(c) && (a)->inmap[(int)c] < (a)->Kp)
-#define esl_abc_CIsResidue(a, c)     ((a)->inmap[(int)c] < (a)->K || ((a)->inmap[(int)c] > (a)->K && (a)->inmap[(int)c] < (a)->Kp-1))
+#define esl_abc_CIsResidue(a, c)     ((a)->inmap[(int)c] < (a)->K || ((a)->inmap[(int)c] > (a)->K && (a)->inmap[(int)c] < (a)->Kp-2))
 #define esl_abc_CIsCanonical(a, c)   ((a)->inmap[(int)c] < (a)->K)
 #define esl_abc_CIsGap(a, c)         ((a)->inmap[(int)c] == (a)->K)
-#define esl_abc_CIsDegenerate(a, c)  ((a)->inmap[(int)c] > (a)->K  && (a)->inmap[(int)c] < (a)->Kp-1)
-#define esl_abc_CIsUnknown(a, c)     ((a)->inmap[(int)c] == (a)->Kp-2)
+#define esl_abc_CIsDegenerate(a, c)  ((a)->inmap[(int)c] > (a)->K  && (a)->inmap[(int)c] < (a)->Kp-2)
+#define esl_abc_CIsUnknown(a, c)     ((a)->inmap[(int)c] == (a)->Kp-3)
+#define esl_abc_CIsNonresidue(a, c)  ((a)->inmap[(int)c] == (a)->Kp-2)
 #define esl_abc_CIsMissing(a, c)     ((a)->inmap[(int)c] == (a)->Kp-1)
-#define esl_abc_CGetGap(a)           ((a)->inmap[(int)(a)->K])
-#define esl_abc_CGetUnknown(a)       ((a)->inmap[(int)(a)->Kp-2])
-#define esl_abc_CGetMissing(a)       ((a)->inmap[(int)(a)->Kp-1])
-
-
-#endif /*!ESL_ALPHABET_INCLUDED*/
+#define esl_abc_CGetGap(a)           ((a)->sym[(int)(a)->K])
+#define esl_abc_CGetUnknown(a)       ((a)->sym[(int)(a)->Kp-3])
+#define esl_abc_CGetNonresidue(a)    ((a)->sym[(int)(a)->Kp-2])
+#define esl_abc_CGetMissing(a)       ((a)->sym[(int)(a)->Kp-1])
 
+#endif /*eslALPHABET_INCLUDED*/
 /*****************************************************************
- *    This copyrighted source code is freely distributed 
- *    under the terms of the GNU General Public License. See
- *    the files COPYRIGHT and LICENSE for details.
+ * Easel - a library of C functions for biological sequence analysis
+ * Version i1.1.1; July 2014
+ * Copyright (C) 2014 HHMI Janelia Farm Research Campus
+ * Other copyrights also apply. See the COPYRIGHT file for a full list.
+ * 
+ * Easel is distributed under the Janelia Farm Software License, a BSD
+ * license. See the LICENSE file for more details.
  *****************************************************************/
