@@ -1,13 +1,12 @@
-/* esl_dirichlet.h
- * Functions relevant to Beta, gamma, and Dirichlet densities,
+/* Functions relevant to Beta, gamma, and Dirichlet densities,
  * and simple and mixture Dirichlet priors.
  * 
  * SRE, Tue Nov  2 14:35:06 2004 [St. Louis]
- * SVN $Id: esl_dirichlet.h 147 2007-01-07 00:44:19Z eddys $
+ * SVN $Id: esl_dirichlet.h 664 2011-02-27 17:08:36Z eddys $
+ * SVN $URL: https://svn.janelia.org/eddylab/eddys/easel/branches/infernal/1.1/esl_dirichlet.h $
  */
-#ifndef ESL_DIRICHLET_INCLUDED
-#define ESL_DIRICHLET_INCLUDED
-
+#ifndef eslDIRICHLET_INCLUDED
+#define eslDIRICHLET_INCLUDED
 
 /* Structure: MIXDCHLET
  * 
@@ -25,6 +24,9 @@ typedef struct {
 } ESL_MIXDCHLET;
 
 extern ESL_MIXDCHLET *esl_mixdchlet_Create(int N, int K);
+extern int            esl_mixdchlet_Compare(ESL_MIXDCHLET *d1, ESL_MIXDCHLET *d2, double tol);
+extern int            esl_mixdchlet_Copy(ESL_MIXDCHLET *d, ESL_MIXDCHLET *d_dst);
+extern int            esl_mixdchlet_Dump(FILE *fp, ESL_MIXDCHLET *d);
 extern void           esl_mixdchlet_Destroy(ESL_MIXDCHLET *pri);
 extern int            esl_mixdchlet_MPParameters(double *c, int K,
 						 ESL_MIXDCHLET *pri, double *mix, 
@@ -32,9 +34,19 @@ extern int            esl_mixdchlet_MPParameters(double *c, int K,
 
 extern int esl_dirichlet_LogProbData(double *c, double *alpha, int K, 
 				     double *ret_answer);
+extern int esl_dirichlet_LogProbData_Mixture(double *c, ESL_MIXDCHLET *d, 
+					     double *ret_answer);
 extern int esl_dirichlet_LogProbProbs(double *p, double *alpha, int K, 
 				      double *ret_answer);
 
+/* Optional fitting code, when augmented by minimizing module.
+ */
+#ifdef eslAUGMENT_MINIMIZER
+#include "esl_minimizer.h"
+extern int esl_mixdchlet_Fit(double **c, int nc, ESL_MIXDCHLET *d, int be_verbose);
+
+#endif /*eslAUGMENT_MINIMIZER*/
+
 /* Optional sampling code, when augmented by random module.
  */
 #ifdef eslAUGMENT_RANDOM
@@ -52,7 +64,17 @@ extern int esl_dirichlet_SampleBeta(ESL_RANDOMNESS *r, double theta1,
 #ifdef eslAUGMENT_FILEPARSER
 #include <esl_fileparser.h>
 extern int esl_mixdchlet_Read(ESL_FILEPARSER *efp,  ESL_MIXDCHLET **ret_pri);
+extern int esl_mixdchlet_Write(FILE *fp,  ESL_MIXDCHLET *d);
 #endif /*eslAUGMENT_FILEPARSER*/
 
 
-#endif /*ESL_DIRICHLET_INCLUDED*/
+#endif /*eslDIRICHLET_INCLUDED*/
+/*****************************************************************
+ * Easel - a library of C functions for biological sequence analysis
+ * Version i1.1.1; July 2014
+ * Copyright (C) 2014 HHMI Janelia Farm Research Campus
+ * Other copyrights also apply. See the COPYRIGHT file for a full list.
+ * 
+ * Easel is distributed under the Janelia Farm Software License, a BSD
+ * license. See the LICENSE file for more details.
+ *****************************************************************/
