@@ -12,8 +12,6 @@
  *    9. Example.
  *   10. Copyright notice and license.
  *    
- * SVN $Id: esl_distance.c 269 2008-06-19 13:47:41Z eddys $
- * SRE, Mon Apr 17 20:05:43 2006 [St. Louis]
  */
 #include "esl_config.h"
 
@@ -61,7 +59,8 @@ static int jukescantor(int n1, int n2, int alphabet_size, double *opt_distance,
  *            degeneracies in biological alphabets.  For a more
  *            sophisticated and biosequence-aware comparison, use
  *            digitized sequences and the <esl_dst_XPairId()> function
- *            instead.
+ *            instead. Note that currently <esl_dst_XPairId()> does
+ *            not correctly handle degeneracies, but is set up to.
  *
  * Args:      asq1         - aligned character string 1
  *            asq2         - aligned character string 2
@@ -193,10 +192,20 @@ esl_dst_CJukesCantor(int K, const char *as1, const char *as2,
  * Synopsis:  Pairwise identity of two aligned digital seqs.
  * Incept:    SRE, Tue Apr 18 09:24:05 2006 [St. Louis]
  *
- * Purpose:   Digital version of <esl_dst_PairId()>: <adsq1> and
+ * Purpose:   Digital version of <esl_dst_CPairId()>: <adsq1> and
  *            <adsq2> are digitized aligned sequences, in alphabet
- *            <abc>. Otherwise, same as <esl_dst_PairId()>.
+ *            <abc>. Otherwise, same as <esl_dst_CPairId()> except
+ *            that only canonical residues are counted and checked for
+ *            identity, while <esl_dst_CPairId()> (which has no
+ *            alphabet) counts and checks identity of all alphanumeric
+ *            characters.
  *            
+ *            This function does not use <esl_abc_Match()> to handle
+ *            degeneracies but it is set up to do so. Doing that would
+ *            require that <opt_nid> be changed to a float or double,
+ *            or its meaning be changed to be the number of canonical
+ *            identities.
+ *
  * Args:      abc          - digital alphabet in use
  *            ax1          - aligned digital seq 1
  *            ax2          - aligned digital seq 2
@@ -699,7 +708,7 @@ esl_dst_XJukesCantorMx(const ESL_ALPHABET *abc, ESL_DSQ **ax, int nseq,
  *            deep MSAs.
  *            
  *            Each fractional pairwise identity (range $[0..$ pid $..1]$
- *            is calculated using <esl_dsq_CPairId()>.
+ *            is calculated using <esl_dst_CPairId()>.
  *
  * Returns:   <eslOK> on success, and <*ret_id> contains the average
  *            fractional identity.
@@ -735,8 +744,7 @@ esl_dst_CAverageId(char **as, int N, int max_comparisons, double *ret_id)
   /* If nseq is large, calculate average over a stochastic sample. */
   else				
     {
-      ESL_RANDOMNESS *r = esl_randomness_CreateTimeseeded();
-
+      ESL_RANDOMNESS *r = esl_randomness_Create(0);
       for (n = 0; n < max_comparisons; n++)
 	{
 	  do { i = esl_rnd_Roll(r, N); j = esl_rnd_Roll(r, N); } while (j == i); /* make sure j != i */
@@ -769,7 +777,7 @@ esl_dst_CAverageId(char **as, int N, int max_comparisons, double *ret_id)
  *            deep MSAs.
  *            
  *            Each fractional pairwise identity (range $[0..$ pid $..1]$
- *            is calculated using <esl_dsq_XPairId()>.
+ *            is calculated using <esl_dst_XPairId()>.
  *
  * Returns:   <eslOK> on success, and <*ret_id> contains the average
  *            fractional identity.
@@ -784,7 +792,7 @@ esl_dst_XAverageId(const ESL_ALPHABET *abc, ESL_DSQ **ax, int N, int max_compari
 {
   int    status;
   double id;
-  double sum;
+  double sum = 0.;
   int    i,j,n;
   
   if (N <= 1) { *ret_id = 1.; return eslOK; }
@@ -809,8 +817,7 @@ esl_dst_XAverageId(const ESL_ALPHABET *abc, ESL_DSQ **ax, int N, int max_compari
   /* If nseq is large, calculate average over a stochastic sample. */
   else				
     {
-      ESL_RANDOMNESS *r = esl_randomness_CreateTimeseeded();
-
+      ESL_RANDOMNESS *r = esl_randomness_Create(0);
       for (n = 0; n < max_comparisons; n++)
 	{
 	  do { i = esl_rnd_Roll(r, N); j = esl_rnd_Roll(r, N); } while (j == i); /* make sure j != i */
@@ -1204,7 +1211,7 @@ static ESL_OPTIONS options[] = {
   { "-h",     eslARG_NONE,  FALSE, NULL, NULL, NULL, NULL, NULL, "show help and usage",            0},
   { "-N",     eslARG_INT,    "10", NULL,"n>3", NULL, NULL, NULL, "number of iid seqs in alignment",0},
   { "-L",     eslARG_INT,    "50", NULL,"n>0", NULL, NULL, NULL, "length of seqs in alignment",    0},
-  { "--seed", eslARG_INT,    "42", NULL,"n>0", NULL, NULL, NULL, "random # seed",                  0},
+  { "--seed", eslARG_INT,    "42", NULL,"n>=0",NULL, NULL, NULL, "random # seed",                  0},
   {  0, 0, 0, 0, 0, 0, 0, 0, 0, 0 },
 };
 static char usage[] = "Usage: ./testdrive-distance [-options]";
@@ -1328,15 +1335,17 @@ main(int argc, char **argv)
 
 int main(int argc, char **argv)
 {
-  ESL_MSAFILE *afp; 
-  ESL_MSA     *msa;
-  ESL_DMATRIX *P;
-  int          status;
-  int          i,j;
-  double       min, avg, max;
+  ESLX_MSAFILE *afp; 
+  ESL_MSA      *msa;
+  ESL_DMATRIX  *P;
+  int           i,j;
+  double        min, avg, max;
+  int           status;
 
-  esl_msafile_Open(argv[1], eslMSAFILE_UNKNOWN, NULL, &afp);
-  esl_msa_Read(afp, &msa);
+  if ((status = eslx_msafile_Open(NULL, argv[1], NULL, eslMSAFILE_UNKNOWN, NULL, &afp)) != eslOK)
+    eslx_msafile_OpenFailure(afp, status);
+  if ((status = eslx_msafile_Read(afp, &msa)) != eslOK)
+    eslx_msafile_ReadFailure(afp, status);
 
   esl_dst_CPairIdMx(msa->aseq, msa->nseq, &P);
 
@@ -1358,17 +1367,24 @@ int main(int argc, char **argv)
 
   esl_dmatrix_Destroy(P);
   esl_msa_Destroy(msa);
-  esl_msafile_Close(afp);
+  eslx_msafile_Close(afp);
   return 0;
 }
 /*::cexcerpt::distance_example::end::*/
 #endif /*eslDISTANCE_EXAMPLE*/
 
 
- /*****************************************************************
-  *    This copyrighted source code is freely distributed 
-  *    under the terms of the GNU General Public License. See
-  *    the files COPYRIGHT and LICENSE for details.
-  *****************************************************************/
+/*****************************************************************
+ * Easel - a library of C functions for biological sequence analysis
+ * Version i1.1.1; July 2014
+ * Copyright (C) 2014 HHMI Janelia Farm Research Campus
+ * Other copyrights also apply. See the COPYRIGHT file for a full list.
+ * 
+ * Easel is distributed under the Janelia Farm Software License, a BSD
+ * license. See the LICENSE file for more details.
+ *
+ * SVN $Id: esl_distance.c 715 2011-08-03 21:04:24Z eddys $
+ * SVN $URL: https://svn.janelia.org/eddylab/eddys/easel/branches/infernal/1.1/esl_distance.c $
+ *****************************************************************/
 
 
