@@ -1,12 +1,12 @@
-/* esl_distance.h
- * Distances between aligned sequences, including both
+/* Distances between aligned sequences, including both
  * probabilistic evolutionary models and ad hoc measures.
  * 
- * SVN $Id: esl_distance.h 192 2007-06-12 15:06:22Z eddys $
  * SRE, Fri Apr 28 06:41:13 2006 [New York]
+ * SVN $Id: esl_distance.h 664 2011-02-27 17:08:36Z eddys $
+ * SVN $URL: https://svn.janelia.org/eddylab/eddys/easel/branches/infernal/1.1/esl_distance.h $
  */
-#ifndef ESL_DISTANCE_INCLUDED
-#define ESL_DISTANCE_INCLUDED
+#ifndef eslDISTANCE_INCLUDED
+#define eslDISTANCE_INCLUDED
 
 #include "easel.h"		/* ESL_DSQ declaration      */
 #ifdef eslAUGMENT_ALPHABET
@@ -64,9 +64,13 @@ extern int esl_dst_XAverageId(const ESL_ALPHABET *abc, ESL_DSQ **ax, int N, int
 #endif
 
 
-#endif /*ESL_DISTANCE_INCLUDED*/
+#endif /*eslDISTANCE_INCLUDED*/
 /*****************************************************************
- *    This copyrighted source code is freely distributed 
- *    under the terms of the GNU General Public License. See
- *    the files COPYRIGHT and LICENSE for details.
+ * Easel - a library of C functions for biological sequence analysis
+ * Version i1.1.1; July 2014
+ * Copyright (C) 2014 HHMI Janelia Farm Research Campus
+ * Other copyrights also apply. See the COPYRIGHT file for a full list.
+ * 
+ * Easel is distributed under the Janelia Farm Software License, a BSD
+ * license. See the LICENSE file for more details.
  *****************************************************************/
