@@ -19,9 +19,6 @@
  * To do:
  *   - eventually probably want additional matrix types
  *   - unit tests poor 
- *
- * SRE, Tue Jul 13 14:42:14 2004 [St. Louis]
- * SVN $Id: esl_dmatrix.c 241 2008-04-01 19:01:52Z eddys $
  */
 #include "esl_config.h"
 
@@ -138,7 +135,6 @@ esl_dmatrix_CreateUpper(int n)
 }
 
 
-
 /* Function:  esl_dmatrix_Destroy()
  *            
  * Purpose:   Frees an <ESL_DMATRIX> object <A>.
@@ -236,10 +232,10 @@ esl_dmatrix_Clone(const ESL_DMATRIX *A)
 /* Function:  esl_dmatrix_Compare()
  *
  * Purpose:   Compares matrix <A> to matrix <B> element by element,
- *            using <esl_DCompare()> on each cognate element pair, 
- *            with equality defined by a fractional tolerance <tol>.
- *            If all elements are equal, return <eslOK>; if any
- *            elements differ, return <eslFAIL>. 
+ *            using <esl_DCompare()> on each cognate element pair,
+ *            with relative equality defined by a fractional tolerance
+ *            <tol>.  If all elements are equal, return <eslOK>; if
+ *            any elements differ, return <eslFAIL>.
  *            
  *            <A> and <B> may be of different types; for example,
  *            a packed upper triangular matrix A is compared to
@@ -266,10 +262,10 @@ esl_dmatrix_Compare(const ESL_DMATRIX *A, const ESL_DMATRIX *B, double tol)
 	for (j = 0; j < A->m; j++)
 	  {
 	    if (A->type == eslUPPER && i > j) x1 = 0.;
-	    else                                         x1 = A->mx[i][j];
+	    else                              x1 = A->mx[i][j];
 
 	    if (B->type == eslUPPER && i > j) x2 = 0.;
-	    else                                         x2 = B->mx[i][j];
+	    else                              x2 = B->mx[i][j];
 
 	    if (esl_DCompare(x1, x2, tol) == eslFAIL) return eslFAIL;
 	  }
@@ -278,6 +274,51 @@ esl_dmatrix_Compare(const ESL_DMATRIX *A, const ESL_DMATRIX *B, double tol)
 }
 
 
+/* Function:  esl_dmatrix_CompareAbs()
+ *
+ * Purpose:   Compares matrix <A> to matrix <B> element by element,
+ *            using <esl_DCompareAbs()> on each cognate element pair,
+ *            with absolute equality defined by a absolute difference tolerance
+ *            <tol>.  If all elements are equal, return <eslOK>; if
+ *            any elements differ, return <eslFAIL>.
+ *            
+ *            <A> and <B> may be of different types; for example,
+ *            a packed upper triangular matrix A is compared to
+ *            a general matrix B by assuming <A->mx[i][j] = 0.> for
+ *            all $i>j$.
+ */
+int
+esl_dmatrix_CompareAbs(const ESL_DMATRIX *A, const ESL_DMATRIX *B, double tol)
+{
+  int i,j,c;
+  double x1,x2;
+
+  if (A->n != B->n) return eslFAIL;
+  if (A->m != B->m) return eslFAIL;
+
+  if (A->type == B->type) 
+    {  /* simple case. */
+      for (c = 0; c < A->ncells; c++) /* can deal w/ packed or unpacked storage */
+	if (esl_DCompareAbs(A->mx[0][c], B->mx[0][c], tol) == eslFAIL) return eslFAIL;
+    }
+  else 
+    { /* comparing matrices of different types */
+      for (i = 0; i < A->n; i++)
+	for (j = 0; j < A->m; j++)
+	  {
+	    if (A->type == eslUPPER && i > j) x1 = 0.;
+	    else                              x1 = A->mx[i][j];
+
+	    if (B->type == eslUPPER && i > j) x2 = 0.;
+	    else                              x2 = B->mx[i][j];
+
+	    if (esl_DCompareAbs(x1, x2, tol) == eslFAIL) return eslFAIL;
+	  }
+    }
+  return eslOK;
+}
+
+
 /* Function:  esl_dmatrix_Set()
  *
  * Purpose:   Set all elements $a_{ij}$ in matrix <A> to <x>,
@@ -678,7 +719,6 @@ int
 esl_dmx_Exp(const ESL_DMATRIX *Q, double t, ESL_DMATRIX *P)
 {
 /*::cexcerpt::function_comment_example::end::*/
-  int status;
   ESL_DMATRIX *Qz   = NULL;	/* Q/2^z rescaled matrix*/
   ESL_DMATRIX *Qpow = NULL;	/* keeps running product Q^k */
   ESL_DMATRIX *C    = NULL;	/* tmp storage for matrix multiply result */
@@ -687,6 +727,7 @@ esl_dmx_Exp(const ESL_DMATRIX *Q, double t, ESL_DMATRIX *P)
   int    z;
   double zfac;
   int    k;
+  int    status;
     
   /* Contract checks  */
   if (Q->type != eslGENERAL) ESL_EXCEPTION(eslEINVAL, "Q isn't general");
@@ -696,9 +737,9 @@ esl_dmx_Exp(const ESL_DMATRIX *Q, double t, ESL_DMATRIX *P)
   if (P->n    != Q->n)       ESL_EXCEPTION(eslEINVAL, "P isn't same size as Q");
 
   /* Allocation of working space */
-  if ((Qz   = esl_dmatrix_Create(Q->n, Q->n)) == NULL) goto ERROR;
-  if ((Qpow = esl_dmatrix_Create(Q->n, Q->n)) == NULL) goto ERROR;
-  if ((C    = esl_dmatrix_Create(Q->n, Q->n)) == NULL) goto ERROR;
+  if ((Qz   = esl_dmatrix_Create(Q->n, Q->n)) == NULL) { status = eslEMEM; goto ERROR; }
+  if ((Qpow = esl_dmatrix_Create(Q->n, Q->n)) == NULL) { status = eslEMEM; goto ERROR; }
+  if ((C    = esl_dmatrix_Create(Q->n, Q->n)) == NULL) { status = eslEMEM; goto ERROR; }
   
   /* Figure out how much to scale the matrix down by.  This is not
    * magical; we're just knocking its magnitude down in an ad hoc way.
@@ -1406,7 +1447,14 @@ int main(void)
 
 
 /*****************************************************************
- *    This copyrighted source code is freely distributed 
- *    under the terms of the GNU General Public License. See
- *    the files COPYRIGHT and LICENSE for details.
+ * Easel - a library of C functions for biological sequence analysis
+ * Version i1.1.1; July 2014
+ * Copyright (C) 2014 HHMI Janelia Farm Research Campus
+ * Other copyrights also apply. See the COPYRIGHT file for a full list.
+ * 
+ * Easel is distributed under the Janelia Farm Software License, a BSD
+ * license. See the LICENSE file for more details.
+ *
+ * SVN $Id: esl_dmatrix.c 755 2012-03-21 11:30:23Z eddys $
+ * SVN $URL: https://svn.janelia.org/eddylab/eddys/easel/branches/infernal/1.1/esl_dmatrix.c $
  *****************************************************************/
