@@ -7,15 +7,13 @@
  *    4. Test driver.
  *    5. Examples.
  *    6. Copyright/license information.
- * 
- * SRE, Tue Jul 13 14:41:52 2004 [St. Louis]
- * SVN $Id: esl_fileparser.c 255 2008-05-30 17:49:15Z eddys $
  */
 #include "esl_config.h"
 
 #include <stdlib.h>
 #include <stdio.h>
 #include <string.h>
+#include <ctype.h>
 
 #include "easel.h"
 #include "esl_fileparser.h"
@@ -33,6 +31,20 @@ static int nextline(ESL_FILEPARSER *efp);
  * 
  *            As a special case, if <filename> is "-", set up the
  *            fileparser to read and parse <stdin>.
+ *            
+ *            <envvar> is optional name of an environment variable,
+ *            such as <BLASTDB>. This environment variable contains a
+ *            colon-delimited list of directories in which the
+ *            <filename> may lie relative to.  We looks first relative
+ *            to the current working directory, then in any
+ *            directories specified by <envvar>. If <envvar> is <NULL>,
+ *            we only look in the current working directory.
+ *            
+ * Args:      filename  - filename, relative path, or fully qualified path
+ *            envvar    - optional environment variable name to find 
+ *                        colon-delimited list of directories <filename>
+ *                        may reside in; or <NULL>
+ *            ret_efp   - RETURN: opened <ESL_FILEPARSER>            
  *
  * Returns:   <eslOK> on success, and <ret_fp> points
  *            to a new <ESL_FILEPARSER> object.
@@ -44,18 +56,21 @@ static int nextline(ESL_FILEPARSER *efp);
  * Throws:    <eslEMEM> on allocation failure.
  */
 int
-esl_fileparser_Open(const char *filename, ESL_FILEPARSER **ret_efp)
+esl_fileparser_Open(const char *filename, const char *envvar, ESL_FILEPARSER **ret_efp)
 {
   int             status;
   ESL_FILEPARSER *efp = NULL;
 
   if ((efp = esl_fileparser_Create(NULL)) == NULL) { status = eslEMEM;      goto ERROR; }
 
-  if (strcmp(filename, "-") == 0)
+  if (strcmp(filename, "-") == 0) 
     efp->fp = stdin;
-  else {
-    if ((efp->fp = fopen(filename, "r")) == NULL)    { status = eslENOTFOUND; goto ERROR; }
+  else if ((efp->fp = fopen(filename, "r")) != NULL) { 
+    if ((status  = esl_strdup(filename, -1, &(efp->filename))) != eslOK) goto ERROR;
   }
+  else if ((status = esl_FileEnvOpen(filename, envvar, &(efp->fp), &(efp->filename))) != eslOK) 
+    { status = eslENOTFOUND; goto ERROR; }    
+    
   *ret_efp = efp;
   return eslOK;
 
@@ -96,9 +111,14 @@ esl_fileparser_Create(FILE *fp)
   efp->commentchar = '\0';
   efp->tok         = NULL;
   efp->toklen      = 0;
+  efp->tokchar     = '\0';
+  efp->filename    = NULL;
   efp->linenumber  = 0;
   efp->errbuf[0]   = '\0';
-
+  efp->is_buffer   = FALSE;
+  efp->mem_buffer  = NULL;
+  efp->mem_size    = 0;
+  efp->mem_pos     = 0;
   return efp;
   
  ERROR:
@@ -107,10 +127,40 @@ esl_fileparser_Create(FILE *fp)
 }
 
 
+/* Function:  esl_fileparser_CreateMapped()
+ * Incept:    MSF, Mon Aug 16 2010 [Janelia]
+ *
+ * Purpose:   Sets up a memory buffer to be parsed with the
+ *            file parser routines.Take an open file <fp>, and transform it to
+ *            a fileparser object -- preparing to parse it
+ *            one whitespace-delimited field at a time.
+ *
+ * Args:      fp  - open FILE to parse
+ *
+ * Returns:   a new <ESL_FILEPARSER> object, which must be 
+ *            free'd by the caller with <esl_fileparser_Destroy()>.
+ *
+ * Throws:    <eslEMEM> if an allocation failed.
+ *            
+ * Xref:      STL8 p.56.
+ */
+ESL_FILEPARSER *
+esl_fileparser_CreateMapped(void *buffer, int size)
+{
+  ESL_FILEPARSER *efp = NULL;
+
+  if ((efp = esl_fileparser_Create(NULL)) == NULL) return NULL;
+  
+  efp->is_buffer   = TRUE;
+  efp->mem_buffer  = buffer;
+  efp->mem_size    = size;
+  return efp;
+}
+
+
 
 
 /* Function:  esl_fileparser_SetCommentChar()
- * Incept:    SRE, Sat Jul 10 10:18:35 2004 [St. Louis]
  *
  * Purpose:   Defines a single character <c> for comments. Anything
  *            on a line following this character is ignored
@@ -129,48 +179,6 @@ esl_fileparser_SetCommentChar(ESL_FILEPARSER *efp, char c)
 }
 
 
-/* Function:  esl_fileparser_NextLine()
- * Incept:    SRE, Tue Apr  3 08:27:22 2007 [Janelia]
- *
- * Purpose:   Advance the parser to the next non-blank, non-comment
- *            data line that contains at least one token. 
- *
- * Returns:   <eslOK> on success.
- *            <eslEOF> if no more tokens remain in the file.  
- *
- * Throws:    <eslEMEM> on allocation error.
- */
-int
-esl_fileparser_NextLine(ESL_FILEPARSER *efp)
-{
-  int   status;
-  char *tok    = NULL;
-  int   toklen = 0;
-  int   tokcode;
-
-  while ((status = nextline(efp)) == eslOK) 
-    {
-      tokcode = esl_strtok(&(efp->s), " \t\r\n", &tok, &toklen);
-      if (tokcode == eslEOL || (tokcode == eslOK && *tok == efp->commentchar)) continue; /* no tokens on line */
-      if (tokcode != eslOK) ESL_XFAIL(tokcode, efp->errbuf, "esl_strtok() failed");
-      break;
-    } 
-  if (status == eslEOF) return eslEOF;
-  if (status != eslOK)  ESL_XFAIL(status, efp->errbuf, "nextline() failed");
-  
-  /* Remember this token. The next GetToken call will regurgitate it instead
-   * of finding its own. efp->s now points to the character after tok ends. 
-   */
-  efp->tok    = tok;
-  efp->toklen = toklen;
-  return eslOK;
-
- ERROR:
-  efp->tok    = NULL;
-  efp->toklen = 0;
-  return status;
-}  
-
 
 /* Function:  esl_fileparser_GetToken()
  * Incept:    SRE, Fri Jul  9 13:03:50 2004 [St. Louis]
@@ -210,13 +218,12 @@ esl_fileparser_GetToken(ESL_FILEPARSER *efp, char **opt_tok, int *opt_toklen)
   if (efp->tok != NULL) {
     if (opt_tok    != NULL) *opt_tok    = efp->tok;
     if (opt_toklen != NULL) *opt_toklen = efp->toklen;
-    efp->tok    = NULL;
-    efp->toklen = 0;
+    efp->tok     = NULL;
+    efp->toklen  = 0;
+    efp->tokchar = '\0';
     return eslOK;
   }
-
-  /* If not, then find next token.
-   */
+  /* If not, then find next token. */
 
   /* First, make sure we have a line loaded. 
    * On the first call to GetToken, we won't.
@@ -232,9 +239,8 @@ esl_fileparser_GetToken(ESL_FILEPARSER *efp, char **opt_tok, int *opt_toklen)
    */
   do {
     goodtok = FALSE;
-    tokcode = esl_strtok(&(efp->s), " \t\r\n", &tok, &toklen);
-    if (tokcode == eslEOL ||
-	(tokcode == eslOK && *tok == efp->commentchar)) 
+    tokcode = esl_strtok_adv(&(efp->s), " \t\r\n", &tok, &toklen, NULL);
+    if (tokcode == eslEOL || (tokcode == eslOK && *tok == efp->commentchar)) 
       {
 	fcode = nextline(efp);
 	if (fcode != eslOK) return fcode;
@@ -244,11 +250,141 @@ esl_fileparser_GetToken(ESL_FILEPARSER *efp, char **opt_tok, int *opt_toklen)
       { sprintf(efp->errbuf, "esl_strtok() failed"); return tokcode;}
   } while (! goodtok);
 
-  if (opt_tok != NULL)    *opt_tok    = tok;
+  if (opt_tok    != NULL) *opt_tok    = tok;
   if (opt_toklen != NULL) *opt_toklen = toklen;
   return eslOK;
 }
 
+
+/* Function:  esl_fileparser_NextLine()
+ * Incept:    SRE, Tue Apr  3 08:27:22 2007 [Janelia]
+ *
+ * Purpose:   Advance the parser to the next non-blank, non-comment
+ *            data line that contains at least one token. 
+ *
+ * Returns:   <eslOK> on success.
+ *            <eslEOF> if no more tokens remain in the file.  
+ *
+ * Throws:    <eslEMEM> on allocation error.
+ */
+int
+esl_fileparser_NextLine(ESL_FILEPARSER *efp)
+{
+  int   status;
+  char *tok     = NULL;
+  int   toklen  = 0;
+  char  tokchar = '\0';
+  int   tokcode;
+
+  while ((status = nextline(efp)) == eslOK) 
+    {
+      tokcode = esl_strtok_adv(&(efp->s), " \t\r\n", &tok, &toklen, &tokchar);
+      if (tokcode == eslEOL || (tokcode == eslOK && *tok == efp->commentchar)) continue; /* no tokens on line */
+      if (tokcode != eslOK) ESL_XFAIL(tokcode, efp->errbuf, "esl_strtok() failed");
+      break;
+    } 
+  if (status == eslEOF) { status = eslEOF; goto ERROR; }
+  if (status != eslOK)  ESL_XFAIL(status, efp->errbuf, "nextline() failed");
+  
+  /* Remember this token. The next GetToken call will regurgitate it instead
+   * of finding its own. efp->s now points to the character after tok ends. 
+   */
+  efp->tok     = tok;
+  efp->toklen  = toklen;
+  efp->tokchar = tokchar;
+  return eslOK;
+
+ ERROR:
+  efp->tok     = NULL;
+  efp->toklen  = 0;
+  efp->tokchar = '\0';
+  return status;
+}  
+
+
+/* Function:  esl_fileparser_NextLinePeeked()
+ * Synopsis:  Read the next line, prepending a peek.
+ * Incept:    SRE, Wed Oct 15 10:08:37 2008 [Janelia]
+ *
+ * Purpose:   Sometimes we need to peek at the start of an input stream
+ *            to see whether it is in a binary format, before we start
+ *            parsing it as ASCII lines. When this happens, the caller
+ *            will typically have used <fread()> to read a fixed
+ *            number of bytes from the input stream, checked to see if
+ *            they are a magic number representing a binary format,
+ *            and found that they are not. The caller then wants to
+ *            switch to reading in ASCII format with the <fileparser>
+ *            API, but with those bytes included on the first
+ *            line. Because the file might start with comments or
+ *            blank lines that need to be skipped, we want to deal
+ *            with the peeked data in the context of the
+ *            <ESL_FILEPARSER>. The caller cannot simply close and
+ *            reopen the stream, because the stream may be a pipe
+ *            (<stdin> or <gzip -dc> for example).
+ *            
+ *            The caller passes the bytes it peeked at with <fread()>
+ *            in <prefix>, and the number of bytes it peeked at in
+ *            <plen>.
+ *            
+ *            The parser is advanced to the next non-blank,
+ *            non-comment data line that contains at least one token,
+ *            taking the prepended <prefix> into account.
+ *
+ * Args:      efp      - open fileparser
+ *            prefix   - bytes that caller obtained by peeking with fread()
+ *            plen     - number of bytes in prefix
+ *
+ * Returns:   <eslOK> on success.
+ *            <eslEOF> if no more tokens remain in the file.  
+ *
+ * Throws:    <eslEMEM> on allocation error.
+ *
+ * Xref:      For an example, see HMMER's HMM save file input.
+ */
+int
+esl_fileparser_NextLinePeeked(ESL_FILEPARSER *efp, char *prefix, int plen)
+{
+  int   blen;
+  void *tmp;
+  char *tok     = NULL;
+  int   toklen  = 0;
+  char  tokchar = '\0';
+  int   status;
+  
+  if ((status = nextline(efp)) != eslOK)  goto ERROR; /* EOF, EMEM */
+  blen = strlen(efp->buf);
+  
+  if (blen + plen + 1 > efp->buflen) {
+    ESL_RALLOC(efp->buf, tmp, sizeof(char) * (blen + plen + 1));
+    efp->buflen = blen + plen + 1;
+  }
+  memmove(efp->buf+plen, efp->buf, blen+1);
+  memcpy(efp->buf, prefix, plen);
+  efp->s = efp->buf;
+
+  status = esl_strtok_adv(&(efp->s), " \t\r\n", &tok, &toklen, &tokchar);
+  
+  if (status == eslOK && *tok != efp->commentchar) 
+    {
+      efp->tok     = tok;
+      efp->toklen  = toklen;
+      efp->tokchar = tokchar;
+      status       = eslOK;
+    }
+  else status = esl_fileparser_NextLine(efp);
+
+  return status;
+
+ ERROR:
+  efp->tok     = NULL;
+  efp->toklen  = 0;
+  efp->tokchar = '\0';
+  return status;
+}  
+
+
+
+
 /* Function:  esl_fileparser_GetTokenOnLine()
  * Incept:    SRE, Tue Apr  3 08:46:59 2007 [Janelia]
  *
@@ -293,8 +429,9 @@ esl_fileparser_GetTokenOnLine(ESL_FILEPARSER *efp, char **opt_tok, int *opt_tokl
   if (efp->tok != NULL) {
     if (opt_tok    != NULL) *opt_tok    = efp->tok;
     if (opt_toklen != NULL) *opt_toklen = efp->toklen;
-    efp->tok    = NULL;
-    efp->toklen = 0;
+    efp->tok     = NULL;
+    efp->toklen  = 0;
+    efp->tokchar = '\0';
     return eslOK;
   }
 
@@ -302,7 +439,7 @@ esl_fileparser_GetTokenOnLine(ESL_FILEPARSER *efp, char **opt_tok, int *opt_tokl
   if (efp->buf == NULL) { status = eslEOL;  goto ERROR; }
 
   /* Find next token in the line loaded in the parser. */
-  status = esl_strtok(&(efp->s), " \t\r\n", &tok, &toklen);
+  status = esl_strtok_adv(&(efp->s), " \t\r\n", &tok, &toklen, NULL);
   if (status == eslEOL) goto ERROR;
   if (status != eslOK)  goto ERROR;
   if (status == eslOK && *tok == efp->commentchar) { status = eslEOL; goto ERROR; }
@@ -318,6 +455,54 @@ esl_fileparser_GetTokenOnLine(ESL_FILEPARSER *efp, char **opt_tok, int *opt_tokl
 }
 
 
+/* Function:  esl_fileparser_GetRemainingLine()
+ * Synopsis:  Returns pointer to the rest of the current line.
+ * Incept:    SRE, Mon Oct 13 08:59:26 2008 [Janelia]
+ *
+ * Purpose:   Set a pointer <*ret_s> to the rest of the current line
+ *            held by the fileparser <efp>. 
+ *            
+ *            Because <ret_s> points to internal storage in the
+ *            fileparser, the caller should be finished with it before
+ *            making its next call to any fileparser function.
+ *            
+ *            Any comment characters on the rest of the line are
+ *            ignored: this is designed for a case where the rest of
+ *            the line is to be read as free text.
+ *            
+ * Args:      efp    - fileparser 
+ *            ret_s  - RETURN: pointer to the remainder of the line
+ *
+ * Returns:   <eslOK> on success.
+ *            <eslEOL> if nothing remains on the line, and <*ret_s>
+ *            is <NULL>.
+ *
+ * Throws:    (no abnormal error conditions)
+ */
+int
+esl_fileparser_GetRemainingLine(ESL_FILEPARSER *efp, char **ret_s)
+{
+  /* No line loaded? Then we can't find anything on it. */
+  if (efp->buf == NULL) { *ret_s = NULL; return eslEOL; }
+
+  /* If we prestored the next token, we must revert that move. */
+  if (efp->tok != NULL)
+    {
+      *(efp->s-1) = efp->tokchar;
+      efp->s      = efp->tok;
+      efp->tok    = NULL;
+      efp->toklen  = 0;
+      efp->tokchar = '\0';
+    }
+
+  /* skip leading whitespace */
+  while (isspace(*(efp->s))) efp->s++;  
+
+  /* Return everything to end of line as a "token", stripping newline  */
+  return esl_strtok(&(efp->s), "\r\n", ret_s);
+}
+
+
 /* Function:  esl_fileparser_Destroy()
  * Incept:    SRE, Fri Jul  9 13:22:36 2004 [St. Louis]
  *
@@ -346,6 +531,7 @@ esl_fileparser_Close(ESL_FILEPARSER *efp)
   if (efp == NULL) return;
   
   if (efp->fp != NULL && efp->fp != stdin) fclose(efp->fp);
+  if (efp->filename != NULL) free(efp->filename);
   esl_fileparser_Destroy(efp);
 }
 
@@ -375,11 +561,41 @@ nextline(ESL_FILEPARSER *efp)
 {
   int status;
 
-  if ((status = esl_fgets(&(efp->buf), &(efp->buflen), efp->fp)) != eslOK) 
-    { sprintf(efp->errbuf, "esl_fgets() failed"); return status;}
+  /* check if we are reading from a file or a buffer */
+  if (efp->is_buffer) {
+    int   len;
+    int   end;
+
+    char *ptr;
+    char *tmp;
+
+    if (efp->mem_pos >= efp->mem_size) return eslEOF;
+
+    len = 0;
+    end = efp->mem_size - efp->mem_pos;
+    ptr = efp->mem_buffer + efp->mem_pos;
+    while (len < end && *ptr++ != '\n') ++len;
+    if (len < end) ++len;
+
+    if (len + 1 > efp->buflen) {
+      ESL_RALLOC(efp->buf, tmp, len * 2);
+      efp->buflen = len * 2;
+    }
+    memcpy(efp->buf, efp->mem_buffer + efp->mem_pos, len);
+    efp->buf[len] = 0;
+
+    efp->mem_pos += len;
+
+  } else {
+    if ((status = esl_fgets(&(efp->buf), &(efp->buflen), efp->fp)) != eslOK) 
+      { sprintf(efp->errbuf, "esl_fgets() failed"); return status;}
+  }
   efp->s = efp->buf;
   efp->linenumber++;
   return eslOK;
+
+ ERROR:
+  return eslEMEM;
 }
 
 
@@ -434,7 +650,7 @@ utest_GetTokenOnLine(char *filename)
   int             nlines = 0;
   char            expect[32];
 
-  if (esl_fileparser_Open(filename, &efp) != eslOK) esl_fatal("File open failed");
+  if (esl_fileparser_Open(filename, NULL, &efp) != eslOK) esl_fatal("File open failed");
   esl_fileparser_SetCommentChar(efp, '#');
 
   while ((status = esl_fileparser_NextLine(efp)) == eslOK)
@@ -457,6 +673,35 @@ utest_GetTokenOnLine(char *filename)
   esl_fileparser_Close(efp);
   return;
 }
+
+static void
+utest_GetTokenBuffered(char *buffer)
+{
+  int status;
+  ESL_FILEPARSER *efp = NULL;
+  char           *tok = NULL;
+  int             toklen = 0;
+  int             ntok   = 0;
+
+  if ((efp = esl_fileparser_CreateMapped(buffer, strlen(buffer))) == NULL)  
+    esl_fatal("Failed to associate buffer with fileparser");
+
+  esl_fileparser_SetCommentChar(efp, '#');
+  
+  while ((status = esl_fileparser_GetToken(efp, &tok, &toklen)) == eslOK)
+    {
+      if (toklen != 6)                   esl_fatal("bad token %s", tok);
+      if (strncmp(tok, "token", 5) != 0) esl_fatal("bad token %s", tok);
+      ntok++;
+    }
+  if (status != eslEOF)  esl_fatal("Abnormal parse termination");
+  if (ntok != 5)         esl_fatal("bad total token number %d\n", ntok);
+  
+  esl_fileparser_Destroy(efp);
+
+  return;
+}
+
 #endif /*eslFILEPARSER_TESTDRIVE*/
 
 /*****************************************************************
@@ -479,16 +724,18 @@ main(int argc, char **argv)
   char  tmpfile[32] = "esltmpXXXXXX";
   FILE *fp;
 
+  char stream[] = "# Full line comment\n"
+                  "token1  # Trailing comment\n"
+                  "\n"                                   /* blank line */
+                  "   \n"                                /* whitespace line */
+                  "   # sowing comment/whitespace confusion...\n"
+                  "token2\ttoken3  token4\n"
+                  "token5";                              /* file ends w/ no \n */
+
   /* Create a test file to read.
    */
   if (esl_tmpfile_named(tmpfile, &fp) != eslOK) esl_fatal("File open failed");
-  fprintf(fp, "# Full line comment\n");
-  fprintf(fp, "token1  # Trailing comment\n");
-  fprintf(fp, "\n");		/* blank line */
-  fprintf(fp, "   \n");		/* whitespace line */
-  fprintf(fp, "   # sowing comment/whitespace confusion...\n"); 
-  fprintf(fp, "token2\ttoken3  token4\n");
-  fprintf(fp, "token5");	/* file ends w/ no \n */
+  fprintf(fp, "%s", stream);
   fclose(fp);
 
   /* Run unit tests using that file.
@@ -496,6 +743,7 @@ main(int argc, char **argv)
    */
   utest_GetToken(tmpfile);
   utest_GetTokenOnLine(tmpfile);
+  utest_GetTokenBuffered(stream);
 
   remove(tmpfile);
   return 0;
@@ -528,7 +776,7 @@ main(int argc, char **argv)
   char           *tok;
   int             toklen;
 
-  if (esl_fileparser_Open(filename, &efp) != eslOK) esl_fatal("File open failed");
+  if (esl_fileparser_Open(filename, NULL, &efp) != eslOK) esl_fatal("File open failed");
   esl_fileparser_SetCommentChar(efp, '#');
   
   while (esl_fileparser_GetToken(efp, &tok, &toklen) == eslOK) { 
@@ -562,7 +810,7 @@ main(int argc, char **argv)
   char           *tok;
   int             toklen;
 
-  if (esl_fileparser_Open(filename, &efp) != eslOK) esl_fatal("File open failed");
+  if (esl_fileparser_Open(filename, NULL, &efp) != eslOK) esl_fatal("File open failed");
   esl_fileparser_SetCommentChar(efp, '#');
   
   while (esl_fileparser_NextLine(efp) == eslOK)
@@ -586,8 +834,15 @@ main(int argc, char **argv)
 
 
 /*****************************************************************
- *    This copyrighted source code is freely distributed 
- *    under the terms of the GNU General Public License. See
- *    the files COPYRIGHT and LICENSE for details.
+ * Easel - a library of C functions for biological sequence analysis
+ * Version i1.1.1; July 2014
+ * Copyright (C) 2014 HHMI Janelia Farm Research Campus
+ * Other copyrights also apply. See the COPYRIGHT file for a full list.
+ * 
+ * Easel is distributed under the Janelia Farm Software License, a BSD
+ * license. See the LICENSE file for more details.
+ * 
+ * SVN $Id: esl_fileparser.c 833 2012-12-07 18:57:36Z nawrockie $
+ * SVN $URL: https://svn.janelia.org/eddylab/eddys/easel/branches/infernal/1.1/esl_fileparser.c $
  *****************************************************************/
 
