@@ -1,11 +1,11 @@
-/* esl_gamma.h
- * Gamma distributions.
+/* Gamma distributions.
  * 
  * SRE, Wed Nov 16 19:15:33 2005 [St. Louis]
- * SVN $Id: esl_gamma.h 83 2005-12-13 20:54:07Z eddy $
+ * SVN $Id: esl_gamma.h 664 2011-02-27 17:08:36Z eddys $
+ * SVN $URL: https://svn.janelia.org/eddylab/eddys/easel/branches/infernal/1.1/esl_gamma.h $
  */
-#ifndef ESL_GAMMA_INCLUDED
-#define ESL_GAMMA_INCLUDED
+#ifndef eslGAMMA_INCLUDED
+#define eslGAMMA_INCLUDED
 
 #ifdef eslAUGMENT_RANDOM
 #include <esl_random.h>
@@ -34,4 +34,13 @@ extern double esl_gam_Sample(ESL_RANDOMNESS *r, double mu, double lambda, double
 
 extern int esl_gam_FitComplete(double *x, int n, double mu, double *ret_lambda, double *ret_tau);
 
-#endif /*ESL_GAMMA_INCLUDED*/
+#endif /*eslGAMMA_INCLUDED*/
+/*****************************************************************
+ * Easel - a library of C functions for biological sequence analysis
+ * Version i1.1.1; July 2014
+ * Copyright (C) 2014 HHMI Janelia Farm Research Campus
+ * Other copyrights also apply. See the COPYRIGHT file for a full list.
+ * 
+ * Easel is distributed under the Janelia Farm Software License, a BSD
+ * license. See the LICENSE file for more details.
+ *****************************************************************/
