@@ -2,11 +2,9 @@
  * configuration of an application. Extends standard
  * UNIX/POSIX/GNU getopt().
  * 
- * SVN $Id: esl_getopts.h 241 2008-04-01 19:01:52Z eddys $
- * SRE, Thu Jan 13 08:38:28 2005 [St. Louis]
  */
-#ifndef ESL_GETOPTS_INCLUDED
-#define ESL_GETOPTS_INCLUDED
+#ifndef eslGETOPTS_INCLUDED
+#define eslGETOPTS_INCLUDED
 
 #include "easel.h"
 
@@ -63,7 +61,9 @@ typedef struct {
   int   *setby;		  /* array [0..nopts-1] for who set option i   */
   int   *valloc;          /* 0, or length of alloc for val[i]          */
 
-  char  *optstring;	  /* internal: ptr into string of 1-char opts in argv[] */
+  char  *optstring;	  /* internal: ptr into string of 1-char opts in argv[]          */
+  char  *spoof;	    	  /* internal allocation: ProcessSpoof() stores cmdline          */
+  char **spoof_argv;	  /* internal allocation: ProcessSpoof()'s ptrs into its cmdline */
 
   char  errbuf[eslERRBUFSIZE];	/* buffer for reporting user error     */
 } ESL_GETOPTS;
@@ -84,16 +84,24 @@ typedef struct {
  */
 extern ESL_GETOPTS *esl_getopts_Create(ESL_OPTIONS *opt);
 extern ESL_GETOPTS *esl_getopts_CreateDefaultApp(ESL_OPTIONS *options, int nargs, int argc, char **argv, char *banner, char *usage);
+extern int          esl_getopts_Reuse  (ESL_GETOPTS *g);
 extern void         esl_getopts_Destroy(ESL_GETOPTS *g);
 extern void         esl_getopts_Dump(FILE *ofp, ESL_GETOPTS *g);
 
-extern int esl_opt_ProcessConfigfile(ESL_GETOPTS *g, char *filename, FILE *fp);
+extern int esl_opt_ProcessConfigfile (ESL_GETOPTS *g, char *filename, FILE *fp);
 extern int esl_opt_ProcessEnvironment(ESL_GETOPTS *g);
-extern int esl_opt_ProcessCmdline(ESL_GETOPTS *g, int argc, char **argv);
-extern int esl_opt_VerifyConfig(ESL_GETOPTS *g);
-extern int esl_opt_ArgNumber(const ESL_GETOPTS *g);
+extern int esl_opt_ProcessCmdline    (ESL_GETOPTS *g, int argc, char **argv);
+extern int esl_opt_ProcessSpoof      (ESL_GETOPTS *g, const char *cmdline);
+extern int esl_opt_VerifyConfig      (ESL_GETOPTS *g);
+extern int esl_opt_ArgNumber   (const ESL_GETOPTS *g);
+extern int esl_opt_SpoofCmdline(const ESL_GETOPTS *g, char **ret_cmdline);
+
+extern int esl_opt_GetSetter(const ESL_GETOPTS *g, char *optname);
 
 extern int    esl_opt_IsDefault (const ESL_GETOPTS *g, char *optname);
+extern int    esl_opt_IsOn      (const ESL_GETOPTS *g, char *optname);
+extern int    esl_opt_IsUsed    (const ESL_GETOPTS *g, char *optname);
+
 extern int    esl_opt_GetBoolean(const ESL_GETOPTS *g, char *optname);
 extern int    esl_opt_GetInteger(const ESL_GETOPTS *g, char *optname);
 extern double esl_opt_GetReal   (const ESL_GETOPTS *g, char *optname);
@@ -103,9 +111,17 @@ extern char  *esl_opt_GetArg    (const ESL_GETOPTS *g, int which);
 
 extern int esl_opt_DisplayHelp(FILE *ofp, ESL_GETOPTS *go, int docgroup, int indent, int textwidth);
 
-#endif /* ESL_GETOPTS_INCLUDED */
+#endif /*eslGETOPTS_INCLUDED*/
+
 /*****************************************************************
- *    This copyrighted source code is freely distributed 
- *    under the terms of the GNU General Public License. See
- *    the files COPYRIGHT and LICENSE for details.
+ * Easel - a library of C functions for biological sequence analysis
+ * Version i1.1.1; July 2014
+ * Copyright (C) 2014 HHMI Janelia Farm Research Campus
+ * Other copyrights also apply. See the COPYRIGHT file for a full list.
+ * 
+ * Easel is distributed under the Janelia Farm Software License, a BSD
+ * license. See the LICENSE file for more details.
+ * 
+ * SVN $Id: esl_getopts.h 697 2011-06-15 20:14:56Z eddys $
+ * SVN $URL: https://svn.janelia.org/eddylab/eddys/easel/branches/infernal/1.1/esl_getopts.h $
  *****************************************************************/
