@@ -1,5 +1,4 @@
-/* esl_gev.c
- * Statistical routines for generalized extreme value (GEV) distributions.
+/* Statistical routines for generalized extreme value (GEV) distributions.
  *
  * GEV distribution 
  *     G(x) = exp{ -[1 + \alpha \lambda(x - \mu)]^{-1/\alpha} }
@@ -19,9 +18,6 @@
  *            
  * Xref: 
  *   STL9/118, 2005/0712-easel-gev-impl. Verified against evd package in R.
- *
- * SRE, Tue Jul 12 09:02:08 2005
- * SVN $Id: esl_gev.c 241 2008-04-01 19:01:52Z eddys $
  */
 #include "esl_config.h"
 
@@ -45,7 +41,6 @@
  ****************************************************************************/ 
 
 /* Function:  esl_gev_pdf()
- * Incept:    SRE, Tue Jul 12 10:53:19 2005 [St. Louis]
  *
  * Purpose:   Calculates the probability density function for the 
  *            generalized extreme value distribution, $P(X=x)$, given
@@ -71,7 +66,6 @@ esl_gev_pdf(double x, double mu, double lambda, double alpha)
 }
 
 /* Function:  esl_gev_logpdf()
- * Incept:    SRE, Tue Jul 12 16:43:13 2005 [St. Louis]
  *
  * Purpose:   Calculates the log probability density function for the
  *            generalized extreme value distribution, $\log P(X=x)$, 
@@ -86,7 +80,7 @@ esl_gev_logpdf(double x, double mu, double lambda, double alpha)
   double lya1;
 
   /* Special case: if alpha is tiny, approx by a Gumbel */
-  if (fabs(y*alpha) < 1e-12) return (log(lambda) -y - exp(-y));
+  if (fabs(y*alpha) < 1e-12) return ((log(lambda) - y) - exp(-y));
 
   /* It's important not to return NaN for this domain error;
    * minimizer relies on being able to compare logL's for any parameter,
@@ -95,12 +89,11 @@ esl_gev_logpdf(double x, double mu, double lambda, double alpha)
   if (ya1 <= 0) return -eslINFINITY;
 
   lya1 = log(ya1);
-  return (log(lambda) - (1.+1./alpha)*lya1 - exp(-lya1/alpha));
+  return ( (log(lambda) - (1.+1./alpha)*lya1) - exp(-lya1/alpha));
 }
 
 
 /* Function:  esl_gev_cdf()
- * Incept:    SRE, Tue Jul 12 16:45:55 2005 [St. Louis]
  *
  * Purpose:   Calculates the cumulative distribution function for the
  *            generalized extreme value distribution, $P(X \leq x)$, 
@@ -128,7 +121,6 @@ esl_gev_cdf(double x, double mu, double lambda, double alpha)
 
 
 /* Function:  esl_gev_logcdf()
- * Incept:    SRE, Tue Jul 12 17:15:49 2005 [St. Louis]
  *
  * Purpose:   Calculates the log of the cumulative distribution function for a
  *            generalized extreme value distribution, $\log P(X \leq x)$, 
@@ -156,7 +148,6 @@ esl_gev_logcdf(double x, double mu, double lambda, double alpha)
 
 
 /* Function:  esl_gev_surv()
- * Incept:    SRE, Wed Jul 13 07:41:12 2005 [St. Louis]
  *
  * Purpose:   Calculates the survivor function, $P(X>x)$ (that is, 1-cdf),
  *            the right tail's probability mass,  given quantile <x> and
@@ -183,7 +174,6 @@ esl_gev_surv(double x, double mu, double lambda, double alpha)
 
 
 /* Function:  esl_gev_logsurv()
- * Incept:    SRE, Wed Jul 13 08:14:48 2005 [St. Louis]
  *
  * Purpose:   Calculates the log survivor function $\log P(X>x)$ for a 
  *            generalized extreme value distribution (that is, 
@@ -221,7 +211,6 @@ esl_gev_logsurv(double x, double mu, double lambda, double alpha)
 }
 
 /* Function:  esl_gev_invcdf()
- * Incept:    SRE, Sun Aug 21 14:14:14 2005 [St. Louis]
  *
  * Purpose:   Calculates the inverse CDF of the GEV: given a probability
  *            <p> ($0 < p < 1$), returns the quantile <x> which would
@@ -245,7 +234,6 @@ esl_gev_invcdf(double p, double mu, double lambda, double alpha)
  *****************************************************************/ 
 
 /* Function:  esl_gev_generic_pdf()
- * Incept:    SRE, Thu Aug 25 08:00:06 2005 [St. Louis]
  *
  * Purpose:   Generic-API version of PDF.
  */
@@ -257,7 +245,6 @@ esl_gev_generic_pdf(double x, void *params)
 }
 
 /* Function:  esl_gev_generic_cdf()
- * Incept:    SRE, Sun Aug 21 14:18:19 2005 [St. Louis]
  *
  * Purpose:   Generic-API version of CDF.
  */
@@ -269,7 +256,6 @@ esl_gev_generic_cdf(double x, void *params)
 }
 
 /* Function:  esl_gev_generic_surv()
- * Incept:    SRE, Thu Aug 25 08:02:17 2005 [St. Louis]
  *
  * Purpose:   Generic-API version of survival function.
  */
@@ -281,7 +267,6 @@ esl_gev_generic_surv(double x, void *params)
 }
 
 /* Function:  esl_gev_generic_invcdf()
- * Incept:    SRE, Sun Aug 21 14:18:41 2005 [St. Louis]
  *
  * Purpose:   Generic-API version of inverse CDF.
  */
@@ -300,14 +285,15 @@ esl_gev_generic_invcdf(double p, void *params)
  ****************************************************************************/ 
 
 /* Function:  esl_gev_Plot()
- * Incept:    SRE, Sun Aug 21 14:20:34 2005 [St. Louis]
  *
  * Purpose:   Plot some GEV function <func> (for instance,
  *            <esl_gev_pdf()>) for parameters <mu> and <lambda>, for
  *            a range of quantiles x from <xmin> to <xmax> in steps of <xstep>;
  *            output to an open stream <fp> in xmgrace XY input format.
  *
- * Returns:   <eslOK>.
+ * Returns:   <eslOK> on success.
+ *
+ * Throws:    <eslEWRITE> on any system write error, such as filled disk.
  */
 int
 esl_gev_Plot(FILE *fp, double mu, double lambda, double alpha,
@@ -316,8 +302,8 @@ esl_gev_Plot(FILE *fp, double mu, double lambda, double alpha,
 {
   double x;
   for (x = xmin; x <= xmax; x += xstep)
-    fprintf(fp, "%f\t%g\n", x, (*func)(x, mu, lambda, alpha));
-  fprintf(fp, "&\n");
+    if (fprintf(fp, "%f\t%g\n", x, (*func)(x, mu, lambda, alpha)) < 0) ESL_EXCEPTION_SYS(eslEWRITE, "gev plot write failed");
+  if (fprintf(fp, "&\n") < 0) ESL_EXCEPTION_SYS(eslEWRITE, "gev plot write failed");
   return eslOK;
 }
 /*-------------------- end plot dumping routines ---------------------------*/
@@ -330,7 +316,6 @@ esl_gev_Plot(FILE *fp, double mu, double lambda, double alpha,
  ****************************************************************************/ 
 #ifdef eslAUGMENT_RANDOM
 /* Function:  esl_gev_Sample()
- * Incept:    SRE, Wed Jul 13 08:30:49 2005 [St. Louis]
  *
  * Purpose:   Sample a GEV-distributed random variate,
  *            by the transformation method.
@@ -546,7 +531,6 @@ fitting_engine(struct gev_data *data,
 
 
 /* Function:  esl_gev_FitComplete()
- * Incept:    SRE, Mon Jul 18 17:36:02 2005 [St. Louis]
  *
  * Purpose:   Given an array of <n> GEV-distributed samples <x[0]..x[n-1>,
  *            return maximum likelihood parameters <ret_mu>, 
@@ -587,7 +571,6 @@ esl_gev_FitComplete(double *x, int n,
 }
 
 /* Function:  esl_gev_FitCensored()
- * Incept:    SRE, Fri Jul 29 09:44:39 2005 [St. Louis]
  *
  * Purpose:   Given a left-censored array of <n> GEV-distributed samples
  *            <x[0]..x[n-1>, the number of censored samples <z>, and
@@ -667,7 +650,7 @@ main(int argc, char **argv)
   double  min       =  9999.;
   double  max       = -9999.;
   double *x         = malloc(sizeof(double) * n);
-  ESL_RANDOMNESS *r = esl_randomness_CreateTimeseeded();;
+  ESL_RANDOMNESS *r = esl_randomness_Create(0);;
 
   for (i = 0; i < n; i++)	/* generate the 10,000 samples */
     { 
@@ -991,7 +974,14 @@ stats_fittest(FILE *fp, int ntrials, int n, double mu, double lambda, double alp
 
 
 /*****************************************************************
- *    This copyrighted source code is freely distributed 
- *    under the terms of the GNU General Public License. See
- *    the files COPYRIGHT and LICENSE for details.
+ * Easel - a library of C functions for biological sequence analysis
+ * Version i1.1.1; July 2014
+ * Copyright (C) 2014 HHMI Janelia Farm Research Campus
+ * Other copyrights also apply. See the COPYRIGHT file for a full list.
+ * 
+ * Easel is distributed under the Janelia Farm Software License, a BSD
+ * license. See the LICENSE file for more details.
+ *
+ * SVN $Id: esl_gev.c 727 2011-10-24 17:17:32Z eddys $
+ * SVN $URL: https://svn.janelia.org/eddylab/eddys/easel/branches/infernal/1.1/esl_gev.c $
  *****************************************************************/
