@@ -2,10 +2,11 @@
  * Generalized extreme value (GEV) distributions.
  * 
  * SRE, Tue Jul 12 09:15:56 2005
- * SVN $Id: esl_gev.h 83 2005-12-13 20:54:07Z eddy $
+ * SVN $Id: esl_gev.h 664 2011-02-27 17:08:36Z eddys $
+ * SVN $URL: https://svn.janelia.org/eddylab/eddys/easel/branches/infernal/1.1/esl_gev.h $
  */
-#ifndef ESL_GEV_INCLUDED
-#define ESL_GEV_INCLUDED
+#ifndef eslGEV_INCLUDED
+#define eslGEV_INCLUDED
 
 #ifdef eslAUGMENT_RANDOM
 #include <esl_random.h>
@@ -43,9 +44,13 @@ extern int esl_gev_FitCensored(double *x, int n, int z, double phi,
 #endif /*eslAUGMENT_MINIMIZER*/
 
 
-#endif /*ESL_GEV_INCLUDED*/
+#endif /*eslGEV_INCLUDED*/
 /*****************************************************************
- *    This copyrighted source code is freely distributed 
- *    under the terms of the GNU General Public License. See
- *    the files COPYRIGHT and LICENSE for details.
+ * Easel - a library of C functions for biological sequence analysis
+ * Version i1.1.1; July 2014
+ * Copyright (C) 2014 HHMI Janelia Farm Research Campus
+ * Other copyrights also apply. See the COPYRIGHT file for a full list.
+ * 
+ * Easel is distributed under the Janelia Farm Software License, a BSD
+ * license. See the LICENSE file for more details.
  *****************************************************************/
