@@ -2,10 +2,11 @@
  * Gumbel (type I extreme value) distributions.
  * 
  * SRE, Mon Jun 27 08:44:41 2005 [St. Louis]
- * SVN $Id: esl_gumbel.h 93 2006-02-26 18:15:26Z eddy $
+ * SVN $Id: esl_gumbel.h 664 2011-02-27 17:08:36Z eddys $
+ * SVN $URL: https://svn.janelia.org/eddylab/eddys/easel/branches/infernal/1.1/esl_gumbel.h $
  */
-#ifndef ESL_GUMBEL_INCLUDED
-#define ESL_GUMBEL_INCLUDED
+#ifndef eslGUMBEL_INCLUDED
+#define eslGUMBEL_INCLUDED
 
 #ifdef eslAUGMENT_RANDOM
 #include <esl_random.h>
@@ -18,6 +19,8 @@ extern double  esl_gumbel_logcdf (double x, double mu, double lambda);
 extern double  esl_gumbel_surv   (double x, double mu, double lambda);
 extern double  esl_gumbel_logsurv(double x, double mu, double lambda);
 extern double  esl_gumbel_invcdf (double p, double mu, double lambda);
+extern double  esl_gumbel_invsurv(double p, double mu, double lambda);
+
 
 extern double  esl_gumbel_generic_pdf   (double x, void *params);
 extern double  esl_gumbel_generic_cdf   (double x, void *params);
@@ -46,11 +49,13 @@ extern int esl_gumbel_FitTruncated(double *x, int n, double phi,
 #endif
 
 
-#endif /*ESL_GUMBEL_INCLUDED*/
-
-
+#endif /*eslGUMBEL_INCLUDED*/
 /*****************************************************************
- *    This copyrighted source code is freely distributed 
- *    under the terms of the GNU General Public License. See
- *    the files COPYRIGHT and LICENSE for details.
+ * Easel - a library of C functions for biological sequence analysis
+ * Version i1.1.1; July 2014
+ * Copyright (C) 2014 HHMI Janelia Farm Research Campus
+ * Other copyrights also apply. See the COPYRIGHT file for a full list.
+ * 
+ * Easel is distributed under the Janelia Farm Software License, a BSD
+ * license. See the LICENSE file for more details.
  *****************************************************************/
