@@ -7,9 +7,6 @@
  *  5. Output and display of binned data.
  *  6. Test driver.
  *  7. Examples.
- *  
- * SRE, Fri Jul  1 13:21:45 2005 [St. Louis]
- * SVN $Id: esl_histogram.c 253 2008-05-06 17:45:23Z nawrockie $
  */
 #include "esl_config.h"
 
@@ -17,6 +14,7 @@
 #include <string.h>
 #include <math.h>
 #include <float.h>
+#include <limits.h>
 
 #include "easel.h"
 #include "esl_stats.h"
@@ -32,7 +30,6 @@ static int esl_histogram_sort(ESL_HISTOGRAM *h);
 
 /* Function:  esl_histogram_Create()
  * Synopsis:  Create a new <ESL_HISTOGRAM>.
- * Incept:    SRE, Fri Jul  1 13:40:26 2005 [St. Louis]
  *
  * Purpose:   Creates and returns a new histogram object, initially
  *            allocated to count scores $>$ <xmin> and $<=$ <xmax> into
@@ -122,7 +119,6 @@ esl_histogram_Create(double xmin, double xmax, double w)
 
 /* Function:  esl_histogram_CreateFull()
  * Synopsis:  A <ESL_HISTOGRAM> to keep all data samples.
- * Incept:    SRE, Tue Jul 26 13:19:27 2005 [St. Louis]
  *
  * Purpose:   Alternative form of <esl_histogram_Create()> that 
  *            creates a more complex histogram that will contain not just the 
@@ -152,7 +148,6 @@ esl_histogram_CreateFull(double xmin, double xmax, double w)
 
 /* Function:  esl_histogram_Destroy()
  * Synopsis:  Frees a <ESL_HISTOGRAM>.
- * Incept:    SRE, Sat Jul  2 19:41:17 2005 [St. Louis]
  *
  * Purpose:   Frees an <ESL_HISTOGRAM> object <h>.
  */
@@ -167,9 +162,48 @@ esl_histogram_Destroy(ESL_HISTOGRAM *h)
   return;
 }
 
+/* Function:  esl_histogram_Score2Bin()
+ * Synopsis:  Given a real-valued <x>; calculate integer bin <b>
+ *
+ * Purpose:   For a real-valued <x>, figure out what bin it would
+ *            go into in the histogram <h>; return this value in
+ *            <*ret_b>.
+ *
+ * Returns:   <eslOK> on success.
+ *
+ * Throws:    <eslERANGE> if bin <b> would exceed the range of
+ *            an integer; for instance, if <x> isn't finite.
+ *
+ * Xref:      J5/122. Replaces earlier macro implementation;
+ *            we needed to range check <x> better.
+ */
+int
+esl_histogram_Score2Bin(ESL_HISTOGRAM *h, double x, int *ret_b)
+{
+  int status;
+
+  if (! isfinite(x)) ESL_XEXCEPTION(eslERANGE, "value added to histogram is not finite");
+
+  x = ceil( ((x - h->bmin) / h->w) - 1.); 
+  
+  /* x is now the bin number as a double, which we will convert to
+   * int. Because x is a double (64-bit), we know all ints are exactly
+   * represented.  Check for under/overflow before conversion.
+   */
+  if (x < (double) INT_MIN || x > (double) INT_MAX) 
+    ESL_XEXCEPTION(eslERANGE, "value %f isn't going to fit in histogram", x);
+
+  *ret_b = (int) x;
+  return eslOK;
+
+ ERROR:
+  *ret_b = 0;
+  return status;
+}
+
+
 /* Function:  esl_histogram_Add()
  * Synopsis:  Add a sample to the histogram.
- * Incept:    SRE, Sat Jul  2 19:41:45 2005 [St. Louis]
  *
  * Purpose:   Adds score <x> to a histogram <h>.
  *           
@@ -180,6 +214,12 @@ esl_histogram_Destroy(ESL_HISTOGRAM *h)
  * Returns:   <eslOK> on success.
  *
  * Throws:    <eslEMEM> on reallocation failure.
+ *
+ *            <eslERANGE> if <x> is beyond the reasonable range for
+ *            the histogram to store -- either because it isn't finite,
+ *            or because the histogram would need to allocate a number
+ *            of bins that exceeds <INT_MAX>.
+ *
  *            Throws <eslEINVAL> for cases where something has been done
  *            to the histogram that requires it to be 'finished', and
  *            adding more data is prohibited; for example, 
@@ -212,7 +252,7 @@ esl_histogram_Add(ESL_HISTOGRAM *h, double x)
 
   /* Which bin will we want to put x into?
    */
-  b = esl_histogram_Score2Bin(h,x);
+  if ((status = esl_histogram_Score2Bin(h,x, &b)) != eslOK) return status;
 
   /* Make sure we have that bin. Realloc as needed.
    * If that reallocation succeeds, we can no longer fail;
@@ -221,6 +261,8 @@ esl_histogram_Add(ESL_HISTOGRAM *h, double x)
   if (b < 0)    /* Reallocate below? */
     {				
       nnew = -b*2;	/* overallocate by 2x */
+      if (nnew > INT_MAX - h->nb)
+	ESL_EXCEPTION(eslERANGE, "value %f requires unreasonable histogram bin number", x);
       ESL_RALLOC(h->obs, tmp, sizeof(uint64_t) * (nnew+ h->nb));
       
       memmove(h->obs+nnew, h->obs, sizeof(uint64_t) * h->nb);
@@ -235,6 +277,8 @@ esl_histogram_Add(ESL_HISTOGRAM *h, double x)
   else if (b >= h->nb)  /* Reallocate above? */
     {
       nnew = (b-h->nb+1) * 2; /* 2x overalloc */
+      if (nnew > INT_MAX - h->nb) 
+	ESL_EXCEPTION(eslERANGE, "value %f requires unreasonable histogram bin number", x);
       ESL_RALLOC(h->obs, tmp, sizeof(uint64_t) * (nnew+ h->nb));
       for (bi = h->nb; bi < h->nb+nnew; bi++) h->obs[bi] = 0;
       if (h->imin == h->nb) { /* boundary condition of no data yet*/
@@ -270,7 +314,6 @@ esl_histogram_Add(ESL_HISTOGRAM *h, double x)
   
 
 /* esl_histogram_sort()
- * Incept:    SRE, Thu Aug 18 10:45:46 2005 [St. Louis]
  *
  * Purpose:   Sort the raw scores in a full histogram, from smallest to
  *            largest. Has no effect on a normal histogram, or on a full
@@ -297,7 +340,6 @@ esl_histogram_sort(ESL_HISTOGRAM *h)
 
 /* Function:  esl_histogram_DeclareCensoring()
  * Synopsis:  Collected data were left-censored.
- * Incept:    SRE, Tue Aug 23 10:00:14 2005 [St. Louis]
  *
  * Purpose:   Declare that the dataset collected in <h> is known to be a
  *            censored distribution, where <z> samples were unobserved because
@@ -333,7 +375,6 @@ esl_histogram_DeclareCensoring(ESL_HISTOGRAM *h, int z, double phi)
 
 /* Function:  esl_histogram_DeclareRounding()
  * Synopsis:  Declare collected data were no more accurate than bins.
- * Incept:    SRE, Tue Jan 31 13:52:10 2006 [St. Louis]
  *
  * Purpose:   Declare that the data sample values in the histogram <h>
  *            are rounded off. Ideally, your bins in <h> should exactly 
@@ -352,7 +393,6 @@ esl_histogram_DeclareRounding(ESL_HISTOGRAM *h)
 
 /* Function:  esl_histogram_SetTail()
  * Synopsis:  Declare only tail $>$ some threshold is considered "observed".
- * Incept:    SRE, Tue Aug 23 09:01:10 2005 [St. Louis]
  *
  * Purpose:   Suggest a threshold <phi> to split a histogram <h>
  *            into "unobserved" data (values $\leq \phi$) and "observed" 
@@ -382,17 +422,21 @@ esl_histogram_DeclareRounding(ESL_HISTOGRAM *h)
  *            tail.
  *            
  * Returns:   <eslOK> on success.
+ *
+ * Throws:    <eslERANGE> if <phi> is an unreasonable value that
+ *            can't be converted to an integer bin value.
  */
 int
 esl_histogram_SetTail(ESL_HISTOGRAM *h, double phi, double *ret_newmass)
 {
   int b;
+  int status;
 
   /* Usually, put true phi at the next bin lower bound, but
    * watch for a special case where phi is already exactly equal to a 
    * bin upper bound.
    */
-  h->cmin = esl_histogram_Score2Bin(h,phi);
+  if ((status = esl_histogram_Score2Bin(h,phi, &(h->cmin))) != eslOK) return status;
   if (phi == esl_histogram_Bin2UBound(h,h->cmin)) h->phi = phi;
   else   h->phi  = esl_histogram_Bin2LBound(h, h->cmin);
 
@@ -409,7 +453,6 @@ esl_histogram_SetTail(ESL_HISTOGRAM *h, double phi, double *ret_newmass)
 
 /* Function:  esl_histogram_SetTailByMass()
  * Synopsis:  Declare only right tail mass is considered "observed".
- * Incept:    SRE, Tue Aug 23 08:10:39 2005 [St. Louis]
  *
  * Purpose:   Given a histogram <h> (with or without raw data samples),
  *            find a cutoff score that at least fraction <pmass> of the samples
@@ -466,7 +509,6 @@ esl_histogram_SetTailByMass(ESL_HISTOGRAM *h, double pmass, double *ret_newmass)
 
 /* Function:  esl_histogram_GetRank()
  * Synopsis:  Retrieve n'th high score.
- * Incept:    SRE, Thu Jul 28 08:39:52 2005 [St. Louis]
  *
  * Purpose:   Retrieve the <rank>'th highest score from a 
  *            full histogram <h>. <rank> is <1..n>, for
@@ -503,7 +545,6 @@ esl_histogram_GetRank(ESL_HISTOGRAM *h, int rank, double *ret_x)
 
 /* Function:  esl_histogram_GetData()
  * Synopsis:  Retrieve vector of all raw scores.
- * Incept:    SRE, Fri Jan 27 07:57:21 2006 [St. Louis]
  *
  * Purpose:   Retrieve the raw data values from the histogram <h>.
  *            Return them in the vector <ret_x>, and the number
@@ -552,7 +593,6 @@ esl_histogram_GetData(ESL_HISTOGRAM *h, double **ret_x, int *ret_n)
 
 /* Function:  esl_histogram_GetTail()
  * Synopsis:  Retrieve all raw scores above some threshold.
- * Incept:    SRE, Fri Jan 27 07:56:38 2006 [St. Louis]
  *
  * Purpose:   Given a full histogram <h>, retrieve all data values 
  *            above the threshold <phi> in the right (high scoring) 
@@ -619,7 +659,6 @@ esl_histogram_GetTail(ESL_HISTOGRAM *h, double phi,
 
 /* Function:  esl_histogram_GetTailByMass()
  * Synopsis:  Retrieve all raw scores in right tail mass.
- * Incept:    SRE, Sun Jan 29 17:56:37 2006 [St. Louis]
  *
  * Purpose:   Given a full histogram <h>, retrieve the data values in
  *            the right (high scoring) tail, as a pointer <ret_x>
@@ -685,7 +724,6 @@ esl_histogram_GetTailByMass(ESL_HISTOGRAM *h, double pmass,
 
 /* Function:  esl_histogram_SetExpect()
  * Synopsis:  Set expected counts for complete distribution.
- * Incept:    SRE, Wed Aug 17 17:36:58 2005 [St. Louis]
  *
  * Purpose:   Given a histogram <h> containing some number of empirically
  *            observed binned counts, and a pointer to a function <(*cdf)()>
@@ -737,7 +775,6 @@ esl_histogram_SetExpect(ESL_HISTOGRAM *h,
 
 /* Function:  esl_histogram_SetExpectedTail()
  * Synopsis:  Set expected counts for right tail.
- * Incept:    SRE, Mon Jan 30 08:57:57 2006 [St. Louis]
  *
  * Purpose:   Given a histogram <h>, and a pointer to a generic function
  *            <(*cdf)()> that describes the expected cumulative
@@ -761,6 +798,9 @@ esl_histogram_SetExpect(ESL_HISTOGRAM *h,
  * Returns:   <eslOK> on success.
  *
  * Throws:    <eslEMEM> on memory allocation failure.
+ *            <eslERANGE> if <base_val> isn't a reasonable value within
+ *              the histogram (it converts to a bin value outside
+ *              integer range).
  */
 int
 esl_histogram_SetExpectedTail(ESL_HISTOGRAM *h, double base_val, double pmass,
@@ -773,7 +813,8 @@ esl_histogram_SetExpectedTail(ESL_HISTOGRAM *h, double base_val, double pmass,
 
   if (h->expect == NULL)  ESL_ALLOC(h->expect, sizeof(double) * h->nb);
 
-  h->emin = 1 + esl_histogram_Score2Bin(h, base_val);
+  if ((status = esl_histogram_Score2Bin(h, base_val, &(h->emin))) != eslOK) return status;
+  h->emin += 1;
   esl_vec_DSet(h->expect, h->emin, 0.);
 
   for (b = h->emin; b < h->nb; b++)
@@ -801,28 +842,30 @@ esl_histogram_SetExpectedTail(ESL_HISTOGRAM *h, double base_val, double pmass,
  * 5. Output and display of binned data.
  *****************************************************************/ 
 
-/* Function:  esl_histogram_Print() 
- * Synopsis:  Print a "pretty" ASCII histogram.
- * Incept:    SRE, Sat Jul  2 16:03:37 2005 [St. Louis]
+/* Function:  esl_histogram_Write() 
+ * Synopsis:  Write a "pretty" ASCII histogram to a stream.
  *
- * Purpose:   Print a "prettified" display histogram <h> to a file 
- *            pointer <fp>.
- *            Deliberately a look-and-feel clone of Bill Pearson's 
- *            excellent FASTA output.
+ * Purpose:   Print a "prettified" display histogram <h> to a file
+ *            pointer <fp>.  Deliberately a look-and-feel clone of
+ *            Bill Pearson's excellent FASTA output.
  *            
  *            Also displays expected binned counts, if they've been
  *            set.
  *            
- *            Display will only work well if the bin width (w) is 0.1 or more,
- *            because the score labels are only shown to one decimal point.
+ *            Display will only work well if the bin width (w) is 0.1
+ *            or more, because the score labels are only shown to one
+ *            decimal point.
  * 
  * Args:      fp     - open file to print to (stdout works)
  *            h      - histogram to print
  *
  * Returns:   <eslOK> on success.
+ * 
+ * Throws:    <eslEWRITE> on any system write error, such as a
+ *                        filled disk.
  */
 int
-esl_histogram_Print(FILE *fp, ESL_HISTOGRAM *h)
+esl_histogram_Write(FILE *fp, ESL_HISTOGRAM *h)
 {
   int      i;
   double   x;
@@ -878,9 +921,10 @@ esl_histogram_Print(FILE *fp, ESL_HISTOGRAM *h)
 
   /* Print the histogram
    */
-  fprintf(fp, "%6s %6s %6s  (one = represents %llu sequences)\n", 
-	  "score", "obs", "exp", (unsigned long long) units);
-  fprintf(fp, "%6s %6s %6s\n", "-----", "---", "---");
+  if (fprintf(fp, "%6s %6s %6s  (one = represents %llu sequences)\n", 
+	      "score", "obs", "exp", (unsigned long long) units) < 0) 
+    ESL_EXCEPTION_SYS(eslEWRITE, "histogram write failed");
+  if (fprintf(fp, "%6s %6s %6s\n", "-----", "---", "---") < 0) ESL_EXCEPTION_SYS(eslEWRITE, "histogram write failed");
   buffer[80] = '\0';
   buffer[79] = '\n';
   for (i = h->imin; i <= h->imax; i++)
@@ -899,7 +943,7 @@ esl_histogram_Print(FILE *fp, ESL_HISTOGRAM *h)
 	    num = 1+(lowcount-1) / units;
 	    for (pos = 21; num > 0; num--)  buffer[pos++] = '=';
 	  }
-	  fputs(buffer, fp);
+	  if (fputs(buffer, fp) < 0) ESL_EXCEPTION_SYS(eslEWRITE, "histogram write failed");
 	  continue;
 	}
       else if (i == ihighbound && i != h->imax)
@@ -909,7 +953,7 @@ esl_histogram_Print(FILE *fp, ESL_HISTOGRAM *h)
 	    num = 1+(highcount-1) / units;
 	    for (pos = 21; num > 0; num--)  buffer[pos++] = '=';
 	  }
-	  fputs(buffer, fp);
+	  if (fputs(buffer, fp) < 0) ESL_EXCEPTION_SYS(eslEWRITE, "histogram write failed");
 	  continue;
 	}
 
@@ -952,7 +996,7 @@ esl_histogram_Print(FILE *fp, ESL_HISTOGRAM *h)
 
       /* Print the line
        */
-      fputs(buffer, fp);
+      if (fputs(buffer, fp) < 0) ESL_EXCEPTION_SYS(eslEWRITE, "histogram write failed");
     }
 
   return eslOK;
@@ -960,13 +1004,14 @@ esl_histogram_Print(FILE *fp, ESL_HISTOGRAM *h)
   
 /* Function:  esl_histogram_Plot()
  * Synopsis:  Output a histogram in xmgrace XY format.
- * Incept:    SRE, Mon Jan 30 11:09:01 2006 [St. Louis]
  *
  * Purpose:   Print observed (and expected, if set) binned counts
  *            in a histogram <h> to open file pointer <fp>
  *            in xmgrace XY input file format.
  *
  * Returns:   <eslOK> on success.
+ *
+ * Throws:    <eslEWRITE> on any system write error.
  */
 int
 esl_histogram_Plot(FILE *fp, ESL_HISTOGRAM *h)
@@ -980,9 +1025,9 @@ esl_histogram_Plot(FILE *fp, ESL_HISTOGRAM *h)
     if (h->obs[i] > 0)
       {
 	x = esl_histogram_Bin2LBound(h,i);
-	fprintf(fp, "%f %llu\n", x, (unsigned long long) h->obs[i]);
+	if (fprintf(fp, "%f %llu\n", x, (unsigned long long) h->obs[i]) < 0) ESL_EXCEPTION_SYS(eslEWRITE, "histogram plot write failed");
       }
-  fprintf(fp, "&\n");
+  if (fprintf(fp, "&\n") < 0) ESL_EXCEPTION_SYS(eslEWRITE, "histogram plot write failed");
 
   /* Second data set is the theoretical (expected) histogram
    */
@@ -992,16 +1037,15 @@ esl_histogram_Plot(FILE *fp, ESL_HISTOGRAM *h)
 	if (h->expect[i] > 0.)	/* >0 suffices to remove censored region */
 	  {
 	    x = esl_histogram_Bin2LBound(h,i);
-	    fprintf(fp, "%.2f %g\n", x, h->expect[i]);
+	    if (fprintf(fp, "%.2f %g\n", x, h->expect[i]) < 0) ESL_EXCEPTION_SYS(eslEWRITE, "histogram plot write failed");
 	  }
-      fprintf(fp, "&\n");
+      if (fprintf(fp, "&\n") < 0) ESL_EXCEPTION_SYS(eslEWRITE, "histogram plot write failed");
     }
   return eslOK;
 }
 
 /* Function:  esl_histogram_PlotSurvival()
  * Synopsis:  Output $P(X>x)$ in xmgrace XY format.
- * Incept:    SRE, Mon Jan 30 11:11:05 2006 [St. Louis]
  *
  * Purpose:   Given a histogram <h>, output the observed (and
  *            expected, if available) survival function $P(X>x)$
@@ -1017,6 +1061,8 @@ esl_histogram_Plot(FILE *fp, ESL_HISTOGRAM *h)
  *            looking like it was artificially truncated.
  *
  * Returns:   <eslOK> on success.
+ *
+ * Throws:    <eslEWRITE> on any system write error.
  */
 int
 esl_histogram_PlotSurvival(FILE *fp, ESL_HISTOGRAM *h)
@@ -1029,16 +1075,16 @@ esl_histogram_PlotSurvival(FILE *fp, ESL_HISTOGRAM *h)
   /* The observed binned counts:
    */
   if (h->obs[h->imax] > 1) 
-    fprintf(fp, "%f\t%g\n", h->xmax, 1.0 / (double) h->Nc);
+    if (fprintf(fp, "%f\t%g\n", h->xmax, 1.0 / (double) h->Nc) < 0) ESL_EXCEPTION_SYS(eslEWRITE, "histogram survival plot write failed");
   for (i = h->imax; i >= h->imin; i--)
     {
       if (h->obs[i] > 0) {
 	c   += h->obs[i];
 	ai = esl_histogram_Bin2LBound(h, i);
-	fprintf(fp, "%f\t%g\n", ai, (double) c / (double) h->Nc);
+	if (fprintf(fp, "%f\t%g\n", ai, (double) c / (double) h->Nc) < 0) ESL_EXCEPTION_SYS(eslEWRITE, "histogram survival plot write failed");
       }
     }
-  fprintf(fp, "&\n");
+  if (fprintf(fp, "&\n") < 0) ESL_EXCEPTION_SYS(eslEWRITE, "histogram survival plot write failed");
 
   /* The expected binned counts:
    */
@@ -1050,17 +1096,16 @@ esl_histogram_PlotSurvival(FILE *fp, ESL_HISTOGRAM *h)
 	  if (h->expect[i] > 0.) { 
 	    esum += h->expect[i];        /* some worry about 1+eps=1 problem here */
 	    ai = esl_histogram_Bin2LBound(h, i);
-	    fprintf(fp, "%f\t%g\n", ai, esum / (double) h->Nc);
+	    if (fprintf(fp, "%f\t%g\n", ai, esum / (double) h->Nc) < 0) ESL_EXCEPTION_SYS(eslEWRITE, "histogram survival plot write failed");
 	  }
 	}
-      fprintf(fp, "&\n");
+      if (fprintf(fp, "&\n") < 0) ESL_EXCEPTION_SYS(eslEWRITE, "histogram survival plot write failed");
     }
   return eslOK;
 }
 
 /* Function:  esl_histogram_PlotQQ()
  * Synopsis:  Output a Q-Q plot in xmgrace XY format.
- * Incept:    SRE, Sat Aug 20 14:15:01 2005 [St. Louis]
  *
  * Purpose:   Given a histogram <h> containing an empirically observed
  *            distribution, and a pointer to a function <(*invcdf)()>
@@ -1074,6 +1119,8 @@ esl_histogram_PlotSurvival(FILE *fp, ESL_HISTOGRAM *h)
  *            if only a tail was fit).
  *
  * Returns:   <eslOK> on success.
+ *
+ * Throws:    <eslEWRITE> on any system write error.
  */
 int
 esl_histogram_PlotQQ(FILE *fp, ESL_HISTOGRAM *h, 
@@ -1108,16 +1155,16 @@ esl_histogram_PlotQQ(FILE *fp, ESL_HISTOGRAM *h,
       if (h->is_tailfit) cdf = (cdf + h->tailmass - 1.) / (h->tailmass);
 
       bi = esl_histogram_Bin2UBound(h, i);
-      fprintf(fp, "%f\t%f\n", bi, (*invcdf)(cdf, params));
+      if (fprintf(fp, "%f\t%f\n", bi, (*invcdf)(cdf, params)) < 0) ESL_EXCEPTION_SYS(eslEWRITE, "histogram QQ plot write failed");
     }
-  fprintf(fp, "&\n");
+  if (fprintf(fp, "&\n") < 0) ESL_EXCEPTION_SYS(eslEWRITE, "histogram QQ plot write failed");
 
   /* Plot a 45-degree expected QQ line:
    */
   bi = esl_histogram_Bin2LBound(h, bbase);
-  fprintf(fp, "%f\t%f\n", bi,  bi);
-  fprintf(fp, "%f\t%f\n", h->xmax, h->xmax);
-  fprintf(fp, "&\n");
+  if (fprintf(fp, "%f\t%f\n", bi,  bi)          < 0) ESL_EXCEPTION_SYS(eslEWRITE, "histogram QQ plot write failed");
+  if (fprintf(fp, "%f\t%f\n", h->xmax, h->xmax) < 0) ESL_EXCEPTION_SYS(eslEWRITE, "histogram QQ plot write failed");
+  if (fprintf(fp, "&\n")                        < 0) ESL_EXCEPTION_SYS(eslEWRITE, "histogram QQ plot write failed");
 
   return eslOK;
 }
@@ -1126,7 +1173,6 @@ esl_histogram_PlotQQ(FILE *fp, ESL_HISTOGRAM *h,
 
 /* Function:  esl_histogram_Goodness()
  * Synopsis:  Evaluate fit between observed, expected. 
- * Incept:    SRE, Wed Aug 17 12:46:05 2005 [St. Louis]
  *
  * Purpose:   Given a histogram <h> with observed and expected counts,
  *            where, for the expected counts, <nfitted> ($\geq 0$)
@@ -1374,9 +1420,10 @@ static ESL_OPTIONS options[] = {
   { 0,0,0,0,0,0,0,0,0,0 },
 };
 
-static int
-binmacro_test(void)
+static void
+binmacro_utest(void)
 {
+  char          *msg = "esl_histogram: binmacro unit test failure";
   ESL_HISTOGRAM *h = esl_histogram_Create(-100, 100, 1.0);
   double trialx[3]  = { -42.42, 0, 42.42 };
   double x, ai, bi;  
@@ -1387,20 +1434,35 @@ binmacro_test(void)
   for (i = 0; i < 3; i++)
     {
       x  = trialx[i];
-      b  = esl_histogram_Score2Bin(h, x);
+      esl_histogram_Score2Bin(h, x, &b);
       ai = esl_histogram_Bin2LBound(h, b);
       bi = esl_histogram_Bin2UBound(h, b);
-      if (x <= ai || x > bi) {
-	fprintf(stderr,
-		"failed: (ai=%.1f) <= (x=%.2f) < (bi=%.1f) in bin %d, bin macro test\n",
-		ai, x, bi, b);
-	esl_histogram_Destroy(h);
-	return 0;
-      }
+      if (x <= ai || x > bi) esl_fatal(msg);
     }
   esl_histogram_Destroy(h);
-  return 1;
+  return;
 }
+
+static void
+valuerange_utest(void)
+{
+  char          *msg = "esl_histogram: value range unit test failure";
+  ESL_HISTOGRAM *h   = esl_histogram_Create(-100, 100, 1.0);
+  int            b;
+
+  esl_exception_SetHandler(&esl_nonfatal_handler);
+
+  if (esl_histogram_Score2Bin(h,  eslINFINITY, &b) != eslERANGE) esl_fatal(msg);
+  if (esl_histogram_Score2Bin(h, -eslINFINITY, &b) != eslERANGE) esl_fatal(msg);
+  if (esl_histogram_Score2Bin(h,  eslNaN,      &b) != eslERANGE) esl_fatal(msg);
+  if (esl_histogram_Score2Bin(h,  1e20,        &b) != eslERANGE) esl_fatal(msg);
+  if (esl_histogram_Score2Bin(h, -1e20,        &b) != eslERANGE) esl_fatal(msg);
+
+  esl_exception_ResetDefaultHandler();
+  esl_histogram_Destroy(h);
+  return;
+}
+
 int
 main(int argc, char **argv)
 {
@@ -1610,7 +1672,7 @@ main(int argc, char **argv)
       if (ascfile != NULL)
 	{
 	  outfp = fopen(ascfile, "w");
-	  esl_histogram_Print(outfp, h1);
+	  esl_histogram_Write(outfp, h1);
 	  fclose(outfp);
 	}
       if (plotfile != NULL)
@@ -1652,7 +1714,8 @@ main(int argc, char **argv)
 
   /* Smaller final tests
    */
-  if (! binmacro_test()) exit(1);
+  binmacro_utest();
+  valuerange_utest();
   
   esl_randomness_Destroy(r);
   return 0;
@@ -1676,11 +1739,11 @@ main(int argc, char **argv)
  * (These same five cases are tested by ./test -t1 through ./test -t5.)
  *****************************************************************/
 /* Case 1. Complete data fit to complete Gumbel.
- * compile: gcc -I. -L. -o example -DeslHISTOGRAM_EXAMPLE1 esl_histogram.c -leasel -lm
+ * compile: gcc -I. -L. -o example -DeslHISTOGRAM_EXAMPLE esl_histogram.c -leasel -lm
  * run:     ./example 
  */
-#ifdef eslHISTOGRAM_EXAMPLE1
-/*::cexcerpt::histogram_example1::begin::*/
+#ifdef eslHISTOGRAM_EXAMPLE
+/*::cexcerpt::histogram_example::begin::*/
 #include "easel.h"
 #include "esl_random.h"
 #include "esl_histogram.h"
@@ -1689,7 +1752,7 @@ main(int argc, char **argv)
 int
 main(int argc, char **argv)
 {
-  ESL_RANDOMNESS *r  = esl_randomness_CreateTimeseeded();
+  ESL_RANDOMNESS *r  = esl_randomness_Create(0);
   ESL_HISTOGRAM  *h  = esl_histogram_CreateFull(-100, 100, 0.2);
   int     nsamples    = 10000;
   double  mu          = 10.0;
@@ -1713,7 +1776,7 @@ main(int argc, char **argv)
   params[1] = lambda;
   esl_histogram_SetExpect(h, &esl_gumbel_generic_cdf, &params);
 
-  esl_histogram_Print(stdout, h);
+  esl_histogram_Write(stdout, h);
   esl_histogram_Goodness(h, 0, NULL, &G, &Gp, &X2, &X2p);
   printf("G   = %f  p = %f\n", G, Gp);
   printf("X^2 = %f  p = %f\n", X2, X2p);
@@ -1722,8 +1785,8 @@ main(int argc, char **argv)
   esl_randomness_Destroy(r);
   return 0;
 }
-/*::cexcerpt::histogram_example1::end::*/
-#endif /*eslHISTOGRAM_EXAMPLE1*/
+/*::cexcerpt::histogram_example::end::*/
+#endif /*eslHISTOGRAM_EXAMPLE*/
 
 
 
@@ -1741,7 +1804,7 @@ main(int argc, char **argv)
 int
 main(int argc, char **argv)
 {
-  ESL_RANDOMNESS *r  = esl_randomness_CreateTimeseeded();
+  ESL_RANDOMNESS *r  = esl_randomness_Create(0);
   ESL_HISTOGRAM  *h  = esl_histogram_CreateFull(-100, 100, 0.2);
   int     nsamples    = 10000;
   double  mu          = 10.0;
@@ -1765,7 +1828,7 @@ main(int argc, char **argv)
   params[1] = lambda;
   esl_histogram_SetExpect(h, &esl_gumbel_generic_cdf, &params);
 
-  esl_histogram_Print(stdout, h);
+  esl_histogram_Write(stdout, h);
   esl_histogram_Goodness(h, 0, NULL, &G, &Gp, &X2, &X2p);
   printf("G   = %f  p = %f\n", G, Gp);
   printf("X^2 = %f  p = %f\n", X2, X2p);
@@ -1793,7 +1856,7 @@ main(int argc, char **argv)
 int
 main(int argc, char **argv)
 {
-  ESL_RANDOMNESS *r  = esl_randomness_CreateTimeseeded();
+  ESL_RANDOMNESS *r  = esl_randomness_Create(0);
   ESL_HISTOGRAM  *h  = esl_histogram_CreateFull(-100, 100, 0.2);
   int     nsamples    = 10000;
   double  mu          = 10.0;
@@ -1817,7 +1880,7 @@ main(int argc, char **argv)
   params[1] = lambda;
   esl_histogram_SetExpectedTail(h, mu, 0.1, &esl_exp_generic_cdf, &params);
 
-  esl_histogram_Print(stdout, h);
+  esl_histogram_Write(stdout, h);
   esl_histogram_Goodness(h, 0, NULL, &G, &Gp, &X2, &X2p);
   printf("G   = %f  p = %f\n", G, Gp);
   printf("X^2 = %f  p = %f\n", X2, X2p);
@@ -1844,7 +1907,7 @@ main(int argc, char **argv)
 int
 main(int argc, char **argv)
 {
-  ESL_RANDOMNESS *r  = esl_randomness_CreateTimeseeded();
+  ESL_RANDOMNESS *r  = esl_randomness_Create(0);
   ESL_HISTOGRAM  *h  = esl_histogram_CreateFull(-100, 100, 0.2);
   int     nsamples    = 10000;
   double  mu          = 10.0;
@@ -1871,7 +1934,7 @@ main(int argc, char **argv)
   params[1] = lambda;
   esl_histogram_SetExpect(h, &esl_gumbel_generic_cdf, &params);
 
-  esl_histogram_Print(stdout, h);
+  esl_histogram_Write(stdout, h);
   esl_histogram_Goodness(h, 0, NULL, &G, &Gp, &X2, &X2p);
   printf("G   = %f  p = %f\n", G, Gp);
   printf("X^2 = %f  p = %f\n", X2, X2p);
@@ -1899,7 +1962,7 @@ main(int argc, char **argv)
 int
 main(int argc, char **argv)
 {
-  ESL_RANDOMNESS *r  = esl_randomness_CreateTimeseeded();
+  ESL_RANDOMNESS *r  = esl_randomness_Create(0);
   ESL_HISTOGRAM  *h  = esl_histogram_Create(-100, 100, 1.0);
   int     nsamples    = 10000;
   double  mu          = 10.0;
@@ -1924,7 +1987,7 @@ main(int argc, char **argv)
   params[1] = lambda;
   esl_histogram_SetExpectedTail(h, mu, actual_mass, &esl_exp_generic_cdf, &params);
 
-  esl_histogram_Print(stdout, h);
+  esl_histogram_Write(stdout, h);
   esl_histogram_Goodness(h, 0, NULL, &G, &Gp, &X2, &X2p);
   printf("G   = %f  p = %f\n", G, Gp);
   printf("X^2 = %f  p = %f\n", X2, X2p);
@@ -1936,5 +1999,17 @@ main(int argc, char **argv)
 /*::cexcerpt::histogram_example5::end::*/
 #endif /*eslHISTOGRAM_EXAMPLE5*/
 
+/*****************************************************************
+ * Easel - a library of C functions for biological sequence analysis
+ * Version i1.1.1; July 2014
+ * Copyright (C) 2014 HHMI Janelia Farm Research Campus
+ * Other copyrights also apply. See the COPYRIGHT file for a full list.
+ * 
+ * Easel is distributed under the Janelia Farm Software License, a BSD
+ * license. See the LICENSE file for more details.
+ *
+ * SVN $Id: esl_histogram.c 727 2011-10-24 17:17:32Z eddys $
+ * SVN $URL: https://svn.janelia.org/eddylab/eddys/easel/branches/infernal/1.1/esl_histogram.c $
+ *****************************************************************/
 
 
