@@ -1,10 +1,11 @@
 /* Collection and display of score histograms.
  * 
  * SRE, Fri Jul  1 13:22:45 2005 [St. Louis]
- * SVN $Id: esl_histogram.h 181 2007-05-04 11:00:44Z eddys $
+ * SVN $Id: esl_histogram.h 727 2011-10-24 17:17:32Z eddys $
+ * SVN $URL: https://svn.janelia.org/eddylab/eddys/easel/branches/infernal/1.1/esl_histogram.h $
  */
-#ifndef ESL_HISTOGRAM_INCLUDED
-#define ESL_HISTOGRAM_INCLUDED
+#ifndef eslHISTOGRAM_INCLUDED
+#define eslHISTOGRAM_INCLUDED
 
 #include <math.h>   /* floor() is in one of the macros */
 
@@ -68,14 +69,14 @@ typedef struct {
 
 #define esl_histogram_Bin2LBound(h,b)  ((h)->w*(b) + (h)->bmin)
 #define esl_histogram_Bin2UBound(h,b)  ((h)->w*((b)+1) + (h)->bmin)
-#define esl_histogram_Score2Bin(h,x)   ((int) ceil( ((x - (h)->bmin) / h->w) - 1.))
 
 /* Creating/destroying histograms and collecting data:
  */
 extern ESL_HISTOGRAM *esl_histogram_Create    (double bmin, double bmax, double w);
 extern ESL_HISTOGRAM *esl_histogram_CreateFull(double bmin, double bmax, double w);
-extern void           esl_histogram_Destroy(ESL_HISTOGRAM *h);
-extern int            esl_histogram_Add(ESL_HISTOGRAM *h, double x);
+extern void           esl_histogram_Destroy  (ESL_HISTOGRAM *h);
+extern int            esl_histogram_Score2Bin(ESL_HISTOGRAM *h, double x, int *ret_b);
+extern int            esl_histogram_Add      (ESL_HISTOGRAM *h, double x);
 
 /* Declarations about the binned data before parameter fitting:
  */
@@ -108,7 +109,7 @@ extern int esl_histogram_SetExpectedTail(ESL_HISTOGRAM *h, double base_val,
 
 /* Output/display of binned data:
  */
-extern int esl_histogram_Print       (FILE *fp, ESL_HISTOGRAM *h);
+extern int esl_histogram_Write       (FILE *fp, ESL_HISTOGRAM *h);
 extern int esl_histogram_Plot        (FILE *fp, ESL_HISTOGRAM *h);
 extern int esl_histogram_PlotSurvival(FILE *fp, ESL_HISTOGRAM *h);
 extern int esl_histogram_PlotQQ      (FILE *fp, ESL_HISTOGRAM *h, 
@@ -123,9 +124,13 @@ extern int esl_histogram_Goodness(ESL_HISTOGRAM *h, int nfitted,
 
 
 
-#endif /*!ESL_HISTOGRAM_INCLUDED*/
+#endif /*eslHISTOGRAM_INCLUDED*/
 /*****************************************************************
- *    This copyrighted source code is freely distributed 
- *    under the terms of the GNU General Public License. See
- *    the files COPYRIGHT and LICENSE for details.
+ * Easel - a library of C functions for biological sequence analysis
+ * Version i1.1.1; July 2014
+ * Copyright (C) 2014 HHMI Janelia Farm Research Campus
+ * Other copyrights also apply. See the COPYRIGHT file for a full list.
+ * 
+ * Easel is distributed under the Janelia Farm Software License, a BSD
+ * license. See the LICENSE file for more details.
  *****************************************************************/
