@@ -1,11 +1,8 @@
-/* esl_hyperexp.h
- * Hyperexponential (mixture exponential) distributions.
+/* Hyperexponential (mixture exponential) distributions.
  * 
- * SRE, Mon Aug 15 08:32:33 2005 [St. Louis]
- * SVN $Id: esl_hyperexp.h 83 2005-12-13 20:54:07Z eddy $
  */
-#ifndef ESL_HYPEREXP_INCLUDED
-#define ESL_HYPEREXP_INCLUDED
+#ifndef eslHYPEREXP_INCLUDED
+#define eslHYPEREXP_INCLUDED
 
 #ifdef eslAUGMENT_RANDOM
 #include <esl_random.h>
@@ -34,7 +31,7 @@ extern int           esl_hyperexp_Copy(ESL_HYPEREXP *src, ESL_HYPEREXP *dest);
 extern int           esl_hyperexp_FixedUniformMixture(ESL_HYPEREXP *h);
 extern int           esl_hyperexp_SortComponents(ESL_HYPEREXP *h);
 extern int           esl_hyperexp_Write(FILE *fp, ESL_HYPEREXP *hxp);
-extern int           esl_hyperexp_WriteOneLine(FILE *fp, ESL_HYPEREXP *hxp);
+extern int           esl_hyperexp_Dump(FILE *fp, ESL_HYPEREXP *hxp);
 #ifdef eslAUGMENT_FILEPARSER
 extern int           esl_hyperexp_Read(ESL_FILEPARSER *ef, ESL_HYPEREXP **ret_hxp);
 extern int           esl_hyperexp_ReadFile(char *filename, ESL_HYPEREXP **ret_hxp);
@@ -69,10 +66,16 @@ extern int esl_hxp_FitCompleteBinned(ESL_HISTOGRAM *g, ESL_HYPEREXP *h);
 #endif
 #endif
 
-#endif /*ESL_HYPEREXP_INCLUDED*/
-
+#endif /*eslHYPEREXP_INCLUDED*/
 /*****************************************************************
- *    This copyrighted source code is freely distributed 
- *    under the terms of the GNU General Public License. See
- *    the files COPYRIGHT and LICENSE for details.
+ * Easel - a library of C functions for biological sequence analysis
+ * Version i1.1.1; July 2014
+ * Copyright (C) 2014 HHMI Janelia Farm Research Campus
+ * Other copyrights also apply. See the COPYRIGHT file for a full list.
+ * 
+ * Easel is distributed under the Janelia Farm Software License, a BSD
+ * license. See the LICENSE file for more details.
+ *
+ * SVN $Id: esl_hyperexp.h 727 2011-10-24 17:17:32Z eddys $
+ * SVN $URL: https://svn.janelia.org/eddylab/eddys/easel/branches/infernal/1.1/esl_hyperexp.h $
  *****************************************************************/
