@@ -2,9 +2,6 @@
  * 
  * Can be used even without derivative information; falls back to
  * a numeric gradient if analytic gradient is unavailable.
- * 
- * SRE, Wed Jun 22 11:37:29 2005
- * SVN $Id: esl_minimizer.c 241 2008-04-01 19:01:52Z eddys $
  */
 #include "esl_config.h"
 
@@ -185,8 +182,8 @@ bracket(double *ori, double *d, int n, double firststep,
       if (ax != bx && bx != cx && fa == fb && fb == fc) break;
 
       niter++;
-      if (niter > 100) 
-	ESL_EXCEPTION(eslENOHALT, "Failed to bracket a minimum.");
+      if (niter > 100)
+    	  ESL_EXCEPTION(eslENORESULT, "Failed to bracket a minimum.");
     }
 
   /* We're about to return. Assure the caller that the points
@@ -441,7 +438,8 @@ esl_min_ConjugateGradientDescent(double *x, double *u, int n,
    * has screwed something up, or has chosen a bad start point.
    */
   if (oldfx == eslINFINITY || oldfx == -eslINFINITY)
-    ESL_EXCEPTION(eslERANGE, "minimum not finite");
+	  ESL_EXCEPTION(eslERANGE, "minimum not finite");
+
 
   if (dfunc != NULL) 
     {
@@ -452,6 +450,8 @@ esl_min_ConjugateGradientDescent(double *x, double *u, int n,
 
   esl_vec_DCopy(dx, n, cg);	/* and make that the first conjugate direction, cg  */
 
+
+
   /* (failsafe) convergence test: a zero direction can happen, 
    * and it either means we're stuck or we're finished (most likely stuck)
    */
@@ -463,7 +463,8 @@ esl_min_ConjugateGradientDescent(double *x, double *u, int n,
   }
   
   for (i = 0; i < MAXITERATIONS; i++)
-    {
+  {
+
       /* Figure out the initial step size.
        */
        bx = fabs(u[0] / cg[0]);
@@ -487,19 +488,20 @@ esl_min_ConjugateGradientDescent(double *x, double *u, int n,
        * has screwed something up.
        */
       if (fx == eslINFINITY || fx == -eslINFINITY)
-	ESL_EXCEPTION(eslERANGE, "minimum not finite");
+    	  ESL_EXCEPTION(eslERANGE, "minimum not finite");
+
 
       /* Find the negative gradient at that point (temporarily in w1) */
       if (dfunc != NULL) 
-	{
-	  (*dfunc)(x, n, prm, w1);
-	  esl_vec_DScale(w1, n, -1.0);
-	}
+	  {
+	    (*dfunc)(x, n, prm, w1);
+	    esl_vec_DScale(w1, n, -1.0);
+	  }
       else numeric_derivative(x, u, n, func, prm, 1e-4, w1); /* resort to brute force */
 
       /* Calculate the Polak-Ribiere coefficient */
       for (coeff = 0., i1 = 0; i1 < n; i1++)
-	coeff += (w1[i1] - dx[i1]) * w1[i1];
+	      coeff += (w1[i1] - dx[i1]) * w1[i1];
       coeff /= esl_vec_DDot(dx, dx, n);
       
       /* Calculate the next conjugate gradient direction in w2 */
@@ -519,26 +521,29 @@ esl_min_ConjugateGradientDescent(double *x, double *u, int n,
       /* Main convergence test. 1e-9 factor is fudging the case where our
        * minimum is at exactly f()=0.
        */
-      cvg = 2.0 * fabs((oldfx-fx)) / (1e-9 + fabs(oldfx) + fabs(fx));
+      cvg = 2.0 * fabs((oldfx-fx)) / (1e-10 + fabs(oldfx) + fabs(fx));
+
+//      fprintf(stderr, "(%d): Old f() = %.9f    New f() = %.9f    Convergence = %.9f\n", i, oldfx, fx, cvg);
+//      fprintf(stdout, "(%d): Old f() = %.9f    New f() = %.9f    Convergence = %.9f\n", i, oldfx, fx, cvg);
 
 #if eslDEBUGLEVEL >= 2
       printf("\nesl_min_ConjugateGradientDescent():\n");
       printf("new point:     ");
       for (i1 = 0; i1 < n; i1++)
-	printf("%g ", x[i1]);
+	    printf("%g ", x[i1]);
 
       printf("\nnew gradient:    ");
       for (i1 = 0; i1 < n; i1++)
-	printf("%g ", dx[i1]);
+	    printf("%g ", dx[i1]);
 
       numeric_derivative(x, u, n, func, prm, 1e-4, w1);
       printf("\n(numeric grad):  ");
       for (i1 = 0; i1 < n; i1++)
-	printf("%g ", w1[i1]);
+	    printf("%g ", w1[i1]);
 
       printf("\nnew direction: ");
       for (i1 = 0; i1 < n; i1++)
-	printf("%g ", cg[i1]);
+	    printf("%g ", cg[i1]);
 
       printf("\nOld f() = %g    New f() = %g    Convergence = %g\n\n", oldfx, fx, cvg);
 #endif
@@ -549,15 +554,21 @@ esl_min_ConjugateGradientDescent(double *x, double *u, int n,
        * and it either means we're stuck or we're finished (most likely stuck)
        */
       for (i1 = 0; i1 < n; i1++) 
-	if (cg[i1] != 0.) break;
+	     if (cg[i1] != 0.) break;
       if  (i1 == n) break;
 
       oldfx = fx;
     }
-  if (i == MAXITERATIONS) 
-    ESL_EXCEPTION(eslENOHALT, "Failed to converge in ConjugateGradientDescent()");
 
-  if (ret_fx != NULL) *ret_fx = fx;
+
+	if (ret_fx != NULL) *ret_fx = fx;
+
+    if (i == MAXITERATIONS)
+	  ESL_FAIL(eslENOHALT, NULL, " ");
+// 	  ESL_EXCEPTION(eslENOHALT, "Failed to converge in ConjugateGradientDescent()");
+
+
+
   return eslOK;
 }
 
@@ -638,7 +649,14 @@ main(int argc, char **argv)
 
 
 /*****************************************************************  
- *    This copyrighted source code is freely distributed 
- *    under the terms of the GNU General Public License. See
- *    the files COPYRIGHT and LICENSE for details.
+ * Easel - a library of C functions for biological sequence analysis
+ * Version i1.1.1; July 2014
+ * Copyright (C) 2014 HHMI Janelia Farm Research Campus
+ * Other copyrights also apply. See the COPYRIGHT file for a full list.
+ * 
+ * Easel is distributed under the Janelia Farm Software License, a BSD
+ * license. See the LICENSE file for more details.
+ * 
+ * SVN $Id: esl_minimizer.c 685 2011-05-23 14:27:52Z eddys $
+ * SVN $URL: https://svn.janelia.org/eddylab/eddys/easel/branches/infernal/1.1/esl_minimizer.c $
  *****************************************************************/
