@@ -1,10 +1,11 @@
 /* Multidimensional optimization by conjugate gradient descent.
  * 
  * SRE, Wed Jun 22 09:53:05 2005
- * SVN $Id: esl_minimizer.h 162 2007-04-10 23:50:12Z eddys $
+ * SVN $Id: esl_minimizer.h 664 2011-02-27 17:08:36Z eddys $
+ * SVN $URL: https://svn.janelia.org/eddylab/eddys/easel/branches/infernal/1.1/esl_minimizer.h $
  */
-#ifndef ESL_MINIMIZER_INCLUDED
-#define ESL_MINIMIZER_INCLUDED
+#ifndef eslMINIMIZER_INCLUDED
+#define eslMINIMIZER_INCLUDED
 
 #define MAXITERATIONS 100
 
@@ -22,10 +23,13 @@ extern int esl_min_ConjugateGradientDescent(double *x, double *u, int n,
 					    void (*dfunc)(double *, int, void *, double *),
 					    void *prm, double tol, double *wrk, double *ret_fx);
 
-#endif /*ESL_MINIMIZER_INCLUDED*/
-
+#endif /*eslMINIMIZER_INCLUDED*/
 /*****************************************************************  
- *    This copyrighted source code is freely distributed 
- *    under the terms of the GNU General Public License. See
- *    the files COPYRIGHT and LICENSE for details.
+ * Easel - a library of C functions for biological sequence analysis
+ * Version i1.1.1; July 2014
+ * Copyright (C) 2014 HHMI Janelia Farm Research Campus
+ * Other copyrights also apply. See the COPYRIGHT file for a full list.
+ * 
+ * Easel is distributed under the Janelia Farm Software License, a BSD
+ * license. See the LICENSE file for more details.
  *****************************************************************/
