@@ -1,14 +1,14 @@
-/* esl_mixgev.h
- * Mixture generalized extreme value distributions.
+/* Mixture generalized extreme value distributions.
  * 
  * SRE, Mon Aug 15 08:33:57 2005 [St. Louis]
- * SVN $Id: esl_mixgev.h 83 2005-12-13 20:54:07Z eddy $
+ * SVN $Id: esl_mixgev.h 664 2011-02-27 17:08:36Z eddys $
+ * SVN $URL: https://svn.janelia.org/eddylab/eddys/easel/branches/infernal/1.1/esl_mixgev.h $
  */
-#ifndef ESL_MIXGEV_INCLUDED
-#define ESL_MIXGEV_INCLUDED
+#ifndef eslMIXGEV_INCLUDED
+#define eslMIXGEV_INCLUDED
 
 #ifdef eslAUGMENT_RANDOM
-#include <esl_random.h>
+#include "esl_random.h"
 #endif
 
 typedef struct {
@@ -57,9 +57,13 @@ extern int         esl_mixgev_FitComplete(double *x, int n, ESL_MIXGEV *mg);
 #endif /*eslAUGMENT_MINIMIZER*/
 
 
-#endif /*ESL_MIXGEV_INCLUDED*/
+#endif /*eslMIXGEV_INCLUDED*/
 /*****************************************************************
- *    This copyrighted source code is freely distributed 
- *    under the terms of the GNU General Public License. See
- *    the files COPYRIGHT and LICENSE for details.
+ * Easel - a library of C functions for biological sequence analysis
+ * Version i1.1.1; July 2014
+ * Copyright (C) 2014 HHMI Janelia Farm Research Campus
+ * Other copyrights also apply. See the COPYRIGHT file for a full list.
+ * 
+ * Easel is distributed under the Janelia Farm Software License, a BSD
+ * license. See the LICENSE file for more details.
  *****************************************************************/
