@@ -11,16 +11,12 @@
  * 
  *  Augmentations:
  *    eslAUGMENT_ALPHABET:  adds support for digital MSAs
- *    
  *  
- * (Why isn't this just part of the cluster or MSA modules?  cluster
- * itself is a core module, dependent only on easel. MSA clustering
- * involves at least the distance, cluster, and msa modules. So we're
- * better off separating its functionality away into a more highly
- * derived module.)
- *   
- * SRE, Sun Nov  5 10:06:53 2006 [Janelia]
- * SVN $Id: esl_msacluster.c 269 2008-06-19 13:47:41Z eddys $
+ * (Wondering why isn't this just part of the cluster or MSA modules?
+ * esl_cluster itself is a core module, dependent only on easel. MSA
+ * clustering involves at least the distance, cluster, and msa
+ * modules. We're better off separating its functionality away into a
+ * more highly derived module.)
  */
 #include "esl_config.h"
 
@@ -194,7 +190,7 @@ msacluster_clinkage(const void *v1, const void *v2, const void *p, int *ret_link
   char  *as2   = *(char **) v2;
   double maxid = *(double *) p;
   double pid;
-  int    status;
+  int    status = eslOK;
 
 #if defined(eslMSACLUSTER_REGRESSION) || defined(eslMSAWEIGHT_REGRESSION)
   pid = 1. - squid_distance(as1, as2);
@@ -203,7 +199,7 @@ msacluster_clinkage(const void *v1, const void *v2, const void *p, int *ret_link
 #endif
 
   *ret_link = (pid >= maxid ? TRUE : FALSE); 
-  return eslOK;
+  return status;
 }
   
 /* Definition of % id linkage in digital aligned seqs (>= maxid) */
@@ -215,7 +211,7 @@ msacluster_xlinkage(const void *v1, const void *v2, const void *p, int *ret_link
   ESL_DSQ *ax2              = *(ESL_DSQ **) v2;
   struct msa_param_s *param = (struct msa_param_s *) p;
   double   pid;
-  int      status;
+  int      status = eslOK;
 
 #if defined(eslMSACLUSTER_REGRESSION) || defined(eslMSAWEIGHT_REGRESSION)
   pid = 1. - squid_xdistance(param->abc, ax1, ax2);
@@ -224,7 +220,7 @@ msacluster_xlinkage(const void *v1, const void *v2, const void *p, int *ret_link
 #endif
 
   *ret_link = (pid >= param->maxid ? TRUE : FALSE); 
-  return eslOK;
+  return status;
 }
 #endif
 
@@ -309,7 +305,9 @@ utest_SingleLinkage(ESL_GETOPTS *go, const ESL_MSA *msa, double maxid, int expec
 #include "easel.h"
 #include "esl_alphabet.h"
 #include "esl_getopts.h"
+#include "esl_msa.h"
 #include "esl_msacluster.h"
+#include "esl_msafile.h"
 
 static ESL_OPTIONS options[] = {
   /* name           type      default  env  range toggles reqs incomp  help                                       docgroup*/
@@ -347,7 +345,7 @@ seq11 MMMMMMMMMM\n\
   utest_SingleLinkage(go, msa, 0.0,  1,  0);    /* at 0% id, everything clusters      */
 
   /* Do the same tests, but now with a digital MSA */
-  esl_msa_Digitize(abc, msa);
+  esl_msa_Digitize(abc, msa, NULL);
   utest_SingleLinkage(go, msa, 1.0, 11, 10);    /* at 100% id, only seq0/seq1 cluster */
   utest_SingleLinkage(go, msa, 0.5,  6,  5);    /* at 50% id, seq0-seq6 cluster       */
   utest_SingleLinkage(go, msa, 0.0,  1,  0);    /* at 0% id, everything clusters      */
@@ -369,22 +367,22 @@ seq11 MMMMMMMMMM\n\
 #ifdef eslMSACLUSTER_EXAMPLE
 /*::cexcerpt::msacluster_example::begin::*/
 /*
-   gcc -g -Wall -o example -I. -DeslMSACLUSTER_EXAMPLE\
-        esl_msacluster.c esl_msa.c esl_cluster.c esl_distance.c easel.c  -lm
-   gcc -g -Wall -o example -DeslAUGMENT_ALPHABET -I. -DeslMSACLUSTER_EXAMPLE\
-        esl_msacluster.c esl_msa.c esl_cluster.c esl_distance.c esl_alphabet.c easel.c  -lm
-  ./example <MSA file>
+   gcc -g -Wall -o msacluster_example -I. -L. -DeslMSACLUSTER_EXAMPLE esl_msacluster.c -leasel -lm
+   ./msacluster_example <MSA file>
  */
 #include <stdio.h>
 #include "easel.h"
+#include "esl_msa.h"
 #include "esl_msacluster.h"
+#include "esl_msafile.h"
 
 int
 main(int argc, char **argv)
 {
   char        *filename   = argv[1];
   int          fmt        = eslMSAFILE_UNKNOWN; 
-  ESL_MSAFILE *afp        = NULL;
+  ESL_ALPHABET *abc       = NULL;
+  ESLX_MSAFILE *afp       = NULL;
   ESL_MSA     *msa        = NULL;
   double       maxid      = 0.62; /* cluster at 62% identity: the BLOSUM62 rule */
   int         *assignment = NULL;
@@ -393,19 +391,15 @@ main(int argc, char **argv)
   int          c, i;		  
   int          status;
 
-  status = esl_msafile_OpenDigital(filename, fmt, NULL, &afp);
-  if (status == eslENOTFOUND)    esl_fatal("Alignment file %s isn't readable\n", filename);
-  else if (status == eslEFORMAT) esl_fatal("Couldn't determine format of %s\n",  filename);
-  else if (status != eslOK)      esl_fatal("Alignment file open failed (error %d)\n", status);
-
-  status = esl_msa_Read(afp, &msa);
-  if (status == eslEFORMAT)  
-    esl_fatal("parse error, line %d of file %s:\n%s\nOffending line is:\n%s\n", 
-	      afp->linenumber, afp->fname, afp->errbuf, afp->buf);
-  else if (status != eslOK)
-    esl_fatal("Alignment file read failed with error code %d\n", status);
+  /* Open; guess alphabet; set to digital mode */
+  if ((status = eslx_msafile_Open(&abc, filename, NULL, fmt, NULL, &afp)) != eslOK)
+    eslx_msafile_OpenFailure(afp, status);
 
+  /* read one alignment */
+  if ((status = eslx_msafile_Read(afp, &msa)) != eslOK)
+    eslx_msafile_ReadFailure(afp, status);
 
+  /* do the clustering */
   esl_msacluster_SingleLinkage(msa, maxid, &assignment, &nin, &nclusters);
 
   printf("%d clusters at threshold of %f fractional identity\n", nclusters, maxid);
@@ -416,7 +410,7 @@ main(int argc, char **argv)
   }
 
   esl_msa_Destroy(msa);
-  esl_msafile_Close(afp);
+  eslx_msafile_Close(afp);
   free(assignment);
   free(nin);
   return 0;
@@ -429,8 +423,15 @@ main(int argc, char **argv)
 
 
 /*****************************************************************
- *    This copyrighted source code is freely distributed 
- *    under the terms of the GNU General Public License. See
- *    the files COPYRIGHT and LICENSE for details.
+ * Easel - a library of C functions for biological sequence analysis
+ * Version i1.1.1; July 2014
+ * Copyright (C) 2014 HHMI Janelia Farm Research Campus
+ * Other copyrights also apply. See the COPYRIGHT file for a full list.
+ * 
+ * Easel is distributed under the Janelia Farm Software License, a BSD
+ * license. See the LICENSE file for more details.
+ *
+ * SVN $Id: esl_msacluster.c 715 2011-08-03 21:04:24Z eddys $
+ * SVN $URL: https://svn.janelia.org/eddylab/eddys/easel/branches/infernal/1.1/esl_msacluster.c $
  *****************************************************************/
 
