@@ -15,11 +15,7 @@
  *   6. Stats driver.
  *   7. Examples.
  *   8. Copyright notice and license.
- * 
- * SVN $Id: esl_msaweight.c 344 2009-06-15 15:32:24Z nawrockie $
- * SRE, Fri Nov  3 12:55:45 2006 [Janelia]
  */
-
 #include "esl_config.h"
 
 #include <math.h>
@@ -539,7 +535,7 @@ utest_GSC(ESL_ALPHABET *abc, ESL_MSA *msa, double *expect)
   
   if (abc != NULL) 
     {
-      if (esl_msa_Digitize(abc, msa)                           != eslOK) esl_fatal(msg);
+      if (esl_msa_Digitize(abc, msa, NULL)                     != eslOK) esl_fatal(msg);
       if (esl_msaweight_GSC(msa)                               != eslOK) esl_fatal(msg);
       if (esl_vec_DCompare(msa->wgt, expect, msa->nseq, 0.001) != eslOK) esl_fatal(msg);
       if (esl_msa_Textize(msa)                                 != eslOK) esl_fatal(msg);
@@ -557,7 +553,7 @@ utest_PB(ESL_ALPHABET *abc, ESL_MSA *msa, double *expect)
   
   if (abc != NULL) 
     {
-      if (esl_msa_Digitize(abc, msa)                           != eslOK) esl_fatal(msg);
+      if (esl_msa_Digitize(abc, msa, NULL)                     != eslOK) esl_fatal(msg);
       if (esl_msaweight_PB(msa)                                != eslOK) esl_fatal(msg);
       if (esl_vec_DCompare(msa->wgt, expect, msa->nseq, 0.001) != eslOK) esl_fatal(msg);
       if (esl_msa_Textize(msa)                                 != eslOK) esl_fatal(msg);
@@ -575,7 +571,7 @@ utest_BLOSUM(ESL_ALPHABET *abc, ESL_MSA *msa, double maxid, double *expect)
   
   if (abc != NULL) 
     {
-      if (esl_msa_Digitize(abc, msa)                           != eslOK) esl_fatal(msg);
+      if (esl_msa_Digitize(abc, msa, NULL)                     != eslOK) esl_fatal(msg);
       if (esl_msaweight_BLOSUM(msa, maxid)                     != eslOK) esl_fatal(msg);
       if (esl_vec_DCompare(msa->wgt, expect, msa->nseq, 0.001) != eslOK) esl_fatal(msg);
       if (esl_msa_Textize(msa)                                 != eslOK) esl_fatal(msg);
@@ -598,6 +594,7 @@ utest_BLOSUM(ESL_ALPHABET *abc, ESL_MSA *msa, double maxid, double *expect)
  */
 #include "easel.h"
 #include "esl_msa.h"
+#include "esl_msafile.h"
 #include "esl_msaweight.h"
 
 int
@@ -700,7 +697,7 @@ main(int argc, char **argv)
 #ifdef eslMSAWEIGHT_REGRESSION
 /* Verify same results as SQUID.
  * To compile:
-       gcc -g -Wall -o regression -I. -L. -L ~/src/squid -I ~/src/squid -DeslMSAWEIGHT_REGRESSION\
+      gcc -g -Wall -o msaweight_regression -I. -L. -L ~/src/squid -I ~/src/squid -DeslMSAWEIGHT_REGRESSION \
           esl_msaweight.c esl_msacluster.c -leasel -lsquid -lm
  * To run: 
  *     ./regression <MSA file>
@@ -716,6 +713,7 @@ main(int argc, char **argv)
 #include "easel.h"
 #include "esl_getopts.h"
 #include "esl_msa.h"
+#include "esl_msafile.h"
 #include "esl_msaweight.h"
 #include "esl_vectorops.h"
 
@@ -742,11 +740,11 @@ static char usage[] = "Usage: ./regression [-options] <msa_file>";
 int 
 main(int argc, char **argv)
 {
-  ESL_GETOPTS *go;
-  char        *msafile;
-  ESL_MSAFILE *afp;
-  ESL_MSA     *msa;
-  float       *sqd;
+  ESL_GETOPTS  *go;
+  char         *msafile;
+  ESLX_MSAFILE *afp;
+  ESL_MSA      *msa;
+  float        *sqd;
   int          status;
   int          nbad;
   int          nali    = 0;
@@ -790,9 +788,13 @@ main(int argc, char **argv)
 
   /* Weight one or more alignments from input file
    */
-  esl_msafile_Open(msafile, eslMSAFILE_UNKNOWN, NULL, &afp);
-  while (esl_msa_Read(afp, &msa) == eslOK)
+  if ((status = eslx_msafile_Open(NULL, msafile, NULL, eslMSAFILE_UNKNOWN, NULL, &afp)) != eslOK)
+    eslx_msafile_OpenFailure(afp, status);
+
+  while ( (status = eslx_msafile_Read(afp, &msa)) != eslEOF)
     {
+      if (status != eslOK) eslx_msafile_ReadFailure(afp, status);
+
       if (maxN > 0 && msa->nseq > maxN) { esl_msa_Destroy(msa); continue; }
 
       nali++;
@@ -832,7 +834,7 @@ main(int argc, char **argv)
       esl_msa_Destroy(msa);
       free(sqd);
     } 
-  esl_msafile_Close(afp);
+  eslx_msafile_Close(afp);
 
   if (nbadali == 0) 
     printf("OK: all weights identical between squid and Easel in %d alignment(s)\n", nali);
@@ -866,6 +868,7 @@ main(int argc, char **argv)
 #include "easel.h"
 #include "esl_getopts.h"
 #include "esl_msa.h"
+#include "esl_msafile.h"
 #include "esl_msaweight.h"
 #include "esl_vectorops.h"
 #include "esl_stopwatch.h"
@@ -889,16 +892,17 @@ int
 main(int argc, char **argv)
 {
   ESL_STOPWATCH *w;
-  ESL_GETOPTS *go;
-  char        *msafile;
-  ESL_MSAFILE *afp;
-  ESL_MSA     *msa;
-  int          do_gsc;
-  int          do_pb;
-  int          do_blosum;
-  int          maxN;
-  double       maxid;
-  double       cpu;
+  ESL_GETOPTS   *go;
+  char          *msafile;
+  ESLX_MSAFILE  *afp;
+  ESL_MSA       *msa;
+  int            do_gsc;
+  int            do_pb;
+  int            do_blosum;
+  int            maxN;
+  double         maxid;
+  double         cpu;
+  int            status;
 
   /* Process command line
    */
@@ -928,9 +932,12 @@ main(int argc, char **argv)
 
   /* Weight one or more alignments from input file
    */
-  esl_msafile_Open(msafile, eslMSAFILE_UNKNOWN, NULL, &afp);
-  while (esl_msa_Read(afp, &msa) == eslOK)
+  if ((status = eslx_msafile_Open(NULL, msafile, NULL, eslMSAFILE_UNKNOWN, NULL, &afp)) != eslOK)
+    eslx_msafile_OpenFailure(afp, status);
+
+  while ( (status = eslx_msafile_Read(afp, &msa)) != eslEOF) 
     {
+      if (status != eslOK) eslx_msafile_ReadFailure(afp, status);
       if (maxN > 0 && msa->nseq > maxN) { esl_msa_Destroy(msa); continue; }
 
       esl_stopwatch_Start(w);
@@ -944,7 +951,7 @@ main(int argc, char **argv)
       printf("%-20s %6d  %6d  %.3f\n", msa->name, msa->alen, msa->nseq, cpu);
       esl_msa_Destroy(msa);
     } 
-  esl_msafile_Close(afp);
+  eslx_msafile_Close(afp);
 
   esl_stopwatch_Destroy(w);
   return eslOK;
@@ -970,6 +977,7 @@ main(int argc, char **argv)
 #include "easel.h"
 #include "esl_getopts.h"
 #include "esl_msa.h"
+#include "esl_msafile.h"
 #include "esl_msaweight.h"
 #include "esl_vectorops.h"
 
@@ -991,21 +999,20 @@ static char usage[] = "Usage: ./stats [-options] <msa_file>";
 int 
 main(int argc, char **argv)
 {
-  ESL_GETOPTS *go;
-  char        *msafile;
-  ESL_MSAFILE *afp;
-  ESL_MSA     *msa;
-  int          do_gsc;
-  int          do_pb;
-  int          do_blosum;
-  int          maxN;
-  double       maxid;
-  int          nsmall, nbig;
-  int          i;
+  ESL_GETOPTS  *go;
+  char         *msafile;
+  ESLX_MSAFILE *afp;
+  ESL_MSA      *msa;
+  int           do_gsc;
+  int           do_pb;
+  int           do_blosum;
+  int           maxN;
+  double        maxid;
+  int           nsmall, nbig;
+  int           i;
+  int           status;
 
-
-  /* Process command line
-   */
+  /* Process command line  */
   go = esl_getopts_Create(options);
   if (esl_opt_ProcessCmdline(go, argc, argv) != eslOK) esl_fatal("%s", go->errbuf);
   if (esl_opt_VerifyConfig(go)               != eslOK) esl_fatal("%s", go->errbuf);
@@ -1030,9 +1037,12 @@ main(int argc, char **argv)
 
   /* Weight one or more alignments from input file
    */
-  esl_msafile_Open(msafile, eslMSAFILE_UNKNOWN, NULL, &afp);
-  while (esl_msa_Read(afp, &msa) == eslOK)
+  if ((status = eslx_msafile_Open(NULL, msafile, NULL, eslMSAFILE_UNKNOWN, NULL, &afp)) != eslOK)
+    eslx_msafile_OpenFailure(afp, status);
+
+  while ( (status = eslx_msafile_Read(afp, &msa)) != eslEOF)
     {
+      if (status != eslOK) eslx_msafile_ReadFailure(afp, status);
       if (maxN > 0 && msa->nseq > maxN) { esl_msa_Destroy(msa); continue; }
 
       if      (do_gsc) 	  esl_msaweight_GSC(msa);
@@ -1054,7 +1064,7 @@ main(int argc, char **argv)
 	     nbig);
       esl_msa_Destroy(msa);
     } 
-  esl_msafile_Close(afp);
+  eslx_msafile_Close(afp);
   return eslOK;
 }
 #endif /* eslMSAWEIGHT_STATS */
@@ -1077,17 +1087,21 @@ main(int argc, char **argv)
  */
 #include "easel.h"
 #include "esl_msa.h"
+#include "esl_msafile.h"
 #include "esl_msaweight.h"
 
 int main(int argc, char **argv)
 {
-  ESL_MSAFILE *afp;
-  ESL_MSA     *msa;
-  int i;
+  ESLX_MSAFILE *afp;
+  ESL_MSA      *msa;
+  int           i;
+  int           status;
 
-  esl_msafile_Open(argv[1], eslMSAFILE_UNKNOWN, NULL, &afp);
-  esl_msa_Read(afp, &msa);
-  esl_msafile_Close(afp);
+  if ( (status = eslx_msafile_Open(NULL, argv[1], NULL, eslMSAFILE_UNKNOWN, NULL, &afp)) != eslOK)
+    eslx_msafile_OpenFailure(afp, status);
+  if ( (status = eslx_msafile_Read(afp, &msa)) != eslOK)
+    eslx_msafile_ReadFailure(afp, status);
+  eslx_msafile_Close(afp);
 
   esl_msaweight_GSC(msa);
   
@@ -1102,7 +1116,14 @@ int main(int argc, char **argv)
 
 
 /*****************************************************************
- *    This copyrighted source code is freely distributed 
- *    under the terms of the GNU General Public License. See
- *    the files COPYRIGHT and LICENSE for details.
+ * Easel - a library of C functions for biological sequence analysis
+ * Version i1.1.1; July 2014
+ * Copyright (C) 2014 HHMI Janelia Farm Research Campus
+ * Other copyrights also apply. See the COPYRIGHT file for a full list.
+ * 
+ * Easel is distributed under the Janelia Farm Software License, a BSD
+ * license. See the LICENSE file for more details.
+ * 
+ * SVN $Id: esl_msaweight.c 715 2011-08-03 21:04:24Z eddys $
+ * SVN $URL: https://svn.janelia.org/eddylab/eddys/easel/branches/infernal/1.1/esl_msaweight.c $
  *****************************************************************/
