@@ -12,9 +12,6 @@
  *   9. Test driver.
  *  10. Example.
  *  11. Copyright and license information 
- * 
- * SRE, Thu Apr 24 08:59:26 2008 [Janelia]
- * SVN $Id$
  */
 #include "esl_config.h"
 
@@ -328,10 +325,78 @@ esl_rsq_CShuffleDP(ESL_RANDOMNESS *r, const char *s, char *shuffled)
  ERROR:
   esl_Free2D((void **) E, 26);
   if (nE != NULL) free(nE);
-  if (iE != NULL) free(nE);
+  if (iE != NULL) free(iE);
   return status;
 }
 
+
+/* Function:  esl_rsq_CShuffleKmers()
+ * Synopsis:  Shuffle k-mers in a text sequence.
+ * Incept:    SRE, Tue Nov 17 16:55:57 2009 [NHGRI retreat, Gettysburg]
+ *
+ * Purpose:   Consider a text sequence <s> as a string of nonoverlapping
+ *            k-mers of length <K>. Shuffle the k-mers, given a random
+ *            number generator <r>. Put the shuffled sequence in
+ *            <shuffled>.
+ *            
+ *            If the length of <s> is not evenly divisible by <K>, the
+ *            remaining residues are left (unshuffled) as a prefix to
+ *            the shuffled k-mers.
+ *            
+ *            For example, shuffling ABCDEFGHIJK as k=3-mers might
+ *            result in ABFIJKFGHCDE.
+ *            
+ *            Caller provides allocated storage for <shuffled>,
+ *            for at least the same length as <s>. 
+ *            
+ *            <shuffled> may also point to the same storage as <s>,
+ *            in which case <s> is shuffled in place.
+ *            
+ *            There is almost no formally justifiable reason why you'd
+ *            use this shuffle -- it's not like it preserves any
+ *            particularly well-defined statistical properties of the
+ *            sequence -- but it's a quick and dirty way to sort of
+ *            maybe possibly preserve some higher-than-monomer
+ *            statistics.
+ *
+ * Args:      r        - an <ESL_RANDOMNESS> random generator
+ *            s        - sequence to shuffle
+ *            K        - size of k-mers to break <s> into
+ *            shuffled - RESULT: the shuffled sequence
+ *
+ * Returns:   <eslOK> on success.
+ *
+ * Throws:    <eslEMEM> on allocation error.
+ */
+int
+esl_rsq_CShuffleKmers(ESL_RANDOMNESS *r, const char *s, int K, char *shuffled)
+{
+  int   L = strlen(s);
+  int   W = L / K;		/* number of kmers "words" excluding leftover prefix */
+  int   P = L % K;		/* leftover residues in prefix */
+  int   i;
+  char *swap = NULL;
+  int   status;
+
+  if (shuffled != s) strcpy(shuffled, s);
+  ESL_ALLOC(swap, sizeof(char) * K);
+  while (W > 1) 
+    {	/* use memmove, not strncpy or memcpy, because i==W-1 creates an overlap case */
+      i = esl_rnd_Roll(r, W);	                                                 /* pick a word          */
+      memmove(swap,                   shuffled + P + i*K,     K * sizeof(char)); /* copy it to tmp space */
+      memmove(shuffled + P + i*K,     shuffled + P + (W-1)*K, K * sizeof(char)); /* move word W-1 to i   */
+      memmove(shuffled + P + (W-1)*K, swap,                   K * sizeof(char)); /* move word i to W-1   */
+      W--;
+    }
+  free(swap);
+  return eslOK;
+
+ ERROR:
+  free(swap);
+  return status;
+}
+
+
 /* Function:  esl_rsq_CReverse()
  * Synopsis:  Reverse a string.
  * Incept:    SRE, Sat Feb 24 10:06:34 2007 [Casa de Gatos]
@@ -787,12 +852,55 @@ esl_rsq_XShuffleDP(ESL_RANDOMNESS *r, const ESL_DSQ *dsq, int L, int K, ESL_DSQ
  ERROR:
   esl_Free2D((void **) E, K);
   if (nE != NULL) free(nE);
-  if (iE != NULL) free(nE);
+  if (iE != NULL) free(iE);
   if (Z  != NULL) free(Z);
   return status;
 }
 
 
+/* Function:  esl_rsq_XShuffleKmers()
+ * Synopsis:  Shuffle k-mers in a digital sequence.
+ *
+ * Purpose:   Same as <esl_rsq_CShuffleKmers()>, but shuffle digital 
+ *            sequence <dsq> of length <L> into digital result <shuffled>.
+ *
+ * Args:      r        - an <ESL_RANDOMNESS> random generator
+ *            dsq      - sequence to shuffle
+ *            K        - size of k-mers to break <s> into
+ *            shuffled - RESULT: the shuffled sequence
+ *
+ * Returns:   <eslOK> on success.
+ *
+ * Throws:    <eslEMEM> on allocation error.
+ */
+int
+esl_rsq_XShuffleKmers(ESL_RANDOMNESS *r, const ESL_DSQ *dsq, int L, int K, ESL_DSQ *shuffled)
+{
+  int   W = L / K;		/* number of kmers "words" excluding leftover prefix */
+  int   P = L % K;		/* leftover residues in prefix */
+  int   i;
+  char *swap = NULL;
+  int   status;
+
+  if (shuffled != dsq) esl_abc_dsqcpy(dsq, L, shuffled);
+  ESL_ALLOC(swap, sizeof(char) * K);
+  while (W > 1) 
+    {				/* use memmove, not memcpy, because i==W-1 is an overlap case */
+      i = esl_rnd_Roll(r, W);	                                                 /* pick a word          */
+      memmove(swap,                   shuffled + P + i*K,     K * sizeof(char)); /* copy it to tmp space */
+      memmove(shuffled + P + i*K,     shuffled + P + (W-1)*K, K * sizeof(char)); /* move word W-1 to i   */
+      memmove(shuffled + P + (W-1)*K, swap,                   K * sizeof(char)); /* move word i to W-1   */
+      W--;
+    }
+  free(swap);
+  return eslOK;
+
+ ERROR:
+  free(swap);
+  return status;
+}
+
+
 /* Function:  esl_rsq_XReverse()
  * Synopsis:  Reverse a digital sequence.
  * Incept:    SRE, Sat Feb 24 10:13:30 2007 [Casa de Gatos]
@@ -1072,7 +1180,6 @@ static ESL_OPTIONS options[] = {
   { "-R",        eslARG_NONE,   FALSE,  NULL, NULL,  NULL,  NULL, NULL, "reverse the sequence",                             0 },
   { "-2",        eslARG_NONE,   FALSE,  NULL, NULL,  NULL,  NULL, NULL, "resample an independent sequence",                 0 },
   { "-N",        eslARG_INT,  "10000",  NULL, NULL,  NULL,  NULL, NULL, "number of sampled sequences per length",           0 },
-  { "-r",        eslARG_NONE,   NULL,   NULL, NULL,  NULL,  NULL, NULL, "use arbitrary random number seed",                 0 },
   { "-s",        eslARG_INT,     "42",  NULL, NULL,  NULL,  NULL, NULL, "set random number seed to <n>",                    0 },
   { "--minL",    eslARG_INT,      "5",  NULL, NULL,  NULL,  NULL, NULL, "xaxis minimum L",                                  0 },
   { "--maxL",    eslARG_INT,    "200",  NULL, NULL,  NULL,  NULL, NULL, "xaxis maximum L",                                  0 },
@@ -1086,7 +1193,7 @@ int
 main(int argc, char **argv)
 {
   ESL_GETOPTS    *go       = esl_getopts_CreateDefaultApp(options, 0, argc, argv, banner, usage);
-  ESL_RANDOMNESS *r        = NULL;
+  ESL_RANDOMNESS *r        = esl_randomness_Create(esl_opt_GetInteger(go, "-s"));
   ESL_ALPHABET   *abc      = esl_alphabet_Create(eslAMINO);
   int             N        = esl_opt_GetInteger(go, "-N");
   int             minL     = esl_opt_GetInteger(go, "--minL");
@@ -1100,9 +1207,6 @@ main(int argc, char **argv)
   int             L;
   int             i;
 
-  if (esl_opt_GetBoolean(go, "-r")) r = esl_randomness_CreateTimeseeded();
-  else                              r = esl_randomness_Create(esl_opt_GetInteger(go, "-s"));
-  
   esl_vec_DSet(fq, abc->K, 1.0 / (double) abc->K );
 
   for (L = minL; L <= maxL; L += stepL)
@@ -1302,6 +1406,22 @@ utest_CShufflers(ESL_RANDOMNESS *r, int L, char *alphabet, int K)
   if (composition_compare(m1, di1, m2, di2, 26)   != eslOK) esl_fatal(logmsg);
   if (strcmp(s2, s) == 0)                                   esl_fatal(logmsg); 
   
+  /* esl_rsq_CShuffleKmers: mono composition stays the same */
+  memset(s2, 0, (L+1)*sizeof(char));
+  if (composition(s, L, m1,  di1)                 != eslOK) esl_fatal(logmsg);
+  if (esl_rsq_CShuffleKmers(r, s, 3, s2)          != eslOK) esl_fatal(logmsg);      
+  if (composition(s2, L, m2, di2)                 != eslOK) esl_fatal(logmsg);
+  if (composition_compare(m1, NULL, m2, NULL, 26) != eslOK) esl_fatal(logmsg);
+  if (strcmp(s2, s) == 0)                                   esl_fatal(logmsg); 
+  
+  /* esl_rsq_CShuffleKmers, in place */
+  strcpy(s, s2);
+  if (composition(s2, L, m1, di1)                 != eslOK) esl_fatal(logmsg);
+  if (esl_rsq_CShuffleKmers(r, s2, 3, s2)         != eslOK) esl_fatal(logmsg);      
+  if (composition(s2, L, m2, di2)                 != eslOK) esl_fatal(logmsg);
+  if (composition_compare(m1, NULL, m2, NULL, 26) != eslOK) esl_fatal(logmsg);
+  if (strcmp(s2, s) == 0)                                   esl_fatal(logmsg); 
+
   /* esl_rsq_CShuffleWindows(): mono composition stays the same */
   memset(s2, 0, (L+1)*sizeof(char));
   if (composition(s,   L, m1, di1)                != eslOK) esl_fatal(logmsg);
@@ -1501,6 +1621,20 @@ utest_XShufflers(ESL_RANDOMNESS *r, int L, int K)
   if (xcomposition(ds2, L, K, m2, di2)           != eslOK) esl_fatal(logmsg);
   if (composition_compare(m1, di1, m2, di2, K)   != eslOK) esl_fatal(logmsg);
   
+  /* esl_rsq_XShuffleKmers: mono compositions stay exactly the same */
+  memset(ds2, eslDSQ_SENTINEL, (L+2));
+  if (xcomposition(dsq, L, K, m1,  di1)          != eslOK) esl_fatal(logmsg);
+  if (esl_rsq_XShuffleKmers(r, dsq, L, 3, ds2)   != eslOK) esl_fatal(logmsg);      
+  if (xcomposition(ds2, L, K, m2, di2)           != eslOK) esl_fatal(logmsg);
+  if (composition_compare(m1, NULL, m2, NULL, K) != eslOK) esl_fatal(logmsg);
+
+  /* esl_rsq_XShuffleKmers, in place */
+  if (esl_abc_dsqcpy(ds2, L, dsq)                != eslOK) esl_fatal(logmsg);
+  if (xcomposition(ds2, L, K, m1, di1)           != eslOK) esl_fatal(logmsg);
+  if (esl_rsq_XShuffleKmers(r, ds2, L, 3, ds2)   != eslOK) esl_fatal(logmsg);      
+  if (xcomposition(ds2, L, K, m2, di2)           != eslOK) esl_fatal(logmsg);
+  if (composition_compare(m1, NULL, m2, NULL, K) != eslOK) esl_fatal(logmsg);
+
   /* esl_rsq_XShuffleWindows(): mono composition stays the same */
   memset(ds2, eslDSQ_SENTINEL, (L+2));
   if (xcomposition(dsq, L, K, m1, di1)           != eslOK) esl_fatal(logmsg);
@@ -1700,7 +1834,6 @@ static ESL_OPTIONS options[] = {
   /* name           type      default  env  range toggles reqs incomp  help                                       docgroup*/
   { "-h",        eslARG_NONE,   FALSE,  NULL, NULL,  NULL,  NULL, NULL, "show brief help on version and usage",             0 },
   { "-L",        eslARG_INT,   "1000",  NULL, NULL,  NULL,  NULL, NULL, "length of random sequences",                       0 },
-  { "-r",        eslARG_NONE,   NULL,   NULL, NULL,  NULL,  NULL, NULL, "use arbitrary random number seed",                 0 },
   { "-s",        eslARG_INT,     "42",  NULL, NULL,  NULL,  NULL, NULL, "set random number seed to <n>",                    0 },
   {  0, 0, 0, 0, 0, 0, 0, 0, 0, 0 },
 };
@@ -1711,14 +1844,11 @@ int
 main(int argc, char **argv)
 {
   ESL_GETOPTS    *go       = esl_getopts_CreateDefaultApp(options, 0, argc, argv, banner, usage);
-  ESL_RANDOMNESS *r        = NULL;
+  ESL_RANDOMNESS *r        = esl_randomness_Create(esl_opt_GetInteger(go, "-s"));
   char           *alphabet = "ACGT";
   int             K        = strlen(alphabet);
   int             L        = esl_opt_GetInteger(go, "-L");
 
-  if (esl_opt_GetBoolean(go, "-r")) r = esl_randomness_CreateTimeseeded();
-  else                              r = esl_randomness_Create(esl_opt_GetInteger(go, "-s"));
-  
   utest_CShufflers(r, L, alphabet, K);
   utest_CMarkovs  (r, L, alphabet);
   utest_XShufflers(r, L, K);
@@ -1757,7 +1887,7 @@ main(int argc, char **argv)
   int             format  = eslSQFILE_UNKNOWN;
   ESL_SQFILE     *sqfp    = NULL;
   ESL_SQ         *sq      = esl_sq_Create();
-  ESL_RANDOMNESS *r       = esl_randomness_CreateTimeseeded();
+  ESL_RANDOMNESS *r       = esl_randomness_Create(0);
   int             status;
 
   if (esl_sqfile_Open(seqfile, format, NULL, &sqfp) != eslOK) 
@@ -1786,7 +1916,14 @@ main(int argc, char **argv)
 /*--------------------- end, example ----------------------------*/
 
 /*****************************************************************
- *    This copyrighted source code is freely distributed 
- *    under the terms of the GNU General Public License. See
- *    the files COPYRIGHT and LICENSE for details.
+ * Easel - a library of C functions for biological sequence analysis
+ * Version i1.1.1; July 2014
+ * Copyright (C) 2014 HHMI Janelia Farm Research Campus
+ * Other copyrights also apply. See the COPYRIGHT file for a full list.
+ * 
+ * Easel is distributed under the Janelia Farm Software License, a BSD
+ * license. See the LICENSE file for more details.
+ * 
+ * SVN $Id: esl_randomseq.c 715 2011-08-03 21:04:24Z eddys $
+ * SVN $URL: https://svn.janelia.org/eddylab/eddys/easel/branches/infernal/1.1/esl_randomseq.c $
  *****************************************************************/
