@@ -34,9 +34,6 @@
  * See also:
  *   paml   - i/o of rate matrices from/to data files in PAML format
  * 
- *         original: SRE, Tue Jul 13 15:51:23 2004 [St. Louis]
- * upgrade to Easel: SRE, Thu Mar  8 17:59:35 2007 [Janelia]
- * SVN $Id: esl_ratematrix.c 241 2008-04-01 19:01:52Z eddys $
  */
 #include "esl_config.h"
 
@@ -625,7 +622,7 @@ main(void)
  * undocumented implementation of a matrix exponential.
  */
 /* 
-  gcc -g -Wall -I. -L. -o regression -DeslRATEMATRIX_REGRESSION -DHAVE_LIBGSL esl_dmatrix.c esl_ratematrix.c -leasel -lgsl -lgslcblas -lm
+  gcc -g -Wall -I. -L. -o ratematrix_regression -DeslRATEMATRIX_REGRESSION -DHAVE_LIBGSL esl_dmatrix.c esl_ratematrix.c -leasel -lgsl -lgslcblas -lm
  */
 
 #include "esl_config.h"
@@ -675,8 +672,13 @@ main(void)
   gsl_matrix_free(Pg);
   return 0;
 }
-
+#else 
+  /* if we don't have GSL, then compile in a dummy main(), solely 
+   *  to quiet any tests that are verifying that all drivers compile
+   *  and run. */
+int main(void) { return 0; }
 #endif /*HAVE_LIBGSL*/
+
 #endif /*eslRATEMATRIX_REGRESSION*/
 
 
@@ -812,7 +814,14 @@ main(void)
 
 
 /*****************************************************************
- *    This copyrighted source code is freely distributed 
- *    under the terms of the GNU General Public License. See
- *    the files COPYRIGHT and LICENSE for details.
+ * Easel - a library of C functions for biological sequence analysis
+ * Version i1.1.1; July 2014
+ * Copyright (C) 2014 HHMI Janelia Farm Research Campus
+ * Other copyrights also apply. See the COPYRIGHT file for a full list.
+ * 
+ * Easel is distributed under the Janelia Farm Software License, a BSD
+ * license. See the LICENSE file for more details.
+ *
+ * SVN $Id: esl_ratematrix.c 685 2011-05-23 14:27:52Z eddys $
+ * SVN $URL: https://svn.janelia.org/eddylab/eddys/easel/branches/infernal/1.1/esl_ratematrix.c $
  *****************************************************************/ 
