@@ -9,10 +9,7 @@
  *   6. Unit tests.
  *   7. Test driver.
  *   8. Examples.
- *   9. License and copyright.
- * 
- * SRE, Mon Mar 31 17:18:59 2008 [Janelia]
- * SVN $Id: esl_sq.c 258 2008-06-04 22:12:12Z eddys $
+ *   9. Copyright and license information.
  */
 #include "esl_config.h"
 
@@ -34,6 +31,10 @@
 static ESL_SQ *sq_create(int do_digital);
 static ESL_SQ *sq_create_from(const char *name, const char *desc, const char *acc);
 
+static ESL_SQ_BLOCK *sq_createblock(int count, int do_digital);
+
+static int  sq_init(ESL_SQ *sq, int do_digital);
+static void sq_free(ESL_SQ *sq);
 
 /*****************************************************************
  *# 1. Text version of the <ESL_SQ> object.
@@ -298,6 +299,7 @@ esl_sq_Copy(const ESL_SQ *src, ESL_SQ *dst)
   /* don't copy allocations (nalloc, etc); dst knows its own memory */
   dst->roff  = src->roff;
   dst->doff  = src->doff;
+  dst->hoff  = src->hoff;
   dst->eoff  = src->eoff;
   return eslOK;
 
@@ -354,6 +356,7 @@ esl_sq_Compare(ESL_SQ *sq1, ESL_SQ *sq2)
   /* Disk offset comparison */
   if (sq1->roff != -1 && sq2->roff != -1 && sq1->roff != sq2->roff) return eslFAIL;
   if (sq1->doff != -1 && sq2->doff != -1 && sq1->doff != sq2->doff) return eslFAIL;
+  if (sq1->hoff != -1 && sq2->hoff != -1 && sq1->hoff != sq2->hoff) return eslFAIL;
   if (sq1->eoff != -1 && sq2->eoff != -1 && sq1->eoff != sq2->eoff) return eslFAIL;
   
   /* alphabet comparison */
@@ -382,6 +385,7 @@ esl_sq_Reuse(ESL_SQ *sq)
   sq->name[0]   = '\0';
   sq->acc[0]    = '\0';
   sq->desc[0]   = '\0';
+  sq->tax_id    = -1;
   sq->source[0] = '\0';
   if (sq->seq != NULL) sq->seq[0] = '\0';
   if (sq->dsq != NULL) sq->dsq[0] = sq->dsq[1] = eslDSQ_SENTINEL;
@@ -395,12 +399,42 @@ esl_sq_Reuse(ESL_SQ *sq)
   sq->C     = 0;
   sq->W     = 0;
   sq->L     = -1;
+  sq->idx   = -1;
   sq->doff  = -1;
+  sq->hoff  = -1;
   sq->roff  = -1;
   sq->eoff  = -1;
   return eslOK;
 }
 
+/* Function:  esl_sq_IsDigital()
+ * Synopsis:  Return <TRUE> if <sq> is digital.
+ * Incept:    SRE, Mon Mar  2 18:05:34 2009 [Casa de Gatos]
+ *
+ * Purpose:   Return <TRUE> if <sq> is in digital mode,
+ *            and <FALSE> if not.
+ */
+int
+esl_sq_IsDigital(const ESL_SQ *sq)
+{
+  return ((sq->dsq != NULL) ? TRUE : FALSE);
+}
+
+
+/* Function:  esl_sq_IsText()
+ * Synopsis:  Return <TRUE> if <sq> is text mode.
+ * Incept:    SRE, Mon Mar  2 18:06:22 2009 [Casa de Gatos]
+ *
+ * Purpose:   Return <TRUE> if <sq> is in text mode,
+ *            and <FALSE> if not.
+ */
+int
+esl_sq_IsText(const ESL_SQ *sq)
+{
+  return ((sq->seq != NULL) ? TRUE : FALSE);
+}
+
+
 /* Function:  esl_sq_Destroy()
  * Synopsis:  Frees an <ESL_SQ>.
  * Incept:    SRE, Thu Dec 23 12:28:07 2004 [Zaragoza]
@@ -422,6 +456,83 @@ esl_sq_Destroy(ESL_SQ *sq)
   free(sq);
   return;
 }
+
+/* Function:  esl_sq_CreateBlock()
+ * Synopsis:  Create a new block of empty <ESL_SQ>.
+ * Incept:    
+ *
+ * Purpose:   Creates a block of empty <ESL_SQ> sequence objects.
+ *            
+ * Returns:   a pointer to the new <ESL_SQ_BLOCK>. Caller frees this
+ *            with <esl_sq_DestroyBlock()>.
+ *
+ * Throws:    <NULL> if allocation fails.
+ */
+ESL_SQ_BLOCK *
+esl_sq_CreateBlock(int count)
+{
+  return sq_createblock(count, FALSE);
+}
+
+/* Function:  esl_sq_DestroyBlock()
+ * Synopsis:  Frees an <ESL_SQ_BLOCK>.
+ * Incept:    
+ *
+ * Purpose:   Free a Create()'d block of <sq>.
+ */
+void
+esl_sq_DestroyBlock(ESL_SQ_BLOCK *block)
+{
+  int i;
+
+  if (block == NULL) return;
+
+  for (i = 0; i < block->listSize; ++i)
+    {
+      sq_free(block->list + i);
+    }
+
+  free(block->list);
+  free(block);
+  return;
+}
+
+#ifdef eslAUGMENT_ALPHABET
+
+/* Function:  esl_sq_CreateDigitalBlock()
+ * Synopsis:  Create a new block of empty <ESL_SQ> in digital mode.
+ * Incept:    
+ *
+ * Purpose:   Same as <esl_sq_CreateBlock()>, except the returned <sq>
+ *            is configured for a digital sequence using internal
+ *            alphabet <abc>, rather than a text sequence. Creates an
+ *            empty digital <ESL_SQ> sequence object, with internal
+ *            fields allocated to reasonable initial sizes.
+ *            
+ * Returns:   a pointer to the new <ESL_SQ_BLOCK>. Caller frees this with
+ *            <esl_sq_DestroyBlock()>.
+ * 
+ * Throws:    <NULL> if an allocation fails.
+ *
+ * Xref:      
+ */
+ESL_SQ_BLOCK *
+esl_sq_CreateDigitalBlock(int count, const ESL_ALPHABET *abc)
+{
+  int i;
+  ESL_SQ_BLOCK *block;
+  if ((block = sq_createblock(count, TRUE)) == NULL) return NULL;
+  
+  for (i = 0; i < count; ++i)
+    {
+      block->list[i].abc = abc;
+    }
+
+  return block;
+}
+
+#endif /* eslAUGMENT_ALPHABET */
+
 /*--------------- end of ESL_SQ object functions ----------------*/
 
 
@@ -645,7 +756,6 @@ esl_sq_Textize(ESL_SQ *sq)
 
 /* Function:  esl_sq_GuessAlphabet()
  * Synopsis:  Guess alphabet type of a single sequence.
- * Incept:    SRE, Wed May 16 11:03:44 2007 [Janelia]
  *
  * Purpose:   Guess the alphabet type of biosequence <sq>, and store the
  *            guess in <*ret_type>.
@@ -658,47 +768,29 @@ esl_sq_Textize(ESL_SQ *sq)
  *            
  *            The sequence must contain more than 10 residues, or it
  *            is called <eslUNKNOWN>.
- *            
- *            Specifically, this routine calls the sequence <eslDNA>
- *            if it consists only of the residues <ACGTN> and all four
- *            of <ACGT> occur. (And analogously for <eslRNA>,
- *            <ACGUN>.)  It calls the sequence <eslAMINO> either if it
- *            contains an amino-specific letter (<EFIJLOPQZ>), or if
- *            it contains at least 15 of the 20 canonical amino acids
- *            and consists only of canonical amino acids or <X>.
-
- *            Thus DNA sequences containing IUPAC degeneracies other
- *            than N are called <eslUNKNOWN>, rather than hazarding a
- *            guess. It may be possible to improve on this in the
- *            future by using residue occurrence frequencies.
- *            
- *            Note that a sequence of <ATATATA...> will be called
- *            <eslUNKNOWN>, whereas a sequence <ACGTACGTACGT...>
- *            (which could conceivably be "ala-cys-gly-thr...") will
- *            be called <eslDNA>. Peptides of simple mono and di-amino
- *            acid compositions do occur, but I have not (yet) seen a
- *            peptide consisting only of all four residues <ACGT>.
- *            
- *            The routine is designed to be conservative, calling
- *            <eslUNKNOWN> rather than making errors. In a test on the
- *            Oct 2006 version of the NCBI nonredundant databases,
- *            this routine called 0 <eslDNA> and 5694 <eslUNKNOWN> on
- *            4.0M protein sequences (99.9\% classification with no
- *            false positives) and 0 <eslAMINO> and 155756
- *            <eslUNKNOWN> in 4.4M DNA sequences (96\% classification
- *            with no false positives). (Well, actually, one DNA call
- *            was made in the protein database, but this was an
- *            exception that proves the rule; that entry was indeed a
- *            DNA contaminant. It has since been removed by NCBI.)
+ *
+ *            For details on the rules used to classify a residue
+ *            composition, see <esl_abc_GuessAlphabet()>. The rules
+ *            are good but not perfect. We err on the conservative
+ *            side, calling <eslUNKNOWN> rather than making
+ *            classification errors. However, errors are possible; an
+ *            example is a protein sequence <ACGTACGTACGT...>
+ *            ("ala-cys-gly-thr..."), which will be called <eslDNA>,
+ *            because it contains all and only DNA residues.
+ *
+ *            The routine is tested on large sequence databases to
+ *            make sure there are zero false positive classifications
+ *            on known sequences. See <esl_abc_GuessAlphabet() for
+ *            details of these tests, and crossreferences.
  *
  * Returns:   <eslOK> on success, and <*ret_type> is set to
  *            <eslAMINO>, <eslRNA>, or <eslDNA>.
  *
- *            Returns <eslEAMBIGUOUS> if unable to determine the
+ *            Returns <eslENOALPHABET> if unable to determine the
  *            alphabet type; in this case, <*ret_type> is set to 
  *            <eslUNKNOWN>.
  *
- * Xref:      J1/62; 2007/0517-easel-guess-alphabet
+ * Xref:      See notes on esl_alphabet.c::esl_abc_GuessAlphabet()
  */
 int
 esl_sq_GuessAlphabet(ESL_SQ *sq, int *ret_type)
@@ -718,8 +810,38 @@ esl_sq_GuessAlphabet(ESL_SQ *sq, int *ret_type)
   }
   return esl_abc_GuessAlphabet(ct, ret_type);
 }
-#endif /*eslAUGMENT_ALPHABET*/
 
+
+/* Function:  esl_sq_ConvertDegen2X()
+ * Synopsis:  Convert all degenerate residues to X/N
+ * Incept:    SRE, Tue Apr 20 08:52:54 2010 [Janelia]
+ *
+ * Purpose:   Convert all the degenerate residue codes in digital
+ *            sequence <sq> to the code for "unknown residue" (maximum
+ *            degeneracy); for example, X for protein, N for nucleic
+ *            acid. 
+ *            
+ *            This is handy when you need to be compatible with
+ *            software that can't deal with unusual residue codes.
+ *            For example, WU-BLAST can't deal with O (pyrrolysine)
+ *            codes.
+ *            
+ * Returns:   <eslOK> on success.
+ *
+ * Throws:    <eslEINVAL> if <sq> isn't in digital mode. 
+ *            (We only know how to interpret the alphabet
+ *            in digital mode. In text mode, letters are
+ *            just letters.)
+ */
+int
+esl_sq_ConvertDegen2X(ESL_SQ *sq)
+{
+  if (! esl_sq_IsDigital(sq)) ESL_EXCEPTION(eslEINVAL, "esl_sq_ConvertDegen2X() only works on digital sequences");
+  return esl_abc_ConvertDegen2X(sq->abc, sq->dsq);
+}
+
+
+#endif /*eslAUGMENT_ALPHABET*/
 /*---------- end of digitized ESL_SQ object functions -----------*/
 
 
@@ -729,12 +851,11 @@ esl_sq_GuessAlphabet(ESL_SQ *sq, int *ret_type)
  *****************************************************************/
 
 /* Function:  esl_sq_SetName()
- * Synopsis:  Format and set a name of a sequence.
+ * Synopsis:  Set the name of a sequence.
  * Incept:    SRE, Thu Jan 11 08:42:53 2007 [Janelia]
  *
  * Purpose:   Set the name of the sequence <sq> to <name>, reallocating
- *            as needed. <name> can be a <printf()>-style format with
- *            arguments; for example, <esl_sq_SetName(sq, "random%d", i)>.
+ *            as needed. For example, <esl_sq_SetName(sq, "random")>.
  * 
  *            A copy of <name> is made, so if caller had <name> allocated, 
  *            it is still responsible for freeing it.
@@ -746,7 +867,163 @@ esl_sq_GuessAlphabet(ESL_SQ *sq, int *ret_type)
  * Xref:      STL11/125
  */
 int
-esl_sq_SetName(ESL_SQ *sq, const char *name, ...)
+esl_sq_SetName(ESL_SQ *sq, const char *name)
+{
+  int   n;
+  void *tmp;
+  int   status;
+
+  if (name == NULL) { sq->name[0] = '\0'; return eslOK; }
+
+  n = strlen(name);
+  if (n >= sq->nalloc) 
+    {
+      ESL_RALLOC(sq->name, tmp, sizeof(char) * (n+1)); 
+      sq->nalloc = n+1;
+    }
+  strcpy(sq->name, name);
+  return eslOK;
+
+ ERROR:
+  return status;
+}
+
+/* Function:  esl_sq_SetAccession()
+ * Synopsis:  Set the accession field in a sequence.
+ * Incept:    SRE, Fri Jan 18 09:48:54 2008 [Westchester airport]
+ *
+ * Purpose:   Set the accession of the sequence <sq> to <acc>, reallocating
+ *            as needed. For example, <esl_sq_SetAccession(sq, "ACC12356")>.
+ * 
+ *            A copy of <acc> is made, so if caller had <acc> allocated, 
+ *            it is still responsible for freeing it.
+ *
+ * Returns:   <eslOK> on success.
+ *
+ * Throws:    <eslEMEM> on allocation error.
+ *
+ * Xref:      STL11/125
+ */
+int
+esl_sq_SetAccession(ESL_SQ *sq, const char *acc)
+{
+  int     n;
+  void   *tmp;
+  int     status;
+
+  if (acc == NULL) { sq->acc[0] = '\0'; return eslOK; }
+
+  n = strlen(acc);
+  if (n >= sq->aalloc)
+    {
+      ESL_RALLOC(sq->acc, tmp, sizeof(char) * (n+1)); 
+      sq->aalloc = n+1;
+    }
+  strcpy(sq->acc, acc);
+  return eslOK;
+
+ ERROR:
+  return status;
+}
+
+
+/* Function:  esl_sq_SetDesc()
+ * Synopsis:  Set the description field in a sequence.
+ * Incept:    SRE, Fri Jan 18 09:46:14 2008 [Westchester airport]
+ *
+ * Purpose:   Set the description of the sequence <sq> to <desc>, reallocating
+ *            as needed. 
+ *            For example, <esl_sq_SetDesc(sq, "this is a random sequence")>.
+ * 
+ *            A copy of <desc> is made, so if caller had <desc> allocated, 
+ *            it is still responsible for freeing it.
+ *
+ * Returns:   <eslOK> on success.
+ *
+ * Throws:    <eslEMEM> on allocation error.
+ *
+ * Xref:      STL11/125
+ */
+int
+esl_sq_SetDesc(ESL_SQ *sq, const char *desc)
+{
+  int     n;
+  void   *tmp;
+  int     status;
+
+  if (desc == NULL) { sq->desc[0] = '\0'; return eslOK; }
+
+  n = strlen(desc);
+  if (n >= sq->dalloc)
+    {
+      ESL_RALLOC(sq->desc, tmp, sizeof(char) * (n+1)); 
+      sq->dalloc = n+1;
+    }
+  strcpy(sq->desc, desc);
+  return eslOK;
+
+ ERROR:
+  return status;
+}
+
+/* Function:  esl_sq_SetSource()
+ * Synopsis:  Set the source name field in a sequence.
+ * Incept:    SRE, Wed May  7 16:17:56 2008 [Janelia]
+ *
+ * Purpose:   Set the source of the sequence <sq> to <source>, reallocating
+ *            as needed. For example, <esl_sq_SetSource(sq, "X123456")>.
+ * 
+ *            A copy of <source> is made, so if caller had <source> allocated, 
+ *            it is still responsible for freeing it.
+ *
+ * Returns:   <eslOK> on success.
+ *
+ * Throws:    <eslEMEM> on allocation error.
+ *
+ * Xref:      STL11/125
+ */
+int
+esl_sq_SetSource(ESL_SQ *sq, const char *source)
+{
+  int     n;
+  void   *tmp;
+  int     status;
+
+  if (source == NULL) { sq->source[0] = '\0'; return eslOK; }
+
+  n = strlen(source);
+  if (n >= sq->srcalloc)
+    {
+      ESL_RALLOC(sq->source, tmp, sizeof(char) * (n+1)); 
+      sq->srcalloc = n+1;
+    }
+  strcpy(sq->source, source);
+  return eslOK;
+
+ ERROR:
+  return status;
+}
+
+
+/* Function:  esl_sq_FormatName()
+ * Synopsis:  Format a name of a sequence, printf()-style.
+ * Incept:    SRE, Fri Sep 11 10:59:01 2009 [Janelia]
+ *
+ * Purpose:   Format the name of the sequence <sq> using
+ *            <printf()>-style format string <name> and corresponding
+ *            <printf()>-style arguments, reallocating as
+ *            needed.
+ *            For example, <esl_sq_FormatName(sq, "random%d", i)>.
+ * 
+ *            A copy of <name> is made, so if caller had <name> allocated, 
+ *            it is still responsible for freeing it.
+ *
+ * Returns:   <eslOK> on success.
+ *
+ * Throws:    <eslEMEM> on allocation error.
+ */
+int
+esl_sq_FormatName(ESL_SQ *sq, const char *name, ...)
 {
   va_list argp;
   va_list argp2;
@@ -772,13 +1049,14 @@ esl_sq_SetName(ESL_SQ *sq, const char *name, ...)
   return status;
 }
 
-/* Function:  esl_sq_SetAccession()
- * Synopsis:  Format and set the accession field in a sequence.
- * Incept:    SRE, Fri Jan 18 09:48:54 2008 [Westchester airport]
+/* Function:  esl_sq_FormatAccession()
+ * Synopsis:  Format the accession field in a sequence, printf()-style.
+ * Incept:    SRE, Fri Sep 11 11:00:37 2009 [Janelia]
  *
- * Purpose:   Set the accession of the sequence <sq> to <acc>, reallocating
- *            as needed. <acc> can be a <printf()>-style format with
- *            arguments; for example, <esl_sq_SetAccession(sq, "ACC%06d", i)>.
+ * Purpose:   Format the accession of the sequence <sq> using <printf()>-style 
+ *            format string <acc> and corresponding  <printf()>-style arguments,
+ *            reallocating as needed. 
+ *            For example, <esl_sq_FormatAccession(sq, "ACC%06d", i)>.
  * 
  *            A copy of <acc> is made, so if caller had <acc> allocated, 
  *            it is still responsible for freeing it.
@@ -786,11 +1064,9 @@ esl_sq_SetName(ESL_SQ *sq, const char *name, ...)
  * Returns:   <eslOK> on success.
  *
  * Throws:    <eslEMEM> on allocation error.
- *
- * Xref:      STL11/125
  */
 int
-esl_sq_SetAccession(ESL_SQ *sq, const char *acc, ...)
+esl_sq_FormatAccession(ESL_SQ *sq, const char *acc, ...)
 {
   va_list argp, argp2;
   int     n;
@@ -816,13 +1092,14 @@ esl_sq_SetAccession(ESL_SQ *sq, const char *acc, ...)
 }
 
 
-/* Function:  esl_sq_SetDesc()
- * Synopsis:  Format and set the description field in a sequence.
- * Incept:    SRE, Fri Jan 18 09:46:14 2008 [Westchester airport]
+/* Function:  esl_sq_FormatDesc()
+ * Synopsis:  Format the description field in a sequence, printf()-style.
+ * Incept:    SRE, Fri Sep 11 11:02:11 2009 [Janelia]
  *
- * Purpose:   Set the description of the sequence <sq> to <desc>, reallocating
- *            as needed. <desc> can be a <printf()>-style format with
- *            arguments; for example, <esl_sq_SetDesc(sq, "random sequence %d", i)>.
+ * Purpose:   Format the description of the sequence <sq> using <printf()>-style 
+ *            format string <desc> and corresponding  <printf()>-style arguments,
+ *            reallocating as needed. 
+ *            For example, <esl_sq_FormatDesc(sq, "random sequence %d", i)>.
  * 
  *            A copy of <desc> is made, so if caller had <desc> allocated, 
  *            it is still responsible for freeing it.
@@ -830,11 +1107,9 @@ esl_sq_SetAccession(ESL_SQ *sq, const char *acc, ...)
  * Returns:   <eslOK> on success.
  *
  * Throws:    <eslEMEM> on allocation error.
- *
- * Xref:      STL11/125
  */
 int
-esl_sq_SetDesc(ESL_SQ *sq, const char *desc, ...)
+esl_sq_FormatDesc(ESL_SQ *sq, const char *desc, ...)
 {
   va_list argp, argp2;
   int     n;
@@ -859,52 +1134,15 @@ esl_sq_SetDesc(ESL_SQ *sq, const char *desc, ...)
   return status;
 }
 
-/* Function:  esl_sq_AppendDesc()
- * Synopsis:  Append a new line to a growing multiline description.
- * Incept:    SRE, Thu May 22 15:33:43 2008 [Janelia]
- *
- * Purpose:   Append line <desc> to the description annotation
- *            in <sq>. 
- *            
- *            The annotation line <sq->desc> doesn't terminate in
- *            \verb+\n+; therefore, caller should make sure <desc> should
- *            not terminate in \verb+\n+. If <sq->desc> already contains
- *            one or more description lines, append \verb+\n+ before adding
- *            this next line \verb+\n+.
- *            
- * Returns:   <eslOK> on success.
- *
- * Throws:    <eslEMEM> on allocation failure.
- */
-int
-esl_sq_AppendDesc(ESL_SQ *sq, const char *desc)
-{
-  void *tmp;
-  int   dlen   = (sq->desc == NULL ? 0 : strlen(sq->desc));
-  int   newlen = (desc     == NULL ? 0 : strlen(desc));
-  int   status;
-  
-  if (dlen + newlen + 1 >= sq->dalloc) { /* +1 for appended \n */
-    ESL_RALLOC(sq->desc, tmp, sizeof(char) * (newlen+dlen+eslSQ_DESCCHUNK));
-    sq->dalloc = newlen+dlen+eslSQ_DESCCHUNK;
-  }
 
-  if (dlen > 0) sq->desc[dlen] = '\n';
-  strcpy(sq->desc + dlen + 1, desc);
-  return eslOK;
-  
- ERROR:
-  return status;
-}
-
-
-/* Function:  esl_sq_SetSource()
- * Synopsis:  Format and set the source name field in a sequence.
- * Incept:    SRE, Wed May  7 16:17:56 2008 [Janelia]
+/* Function:  esl_sq_FormatSource()
+ * Synopsis:  Format the source name field in a sequence, printf()-style.
+ * Incept:    SRE, Fri Sep 11 10:55:10 2009 [Janelia]
  *
- * Purpose:   Set the source of the sequence <sq> to <source>, reallocating
- *            as needed. <source> can be a <printf()>-style format with
- *            arguments; for example, <esl_sq_SetSource(sq, "source %d", i)>.
+ * Purpose:   Format the source of the sequence <sq> using <printf()>-style 
+ *            format string <source> and corresponding  <printf()>-style arguments,
+ *            reallocating as needed. 
+ *            For example, <esl_sq_FormatSource(sq, "source %d", i)>.
  * 
  *            A copy of <source> is made, so if caller had <source> allocated, 
  *            it is still responsible for freeing it.
@@ -912,11 +1150,9 @@ esl_sq_AppendDesc(ESL_SQ *sq, const char *desc)
  * Returns:   <eslOK> on success.
  *
  * Throws:    <eslEMEM> on allocation error.
- *
- * Xref:      STL11/125
  */
 int
-esl_sq_SetSource(ESL_SQ *sq, const char *source, ...)
+esl_sq_FormatSource(ESL_SQ *sq, const char *source, ...)
 {
   va_list argp, argp2;
   int     n;
@@ -942,6 +1178,48 @@ esl_sq_SetSource(ESL_SQ *sq, const char *source, ...)
 }
 
 
+/* Function:  esl_sq_AppendDesc()
+ * Synopsis:  Append a new line to a growing multiline description.
+ * Incept:    SRE, Thu May 22 15:33:43 2008 [Janelia]
+ *
+ * Purpose:   Append line <desc> to the description annotation line
+ *            in <sq>. 
+ *            
+ *            The annotation line <sq->desc> is a single line; it may
+ *            not contain \verb+\n+ newlines. Caller is responsible
+ *            for making sure <desc> does not terminate in \verb+\n+.
+ *            If <sq->desc> already contains a description
+ *            line (presumably because we're reading from a file format
+ *            that's split the description across multiple lines), 
+ *            append a space before adding this next line <desc>.
+ *            
+ * Returns:   <eslOK> on success.
+ *
+ * Throws:    <eslEMEM> on allocation failure.
+ */
+int
+esl_sq_AppendDesc(ESL_SQ *sq, const char *desc)
+{
+  void *tmp;
+  int   dlen   = (sq->desc == NULL ? 0 : strlen(sq->desc));
+  int   newlen = (desc     == NULL ? 0 : strlen(desc));
+  int   status;
+  
+  if (dlen + newlen + 1 >= sq->dalloc) { /* +1 for appended space */
+    ESL_RALLOC(sq->desc, tmp, sizeof(char) * (newlen+dlen+eslSQ_DESCCHUNK));
+    sq->dalloc = newlen+dlen+eslSQ_DESCCHUNK;
+  }
+
+  if (dlen > 0) { sq->desc[dlen] = ' '; dlen++; } 
+  strcpy(sq->desc + dlen, desc);
+  return eslOK;
+  
+ ERROR:
+  return status;
+}
+
+
+
 /* Function:  esl_sq_SetCoordComplete()
  * Synopsis:  Sets coords in a complete sequence.
  * Incept:    SRE, Tue May 13 09:25:33 2008 [Janelia]
@@ -1149,6 +1427,56 @@ esl_sq_ReverseComplement(ESL_SQ *sq)
   return status;
 }
 
+/* Function:  esl_sq_Checksum()
+ * Synopsis:  Calculate a 32-bit checksum for a sequence.
+ * Incept:    SRE, Tue Aug 25 14:32:17 2009 [Janelia]
+ *
+ * Purpose:   Calculate a 32-bit checksum for <sq>.
+ * 
+ *            Only the sequence data are considered, not name or other
+ *            annotation. For text mode sequences, the checksum is
+ *            case sensitive.  The checksum is also sensitive to
+ *            whether the sequence is text or digital mode; the same
+ *            sequence in will yield different checksums in digital
+ *            vs. text mode.
+ *            
+ * Returns:   <eslOK> on success; the checksum is in <*ret_checksum>.
+ */
+int
+esl_sq_Checksum(const ESL_SQ *sq, uint32_t *ret_checksum)
+{
+  uint32_t val = 0;
+  uint64_t pos;
+
+  if (sq->seq != NULL)
+    {
+      for (pos = 0; pos < sq->n; pos++)
+	{
+	  val += sq->seq[pos];
+	  val += (val << 10);
+	  val ^= (val >>  6);
+	}
+    }
+#ifdef eslAUGMENT_ALPHABET
+  else
+    {
+      for (pos = 1; pos <= sq->n; pos++)
+	{
+	  val += sq->dsq[pos];
+	  val += (val << 10);
+	  val ^= (val >>  6);
+	}
+    }
+#endif
+
+  val += (val <<  3);
+  val ^= (val >> 11);
+  val += (val << 15);
+
+  *ret_checksum = val;
+  return eslOK;
+}
+
 /*----------------------  end, other functions -------------------*/
 
 
@@ -1193,7 +1521,7 @@ esl_sq_ReverseComplement(ESL_SQ *sq)
 int
 esl_sq_GetFromMSA(const ESL_MSA *msa, int which, ESL_SQ *sq)
 {
-  char   *gapchars = "-_.";	/* hardcoded for now */
+  char   *gapchars = "-_.~";	/* hardcoded for now */
   char   *acc      = NULL;
   char   *desc     = NULL;
   char   *ss       = NULL;
@@ -1216,7 +1544,7 @@ esl_sq_GetFromMSA(const ESL_MSA *msa, int which, ESL_SQ *sq)
   if ((status = esl_sq_SetSource   (sq, msa->name))          != eslOK) goto ERROR;
   if ((status = esl_sq_GrowTo      (sq, msa->alen))          != eslOK) goto ERROR; /* can't be more than alen residues */
 
-  if (! msa->flags & eslMSA_DIGITAL) /* text mode to text mode */
+  if (! (msa->flags & eslMSA_DIGITAL)) /* text mode to text mode */
     {
       strcpy(sq->seq, msa->aseq[which]);
       if (ss != NULL) { 
@@ -1246,6 +1574,7 @@ esl_sq_GetFromMSA(const ESL_MSA *msa, int which, ESL_SQ *sq)
 
   sq->roff = -1;
   sq->doff = -1;
+  sq->hoff = -1;
   sq->eoff = -1;
   return eslOK;
 
@@ -1267,7 +1596,7 @@ esl_sq_GetFromMSA(const ESL_MSA *msa, int which, ESL_SQ *sq)
  * 
  *            The retrieved sequence is dealigned. For a text mode
  *            sequence, gap characters to be removed are assumed to be
- *            <-_.>. For a digital mode sequence, gap characters are
+ *            <-_.~>. For a digital mode sequence, gap characters are
  *            defined by the digital alphabet.
  *
  * Returns:   <eslOK> on success, and a pointer to the newly fetched
@@ -1284,7 +1613,7 @@ esl_sq_FetchFromMSA(const ESL_MSA *msa, int which, ESL_SQ **ret_sq)
   char   *acc      = NULL;
   char   *desc     = NULL;
   char   *ss       = NULL;
-  char   *gapchars = "-_.";	/* hardcoded for now; only affects text mode, not digital */
+  char   *gapchars = "-_.~";	/* hardcoded for now; only affects text mode, not digital */
   int     status;
 
   if (which >= msa->nseq || which < 0) return eslEOD;
@@ -1346,9 +1675,56 @@ sq_create(int do_digital)
 
   ESL_ALLOC(sq, sizeof(ESL_SQ));
 
+  if (sq_init(sq, do_digital) != eslOK) goto ERROR;
+
+  return sq;
+
+ ERROR:
+  esl_sq_Destroy(sq);
+  return NULL;
+}  
+
+/* Create an <ESL_SQ_BLOCK> object and its list of <ESL_SQ> objects */
+static ESL_SQ_BLOCK *
+sq_createblock(int count, int do_digital)
+{
+  int i = 0;
+
+  ESL_SQ_BLOCK *block = NULL;
+  int status = eslOK;
+
+  ESL_ALLOC(block, sizeof(ESL_SQ_BLOCK));
+
+  block->count = 0;
+  block->first_seqidx = -1;
+  block->list  = NULL;
+  block->complete = TRUE;
+
+  ESL_ALLOC(block->list, sizeof(ESL_SQ) * count);
+  block->listSize = count;
+
+  for (i = 0; i < count; ++i)
+    {
+      if (sq_init(block->list + i, do_digital) != eslOK) goto ERROR;
+    }
+
+  return block;
+
+ ERROR:
+  esl_sq_DestroyBlock(block);
+  return NULL;
+}  
+
+/* Initialize <ESL_SQ> object */
+static int
+sq_init(ESL_SQ *sq, int do_digital)
+{
+  int status;
+
   sq->name     = NULL;
   sq->acc      = NULL;
   sq->desc     = NULL;
+  sq->tax_id   = -1;
   sq->seq      = NULL;
   sq->dsq      = NULL;	
   sq->ss       = NULL;		/* Note that ss is optional - it will only be allocated if needed */
@@ -1368,11 +1744,10 @@ sq_create(int do_digital)
   else            ESL_ALLOC(sq->seq,  sizeof(char)    * sq->salloc);
 
   esl_sq_Reuse(sq);	/* initialization of sq->n, offsets, and strings */
-  return sq;
+  return eslOK;
 
  ERROR:
-  esl_sq_Destroy(sq);
-  return NULL;
+  return eslEMEM;
 }  
 
 /* CreateFrom and CreateDigitalFrom() are almost identical, so
@@ -1445,6 +1820,7 @@ sq_create_from(const char *name, const char *desc, const char *acc)
    * know it here.
    */
   sq->doff = -1;
+  sq->hoff = -1;
   sq->roff = -1;
   sq->eoff = -1;
   return sq;
@@ -1453,6 +1829,20 @@ sq_create_from(const char *name, const char *desc, const char *acc)
   esl_sq_Destroy(sq);
   return NULL;
 }
+
+/* Free <ESL_SQ> object */
+static void
+sq_free(ESL_SQ *sq)
+{
+  if (sq->name   != NULL)   free(sq->name);
+  if (sq->acc    != NULL)   free(sq->acc);
+  if (sq->desc   != NULL)   free(sq->desc);
+  if (sq->source != NULL)   free(sq->source);
+  if (sq->seq    != NULL)   free(sq->seq);
+  if (sq->dsq    != NULL)   free(sq->dsq);
+  if (sq->ss     != NULL)   free(sq->ss);
+}  
+
 /*----------------- end, internal functions ---------------------*/
 
 
@@ -1477,12 +1867,12 @@ utest_Create()
 
   if (sq1 == NULL || sq2 == NULL) esl_fatal(msg);
 
-  if (esl_sq_GrowTo(sq2, n)                                                 != eslOK) esl_fatal(msg);
-  if (esl_sq_SetName     (sq2, "%s%s", "seq", "name")                       != eslOK) esl_fatal(msg);
-  if (esl_sq_SetAccession(sq2, "%s%05d", "XX", 1)                           != eslOK) esl_fatal(msg);
-  if (esl_sq_SetDesc     (sq2, "%s %s %s", "test","sequence","description") != eslOK) esl_fatal(msg);
-  if (esl_sq_SetSource   (sq2, "%s", "source-unknown")                      != eslOK) esl_fatal(msg);
-  if (esl_strdup(ss, -1, &(sq2->ss))                                        != eslOK) esl_fatal(msg);
+  if (esl_sq_GrowTo(sq2, n)                                                    != eslOK) esl_fatal(msg);
+  if (esl_sq_FormatName     (sq2, "%s%s", "seq", "name")                       != eslOK) esl_fatal(msg);
+  if (esl_sq_FormatAccession(sq2, "%s%05d", "XX", 1)                           != eslOK) esl_fatal(msg);
+  if (esl_sq_FormatDesc     (sq2, "%s %s %s", "test","sequence","description") != eslOK) esl_fatal(msg);
+  if (esl_sq_FormatSource   (sq2, "%s", "source-unknown")                      != eslOK) esl_fatal(msg);
+  if (esl_strdup(ss, -1, &(sq2->ss))                                           != eslOK) esl_fatal(msg);
   strcpy(sq2->seq, seq);
   sq2->n = n;
 
@@ -1497,14 +1887,55 @@ utest_Create()
   esl_sq_Destroy(sq2);
 }
 
-/* This tests the Set() functions - 
- * in particular, the way they use vsnprintf().
- */
+/* This tests the Set() functions. */
 static void
 utest_Set(ESL_RANDOMNESS *r)
 {
   char   *msg     = "sqio Set unit test failure";
   ESL_SQ *sq      = esl_sq_Create();
+  int     ntrials = 8;
+  int     maxn    = eslSQ_NAMECHUNK*2;
+  int     maxa    = eslSQ_ACCCHUNK*2;
+  int     maxd    = eslSQ_DESCCHUNK*2;
+  int     n       = ESL_MAX( maxn, ESL_MAX(maxa, maxd));
+  char   *buf     = malloc(sizeof(char) * (n+1));
+  int64_t L;
+  int     i;
+
+  for (i = 0; i < ntrials; i++)
+    {
+      L = esl_rnd_Roll(r, maxn) + 1;
+      memset(buf, 'x', L);
+      buf[L] = '\0';
+      if (esl_sq_SetName(sq, buf) != eslOK) esl_fatal(msg);
+    }
+  for (i = 0; i < ntrials; i++)
+    {
+      L = esl_rnd_Roll(r, maxa) + 1;
+      memset(buf, 'x', L);
+      buf[L] = '\0';
+      if (esl_sq_SetAccession(sq, buf) != eslOK) esl_fatal(msg);
+    }      
+  for (i = 0; i < ntrials; i++)
+    {
+      L = esl_rnd_Roll(r, maxd) + 1;
+      memset(buf, 'x', L);
+      buf[L] = '\0';
+      if (esl_sq_SetDesc(sq, buf) != eslOK) esl_fatal(msg);
+    }      
+  free(buf);
+  esl_sq_Destroy(sq);
+} 
+
+
+/* This tests the Format() functions - 
+ * in particular, the way they use vsnprintf().
+ */
+static void
+utest_Format(ESL_RANDOMNESS *r)
+{
+  char   *msg     = "esl_sq_Format*() unit test failure";
+  ESL_SQ *sq      = esl_sq_Create();
   int     ntrials = 128;
   int     maxn    = eslSQ_NAMECHUNK*2;
   int     maxa    = eslSQ_ACCCHUNK*2;
@@ -1519,21 +1950,21 @@ utest_Set(ESL_RANDOMNESS *r)
       L = esl_rnd_Roll(r, maxn) + 1;
       memset(buf, 'x', L);
       buf[L] = '\0';
-      if (esl_sq_SetName(sq, "%s%d", buf, i) != eslOK) esl_fatal(msg);
+      if (esl_sq_FormatName(sq, "%s%d", buf, i) != eslOK) esl_fatal(msg);
     }
   for (i = 0; i < ntrials; i++)
     {
       L = esl_rnd_Roll(r, maxa) + 1;
       memset(buf, 'x', L);
       buf[L] = '\0';
-      if (esl_sq_SetAccession(sq, "%s%d", buf, i) != eslOK) esl_fatal(msg);
+      if (esl_sq_FormatAccession(sq, "%s%d", buf, i) != eslOK) esl_fatal(msg);
     }      
   for (i = 0; i < ntrials; i++)
     {
       L = esl_rnd_Roll(r, maxd) + 1;
       memset(buf, 'x', L);
       buf[L] = '\0';
-      if (esl_sq_SetDesc(sq, "%s%d", buf, i) != eslOK) esl_fatal(msg);
+      if (esl_sq_FormatDesc(sq, "%s%d", buf, i) != eslOK) esl_fatal(msg);
     }      
   free(buf);
   esl_sq_Destroy(sq);
@@ -1560,13 +1991,13 @@ utest_CreateDigital()
   if (esl_abc_CreateDsq(abc, seq, &dsq)                                     != eslOK) esl_fatal(msg);
   if ((sq1 = esl_sq_CreateDigitalFrom(abc, name, dsq, n, desc, acc, ss))    == NULL)  esl_fatal(msg);
 
-  if ((sq2 = esl_sq_CreateDigital(abc))                                     == NULL)  esl_fatal(msg);
-  if (esl_sq_GrowTo(sq2, n)                                                 != eslOK) esl_fatal(msg);
-  if (esl_sq_SetName     (sq2, "%s%s", "seq", "name")                       != eslOK) esl_fatal(msg);
-  if (esl_sq_SetAccession(sq2, "%s%05d", "XX", 1)                           != eslOK) esl_fatal(msg);
-  if (esl_sq_SetDesc     (sq2, "%s %s %s", "test","sequence","description") != eslOK) esl_fatal(msg);
-  if (esl_sq_SetSource   (sq2, "%s", "source-unknown")                      != eslOK) esl_fatal(msg);
-  if ((sq2->ss    = malloc(sizeof(char) * (n+2)))                           == NULL)  esl_fatal(msg);
+  if ((sq2 = esl_sq_CreateDigital(abc))                                        == NULL)  esl_fatal(msg);
+  if (esl_sq_GrowTo(sq2, n)                                                    != eslOK) esl_fatal(msg);
+  if (esl_sq_FormatName     (sq2, "%s%s", "seq", "name")                       != eslOK) esl_fatal(msg);
+  if (esl_sq_FormatAccession(sq2, "%s%05d", "XX", 1)                           != eslOK) esl_fatal(msg);
+  if (esl_sq_FormatDesc     (sq2, "%s %s %s", "test","sequence","description") != eslOK) esl_fatal(msg);
+  if (esl_sq_FormatSource   (sq2, "%s", "source-unknown")                      != eslOK) esl_fatal(msg);
+  if ((sq2->ss    = malloc(sizeof(char) * (n+2)))                              == NULL)  esl_fatal(msg);
   strcpy(sq2->ss+1, ss);   sq2->ss[0] = '\0';
   if (esl_abc_Digitize(abc, seq, sq2->dsq)                                  != eslOK) esl_fatal(msg);
   sq2->n = n;
@@ -1602,8 +2033,8 @@ utest_CreateDigital()
  * 6. Test driver.
  *****************************************************************/
 #ifdef eslSQ_TESTDRIVE
-/* gcc -g -Wall -o sq_utest -I. -L. -DeslSQ_TESTDRIVE esl_sq.c -leasel -lm
- * ./sq_utest
+/* gcc -g -Wall -o esl_sq_utest -I. -L. -DeslSQ_TESTDRIVE esl_sq.c -leasel -lm
+ * ./esl_sq_utest
  */
 #include "esl_config.h"
 
@@ -1620,7 +2051,6 @@ utest_CreateDigital()
 static ESL_OPTIONS options[] = {
   /* name           type      default  env  range toggles reqs incomp  help                                       docgroup*/
   { "-h",        eslARG_NONE,   FALSE,  NULL, NULL,  NULL,  NULL, NULL, "show brief help on version and usage",             0 },
-  { "-r",        eslARG_NONE,   NULL,   NULL, NULL,  NULL,  NULL, NULL, "use arbitrary random number seed",                 0 },
   { "-s",        eslARG_INT,     "42",  NULL, NULL,  NULL,  NULL, NULL, "set random number seed to <n>",                    0 },
  {  0, 0, 0, 0, 0, 0, 0, 0, 0, 0 },
 };
@@ -1631,13 +2061,11 @@ int
 main(int argc, char **argv)
 {
   ESL_GETOPTS    *go      = esl_getopts_CreateDefaultApp(options, 0, argc, argv, banner, usage);
-  ESL_RANDOMNESS *r       = NULL;
-
-  if (esl_opt_GetBoolean(go, "-r")) r = esl_randomness_CreateTimeseeded();
-  else                              r = esl_randomness_Create(esl_opt_GetInteger(go, "-s"));
+  ESL_RANDOMNESS *r       = esl_randomness_Create(esl_opt_GetInteger(go, "-s"));
 
   utest_Create();
   utest_Set(r);
+  utest_Format(r);
 
 #ifdef eslAUGMENT_ALPHABET
   utest_CreateDigital();
@@ -1683,10 +2111,10 @@ int main(void)
   
   /* Building up a ESL_SQ yourself: */
   sq2 = esl_sq_Create();
-  esl_sq_SetName     (sq2, "seq%d", 1);
-  esl_sq_SetAccession(sq2, "XX%05d", 1);
-  esl_sq_SetDesc     (sq2, "This %s a test", "is");
-  esl_sq_GrowTo      (sq2, n);
+  esl_sq_FormatName     (sq2, "seq%d", 1);
+  esl_sq_FormatAccession(sq2, "XX%05d", 1);
+  esl_sq_FormatDesc     (sq2, "This %s a test", "is");
+  esl_sq_GrowTo         (sq2, n);
   strcpy(sq2->seq, testseq);
   esl_strdup(ss, -1, &(sq2->ss));  
   sq2->n = n;
@@ -1744,10 +2172,10 @@ int main(void)
   
   /* Building up a digital ESL_SQ yourself: */
   sq2 = esl_sq_CreateDigital(abc);
-  esl_sq_SetName     (sq2, "seq%d", 1);
-  esl_sq_SetAccession(sq2, "XX%05d", 1);
-  esl_sq_SetDesc     (sq2, "This %s a test", "is");
-  esl_sq_GrowTo      (sq2, n);
+  esl_sq_FormatName     (sq2, "seq%d", 1);
+  esl_sq_FormatAccession(sq2, "XX%05d", 1);
+  esl_sq_FormatDesc     (sq2, "This %s a test", "is");
+  esl_sq_GrowTo         (sq2, n);
   esl_abc_Digitize(abc, testseq, sq2->dsq);
   sq2->n = n;
 
@@ -1778,7 +2206,14 @@ int main(void)
 /*------------------ end, example drivers ------------------------*/
 
 /*****************************************************************
- *    This copyrighted source code is freely distributed 
- *    under the terms of the GNU General Public License. See
- *    the files COPYRIGHT and LICENSE for details.
+ * Easel - a library of C functions for biological sequence analysis
+ * Version i1.1.1; July 2014
+ * Copyright (C) 2014 HHMI Janelia Farm Research Campus
+ * Other copyrights also apply. See the COPYRIGHT file for a full list.
+ * 
+ * Easel is distributed under the Janelia Farm Software License, a BSD
+ * license. See the LICENSE file for more details.
+ *
+ * SVN $Id: esl_sq.c 741 2012-02-01 15:26:09Z eddys $
+ * SVN $URL: https://svn.janelia.org/eddylab/eddys/easel/branches/infernal/1.1/esl_sq.c $
  *****************************************************************/
