@@ -1,10 +1,11 @@
 /* A sequence.
  * 
  * SRE, Mon Mar 31 17:03:51 2008 [Janelia]
- * SVN $Id: esl_sq.h 255 2008-05-30 17:49:15Z eddys $
+ * SVN $Id: esl_sq.h 699 2011-06-16 18:19:23Z wheelert $
+ * SVN $URL: https://svn.janelia.org/eddylab/eddys/easel/branches/infernal/1.1/esl_sq.h $
  */
-#ifndef ESL_SQ_INCLUDED
-#define ESL_SQ_INCLUDED
+#ifndef eslSQ_INCLUDED
+#define eslSQ_INCLUDED
 
 #ifdef eslAUGMENT_ALPHABET
 #include "esl_alphabet.h"
@@ -13,6 +14,7 @@
 #include "esl_msa.h"
 #endif
 
+
 /* ESL_SQ - a biosequence
  * 
  * Can be either in text mode <seq>, or digital mode <dsq>. 
@@ -77,12 +79,18 @@
  *    for example, so we don't have to read entire huge seqs into
  *    memory just to calculate their lengths for the index.
  *    
+ * Note/TODO: use of "\0" empty string to indicate lack of optional
+ * acc, desc info is now deprecated. Cannot distinguish empty string
+ * from lack of annotation. Should use NULL ptr instead. Fix this in
+ * future.  (21 Nov 09 xref J5/114)
+ *    
  */
 typedef struct {
   /*::cexcerpt::sq_sq::begin::*/
   char    *name;           /* name; one word, no whitespace ("\0" if no name)  */
   char    *acc;            /* optional accession (1 word) ("\0" if none)       */
   char    *desc;           /* description line ("\0" if no description)        */
+  int32_t  tax_id;	   /* NCBI taxonomy id (-1 if none)                    */
   char    *seq;            /* sequence [0..n-1], or NULL if digital            */
   ESL_DSQ *dsq;            /* digitized sequence [1..n], or NULL if text       */
   char    *ss;             /* optional sec structure [0..n-1], [1..n], or NULL */
@@ -100,7 +108,7 @@ typedef struct {
   /* In all the above bookkeeping, a -1 means "unknown" */
   char    *source; /* name of the source of a subseq/window; or MSA name; or ""*/
   
-  /* Memory allocation bookkeeping:                                            */
+  /* Memory allocation bookkeeping:  (all inclusive of \0;  >= strlen()+1)     */
   int      nalloc;         /* allocated length of name                         */
   int      aalloc;         /* allocated length of accession                    */
   int      dalloc;         /* allocated length of description                  */
@@ -108,7 +116,9 @@ typedef struct {
   int      srcalloc;	   /* allocated length for source name                 */
 
   /* Disk offset bookkeeping:                                                  */
+  int64_t  idx;	           /* ctr for which # seq this is; -1 if not counting  */
   off_t    roff;	   /* record offset (start of record); -1 if none      */
+  off_t    hoff;	   /* offset to last byte of header; -1 if unknown     */
   off_t    doff;	   /* data offset (start of sequence data); -1 if none */
   off_t    eoff;	   /* offset to last byte of record; -1 if unknown     */
 
@@ -120,7 +130,13 @@ typedef struct {
 #endif
 } ESL_SQ;
 
-
+typedef struct {
+  int      count;       /* number of <ESL_SQ> objects in the block */
+  int      listSize;    /* maximum number elements in the list     */
+  int      complete;    /*TRUE if the the final ESL_SQ element on the block is complete, FALSE if it's only a partial winow of the full sequence*/
+  int64_t  first_seqidx;/*unique identifier of the first ESL_SQ object on list;  the seqidx of the i'th entry on list is first_seqidx+i */
+  ESL_SQ  *list;        /* array of <ESL_SQ> objects               */
+} ESL_SQ_BLOCK;
 
 /* These control default initial allocation sizes in an ESL_SQ.     */
 #define eslSQ_NAMECHUNK   32	/* allocation unit for name, source */
@@ -135,18 +151,25 @@ extern ESL_SQ *esl_sq_CreateFrom(const char *name, const char *seq,
 extern int     esl_sq_Grow  (ESL_SQ *sq, int64_t *ret_nsafe);
 extern int     esl_sq_GrowTo(ESL_SQ *sq, int64_t  n);
 extern int     esl_sq_Copy(const ESL_SQ *src, ESL_SQ *dst);
-extern int     esl_sq_Compare(ESL_SQ *sq1, ESL_SQ *sq2);
-extern int     esl_sq_Reuse  (ESL_SQ *sq);
-extern void    esl_sq_Destroy(ESL_SQ *sq);
+extern int     esl_sq_Compare  (ESL_SQ *sq1, ESL_SQ *sq2);
+extern int     esl_sq_Reuse    (ESL_SQ *sq);
+extern int     esl_sq_IsDigital(const ESL_SQ *sq);
+extern int     esl_sq_IsText   (const ESL_SQ *sq);
+extern void    esl_sq_Destroy  (ESL_SQ *sq);
 
-extern int     esl_sq_SetName     (ESL_SQ *sq, const char *name,   ...);
-extern int     esl_sq_SetAccession(ESL_SQ *sq, const char *acc,    ...);
-extern int     esl_sq_SetDesc     (ESL_SQ *sq, const char *desc,   ...);
-extern int     esl_sq_AppendDesc  (ESL_SQ *sq, const char *desc);
-extern int     esl_sq_SetSource   (ESL_SQ *sq, const char *source, ...);
+extern int     esl_sq_SetName        (ESL_SQ *sq, const char *name);
+extern int     esl_sq_SetAccession   (ESL_SQ *sq, const char *acc);
+extern int     esl_sq_SetDesc        (ESL_SQ *sq, const char *desc);
+extern int     esl_sq_SetSource      (ESL_SQ *sq, const char *source);
+extern int     esl_sq_FormatName     (ESL_SQ *sq, const char *name,   ...);
+extern int     esl_sq_FormatAccession(ESL_SQ *sq, const char *acc,    ...);
+extern int     esl_sq_FormatDesc     (ESL_SQ *sq, const char *desc,   ...);
+extern int     esl_sq_FormatSource   (ESL_SQ *sq, const char *source, ...);
+extern int     esl_sq_AppendDesc     (ESL_SQ *sq, const char *desc);
 extern int     esl_sq_SetCoordComplete(ESL_SQ *sq, int64_t L);
 extern int     esl_sq_CAddResidue (ESL_SQ *sq, char c);
 extern int     esl_sq_ReverseComplement(ESL_SQ *sq);
+extern int     esl_sq_Checksum(const ESL_SQ *sq, uint32_t *ret_checksum);
 
 #ifdef eslAUGMENT_ALPHABET
 extern ESL_SQ *esl_sq_CreateDigital(const ESL_ALPHABET *abc);
@@ -156,6 +179,7 @@ extern int     esl_sq_Digitize(const ESL_ALPHABET *abc, ESL_SQ *sq);
 extern int     esl_sq_Textize(ESL_SQ *sq);
 extern int     esl_sq_GuessAlphabet(ESL_SQ *sq, int *ret_type);
 extern int     esl_sq_XAddResidue(ESL_SQ *sq, ESL_DSQ x);
+extern int     esl_sq_ConvertDegen2X(ESL_SQ *sq);
 #endif
 
 #ifdef eslAUGMENT_MSA
@@ -163,9 +187,19 @@ extern int     esl_sq_GetFromMSA  (const ESL_MSA *msa, int which, ESL_SQ *sq);
 extern int     esl_sq_FetchFromMSA(const ESL_MSA *msa, int which, ESL_SQ **ret_sq);
 #endif
 
-#endif /*!ESL_SQ_INCLUDED*/
+extern ESL_SQ_BLOCK *esl_sq_CreateBlock(int count);
+#ifdef eslAUGMENT_ALPHABET
+extern ESL_SQ_BLOCK *esl_sq_CreateDigitalBlock(int count, const ESL_ALPHABET *abc);
+#endif
+extern void          esl_sq_DestroyBlock(ESL_SQ_BLOCK *sqBlock);
+
+#endif /*eslSQ_INCLUDED*/
 /*****************************************************************
- *    This copyrighted source code is freely distributed 
- *    under the terms of the GNU General Public License. See
- *    the files COPYRIGHT and LICENSE for details.
+ * Easel - a library of C functions for biological sequence analysis
+ * Version i1.1.1; July 2014
+ * Copyright (C) 2014 HHMI Janelia Farm Research Campus
+ * Other copyrights also apply. See the COPYRIGHT file for a full list.
+ * 
+ * Easel is distributed under the Janelia Farm Software License, a BSD
+ * license. See the LICENSE file for more details.
  *****************************************************************/
