@@ -2,15 +2,14 @@
  * 
  * Table of contents           
  *     1. SIMD logf(), expf()
- *     2. Miscellaneous convenience functions.
+ *     2. Utilities for ps vectors (4 floats in a __m128)
+ *     3. Utilities for epu8 vectors (16 uchars in a __m128i)
  *     3. Benchmark
  *     4. Unit tests
  *     5. Test driver
  *     6. Example
  *     7. Copyright and license
  *     
- * SRE, Sun Dec 16 09:14:51 2007 [Janelia]    
- * SVN $Id: esl_sse.c 269 2008-06-19 13:47:41Z eddys $
  *****************************************************************
  * Credits:
  *
@@ -89,13 +88,13 @@ esl_sse_logf(__m128 x)
   __m128  z;
 
   /* first, split x apart: x = frexpf(x, &e); */
-  ei           = _mm_srli_epi32((__m128i) x, 23);	                             /* shift right 23: IEEE754 floats: ei = biased exponents     */
-  invalid_mask = (__m128) _mm_cmpeq_epi32( _mm_and_si128((__m128i) x, vneg), vneg);  /* mask any elem that's negative; these become NaN           */
-  zero_mask    = (__m128) _mm_cmpeq_epi32(ei, _mm_setzero_si128());                  /* mask any elem zero or subnormal; these become -inf        */
-  inf_mask     = (__m128) _mm_cmpeq_epi32( _mm_and_si128((__m128i) x, vexp), vexp);  /* mask any elem inf or NaN; log(inf)=inf, log(NaN)=NaN      */
-  origx        = x;			                                             /* store original x, used for log(inf) = inf, log(NaN) = NaN */
+  ei           = _mm_srli_epi32( _mm_castps_si128(x), 23);	                                        /* shift right 23: IEEE754 floats: ei = biased exponents     */
+  invalid_mask = _mm_castsi128_ps ( _mm_cmpeq_epi32( _mm_and_si128(_mm_castps_si128(x), vneg), vneg));  /* mask any elem that's negative; these become NaN           */
+  zero_mask    = _mm_castsi128_ps ( _mm_cmpeq_epi32(ei, _mm_setzero_si128()));                          /* mask any elem zero or subnormal; these become -inf        */
+  inf_mask     = _mm_castsi128_ps ( _mm_cmpeq_epi32( _mm_and_si128(_mm_castps_si128(x), vexp), vexp));  /* mask any elem inf or NaN; log(inf)=inf, log(NaN)=NaN      */
+  origx        = x;			                                                                /* store original x, used for log(inf) = inf, log(NaN) = NaN */
 
-  x  = _mm_and_ps(x, (__m128) _mm_set1_epi32(~0x7f800000));          /* x now the stored 23 bits of the 24-bit significand        */
+  x  = _mm_and_ps(x, _mm_castsi128_ps(_mm_set1_epi32(~0x7f800000))); /* x now the stored 23 bits of the 24-bit significand        */
   x  = _mm_or_ps (x, v0p5);                                          /* sets hidden bit b[0]                                      */
 
   ei = _mm_sub_epi32(ei, _mm_set1_epi32(126));                       /* -127 (ei now signed base-2 exponent); then +1             */
@@ -142,14 +141,42 @@ esl_sse_logf(__m128 x)
  * Incept:    SRE, Fri Dec 14 14:46:27 2007 [Janelia]
  *
  * Purpose:   Given a vector <x> containing four floats, returns a
- *            vector <r> in which each element <r[z] = logf(x[z])>.
+ *            vector <r> in which each element <r[z] = expf(x[z])>.
  *            
  *            Valid for all IEEE754 floats $x_z$.
  *            
  * Xref:      J2/71
+ *            J10/62: bugfix, minlogf/maxlogf range was too wide; 
+ *                    (k+127) must be >=0 and <=255, so (k+127)<<23
+ *                    is a valid IEEE754 float, without touching 
+ *                    the sign bit. Pommier had this right in the
+ *                    first place, and I didn't understand.
  * 
  * Note:      Derived from an SSE1 implementation by Julian
  *            Pommier. Converted to SSE2.
+ *            
+ *            Note on maxlogf/minlogf, which are close to but not
+ *            exactly 127.5/log2 [J10/63]. We need -127<=k<=128, so
+ *            k+127 is 0..255, a valid IEEE754 8-bit exponent
+ *            (0..255), so the bit pattern (k+127)<<23 is IEEE754
+ *            single-precision for 2^k.  If k=-127, we get IEEE754 0.
+ *            If k=128, we get IEEE754 +inf.  If k<-127, k+127 is
+ *            negative and we get screwed up.  If k>128, k+127
+ *            overflows the 8-bit exponent and sets the sign bit.  So
+ *            for x' (base 2) < -127.5 we must definitely return e^x ~
+ *            0; for x' < 126.5 we're going to calculate 0 anyway
+ *            (because k=floor(-126.5-epsilon+0.5) = -127).  So any
+ *            minlogf between -126.5 log2 ... -127.5 log2 will suffice
+ *            as the cutoff. Ditto for 126.5 log2 .. 127.5log2.
+ *            That's 87.68312 .. 88.3762655.  I think Pommier's
+ *            thinking is, you don't want to get to close to the
+ *            edges, lest fp roundoff error screw you (he may have
+ *            consider 1 ulp carefully, I can't tell), but otherwise
+ *            you may as well put your bounds close to the outer edge;
+ *            so 
+ *              maxlogf =  127.5 log(2) - epsilon 
+ *              minlogf = -127.5 log(2) + epsilon 
+ *            for an epsilon that happen to be ~ 3e-6.
  */
 __m128 
 esl_sse_expf(__m128 x) 
@@ -157,8 +184,8 @@ esl_sse_expf(__m128 x)
   static float cephes_p[6] = { 1.9875691500E-4f, 1.3981999507E-3f, 8.3334519073E-3f, 
 			       4.1665795894E-2f, 1.6666665459E-1f, 5.0000001201E-1f };
   static float cephes_c[2] = { 0.693359375f,    -2.12194440e-4f };
-  static float maxlogf     =   88.72283905206835;  /* log(2^128)  */
-  static float minlogf     = -103.27892990343185;  /* log(2^-149) */
+  static float maxlogf     =  88.3762626647949f;  /* 127.5 log(2) - epsilon. above this, 0.5+x/log2 gives k>128 and breaks 2^k "float" construction, because (k+127)<<23 must be a valid IEEE754 exponent 0..255 */
+  static float minlogf     = -88.3762626647949f;  /*-127.5 log(2) + epsilon. below this, 0.5+x/log2 gives k<-127 and breaks 2^k, see above */
   __m128i k;
   __m128  mask, tmp, fx, z, y, minmask, maxmask;
   
@@ -197,7 +224,7 @@ esl_sse_expf(__m128 x)
   /* build 2^k by hand, by creating a IEEE754 float */
   k  = _mm_add_epi32(k, _mm_set1_epi32(127));
   k  = _mm_slli_epi32(k, 23);
-  fx = (__m128) k;
+  fx = _mm_castsi128_ps(k);
   
   /* put 2^k e^f together (fx = 2^k,  y = e^f) and we're done */
   y = _mm_mul_ps(y, fx);	
@@ -210,37 +237,9 @@ esl_sse_expf(__m128 x)
 
 
 /*****************************************************************
- * 2. Miscellaneous convenience functions
+ * 2. Utilities for ps vectors (4 floats in a __m128)
  *****************************************************************/ 
 
-/* Function:  esl_sse_select_ps()
- * Synopsis:  SSE equivalent of <vec_sel()>
- * Incept:    SRE, Sun Dec 16 12:39:03 2007 [Janelia]
- *
- * Purpose:   Vector select. Returns a vector <r[z] = a[z]> where <mask[z]>
- *            is all 0's; <r[z] = b[z]> where <mask[z]> is all 1's.
- *            
- *            Useful for avoiding conditional branches. For example,
- *            to implement \ccode{if (a > 0) a += a;}:
- *            
- *            \begin{cchunk}
- *              mask = _mm_cmpgt_ps(a, _mm_setzero_ps());
- *              twoa = _mm_add_ps(a, a);
- *              a    = esl_sse_select_ps(a, twoa, mask);
- *            \end{cchunk}
- *
- * Notes:     As recommended by the Altivec/SSE Migration Guide,
- *            Apple Computer, Inc.
- */
-__m128
-esl_sse_select_ps(__m128 a, __m128 b, __m128 mask)
-{
-  b = _mm_and_ps(b, mask);
-  a = _mm_andnot_ps(mask, a);
-  return _mm_or_ps(a,b);
-}
-
-
 void
 esl_sse_dump_ps(FILE *fp, __m128 v)
 {
@@ -249,6 +248,8 @@ esl_sse_dump_ps(FILE *fp, __m128 v)
 }
 
 
+
+
 /*****************************************************************
  * 3. Benchmark
  *****************************************************************/
@@ -298,8 +299,9 @@ main(int argc, char **argv)
   esl_stopwatch_Display(stdout, w, "# vector CPU time: ");
 
   /* If you don't do something with x and xv, the compiler may optimize them away */
-  printf("%g  => many scalar logf,expf cycles => %g\n", origx, N, x);
-  printf("%g  => many vector logf,expf cycles => ", origx, N); esl_sse_dump_ps(stdout, xv); printf("\n");
+  printf("%g  => many scalar logf,expf cycles => %g\n", origx, x);
+  printf("%g  => many vector logf,expf cycles => ", origx);
+  esl_sse_dump_ps(stdout, xv); printf("\n");
 
   esl_stopwatch_Destroy(w);
   esl_getopts_Destroy(go);
@@ -384,6 +386,30 @@ utest_expf(ESL_GETOPTS *go)
   if (! isinf(r.x[1]))  esl_fatal("expf(large x)  should be inf");
   if (r.x[2] != 0.0f)   esl_fatal("expf(-large x) should be 0");
 
+  /* Make sure we are correct around the problematic ~minlogf boundary:
+   *  (1) e^x for x < -127.5 log2 + epsilon is 0, because that's our minlogf barrier.
+   *  (2) e^x for  -127.5 log2 < x < -126.5 log2 is 0 too, but is actually calculated
+   *  (3) e^x for  -126.5 log2 < x should be finite (and close to FLT_MIN)
+   *
+   *  minlogf = -127.5 log(2) + epsilon = -88.3762626647949;
+   *        and -126.5 log(2)           = -87.68311834
+   *  so for
+   *     (1): expf(-88.3763)  => 0
+   *     (2): expf(-88.3762)  => 0
+   *     (3): expf(-87.6832)   => 0
+   *     (4): expf(-87.6831)   => <FLT_MIN (subnormal) : ~8.31e-39 (may become 0 in flush-to-zero mode for subnormals)
+   */
+  x   = _mm_set_ps(-88.3763, -88.3762, -87.6832, -87.6831);
+  r.v = esl_sse_expf(x); 
+  if (esl_opt_GetBoolean(go, "-v")) {
+    printf("expf");
+    esl_sse_dump_ps(stdout, x);    printf(" ==> ");
+    esl_sse_dump_ps(stdout, r.v);  printf("\n");
+  }
+  if ( r.x[0] >= FLT_MIN) esl_fatal("expf( -126.5 log2 + eps) should be around FLT_MIN");
+  if ( r.x[1] != 0.0f)    esl_fatal("expf( -126.5 log2 - eps) should be 0.0 (by calculation)");
+  if ( r.x[2] != 0.0f)    esl_fatal("expf( -127.5 log2 + eps) should be 0.0 (by calculation)");
+  if ( r.x[3] != 0.0f)    esl_fatal("expf( -127.5 log2 - eps) should be 0.0 (by min bound): %g", r.x[0]);
 }
 
 /* utest_odds():  test accuracy of logf, expf on odds ratios,
@@ -412,7 +438,7 @@ utest_odds(ESL_GETOPTS *go, ESL_RANDOMNESS *r)
       if (odds == 0.0) esl_fatal("whoa, odds ratio can't be 0!\n");
 
       r1.v      = esl_sse_logf(_mm_set1_ps(odds));  /* r1.x[z] = log(p1/p2) */
-      scalar_r1 = logf(odds);
+      scalar_r1 = log(odds);
 
       err1       = (r1.x[0] == 0. && scalar_r1 == 0.) ? 0.0 : 2 * fabs(r1.x[0] - scalar_r1) / fabs(r1.x[0] + scalar_r1);
       if (err1 > maxerr1) maxerr1 = err1;
@@ -420,7 +446,7 @@ utest_odds(ESL_GETOPTS *go, ESL_RANDOMNESS *r)
       if (isnan(avgerr1)) esl_fatal("whoa, what?\n");
 
       r2.v      = esl_sse_expf(r1.v);        /* and back to odds */
-      scalar_r2 = expf(r1.x[0]);
+      scalar_r2 = exp(r1.x[0]);
 
       err2       = (r2.x[0] == 0. && scalar_r2 == 0.) ? 0.0 : 2 * fabs(r2.x[0] - scalar_r2) / fabs(r2.x[0] + scalar_r2);
       if (err2 > maxerr2) maxerr2 = err2;
@@ -430,16 +456,16 @@ utest_odds(ESL_GETOPTS *go, ESL_RANDOMNESS *r)
 	printf("%13.7g  %13.7g  %13.7g  %13.7g  %13.7g  %13.7g  %13.7g\n", odds, scalar_r1, r1.x[0], scalar_r2, r2.x[0], err1, err2);
     }
 
+  if (verbose) {
+    printf("Average [max] logf() relative error in %d odds trials:  %13.8g  [%13.8g]\n", N, avgerr1, maxerr1);
+    printf("Average [max] expf() relative error in %d odds trials:  %13.8g  [%13.8g]\n", N, avgerr2, maxerr2);
+    printf("(random seed : %" PRIu32 ")\n", esl_randomness_GetSeed(r));
+  }
+
   if (avgerr1 > 1e-8) esl_fatal("average error on logf() is intolerable\n");
   if (maxerr1 > 1e-6) esl_fatal("maximum error on logf() is intolerable\n");
   if (avgerr2 > 1e-8) esl_fatal("average error on expf() is intolerable\n");
   if (maxerr2 > 1e-6) esl_fatal("maximum error on expf() is intolerable\n");
-
-  if (verbose) {
-    printf("Average [max] logf() relative error in %d odds trials:  %13.8g  [%13.8g]\n", N, avgerr1, maxerr1);
-    printf("Average [max] expf() relative error in %d odds trials:  %13.8g  [%13.8g]\n", N, avgerr2, maxerr2);
-    printf("(random seed : %ld)\n", esl_randomness_GetSeed(r));
-  }
 }
 #endif /*eslSSE_TESTDRIVE*/
 
@@ -467,7 +493,6 @@ static ESL_OPTIONS options[] = {
   /* name           type      default  env  range toggles reqs incomp  help                                       docgroup*/
   { "-h",        eslARG_NONE,   FALSE,  NULL, NULL,  NULL,  NULL, NULL, "show brief help on version and usage",             0 },
   { "-N",        eslARG_INT,  "10000",  NULL, NULL,  NULL,  NULL, NULL, "number of random test points",                     0 },
-  { "-r",        eslARG_NONE,   NULL,   NULL, NULL,  NULL,  NULL, NULL, "use arbitrary random number seed",                 0 },
   { "-s",        eslARG_INT,     "42",  NULL, NULL,  NULL,  NULL, NULL, "set random number seed to <n>",                    0 },
   { "-v",        eslARG_NONE,   FALSE,  NULL, NULL,  NULL,  NULL, NULL, "be verbose: show test report",                     0 },
   { "--vv",      eslARG_NONE,   FALSE,  NULL, NULL,  NULL,  NULL, NULL, "be very verbose: show individual test samples",    0 },
@@ -479,11 +504,8 @@ static char banner[] = "test driver for sse module";
 int
 main(int argc, char **argv)
 {
-  ESL_GETOPTS    *go      = esl_getopts_CreateDefaultApp(options, 0, argc, argv, banner, usage);
-  ESL_RANDOMNESS *r;
-
-  if (esl_opt_GetBoolean(go, "-r")) r = esl_randomness_CreateTimeseeded();
-  else                              r = esl_randomness_Create(esl_opt_GetInteger(go, "-s"));
+  ESL_GETOPTS    *go = esl_getopts_CreateDefaultApp(options, 0, argc, argv, banner, usage);
+  ESL_RANDOMNESS *r  = esl_randomness_Create(esl_opt_GetInteger(go, "-s"));;
 
   utest_logf(go);
   utest_expf(go);
@@ -533,16 +555,19 @@ main(int argc, char **argv)
 }
 /*::cexcerpt::sse_example::end::*/
 #endif /*eslSSE_EXAMPLE*/
+
+
 #else /* ! HAVE_SSE2*/
 
-/* The remainder of the file is just bookkeeping, for what to do when
- * we aren't compiling with SSE instructions.
+/* If we don't have SSE2 compiled in, provide some nothingness to:
+ *   a. prevent Mac OS/X ranlib from bitching about .o file that "has no symbols" 
+ *   b. prevent compiler from bitching about "empty compilation unit"
+ *   c. automatically pass the automated tests.
  */
+#include "easel.h"
 
-/*
- * Provide a successful unit test on platforms where we don't have SSE instructions.
- */
-#ifdef eslSSE_TESTDRIVE
+void esl_sse_DoAbsolutelyNothing(void) { return; }
+#if defined eslSSE_TESTDRIVE || eslSSE_EXAMPLE || eslSSE_BENCHMARK
 int main(void) { return 0; }
 #endif
 
@@ -550,9 +575,16 @@ int main(void) { return 0; }
 
 
 /*****************************************************************
- *    This copyrighted source code is freely distributed 
- *    under the terms of the GNU General Public License. See
- *    the files COPYRIGHT and LICENSE for details.
+ * Easel - a library of C functions for biological sequence analysis
+ * Version i1.1.1; July 2014
+ * Copyright (C) 2014 HHMI Janelia Farm Research Campus
+ * Other copyrights also apply. See the COPYRIGHT file for a full list.
+ * 
+ * Easel is distributed under the Janelia Farm Software License, a BSD
+ * license. See the LICENSE file for more details.
+ * 
+ * SVN $Id: esl_sse.c 807 2012-09-19 17:51:00Z nawrockie $
+ * SVN $URL: https://svn.janelia.org/eddylab/eddys/easel/branches/infernal/1.1/esl_sse.c $
  *****************************************************************/
 
 /* Additionally, esl_sse_logf() and esl_sse_expf() are 
