@@ -1,12 +1,10 @@
 /* Simple sequence indices: 
  * Fast sequence record lookup in large files by keywords, such
  * as names or accessions.
- * 
- * SVN $Id: esl_ssi.h 269 2008-06-19 13:47:41Z eddys $
- * SRE, Thu Mar  2 15:54:51 2006 [St. Louis]
  */
-#ifndef ESL_SSI_INCLUDED
-#define ESL_SSI_INCLUDED
+#ifndef eslSSI_INCLUDED
+#define eslSSI_INCLUDED
+#ifdef eslAUGMENT_SSI
 
 #ifdef HAVE_SYS_TYPES_H
 #include <sys/types.h>
@@ -77,6 +75,8 @@ typedef struct {		/* Secondary key data: */
 } ESL_SKEY;
 
 typedef struct {
+  char       *ssifile;		/* name of the SSI file we're creating    */
+  FILE       *ssifp;		/* open SSI file being created            */
   int         external;	        /* TRUE if pkeys and skeys are on disk    */
   int         max_ram;	        /* threshold in MB to trigger extern sort */
 
@@ -98,6 +98,8 @@ typedef struct {
   uint64_t    nsecondary;
   char       *stmpfile;		/* secondary key tmpfile name, for extern sort */
   FILE       *stmp;	        /* handle on open ptmpfile */
+
+  char        errbuf[eslERRBUFSIZE];
 } ESL_NEWSSI;
 
 
@@ -119,14 +121,13 @@ extern int  esl_ssi_FileInfo(ESL_SSI *ssi, uint16_t fh, char **ret_filename, int
 
 
 /* 2. Creating (writing) SSI indices. */
-extern ESL_NEWSSI *esl_newssi_Create(void);
-extern int  esl_newssi_AddFile(ESL_NEWSSI *ns, const char *filename, int fmt, uint16_t *ret_fh);
+extern int  esl_newssi_Open(const char *ssifile, int allow_overwrite, ESL_NEWSSI **ret_newssi);
+extern int  esl_newssi_AddFile  (ESL_NEWSSI *ns, const char *filename, int fmt, uint16_t *ret_fh);
 extern int  esl_newssi_SetSubseq(ESL_NEWSSI *ns, uint16_t fh, uint32_t bpl, uint32_t rpl);
-extern int  esl_newssi_AddKey(ESL_NEWSSI *ns, const char *key, uint16_t fh, 
-			      off_t r_off, off_t d_off, int64_t L);
-extern int  esl_newssi_AddAlias(ESL_NEWSSI *ns, const char *alias, const char *key);
-extern int  esl_newssi_Write(FILE *fp, ESL_NEWSSI *ns);
-extern void esl_newssi_Destroy(ESL_NEWSSI *ns);
+extern int  esl_newssi_AddKey   (ESL_NEWSSI *ns, const char *key, uint16_t fh, off_t r_off, off_t d_off, int64_t L);
+extern int  esl_newssi_AddAlias (ESL_NEWSSI *ns, const char *alias, const char *key);
+extern int  esl_newssi_Write    (ESL_NEWSSI *ns);
+extern void esl_newssi_Close    (ESL_NEWSSI *ns);
 
 
 /* 3. Portable binary i/o. */
@@ -151,11 +152,18 @@ extern int      esl_fwrite_i32(FILE *fp, int32_t  n);
 extern int      esl_fwrite_i64(FILE *fp, int64_t  n);
 extern int	esl_fread_offset(FILE *fp, int mode, off_t *ret_offset);
 extern int      esl_fwrite_offset(FILE *fp, off_t offset);
-
-
-#endif /* ESL_SSI_INCLUDED */
+#endif /* eslAUGMENT_SSI*/
+#ifndef eslAUGMENT_SSI
+typedef void ESL_SSI;
+typedef void ESL_NEWSSI;
+#endif
+#endif /* eslSSI_INCLUDED */
 /*****************************************************************
- *    This copyrighted source code is freely distributed 
- *    under the terms of the GNU General Public License. See
- *    the files COPYRIGHT and LICENSE for details.
+ * Easel - a library of C functions for biological sequence analysis
+ * Version i1.1.1; July 2014
+ * Copyright (C) 2014 HHMI Janelia Farm Research Campus
+ * Other copyrights also apply. See the COPYRIGHT file for a full list.
+ * 
+ * Easel is distributed under the Janelia Farm Software License, a BSD
+ * license. See the LICENSE file for more details.
  *****************************************************************/
