@@ -1,14 +1,17 @@
-/* Foundation for the statistics modules.
+/* Foundation and miscellenea for the statistics modules.
  * 
  * Contents:
- *   1. The stats API.
- *   2. Unit tests.
- *   3. Test driver.
- *   4. Example.
- *   5. License and copyright information.
+ *   1. Summary statistics (means, variances)
+ *   2. Special functions
+ *   3. Standard statistical tests
+ *   4. Data fitting.
+ *   5. Unit tests.
+ *   6. Test driver.
+ *   7. Examples.
+ *      - driver for linear regression
+ *      - driver for G-test
+ *   8. License and copyright information.
  * 
- * SRE, Tue Jul 19 10:57:44 2005
- * SVN $Id: esl_stats.c 241 2008-04-01 19:01:52Z eddys $
  */
 #include "esl_config.h"
 
@@ -18,9 +21,12 @@
 #include "esl_stats.h"
 
 
+/*****************************************************************
+ * 1. Summary statistics calculations (means, variances)
+ *****************************************************************/
+
 /* Function:  esl_stats_DMean()
  * Synopsis:  Calculates mean and $\sigma^2$ for samples $x_i$.
- * Incept:    SRE, Tue Jul 19 11:04:00 2005 [St. Louis]
  *
  * Purpose:   Calculates the sample mean and $s^2$, the unbiased
  *            estimator of the population variance, for a
@@ -86,11 +92,16 @@ esl_stats_IMean(const int *x, int n, double *opt_mean, double *opt_var)
   if (opt_var  != NULL)  *opt_var  = (sqsum - sum*sum/(double)n) / ((double)n-1);
   return eslOK;
 }
+/*--------------- end, summary statistics -----------------------*/
 
 
+
+/*****************************************************************
+ * 2. Special functions.
+ *****************************************************************/
+
 /* Function:  esl_stats_LogGamma()
  * Synopsis:  Calculates $\log \Gamma(x)$.
- * Incept:    SRE, Tue Nov  2 13:47:01 2004 [St. Louis]
  *
  * Purpose:   Returns natural log of $\Gamma(x)$, for $x > 0$.
  * 
@@ -113,21 +124,20 @@ esl_stats_LogGamma(double x, double *ret_answer)
   double xx, tx;
   double tmp, value;
   static double cof[11] = {
-    4.694580336184385e+04,
+     4.694580336184385e+04,
     -1.560605207784446e+05,
-    2.065049568014106e+05,
+     2.065049568014106e+05,
     -1.388934775095388e+05,
-    5.031796415085709e+04,
+     5.031796415085709e+04,
     -9.601592329182778e+03,
-    8.785855930895250e+02,
+     8.785855930895250e+02,
     -3.155153906098611e+01,
-    2.908143421162229e-01,
+     2.908143421162229e-01,
     -2.319827630494973e-04,
-    1.251639670050933e-10
+     1.251639670050933e-10
   };
   
-  /* Protect against invalid x<=0
-   */
+  /* Protect against invalid x<=0  */
   if (x <= 0.0)  ESL_EXCEPTION(eslERANGE, "invalid x <= 0 in esl_stats_LogGamma()");
 
   xx       = x - 1.0;
@@ -148,7 +158,6 @@ esl_stats_LogGamma(double x, double *ret_answer)
 
 /* Function:  esl_stats_Psi()
  * Synopsis:  Calculates $\Psi(x)$ (the digamma function).
- * Incept:    SRE, Tue Nov 15 13:57:59 2005 [St. Louis]
  *
  * Purpose:   Computes $\Psi(x)$ (the "digamma" function), which is
  *            the derivative of log of the Gamma function:
@@ -332,6 +341,72 @@ esl_stats_IncompleteGamma(double a, double x, double *ret_pax, double *ret_qax)
   /*NOTREACHED*/
   return eslOK;
 }
+/*----------------- end, special functions ----------------------*/
+
+
+/*****************************************************************
+ * 3. Standard statistical tests.
+ *****************************************************************/
+
+/* Function:  esl_stats_GTest()
+ * Synopsis:  Calculates a G-test on 2 vs. 1 binomials.
+ *
+ * Purpose:   In experiment a, we've drawn <ca> successes in <na> total
+ *            trials; in experiment b, we've drawn <cb> successes in
+ *            <nb> total trials. Are the counts different enough to
+ *            conclude that the two experiments are different? The
+ *            null hypothesis is that the successes in both experiments
+ *            were drawn from the same binomial distribution with
+ *            per-trial probability $p$. The tested hypothesis is that
+ *            experiments a,b have different binomial probabilities
+ *            $p_a,p_b$. The G-test is a log-likelihood-ratio statistic,
+ *            assuming maximum likelihood values for $p,p_a,p_b$. 
+ *            $2G$ is distributed approximately as $\Chi^2(1)$, which
+ *            we use to calculate a P-value for the G statistic. 
+ *            
+ * Args:      ca    - number of positives in experiment a
+ *            na    - total number in experiment a
+ *            cb    - number of positives in experiment b
+ *            nb    - total number in experiment b
+ *            ret_G - RETURN: G statistic, a log likelihood ratio, in nats     
+ *            ret_P - RETURN: P-value for the G-statistic
+ *
+ * Returns:   <eslOK> on success.
+ *
+ * Throws:    (no abnormal error conditions)
+ *
+ * Xref:      Archive1999/0906-sagescore/sagescore.c
+ */
+int
+esl_stats_GTest(int ca, int na, int cb, int nb, double *ret_G, double *ret_P)
+{
+  double a,b,c,d,n;
+  double G = 0.;
+
+  a = (double) ca;
+  b = (double) (na - ca);
+  c = (double) cb;
+  d = (double) (nb - cb);
+  n = (double) na+nb;
+
+  /* Yes, the calculation here is correct; algebraic 
+   * rearrangement of the log-likelihood-ratio with 
+   * p_a = ca/na, p_b = cb/nb, and p = (ca+cb)/(na+nb).
+   * Guard against 0 probabilities; assume 0 log 0 => 0. 
+   */
+  if (a   > 0.) G  = a * log(a);
+  if (b   > 0.) G += b * log(b);
+  if (c   > 0.) G += c * log(c);
+  if (d   > 0.) G += d * log(d);
+  if (n   > 0.) G += n * log(n);
+  if (a+b > 0.) G -= (a+b) * log(a+b);
+  if (c+d > 0.) G -= (c+d) * log(c+d);
+  if (a+c > 0.) G -= (a+c) * log(a+c);
+  if (b+d > 0.) G -= (b+d) * log(b+d);
+
+  *ret_G = G;
+  return esl_stats_IncompleteGamma( 0.5, G, NULL, ret_P);
+}
 
 
 /* Function:  esl_stats_ChiSquaredTest()
@@ -359,8 +434,14 @@ esl_stats_ChiSquaredTest(int v, double x, double *ret_answer)
 {
   return esl_stats_IncompleteGamma((double)v/2, x/2, NULL, ret_answer);
 }
+/*----------------- end, statistical tests  ---------------------*/
 
 
+
+/*****************************************************************
+ * 4. Data fitting.
+ *****************************************************************/
+
 /* Function:  esl_stats_LinearRegression()
  * Synopsis:  Fit data to a straight line.
  * Incept:    SRE, Sat May 26 11:33:46 2007 [Janelia]
@@ -547,16 +628,66 @@ esl_stats_LinearRegression(const double *x, const double *y, const double *sigma
   if (opt_Q       != NULL) *opt_Q       = 0.;
   return status;
 }
-/*---------------- end of API implementation --------------------*/
-
+/*------------------- end, data fitting -------------------------*/
 
 
 
 /*****************************************************************
- * 2. Unit tests.
+ * 5. Unit tests.
  *****************************************************************/
 #ifdef eslSTATS_TESTDRIVE
-#include <esl_random.h>
+#include "esl_random.h"
+#include "esl_stopwatch.h"
+#ifdef HAVE_LIBGSL 
+#include <gsl/gsl_sf_gamma.h>
+#endif
+
+/* The LogGamma() function is rate-limiting in hmmbuild, because it is
+ * used so heavily in mixture Dirichlet calculations.
+ *    ./configure --with-gsl; [compile test driver]
+ *    ./stats_utest -v
+ * runs a comparison of time/precision against GSL.
+ * SRE, Sat May 23 10:04:41 2009, on home Mac:
+ *     LogGamma       = 1.29u  / N=1e8  =  13 nsec/call
+ *     gsl_sf_lngamma = 1.43u  / N=1e8  =  14 nsec/call
+ */
+static void
+utest_LogGamma(ESL_RANDOMNESS *r, int N, int be_verbose)
+{
+  char          *msg = "esl_stats_LogGamma() unit test failed";
+  ESL_STOPWATCH *w   = esl_stopwatch_Create();
+  double        *x   = malloc(sizeof(double) * N);
+  double        *lg  = malloc(sizeof(double) * N);
+  double        *lg2 = malloc(sizeof(double) * N);
+  int            i;
+
+  for (i = 0; i < N; i++) 
+    x[i] = esl_random(r) * 100.;
+  
+  esl_stopwatch_Start(w);
+  for (i = 0; i < N; i++) 
+    if (esl_stats_LogGamma(x[i], &(lg[i])) != eslOK) esl_fatal(msg);
+  esl_stopwatch_Stop(w);
+
+  if (be_verbose) esl_stopwatch_Display(stdout, w, "esl_stats_LogGamma() timing: ");
+
+#ifdef HAVE_LIBGSL
+  esl_stopwatch_Start(w);
+  for (i = 0; i < N; i++) lg2[i] = gsl_sf_lngamma(x[i]);
+  esl_stopwatch_Stop(w);
+
+  if (be_verbose) esl_stopwatch_Display(stdout, w, "gsl_sf_lngamma() timing:     ");
+  
+  for (i = 0; i < N; i++)
+    if (esl_DCompare(lg[i], lg2[i], 1e-2) != eslOK) esl_fatal(msg);
+#endif
+  
+  free(lg2);
+  free(lg);
+  free(x);
+  esl_stopwatch_Destroy(w);
+}
+
 
 /* The test of esl_stats_LinearRegression() is a statistical test,
  * so we can't be too aggressive about testing results. 
@@ -633,10 +764,11 @@ utest_LinearRegression(ESL_RANDOMNESS *r, int use_sigma, int be_verbose)
 
 
 /*****************************************************************
- * 3. Test driver.
+ * 6. Test driver.
  *****************************************************************/
 #ifdef eslSTATS_TESTDRIVE
-/* gcc -g -Wall -o testdrive -L. -I. -DeslSTATS_TESTDRIVE esl_stats.c -leasel -lm
+/* gcc -g -Wall -o stats_utest  -L. -I. -DeslSTATS_TESTDRIVE esl_stats.c -leasel -lm
+ * gcc -DHAVE_LIBGSL -O2 -o stats_utest -L. -I. -DeslSTATS_TESTDRIVE esl_stats.c -leasel -lgsl -lm
  */
 #include <stdio.h>
 #include "easel.h"
@@ -647,41 +779,25 @@ utest_LinearRegression(ESL_RANDOMNESS *r, int use_sigma, int be_verbose)
 static ESL_OPTIONS options[] = {
   /* name  type         default  env   range togs  reqs  incomp  help                docgrp */
   {"-h",  eslARG_NONE,    FALSE, NULL, NULL, NULL, NULL, NULL, "show help and usage",                   0},
-  {"-r",  eslARG_NONE,    FALSE, NULL, NULL, NULL, NULL, "-s", "random:  use current time as RNG seed", 0},
-  {"-s",  eslARG_INT,      "42", NULL, NULL, NULL, NULL, "-r", "set random number seed to <n>",         0},
+  {"-s",  eslARG_INT,      "42", NULL, NULL, NULL, NULL, NULL, "set random number seed to <n>",         0},
   {"-v",  eslARG_NONE,    FALSE, NULL, NULL, NULL, NULL, NULL, "verbose: show verbose output",          0},
+  {"-N",  eslARG_INT,"10000000", NULL, NULL, NULL, NULL, NULL, "number of trials in LogGamma test",     0},
   { 0,0,0,0,0,0,0,0,0,0},
 };
-static char usage[] = "[-options]";
+static char usage[]  = "[-options]";
+static char banner[] = "test driver for stats special functions";
 
 int
 main(int argc, char **argv)
 {
-  ESL_GETOPTS    *go  = NULL;
-  ESL_RANDOMNESS *r   = NULL;
-  int             be_verbose;
+  ESL_GETOPTS    *go         = esl_getopts_CreateDefaultApp(options, 0, argc, argv, banner, usage);
+  ESL_RANDOMNESS *r          = esl_randomness_Create(esl_opt_GetInteger(go, "-s"));
+  int             be_verbose = esl_opt_GetBoolean(go, "-v");
+  int             N          = esl_opt_GetInteger(go, "-N");
 
-  go = esl_getopts_Create(options);
-  if (esl_opt_ProcessCmdline(go, argc, argv) != eslOK ||
-      esl_opt_VerifyConfig(go)               != eslOK) esl_fatal("%s", go->errbuf);
-  if (esl_opt_GetBoolean(go, "-h") == TRUE) {
-    esl_usage(stdout, argv[0], usage);
-    puts("\n  where options are:");
-    esl_opt_DisplayHelp(stdout, go, 0, 2, 80); /* 0=all docgroups; 2=indentation; 80=width */
-    exit(0);
-  }
-  if (esl_opt_ArgNumber(go) != 0) {
-    printf("Incorrect number of command line arguments.\n");
-    esl_usage(stdout, argv[0], usage);
-    exit(1);
-  }
-  be_verbose = esl_opt_GetBoolean(go, "-v");
-
-  if (esl_opt_GetBoolean(go, "-r")) {
-    r = esl_randomness_CreateTimeseeded();
-    if (be_verbose) printf("seed = %ld\n", esl_randomness_GetSeed(r));
-  } else r = esl_randomness_Create(esl_opt_GetInteger(go, "-s"));
+  if (be_verbose) printf("seed = %" PRIu32 "\n", esl_randomness_GetSeed(r));
 
+  utest_LogGamma(r, N, be_verbose);
   utest_LinearRegression(r, TRUE,  be_verbose);
   utest_LinearRegression(r, FALSE, be_verbose);
   
@@ -696,7 +812,7 @@ main(int argc, char **argv)
 
 
 /*****************************************************************
- * 4. Example.
+ * 7. Examples.
  *****************************************************************/
 
 /* Compile:  gcc -g -Wall -o example -I. -DeslSTATS_EXAMPLE esl_stats.c esl_random.c easel.c -lm  
@@ -712,7 +828,7 @@ main(int argc, char **argv)
 
 int main(void)
 {
-  ESL_RANDOMNESS *r   = esl_randomness_CreateTimeseeded();
+  ESL_RANDOMNESS *r   = esl_randomness_Create(0);
   double a            = -3.;
   double b            = 1.;
   double xori         = -20.;
@@ -748,15 +864,57 @@ int main(void)
   exit(0);
 }
 /*::cexcerpt::stats_example::end::*/
-#endif
+#endif /* eslSTATS_EXAMPLE */
 
-/*--------------------- end of example --------------------------*/
+
+#ifdef eslSTATS_EXAMPLE2
+
+#include <stdlib.h>
+
+#include "easel.h"
+#include "esl_getopts.h"
+#include "esl_stats.h"
+
+static ESL_OPTIONS options[] = {
+  /* name           type      default  env  range toggles reqs incomp  help                                       docgroup*/
+  { "-h",        eslARG_NONE,   FALSE,  NULL, NULL,  NULL,  NULL, NULL, "show brief help on version and usage",    0 },
+  {  0, 0, 0, 0, 0, 0, 0, 0, 0, 0 },
+};
+static char usage[]  = "[-options] <ca> <na> <cb> <nb>";
+static char banner[] = "example from the stats module: using a G-test";
+
+int
+main(int argc, char **argv)
+{
+  ESL_GETOPTS  *go  = esl_getopts_CreateDefaultApp(options, 4, argc, argv, banner, usage);
+  int           ca  = strtol(esl_opt_GetArg(go, 1), NULL, 10);
+  int           na  = strtol(esl_opt_GetArg(go, 2), NULL, 10);
+  int           cb  = strtol(esl_opt_GetArg(go, 3), NULL, 10);
+  int           nb  = strtol(esl_opt_GetArg(go, 4), NULL, 10);
+  double        G, P;
+  int           status;
+  
+  if (ca > na || cb > nb) esl_fatal("argument order wrong? expect ca, na, cb, nb for ca/na, cb/nb");
+ 
+  if ( (status = esl_stats_GTest(ca, na, cb, nb, &G, &P)) != eslOK) esl_fatal("G-test failed?");
+  printf("%-10.3g %12.2f\n", P, G);
+  exit(0);
+}
+#endif /* eslSTATS_EXAMPLE2 */
+/*--------------------- end of examples -------------------------*/
 
 
 
 
 /*****************************************************************
- *    This copyrighted source code is freely distributed 
- *    under the terms of the GNU General Public License. See
- *    the files COPYRIGHT and LICENSE for details.
+ * Easel - a library of C functions for biological sequence analysis
+ * Version i1.1.1; July 2014
+ * Copyright (C) 2014 HHMI Janelia Farm Research Campus
+ * Other copyrights also apply. See the COPYRIGHT file for a full list.
+ * 
+ * Easel is distributed under the Janelia Farm Software License, a BSD
+ * license. See the LICENSE file for more details.
+ *
+ * SVN $Id: esl_stats.c 771 2012-06-11 12:53:13Z eddys $
+ * SVN $URL: https://svn.janelia.org/eddylab/eddys/easel/branches/infernal/1.1/esl_stats.c $
  *****************************************************************/
