@@ -1,26 +1,37 @@
-/* esl_stats.h
- * Foundation for the statistics modules.
- * 
- * SRE, Tue Jul 19 11:35:28 2005
- * SVN $Id: esl_stats.h 195 2007-08-09 19:02:55Z eddys $
+/* Foundation, miscellenea for the statistics modules.
  */
-#ifndef ESL_STATS_INCLUDED
-#define ESL_STATS_INCLUDED
+#ifndef eslSTATS_INCLUDED
+#define eslSTATS_INCLUDED
 
+/* 1. Summary statistics calculations */
 extern int esl_stats_DMean(const double *x, int n, double *opt_mean, double *opt_var);
 extern int esl_stats_FMean(const float  *x, int n, double *opt_mean, double *opt_var);
 extern int esl_stats_IMean(const int    *x, int n, double *opt_mean, double *opt_var);
+
+/* 2. Special functions */
 extern int esl_stats_LogGamma(double x, double *ret_answer);
 extern int esl_stats_Psi(double x, double *ret_answer);
 extern int esl_stats_IncompleteGamma(double a, double x, double *ret_pax, double *ret_qax);
+
+/* 3. Standard statistical tests */
+extern int esl_stats_GTest(int ca, int na, int cb, int nb, double *ret_G, double *ret_P);
 extern int esl_stats_ChiSquaredTest(int v, double x, double *ret_answer);
+
+/* 4. Data fitting */
 extern int esl_stats_LinearRegression(const double *x, const double *y, const double *sigma, int n,
 				      double *opt_a,       double *opt_b,
 				      double *opt_sigma_a, double *opt_sigma_b, double *opt_cov_ab,
 				      double *opt_cc,      double *opt_Q);
-#endif /*ESL_STATS_INCLUDED*/
+#endif /*eslSTATS_INCLUDED*/
 /*****************************************************************
- *    This copyrighted source code is freely distributed 
- *    under the terms of the GNU General Public License. See
- *    the files COPYRIGHT and LICENSE for details.
+ * Easel - a library of C functions for biological sequence analysis
+ * Version i1.1.1; July 2014
+ * Copyright (C) 2014 HHMI Janelia Farm Research Campus
+ * Other copyrights also apply. See the COPYRIGHT file for a full list.
+ * 
+ * Easel is distributed under the Janelia Farm Software License, a BSD
+ * license. See the LICENSE file for more details.
+ *
+ * SVN $Id: esl_stats.h 771 2012-06-11 12:53:13Z eddys $
+ * SVN $URL: https://svn.janelia.org/eddylab/eddys/easel/branches/infernal/1.1/esl_stats.h $
  *****************************************************************/
