@@ -1,12 +1,6 @@
-/* esl_stopwatch.c
- * Tracking cpu/system/elapsed time used by a process.
+/* Tracking cpu/system/elapsed time used by a process.
  *
  * Thanks to Warren Gish for assistance.
- * 
- * SRE, Wed Feb 22 20:12:55 2006 [St. Louis] [moved to Easel]
- * SRE, Thu Aug  3 08:11:52 2000 [St. Louis] [moved to SQUID]
- * SRE, Fri Nov 26 14:54:21 1999 [St. Louis] [HMMER]
- * SVN $Id: esl_stopwatch.c 241 2008-04-01 19:01:52Z eddys $
  */
 #include "esl_config.h"
 
@@ -18,7 +12,6 @@
  *****************************************************************/
 
 /* Function:  esl_stopwatch_Create()
- * Incept:    SRE, Wed Feb 22 20:15:05 2006 [St. Louis]
  *
  * Purpose:   Creates a new stopwatch.
  *
@@ -45,7 +38,6 @@ esl_stopwatch_Create(void)
 }
 
 /* Function:  esl_stopwatch_Destroy()
- * Incept:    SRE, Thu Feb 23 07:09:23 2006 [St. Louis]
  *
  * Purpose:   Frees an <ESL_STOPWATCH>.
  */
@@ -59,7 +51,6 @@ esl_stopwatch_Destroy(ESL_STOPWATCH *w)
 
 
 /* Function:  esl_stopwatch_Start()
- * Incept:    SRE, Sat Feb 25 10:41:00 2006 [St. Louis]
  *
  * Purpose:   Start a stopwatch. This sets the base 
  *            for elapsed, cpu, and system time difference
@@ -71,10 +62,10 @@ esl_stopwatch_Destroy(ESL_STOPWATCH *w)
 int 
 esl_stopwatch_Start(ESL_STOPWATCH *w)
 {
-  w->t0 = time(NULL);
 #ifdef HAVE_TIMES /* POSIX */
-  (void) times(&(w->cpu0));
+  w->t0 = times(&(w->cpu0));
 #else             /* fallback to ANSI C */
+  w->t0   = time(NULL);
   w->cpu0 = clock();
 #endif
   w->elapsed = 0.;
@@ -84,7 +75,6 @@ esl_stopwatch_Start(ESL_STOPWATCH *w)
 }
 
 /* Function:  esl_stopwatch_Stop()
- * Incept:    SRE, Sat Feb 25 10:42:26 2006 [St. Louis]
  *
  * Purpose:   Stop a stopwatch. Record and store elapsed,
  *            cpu, and system time difference relative to the
@@ -95,32 +85,32 @@ esl_stopwatch_Start(ESL_STOPWATCH *w)
 int
 esl_stopwatch_Stop(ESL_STOPWATCH *w)
 {
-  time_t t1;
 #ifdef HAVE_TIMES
   struct tms cpu1;
-  long       clk_tck;
+  clock_t    t1;
+  double     clk_tck;
 #else
+  time_t  t1;
   clock_t cpu1;
 #endif
 
-  t1 = time(NULL);
-  w->elapsed = difftime(t1, w->t0);
 
 #ifdef HAVE_TIMES /* POSIX */
-  (void) times(&cpu1);
+  t1         = times(&cpu1);
   
-  clk_tck = sysconf(_SC_CLK_TCK);
-  w->user = (double) (cpu1.tms_utime + cpu1.tms_cutime -
-		      w->cpu0.tms_utime - w->cpu0.tms_cutime) /
-            (double) clk_tck;
+  clk_tck    = (double) sysconf(_SC_CLK_TCK);
+  w->elapsed = (double) (t1 - w->t0) / clk_tck;
+  w->user    = (double) (cpu1.tms_utime + cpu1.tms_cutime -
+			 w->cpu0.tms_utime - w->cpu0.tms_cutime) / clk_tck;
 
-  w->sys  = (double) (cpu1.tms_stime + cpu1.tms_cstime -
-		      w->cpu0.tms_stime - w->cpu0.tms_cstime) /
-            (double) clk_tck;
+  w->sys     = (double) (cpu1.tms_stime + cpu1.tms_cstime -
+			 w->cpu0.tms_stime - w->cpu0.tms_cstime) / clk_tck;
 #else /* fallback to ANSI C */
-  cpu1    = clock();
-  w->user = (double) (cpu1- w->cpu0) / (double) CLOCKS_PER_SEC;
-  w->sys  = 0.;		/* no way to portably get system time in ANSI C */
+  t1         = time(NULL);
+  cpu1       = clock();
+  w->elapsed = difftime(t1, w->t0);
+  w->user    = (double) (cpu1- w->cpu0) / (double) CLOCKS_PER_SEC;
+  w->sys     = 0.;		/* no way to portably get system time in ANSI C */
 
 #endif
   return eslOK;
@@ -153,7 +143,6 @@ format_time_string(char *buf, double sec, int do_frac)
 }
 
 /* Function:  esl_stopwatch_Display()
- * Incept:    SRE, Sat Feb 25 10:51:09 2006 [St. Louis]
  *
  * Purpose:   Output a usage summary line from a stopped
  *            stopwatch, showing elapsed, cpu, and system time
@@ -172,32 +161,31 @@ format_time_string(char *buf, double sec, int do_frac)
  *            prefix  - output line prefix ("" for nothing)
  *
  * Returns:   <eslOK> on success.
+ * 
+ * Throws:    <eslEWRITE> on any system write error, such as filled disk.
+
  */
 int 
 esl_stopwatch_Display(FILE *fp, ESL_STOPWATCH *w, char *prefix)
 {
-  char buf[128];	/* (safely holds up to 10^14 years) */
+  char buf[128];	/* (safely holds up to 10^14 years; I'll be dead by then) */
   
-  if (prefix == NULL)
-    fputs("CPU Time: ", fp);
-  else 
-    fputs(prefix, fp);
+  if (prefix == NULL) { if (fputs("CPU Time: ", fp) < 0) ESL_EXCEPTION_SYS(eslEWRITE, "stopwatch display write failed"); }
+  else                { if (fputs(prefix, fp)       < 0) ESL_EXCEPTION_SYS(eslEWRITE, "stopwatch display write failed"); }
 
   format_time_string(buf, w->user+w->sys, TRUE);
 #ifdef HAVE_TIMES
-  fprintf(fp, "%.2fu %.2fs %s ", w->user, w->sys, buf);
+  if (fprintf(fp, "%.2fu %.2fs %s ", w->user, w->sys, buf) < 0) ESL_EXCEPTION_SYS(eslEWRITE, "stopwatch display write failed"); 
 #else
-  fprintf(fp, "%.2fu %s ", w->user, buf);
+  if (fprintf(fp, "%.2fu %s ", w->user, buf)               < 0) ESL_EXCEPTION_SYS(eslEWRITE, "stopwatch display write failed"); 
 #endif
-
-  format_time_string(buf, w->elapsed, 0);
-  fprintf(fp, "Elapsed: %s\n", buf);
+  format_time_string(buf, w->elapsed, TRUE);
+  if (fprintf(fp, "Elapsed: %s\n", buf)                    < 0) ESL_EXCEPTION_SYS(eslEWRITE, "stopwatch display write failed"); 
   return eslOK;
 }
   
 
 /* Function:  esl_stopwatch_Include()
- * Incept:    SRE, Sat Feb 25 10:47:17 2006 [St. Louis]
  *
  * Purpose:   Merge the cpu and system times from a slave into
  *            a master stopwatch. Both watches must be
@@ -258,3 +246,17 @@ main(void)
 }
 /*::cexcerpt::stopwatch_example::end::*/
 #endif /*ESL_STOPWATCH_EXAMPLE*/
+
+/*****************************************************************
+ * Easel - a library of C functions for biological sequence analysis
+ * Version i1.1.1; July 2014
+ * Copyright (C) 2014 HHMI Janelia Farm Research Campus
+ * Other copyrights also apply. See the COPYRIGHT file for a full list.
+ * 
+ * Easel is distributed under the Janelia Farm Software License, a BSD
+ * license. See the LICENSE file for more details.
+ *
+ * SVN $Id: esl_stopwatch.c 727 2011-10-24 17:17:32Z eddys $
+ * SVN $URL: https://svn.janelia.org/eddylab/eddys/easel/branches/infernal/1.1/esl_stopwatch.c $
+ *****************************************************************/
+
