@@ -1,13 +1,13 @@
-/* esl_stopwatch.h
- * Tracking cpu/system/elapsed time used by a process.
+/* Tracking cpu/system/elapsed time used by a process.
  * 
  * SRE, Wed Feb 22 19:30:36 2006 [St. Louis] [moved to Easel]
  * SRE, Thu Aug  3 08:00:35 2000 [St. Louis] [moved to SQUID]
  * SRE, Fri Nov 26 14:54:21 1999 [St. Louis] [HMMER]
- * SVN $Id: esl_stopwatch.h 111 2006-03-31 13:15:58Z eddy $
+ * SVN $Id: esl_stopwatch.h 664 2011-02-27 17:08:36Z eddys $
+ * SVN $URL: https://svn.janelia.org/eddylab/eddys/easel/branches/infernal/1.1/esl_stopwatch.h $
  */
-#ifndef ESL_STOPWATCH_INCLUDED
-#define ESL_STOPWATCH_INCLUDED
+#ifndef eslSTOPWATCH_INCLUDED
+#define eslSTOPWATCH_INCLUDED
 
 #include <time.h>
 #ifdef HAVE_TIMES
@@ -18,13 +18,13 @@
 #endif
 
 typedef struct {
-  /* t0 and cpu0 keep base, when the watch was Start()'ed 
-   */
-  time_t t0;			/* Wall clock time, ANSI time()  */
+  /* t0 and cpu0 keep base, when the watch was Start()'ed */
 #ifdef HAVE_TIMES
+  clock_t    t0;		/* Wall time, POSIX times()      */
   struct tms cpu0;		/* CPU/system time, POSIX times()*/
 #else
-  clock_t cpu0;			/* CPU time, fallback to ANSI clock() */
+  time_t  t0;			/* Wall time, fallback to ANSI time()  */
+  clock_t cpu0;			/* CPU time, fallback to ANSI clock()  */
 #endif
 
   /* elapsed/user/sys are t-t0 results for the last time the
@@ -46,11 +46,15 @@ extern int esl_stopwatch_Display(FILE *fp, ESL_STOPWATCH *w, char *prefix);
 extern int esl_stopwatch_Include(ESL_STOPWATCH *master, ESL_STOPWATCH *w);
 
 
-#endif /*ESL_STOPWATCH_INCLUDED*/ 
+#endif /*eslSTOPWATCH_INCLUDED*/ 
 /*****************************************************************
- *    This copyrighted source code is freely distributed 
- *    under the terms of the GNU General Public License. See
- *    the files COPYRIGHT and LICENSE for details.
+ * Easel - a library of C functions for biological sequence analysis
+ * Version i1.1.1; July 2014
+ * Copyright (C) 2014 HHMI Janelia Farm Research Campus
+ * Other copyrights also apply. See the COPYRIGHT file for a full list.
+ * 
+ * Easel is distributed under the Janelia Farm Software License, a BSD
+ * license. See the LICENSE file for more details.
  *****************************************************************/
 
 
