@@ -1,9 +1,6 @@
-/* esl_stretchexp.c
- * Statistical routines for stretched exponential distributions.
+/* Statistical routines for stretched exponential distributions.
  * 
- * SRE, Fri Aug 19 11:15:21 2005 [St. Louis] 
  * xref STL9/146
- * SVN $Id: esl_stretchexp.c 241 2008-04-01 19:01:52Z eddys $
  */
 #include <esl_config.h>
 
@@ -35,7 +32,6 @@
  */
 
 /* Function:  esl_sxp_pdf()
- * Incept:    SRE, Fri Aug 19 11:17:47 2005 [St. Louis]
  *
  * Purpose:   Calculates the probability density function for the 
  *            stretched exponential pdf, $P(X=x)$, given
@@ -58,7 +54,6 @@ esl_sxp_pdf(double x, double mu, double lambda, double tau)
 }
 
 /* Function:  esl_sxp_logpdf()
- * Incept:    SRE, Fri Aug 19 11:27:32 2005 [St. Louis]
  *
  * Purpose:   Calculates the log probability density function for the 
  *            stretched exponential pdf, $\log P(X=x)$, given
@@ -80,7 +75,6 @@ esl_sxp_logpdf(double x, double mu, double lambda, double tau)
 }
 
 /* Function:  esl_sxp_cdf()
- * Incept:    SRE, Fri Aug 19 11:30:55 2005 [St. Louis]
  *
  * Purpose:   Calculates the cumulative distribution function for the 
  *            stretched exponential pdf, $P(X \leq x)$, given
@@ -100,7 +94,6 @@ esl_sxp_cdf(double x, double mu, double lambda, double tau)
 }
 
 /* Function:  esl_sxp_logcdf()
- * Incept:    SRE, Fri Aug 19 11:37:20 2005 [St. Louis]
  *
  * Purpose:   Calculates the log of the cumulative distribution function for the 
  *            stretched exponential pdf, $\log P(X \leq x)$, given
@@ -118,7 +111,6 @@ esl_sxp_logcdf(double x, double mu, double lambda, double tau)
 }
 
 /* Function:  esl_sxp_surv()
- * Incept:    SRE, Fri Aug 19 11:38:24 2005 [St. Louis]
  *
  * Purpose:   Calculates the survival function for the 
  *            stretched exponential pdf, $P(X > x)$, given
@@ -137,7 +129,6 @@ esl_sxp_surv(double x, double mu, double lambda, double tau)
 }
 
 /* Function:  esl_sxp_logsurv()
- * Incept:    SRE, Fri Aug 19 11:38:24 2005 [St. Louis]
  *
  * Purpose:   Calculates the log survival function for the 
  *            stretched exponential pdf, $\log P(X > x)$, given
@@ -156,7 +147,6 @@ esl_sxp_logsurv(double x, double mu, double lambda, double tau)
 }
 
 /* Function:  esl_sxp_invcdf()
- * Incept:    SRE, Sat Aug 20 14:42:06 2005 [St. Louis]
  *
  * Purpose:   Calculates the inverse CDF for a stretched exponential
  *            with parameters <mu>, <lambda>, and <tau>, returning
@@ -206,7 +196,6 @@ esl_sxp_invcdf(double p, double mu, double lambda, double tau)
  ****************************************************************************/ 
 
 /* Function:  esl_sxp_generic_pdf()
- * Incept:    SRE, Thu Aug 25 08:06:14 2005 [St. Louis]
  *
  * Purpose:   Generic-API wrapper around <esl_sxp_pdf()>, taking
  *            a void ptr to a double array containing $\mu$, $\lambda$,
@@ -220,7 +209,6 @@ esl_sxp_generic_pdf(double x, void *params)
 }
 
 /* Function:  esl_sxp_generic_cdf()
- * Incept:    SRE, Fri Aug 19 13:54:26 2005 [St. Louis]
  *
  * Purpose:   Generic-API wrapper around <esl_sxp_cdf()>, taking
  *            a void ptr to a double array containing $\mu$, $\lambda$,
@@ -234,7 +222,6 @@ esl_sxp_generic_cdf(double x, void *params)
 }
 
 /* Function:  esl_sxp_generic_surv()
- * Incept:    SRE, Thu Aug 25 08:06:33 2005 [St. Louis]
  *
  * Purpose:   Generic-API wrapper around <esl_sxp_surv()>, taking
  *            a void ptr to a double array containing $\mu$, $\lambda$,
@@ -248,7 +235,6 @@ esl_sxp_generic_surv(double x, void *params)
 }
 
 /* Function:  esl_sxp_generic_invcdf()
- * Incept:    SRE, Sat Aug 20 14:46:55 2005 [St. Louis]
  *
  * Purpose:   Generic-API wrapper around <esl_sxp_invcdf()>, taking
  *            a void ptr to a double array containing $\mu$, $\lambda$,
@@ -269,14 +255,15 @@ esl_sxp_generic_invcdf(double p, void *params)
  ****************************************************************************/ 
 
 /* Function:  esl_sxp_Plot()
- * Incept:    SRE, Fri Aug 19 11:48:27 2005 [St. Louis]
  *
  * Purpose:   Plot some stretched exponential function <func> (for instance,
  *            <esl_sxp_pdf()>) for parameters <mu>, <lambda>, and <tau>, for
  *            a range of quantiles x from <xmin> to <xmax> in steps of <xstep>;
  *            output to an open stream <fp> in xmgrace XY input format.
  *
- * Returns:   <eslOK>.
+ * Returns:   <eslOK> on success.
+ *
+ * Throws:    <eslEWRITE> on any system write error, such as filled disk.
  */
 int
 esl_sxp_Plot(FILE *fp, double mu, double lambda, double tau,
@@ -285,8 +272,8 @@ esl_sxp_Plot(FILE *fp, double mu, double lambda, double tau,
 {
   double x;
   for (x = xmin; x <= xmax; x += xstep)
-    fprintf(fp, "%f\t%g\n", x, (*func)(x, mu, lambda, tau));
-  fprintf(fp, "&\n");
+    if (fprintf(fp, "%f\t%g\n", x, (*func)(x, mu, lambda, tau)) < 0) ESL_EXCEPTION_SYS(eslEWRITE, "stretchexp plot write failed");
+  if (fprintf(fp, "&\n")                                        < 0) ESL_EXCEPTION_SYS(eslEWRITE, "stretchexp plot write failed");
   return eslOK;
 }
 /*-------------------- end plot dumping routines ---------------------------*/
@@ -299,7 +286,6 @@ esl_sxp_Plot(FILE *fp, double mu, double lambda, double tau,
  ****************************************************************************/ 
 #ifdef eslAUGMENT_RANDOM
 /* Function:  esl_sxp_Sample()
- * Incept:    SRE, Fri Aug 19 13:39:36 2005 [St. Louis]
  *
  * Purpose:   Sample a stretched exponential random variate,
  *            by a change of variable from a Gamma sample.
@@ -348,7 +334,6 @@ sxp_complete_func(double *p, int np, void *dptr)
 }
 
 /* Function:  esl_sxp_FitComplete()
- * Incept:    SRE, Fri Aug 19 15:25:42 2005 [St. Louis]
  *
  * Purpose:   Given a vector of <n> observed data samples <x[]>,
  *            find maximum likelihood parameters by conjugate gradient 
@@ -436,7 +421,6 @@ sxp_complete_binned_func(double *p, int np, void *dptr)
 }
 
 /* Function:  esl_sxp_FitCompleteBinned()
- * Incept:    SRE, Sat Aug 20 13:28:00 2005 [St. Louis]
  *
  * Purpose:   Given a histogram <g> with binned observations, where each
  *            bin i holds some number of observed samples x with values from 
@@ -522,7 +506,7 @@ main(int argc, char **argv)
   double lambda     = 2.5;
   double tau        = 0.7;
   ESL_HISTOGRAM  *h = esl_histogram_CreateFull(mu, 100., 0.1);
-  ESL_RANDOMNESS *r = esl_randomness_CreateTimeseeded();
+  ESL_RANDOMNESS *r = esl_randomness_Create(0);
   int    n          = 10000;
   double *data;
   int     ndata;
@@ -565,85 +549,80 @@ main(int argc, char **argv)
  * Test driver
  ****************************************************************************/ 
 #ifdef eslSTRETCHEXP_TESTDRIVE
-/* Compile:
-   gcc -g -Wall -I. -I ~/src/easel -L ~/src/easel -o test -DeslSTRETCHEXP_TESTDRIVE\
-    esl_stretchexp.c -leasel -lm
-*/
+/* gcc -g -Wall -I. -L . -o stretchexp_utest -DeslSTRETCHEXP_TESTDRIVE esl_stretchexp.c -leasel -lm
+ */
+#include "esl_config.h"
+
 #include <stdio.h>
 #include <stdlib.h>
 #include <string.h>
 
 #include "easel.h"
+#include "esl_getopts.h"
 #include "esl_random.h"
 #include "esl_histogram.h"
 #include "esl_stretchexp.h"
 
+static ESL_OPTIONS options[] = {
+  /* name  type         default  env   range togs  reqs  incomp  help                docgrp */
+  {"-h",  eslARG_NONE,    FALSE, NULL, NULL, NULL, NULL, NULL, "show help and usage",               0},
+  {"-l",  eslARG_REAL,    "1.0", NULL,"x>0", NULL, NULL, NULL, "set lambda param to <x>",           0},
+  {"-m",  eslARG_REAL,   "10.0", NULL, NULL, NULL, NULL, NULL, "set mu param to <x>",               0},
+  {"-n",  eslARG_INT,   "10000", NULL,"n>0", NULL, NULL, NULL, "set number of samples to <n>",      0},
+  {"-o",  eslARG_OUTFILE,  NULL, NULL, NULL, NULL, NULL, NULL, "save plots to file <f>",            0},
+  {"-s",  eslARG_INT,      "42", NULL, NULL, NULL, NULL, NULL, "set random number seed to <n>",     0},
+  {"-t",  eslARG_REAL,    "0.7", NULL,"x>0", NULL, NULL, NULL, "set tau param to <x>",              0},
+  {"-v",  eslARG_NONE,    FALSE, NULL, NULL, NULL, NULL, NULL, "show verbose output",               0},
+  {"-w",  eslARG_REAL,    "0.1", NULL,"x>0", NULL, NULL, NULL, "set width of histogram bins to <x>",0},
+  {"--C", eslARG_NONE,    FALSE, NULL, NULL, NULL, NULL, NULL, "plot CDF",                          0},
+  {"--LC",eslARG_NONE,    FALSE, NULL, NULL, NULL, NULL, NULL, "plot log CDF",                      0},
+  {"--P", eslARG_NONE,    FALSE, NULL, NULL, NULL, NULL, NULL, "plot PDF",                          0},
+  {"--LP",eslARG_NONE,    FALSE, NULL, NULL, NULL, NULL, NULL, "plot log PDF",                      0},
+  {"--S", eslARG_NONE,    FALSE, NULL, NULL, NULL, NULL, NULL, "plot survival",                     0},
+  {"--LS",eslARG_NONE,    FALSE, NULL, NULL, NULL, NULL, NULL, "plot log survival",                 0},
+  {"--XL",eslARG_NONE,     NULL, NULL, NULL, NULL, NULL, NULL, "set xmin for plot axis",            0},
+  {"--XH",eslARG_NONE,     NULL, NULL, NULL, NULL, NULL, NULL, "set xmax for plot axis",            0},
+  {"--XS",eslARG_NONE,     NULL, NULL, NULL, NULL, NULL, NULL, "set xstep for plot axis",           0},
+  { 0,0,0,0,0,0,0,0,0,0},
+};
+static char usage[]  = "[-options]";
+static char banner[] = "test driver for random module";
+
 int
 main(int argc, char **argv)
 {
-  ESL_HISTOGRAM  *h;
-  ESL_RANDOMNESS *r;
-  double  mu        = 10.0;
-  double  lambda    =  1.0;  
-  double  tau       =  0.7;
-  int     n         = 10000;
-  double  binwidth  = 0.1;
+  ESL_GETOPTS    *go   = esl_getopts_CreateDefaultApp(options, 0, argc, argv, banner, usage);
+  double  mu           = esl_opt_GetReal(go, "-m");
+  double  lambda       = esl_opt_GetReal(go, "-l");
+  double  tau          = esl_opt_GetReal(go, "-t");
+  ESL_RANDOMNESS *r    = esl_randomness_Create(esl_opt_GetInteger(go, "-s"));
+  ESL_HISTOGRAM  *h    = esl_histogram_CreateFull(mu, 100., esl_opt_GetReal(go, "-w"));;
+  int     n            = esl_opt_GetInteger(go, "-n");
+  int     be_verbose   = esl_opt_GetBoolean(go, "-v");
+  char   *plotfile     = esl_opt_GetString(go, "-o");
+  FILE   *pfp          = stdout;
+  int     plot_pdf     = esl_opt_GetBoolean(go, "--P");
+  int     plot_logpdf  = esl_opt_GetBoolean(go, "--LP");
+  int     plot_cdf     = esl_opt_GetBoolean(go, "--C");
+  int     plot_logcdf  = esl_opt_GetBoolean(go, "--LC");
+  int     plot_surv    = esl_opt_GetBoolean(go, "--S");
+  int     plot_logsurv = esl_opt_GetBoolean(go, "--LS");
+  double  xmin         = esl_opt_IsOn(go, "--XL") ?  esl_opt_GetReal(go, "--XL") :  mu;
+  double  xmax         = esl_opt_IsOn(go, "--XH") ?  esl_opt_GetReal(go, "--XH") :  mu+40*(1./lambda);
+  double  xstep        = esl_opt_IsOn(go, "--XS") ?  esl_opt_GetReal(go, "--XS") :  0.1;
   double  emu, elambda, etau;
   int     i;
   double  x;
   double *data;
   int     ndata;
 
-  int     opti;
-  int     be_verbose   = FALSE;
-  char   *plotfile     = NULL;
-  FILE   *pfp          = stdout;
-  int     plot_pdf     = FALSE;
-  int     plot_logpdf  = FALSE;
-  int     plot_cdf     = FALSE;
-  int     plot_logcdf  = FALSE;
-  int     plot_surv    = FALSE;
-  int     plot_logsurv = FALSE;
-  int     xmin_set     = FALSE;
-  double  xmin;
-  int     xmax_set     = FALSE;
-  double  xmax;
-  int     xstep_set    = FALSE;
-  double  xstep;
-
-  for (opti = 1; opti < argc && *(argv[opti]) == '-'; opti++)
-    {
-      if      (strcmp(argv[opti], "-m")  == 0) mu           = atof(argv[++opti]);
-      else if (strcmp(argv[opti], "-l")  == 0) lambda       = atof(argv[++opti]);
-      else if (strcmp(argv[opti], "-n")  == 0) n            = atoi(argv[++opti]);
-      else if (strcmp(argv[opti], "-o")  == 0) plotfile     = argv[++opti];
-      else if (strcmp(argv[opti], "-t")  == 0) tau          = atof(argv[++opti]);
-      else if (strcmp(argv[opti], "-v")  == 0) be_verbose   = TRUE;
-      else if (strcmp(argv[opti], "-w")  == 0) binwidth     = atof(argv[++opti]);
-      else if (strcmp(argv[opti], "-C")  == 0) plot_cdf     = TRUE;
-      else if (strcmp(argv[opti], "-LC") == 0) plot_logcdf  = TRUE;
-      else if (strcmp(argv[opti], "-P")  == 0) plot_pdf     = TRUE;
-      else if (strcmp(argv[opti], "-LP") == 0) plot_logpdf  = TRUE;
-      else if (strcmp(argv[opti], "-S")  == 0) plot_surv    = TRUE;
-      else if (strcmp(argv[opti], "-LS") == 0) plot_logsurv = TRUE;
-      else if (strcmp(argv[opti], "-XL") == 0) { xmin_set  = TRUE; xmin  = atof(argv[++opti]); }
-      else if (strcmp(argv[opti], "-XH") == 0) { xmax_set  = TRUE; xmax  = atof(argv[++opti]); }
-      else if (strcmp(argv[opti], "-XS") == 0) { xstep_set = TRUE; xstep = atof(argv[++opti]); }
-      else ESL_EXCEPTION(eslEINVAL, "bad option");
-    }
-
   if (be_verbose)
     printf("Parametric:  mu = %f   lambda = %f    tau = %f\n", mu, lambda, tau);
 
-  r = esl_randomness_CreateTimeseeded();
-  h = esl_histogram_CreateFull(mu, 100., binwidth);
   if (plotfile != NULL) {
     if ((pfp = fopen(plotfile, "w")) == NULL) 
       ESL_EXCEPTION(eslFAIL, "Failed to open plotfile");
   }
-  if (! xmin_set)  xmin  = mu;
-  if (! xmax_set)  xmax  = mu+40*(1./lambda);
-  if (! xstep_set) xstep = 0.1;
 
   for (i = 0; i < n; i++)
     {
@@ -684,13 +663,21 @@ main(int argc, char **argv)
   if (plotfile != NULL) fclose(pfp);
   esl_histogram_Destroy(h);
   esl_randomness_Destroy(r);
+  esl_getopts_Destroy(go);
   return 0;
 }
 #endif /*eslSTRETCHEXP_TESTDRIVE*/
 
 
 /*****************************************************************
- *    This copyrighted source code is freely distributed 
- *    under the terms of the GNU General Public License. See
- *    the files COPYRIGHT and LICENSE for details.
+ * Easel - a library of C functions for biological sequence analysis
+ * Version i1.1.1; July 2014
+ * Copyright (C) 2014 HHMI Janelia Farm Research Campus
+ * Other copyrights also apply. See the COPYRIGHT file for a full list.
+ * 
+ * Easel is distributed under the Janelia Farm Software License, a BSD
+ * license. See the LICENSE file for more details.
+ *
+ * SVN $Id: esl_stretchexp.c 727 2011-10-24 17:17:32Z eddys $
+ * SVN $URL: https://svn.janelia.org/eddylab/eddys/easel/branches/infernal/1.1/esl_stretchexp.c $
  *****************************************************************/
