@@ -6,8 +6,6 @@
  * Contents:
  *   
  * 
- * SRE, Fri Apr 13 13:03:17 2007
- * SVN $Id: esl_swat.c 279 2008-07-12 13:17:06Z eddys $
  */
 #include "esl_config.h"
 
@@ -135,7 +133,7 @@ int
 main(int argc, char **argv)
 {
   int status;
-  ESL_RANDOMNESS  *r   = esl_randomness_CreateTimeseeded();
+  ESL_RANDOMNESS  *r   = esl_randomness_Create(0);
   ESL_ALPHABET    *abc = esl_alphabet_Create(eslAMINO);
   ESL_SCOREMATRIX *S   = NULL;
   ESL_DSQ         *x   = NULL;	/* iid query */
@@ -166,12 +164,12 @@ main(int argc, char **argv)
   /* Input an amino acid score matrix from a file. */
   if (mxfile != NULL) {
     ESL_FILEPARSER  *efp = NULL;
-    if ( esl_fileparser_Open(mxfile, &efp)  != eslOK) esl_fatal("failed to open score file %s", mxfile);
-    if ( esl_sco_Read(efp, abc, &S)         != eslOK) esl_fatal("failed to read matrix from %s", mxfile);
+    if ( esl_fileparser_Open(mxfile, NULL, &efp)  != eslOK) esl_fatal("failed to open score file %s", mxfile);
+    if ( esl_scorematrix_Read(efp, abc, &S)               != eslOK) esl_fatal("failed to read matrix from %s", mxfile);
     esl_fileparser_Close(efp);
   } else {			/* default = BLOSUM62 */
     S = esl_scorematrix_Create(abc);
-    esl_scorematrix_SetBLOSUM62(S);
+    esl_scorematrix_Set("BLOSUM62", S);
   }
   esl_composition_BL62(bg);
 
@@ -215,14 +213,18 @@ utest_Score(char *s1, char *s2, ESL_SCOREMATRIX *S, int gop, int gex, int expect
 
 
 
-/*****************************************************************
- *
- *****************************************************************/
 
 
 
 /*****************************************************************
- *    This copyrighted source code is freely distributed 
- *    under the terms of the GNU General Public License. See
- *    the files COPYRIGHT and LICENSE for details.
+ * Easel - a library of C functions for biological sequence analysis
+ * Version i1.1.1; July 2014
+ * Copyright (C) 2014 HHMI Janelia Farm Research Campus
+ * Other copyrights also apply. See the COPYRIGHT file for a full list.
+ * 
+ * Easel is distributed under the Janelia Farm Software License, a BSD
+ * license. See the LICENSE file for more details.
+ * 
+ * SVN $Id: esl_swat.c 685 2011-05-23 14:27:52Z eddys $
+ * SVN $URL: https://svn.janelia.org/eddylab/eddys/easel/branches/infernal/1.1/esl_swat.c $
  *****************************************************************/
