@@ -11,8 +11,6 @@
  *    4. Examples.
  *    5. Copyright and license information.
  * 
- * SRE, Tue Oct  1 15:23:25 2002 [St. Louis]
- * SVN $Id: esl_vectorops.c 241 2008-04-01 19:01:52Z eddys $
  */                      
 #include "esl_config.h"
 
@@ -253,21 +251,21 @@ esl_vec_ICopy(const int *src, const int n, int *dest)
  *            for float and integer vectors.
  */
 int
-esl_vec_DCompare(double *vec1, double *vec2, int n, double tol)
+esl_vec_DCompare(const double *vec1, const double *vec2, int n, double tol)
 {
   int i;
   for (i = 0; i < n; i++) if (esl_DCompare(vec1[i], vec2[i], tol) == eslFAIL) return eslFAIL;
   return eslOK;
 }
 int
-esl_vec_FCompare(float *vec1, float *vec2, int n, float tol)
+esl_vec_FCompare(const float *vec1, const float *vec2, int n, float tol)
 {
   int i;
   for (i = 0; i < n; i++) if (esl_DCompare(vec1[i], vec2[i], tol) == eslFAIL) return eslFAIL;
   return eslOK;
 }
 int
-esl_vec_ICompare(int *vec1, int *vec2, int n)
+esl_vec_ICompare(const int *vec1, const int *vec2, int n)
 {
   int i;
   for (i = 0; i < n; i++) if (vec1[i] != vec2[i]) return eslFAIL;
@@ -319,6 +317,63 @@ esl_vec_ISwap(int *vec1, int *vec2, int n)
 }
 
 
+/* Function:  esl_vec_DReverse()
+ * Synopsis:  Reverse a vector (possibly in place).
+ *
+ * Purpose:   Put the <n> values from vector <vec> in reversed order in
+ *            <rev>. Caller provides storage in <rev> for at least <n>
+ *            values.
+ *            
+ *            <vec> and <rev> can be the same, in which case <vec> is
+ *            reversed in place.
+ *            
+ *            <esl_vec_FReverse()> and <esl_vec_IReverse()>
+ *            do the same, for float and integer values.
+ */
+void
+esl_vec_DReverse(double *vec, double *rev, int n)
+{
+  int    i;
+  double x;
+
+  for (i = 0; i < n/2; i++)
+    {
+      x          = vec[n-i-1];
+      rev[n-i-1] = vec[i];
+      rev[i]     = x;
+    }
+  if (n%2) rev[i] = vec[i];
+}
+void
+esl_vec_FReverse(float *vec, float *rev, int n)
+{
+  int    i;
+  float  x;
+
+  for (i = 0; i < n/2; i++)
+    {
+      x          = vec[n-i-1];
+      rev[n-i-1] = vec[i];
+      rev[i]     = x;
+    }
+  if (n%2) rev[i] = vec[i];
+}
+void
+esl_vec_IReverse(int *vec, int *rev, int n)
+{
+  int i;
+  int x;
+
+  for (i = 0; i < n/2; i++)
+    {
+      x          = vec[n-i-1];
+      rev[n-i-1] = vec[i];
+      rev[i]     = x;
+    }
+  if (n%2) rev[i] = vec[i];
+}
+
+
 
 
 /* Function:  esl_vec_DDot()
@@ -367,7 +422,7 @@ esl_vec_IDot(int *vec1, int *vec2, int n)
  *            for float and integer vectors.
  */
 double
-esl_vec_DMax(double *vec, int n)
+esl_vec_DMax(const double *vec, int n)
 {
   int i;
   double best;
@@ -378,7 +433,7 @@ esl_vec_DMax(double *vec, int n)
   return best;
 }
 float
-esl_vec_FMax(float *vec, int n)
+esl_vec_FMax(const float *vec, int n)
 {
   int   i;
   float best;
@@ -389,7 +444,7 @@ esl_vec_FMax(float *vec, int n)
   return best;
 }
 int
-esl_vec_IMax(int *vec, int n)
+esl_vec_IMax(const int *vec, int n)
 {
   int   i;
   int   best;
@@ -411,7 +466,7 @@ esl_vec_IMax(int *vec, int n)
  *            for float and integer vectors.
  */
 double
-esl_vec_DMin(double *vec, int n)
+esl_vec_DMin(const double *vec, int n)
 {
   int i;
   double best;
@@ -422,7 +477,7 @@ esl_vec_DMin(double *vec, int n)
   return best;
 }
 float
-esl_vec_FMin(float *vec, int n)
+esl_vec_FMin(const float *vec, int n)
 {
   int   i;
   float best;
@@ -433,7 +488,7 @@ esl_vec_FMin(float *vec, int n)
   return best;
 }
 int
-esl_vec_IMin(int *vec, int n)
+esl_vec_IMin(const int *vec, int n)
 {
   int   i;
   int   best;
@@ -449,13 +504,18 @@ esl_vec_IMin(int *vec, int n)
  * Synopsis:  Return index of maximum element in a vector.           
  *
  * Purpose:   Returns the index of the maximum value in the <n> values
- *            in <vec>.
+ *            in <vec>. In case of ties, the element with the smallest index
+ *            is returned. 
  *            
  *            <esl_vec_FArgMax()> and <esl_vec_IArgMax()> do the same,
  *            for float and integer vectors.
+ *            
+ * Note:      Do not change the behavior that the smallest index is
+ *            returned in case of ties. Some functions rely on this
+ *            behavior: optimal accuracy tracebacks in HMMER for example.           
  */
 int
-esl_vec_DArgMax(double *vec, int n)
+esl_vec_DArgMax(const double *vec, int n)
 {
   int i;
   int best = 0;
@@ -465,7 +525,7 @@ esl_vec_DArgMax(double *vec, int n)
   return best;
 }
 int
-esl_vec_FArgMax(float *vec, int n)
+esl_vec_FArgMax(const float *vec, int n)
 {
   int i;
   int best = 0;
@@ -475,7 +535,7 @@ esl_vec_FArgMax(float *vec, int n)
   return best;
 }
 int
-esl_vec_IArgMax(int *vec, int n)
+esl_vec_IArgMax(const int *vec, int n)
 {
   int i;
   int best = 0;
@@ -496,7 +556,7 @@ esl_vec_IArgMax(int *vec, int n)
  *            for float and integer vectors.
  */
 int
-esl_vec_DArgMin(double *vec, int n)
+esl_vec_DArgMin(const double *vec, int n)
 {
   int i;
   int best = 0;
@@ -505,7 +565,7 @@ esl_vec_DArgMin(double *vec, int n)
   return best;
 }
 int
-esl_vec_FArgMin(float *vec, int n)
+esl_vec_FArgMin(const float *vec, int n)
 {
   int   i;
   int   best = 0;
@@ -515,7 +575,7 @@ esl_vec_FArgMin(float *vec, int n)
   return best;
 }
 int
-esl_vec_IArgMin(int *vec, int n)
+esl_vec_IArgMin(const int *vec, int n)
 {
   int   i;
   int   best = 0;
@@ -805,7 +865,7 @@ esl_vec_FLog(float *vec, int n)
 {
   int x;
   for (x = 0; x < n; x++) 
-    if (vec[x] > 0.) vec[x] = log(vec[x]);
+    if (vec[x] > 0.) vec[x] = logf(vec[x]);
     else vec[x] = -FLT_MAX;
 }
 
@@ -817,7 +877,7 @@ esl_vec_FLog(float *vec, int n)
  *            in bits ($\log_2$), defined as \citep{CoverThomas}:
  *            
  *            \[
- *               H = \sum_x p_x \log_2 p_x.
+ *               H = - \sum_x p_x \log_2 p_x.
  *            \]
  *
  *            <esl_vec_FEntropy()> does the same, for a probability vector
@@ -842,7 +902,7 @@ esl_vec_FEntropy(const float *p, int n)
 
   entropy = 0.;
   for(i = 0; i < n; i++)
-    if (p[i] > 0.) entropy += p[i] * log(p[i]);
+    if (p[i] > 0.) entropy += p[i] * logf(p[i]);
   return(-1.44269504 * entropy); /* converts to bits */
 }
 
@@ -900,6 +960,11 @@ esl_vec_FRelEntropy(const float *p, const float *q, int n)
  * Purpose:   Converts a log probability vector <vec> back to a 
  *            probability vector: exponentiates each of the <n> 
  *            values in the vector.
+ *            
+ *            This routine only calls <exp()> on the elements of 
+ *            vector, which are presumed to be log probabilities;
+ *            whether the resulting vector is a properly normalized
+ *            probability vector is the caller's problem.
  *
  *            <esl_vec_FExp()> does the same, for a log probability vector
  *            of floats.
@@ -909,14 +974,12 @@ esl_vec_DExp(double *vec, int n)
 {
   int x;
   for (x = 0; x < n; x++) vec[x] = exp(vec[x]);
-  esl_vec_DNorm(vec, n);
 }
 void
 esl_vec_FExp(float *vec, int n)
 {
   int x;
-  for (x = 0; x < n; x++) vec[x] = exp(vec[x]);
-  esl_vec_FNorm(vec, n);
+  for (x = 0; x < n; x++) vec[x] = expf(vec[x]);
 }
 
 /* Function:  esl_vec_DLogSum()
@@ -960,8 +1023,8 @@ esl_vec_FLogSum(float *vec, int n)
   sum = 0.0;
   for (x = 0; x < n; x++)
     if (vec[x] > max - 50.)
-      sum += exp(vec[x] - max);
-  sum = log(sum) + max;
+      sum += expf(vec[x] - max);
+  sum = logf(sum) + max;
   return sum;
 }
 
@@ -986,7 +1049,8 @@ esl_vec_DLogNorm(double *vec, int n)
   
   denom = esl_vec_DLogSum(vec, n);
   esl_vec_DIncrement(vec, n, -1.*denom);
-  esl_vec_DExp(vec, n);
+  esl_vec_DExp (vec, n);
+  esl_vec_DNorm(vec, n);
 }
 void
 esl_vec_FLogNorm(float *vec, int n)
@@ -995,9 +1059,58 @@ esl_vec_FLogNorm(float *vec, int n)
   
   denom = esl_vec_FLogSum(vec, n);
   esl_vec_FIncrement(vec, n, -1.*denom);
-  esl_vec_FExp(vec, n);
+  esl_vec_FExp (vec, n);
+  esl_vec_FNorm(vec, n);
 }
 
+
+/* Function:  esl_vec_DCDF()
+ * Synopsis:  Calculate cumulative distribution for a discrete prob vector
+ * Incept:    SRE, Wed Jan 12 09:09:42 2011 [Janelia]
+ *
+ * Purpose:   Given a probability vector <p> of length <n>, 
+ *            calculates its cumulate distribution function
+ *            and puts in in caller-allocated space <cdf>.
+ *            Caller must have allocated <cdf> for at least
+ *            <n> elements.
+ *            
+ *            By definition, <cdf[0] == p[0]>, and <cdf[n-1]> ought to
+ *            be 1.0; however, numerical roundoff error must be tolerated
+ *            in the sum. If caller isn't sure about <p>'s provenance,
+ *            it may want to check that <cdf[n-1]> is tolerably close 
+ *            to 1.0 (see <esl_DCompare()>).
+ *
+ *            It is ok for <cdf> to be the same space as <p>
+ *            (<esl_vec_DCDF(p, n, p)> is fine); that is, <p> can be
+ *            overwritten by <cdf>.
+ *
+ * Args:      p    - input probability vector p[0..n-1]
+ *            n    - number of elements in p
+ *            cdf  - RETURN: cumulative distribution for p, in caller-allocated space
+ *
+ * Returns:   (void).
+ */
+void
+esl_vec_DCDF(double *p, int n, double *cdf)
+{
+  int i;
+ 
+  cdf[0] = p[0];
+  for (i = 1; i < n; i++) 
+    cdf[i] = p[i] + cdf[i-1];
+}
+void
+esl_vec_FCDF(float *p, int n, float *cdf)
+{
+  int i;
+ 
+  cdf[0] = p[0];
+  for (i = 1; i < n; i++) 
+    cdf[i] = p[i] + cdf[i-1];
+}
+
+
+
 /* Function:  esl_vec_DValidate()
  * Synopsis:  Verifies that vector is p-vector.
  * Incept:    ER, Tue Dec  5 09:38:54 EST 2006 [janelia]
@@ -1124,6 +1237,58 @@ esl_vec_FLogValidate(float *vec, int n, float tol, char *errbuf)
   return eslOK;
 }
 
+#ifdef eslAUGMENT_RANDOM
+#include "esl_random.h"
+
+/* Function:  esl_vec_DShuffle()
+ * Synopsis:  Shuffle a vector, in place.
+ *
+ * Purpose:   Shuffle a vector <v> of <n> items, using the
+ *            random number generator <r>.
+ */
+int
+esl_vec_DShuffle(ESL_RANDOMNESS *r, double *v, int n)
+{
+  double swap;
+  int    pos;
+  for ( ; n > 1; n--)
+    {
+      pos = esl_rnd_Roll(r, n);
+      swap = v[pos]; 
+      v[pos] = v[n-1];
+      v[n-1] = swap;
+    }
+  return eslOK;
+}
+int
+esl_vec_FShuffle(ESL_RANDOMNESS *r, float *v, int n)
+{
+  float swap;
+  int   pos;
+  for ( ; n > 1; n--)
+    {
+      pos = esl_rnd_Roll(r, n);
+      swap = v[pos]; 
+      v[pos] = v[n-1];
+      v[n-1] = swap;
+    }
+  return eslOK;
+}
+int
+esl_vec_IShuffle(ESL_RANDOMNESS *r, int *v, int n)
+{
+  int swap;
+  int pos;
+  for ( ; n > 1; n--)
+    {
+      pos = esl_rnd_Roll(r, n);
+      swap = v[pos]; 
+      v[pos] = v[n-1];
+      v[n-1] = swap;
+    }
+  return eslOK;
+}
+#endif /*eslAUGMENT_RANDOM*/
 
 
 /*****************************************************************
@@ -1241,7 +1406,14 @@ int main(void)
 #endif /*eslVECTOROPS_EXAMPLE*/
 
 /*****************************************************************  
- *    This copyrighted source code is freely distributed 
- *    under the terms of the GNU General Public License. See
- *    the files COPYRIGHT and LICENSE for details.
+ * Easel - a library of C functions for biological sequence analysis
+ * Version i1.1.1; July 2014
+ * Copyright (C) 2014 HHMI Janelia Farm Research Campus
+ * Other copyrights also apply. See the COPYRIGHT file for a full list.
+ * 
+ * Easel is distributed under the Janelia Farm Software License, a BSD
+ * license. See the LICENSE file for more details.
+ *
+ * SVN $Id: esl_vectorops.c 762 2012-05-11 21:01:15Z eddys $
+ * SVN $URL: https://svn.janelia.org/eddylab/eddys/easel/branches/infernal/1.1/esl_vectorops.c $
  *****************************************************************/
