@@ -9,8 +9,6 @@
  *     6. Example
  *     7. Copyright and license
  *     
- * SRE, Sun Dec 16 09:14:51 2007 [Janelia]    
- * SVN $Id: esl_vmx.c 231 2008-03-25 14:43:57Z eddys $
  *****************************************************************
  * Credits:
  *
@@ -30,7 +28,9 @@
 #include <math.h>
 #include <float.h>
 
+#ifndef __APPLE_ALTIVEC__
 #include <altivec.h>
+#endif
 
 #include "easel.h"
 #include "esl_vmx.h"
@@ -71,11 +71,13 @@
 vector float 
 esl_vmx_logf(vector float x) 
 {
-  static float cephes_p[9] = {  7.0376836292E-2f, -1.1514610310E-1f,  1.1676998740E-1f,
-				-1.2420140846E-1f, 1.4249322787E-1f, -1.6668057665E-1f,
-				2.0000714765E-1f, -2.4999993993E-1f,  3.3333331174E-1f };
+  static vector float cephesv_1 = { 7.0376836292E-2f, -1.1514610310E-1f,  1.1676998740E-1f, -1.2420140846E-1f };
+  static vector float cephesv_2 = { 1.4249322787E-1f, -1.6668057665E-1f,  2.0000714765E-1f, -2.4999993993E-1f };
+  static vector float cephesv_3 = { 3.3333331174E-1f,  0.0f,              0.0f,              0.0f             };
+
+  static vector float constv = { 0.707106781186547524f, -2.12194440e-4f, 0.5f, 0.693359375f };
+
   vector float onev = (vector float) {1.0, 1.0, 1.0, 1.0}; /* all elem = 1.0 */
-  vector float v0p5 = (vector float) {0.5, 0.5, 0.5, 0.5}; /* all elem = 0.5 */
   vector signed int ei;
   vector float e;
   vector bool int invalid_mask, zero_mask, inf_mask;            /* masks used to handle special IEEE754 inputs */
@@ -85,52 +87,52 @@ esl_vmx_logf(vector float x)
   vector float y;
   vector float z;
 
+  vector float zerov = (vector float) vec_splat_u32(0);
+  vector signed int infExpv = { 255, 255, 255, 255 };  
+
   /* first, split x apart: x = frexpf(x, &e); */
   ei           = vec_sr((vector signed int) x, ((vector unsigned int) {23, 23, 23, 23}));
 							             /* shift right 23: IEEE754 floats: ei = biased exponents     */
-  invalid_mask = vec_cmple(x, ((vector float) {0.0, 0.0, 0.0, 0.0}));  /* mask any elem that's negative; these become NaN           */
-  zero_mask    = vec_cmpeq(ei,((vector signed int) {0, 0, 0, 0}));
-								     /* mask any elem zero or subnormal; these become -inf        */
-  inf_mask     = vec_cmpeq(ei,((vector signed int) {255, 255, 255, 255}));
-								     /* mask any elem +inf or NaN; these stay +inf or NaN         */
+  invalid_mask = vec_cmple(x, zerov);                                /* mask any elem that's negative; these become NaN           */
+  zero_mask    = vec_cmpeq(ei,(vector signed int) zerov);            /* mask any elem zero or subnormal; these become -inf        */
+  inf_mask     = vec_cmpeq(ei, infExpv);			     /* mask any elem +inf or NaN; these stay +inf or NaN         */
   origx        = x;			                             /* store original x, used for log(inf) = inf, log(NaN) = NaN */
 
   x  = vec_and(x, (vector float) ((vector unsigned int) {~0x7f800000, ~0x7f800000, ~0x7f800000, ~0x7f800000}));
 						                     /* x now the stored 23 bits of the 24-bit significand        */
-  x  = vec_or (x, v0p5);                                             /* sets hidden bit b[0]                                      */
+  x  = vec_or (x, vec_splat(constv, 2));                             /* sets hidden bit b[0]                                      */
 
-  ei = vec_sub(ei, ((vector signed int) {126, 126, 126, 126}));        /* -127 (ei now signed base-2 exponent); then +1             */
+  ei = vec_sub(ei, ((vector signed int) {126, 126, 126, 126}));      /* -127 (ei now signed base-2 exponent); then +1             */
   e  = vec_ctf(ei, 0);
 
   /* now, calculate the log */
-  mask = vec_cmplt(x, ((vector float) {0.707106781186547524f, 
-         0.707106781186547524f, 0.707106781186547524f, 0.707106781186547524f})); /* avoid conditional branches.           */
-  tmp  = vec_and(x, (vector float) mask);                      /* tmp contains x values < 0.707, else 0 */
+  mask = vec_cmplt(x, vec_splat(constv, 0)); /* avoid conditional branches.           */
+  tmp  = vec_and(x, (vector float) mask);    /* tmp contains x values < 0.707, else 0 */
   x    = vec_sub(x, onev);
   e    = vec_sub(e, vec_and(onev, (vector float) mask));
   x    = vec_add(x, tmp);
-  z    = vec_madd(x, x, ((vector float) {0.0, 0.0, 0.0, 0.0}));
+  z    = vec_madd(x, x, zerov);
 
-  y =            ((vector float) {cephes_p[0], cephes_p[0], cephes_p[0], cephes_p[0]});    y = vec_madd(y, x, ((vector float) {0.0, 0.0, 0.0, 0.0})); 
-  y = vec_add(y, ((vector float) {cephes_p[1], cephes_p[1], cephes_p[1], cephes_p[1]}));   y = vec_madd(y, x, ((vector float) {0.0, 0.0, 0.0, 0.0}));    
-  y = vec_add(y, ((vector float) {cephes_p[2], cephes_p[2], cephes_p[2], cephes_p[2]}));   y = vec_madd(y, x, ((vector float) {0.0, 0.0, 0.0, 0.0}));   
-  y = vec_add(y, ((vector float) {cephes_p[3], cephes_p[3], cephes_p[3], cephes_p[3]}));   y = vec_madd(y, x, ((vector float) {0.0, 0.0, 0.0, 0.0}));   
-  y = vec_add(y, ((vector float) {cephes_p[4], cephes_p[4], cephes_p[4], cephes_p[4]}));   y = vec_madd(y, x, ((vector float) {0.0, 0.0, 0.0, 0.0}));    
-  y = vec_add(y, ((vector float) {cephes_p[5], cephes_p[5], cephes_p[5], cephes_p[5]}));   y = vec_madd(y, x, ((vector float) {0.0, 0.0, 0.0, 0.0}));   
-  y = vec_add(y, ((vector float) {cephes_p[6], cephes_p[6], cephes_p[6], cephes_p[6]}));   y = vec_madd(y, x, ((vector float) {0.0, 0.0, 0.0, 0.0})); 
-  y = vec_add(y, ((vector float) {cephes_p[7], cephes_p[7], cephes_p[7], cephes_p[7]}));   y = vec_madd(y, x, ((vector float) {0.0, 0.0, 0.0, 0.0}));  
-  y = vec_add(y, ((vector float) {cephes_p[8], cephes_p[8], cephes_p[8], cephes_p[8]}));   y = vec_madd(y, x, ((vector float) {0.0, 0.0, 0.0, 0.0}));
-  y = vec_madd(y, z, ((vector float) {0.0, 0.0, 0.0, 0.0}));
+  y =                vec_splat(cephesv_1, 0);
+  y = vec_madd(y, x, vec_splat(cephesv_1, 1)); 
+  y = vec_madd(y, x, vec_splat(cephesv_1, 2));    
+  y = vec_madd(y, x, vec_splat(cephesv_1, 3));   
+  y = vec_madd(y, x, vec_splat(cephesv_2, 0));   
+  y = vec_madd(y, x, vec_splat(cephesv_2, 1));    
+  y = vec_madd(y, x, vec_splat(cephesv_2, 2));   
+  y = vec_madd(y, x, vec_splat(cephesv_2, 3)); 
+  y = vec_madd(y, x, vec_splat(cephesv_3, 0));  
+  y = vec_madd(y, x, zerov);
+  y = vec_madd(y, z, zerov);
 
-  tmp = vec_madd(e, ((vector float) {-2.12194440e-4f, -2.12194440e-4f, -2.12194440e-4f, -2.12194440e-4f}), ((vector float) {0.0, 0.0, 0.0, 0.0}));
+  tmp = vec_madd(e, vec_splat(constv, 1), zerov);
   y   = vec_add(y, tmp);
 
-  tmp = vec_madd(z, v0p5, ((vector float) {0.0, 0.0, 0.0, 0.0}));
+  tmp = vec_madd(z, vec_splat(constv, 2), zerov);
   y   = vec_sub(y, tmp);
 
-  tmp = vec_madd(e, ((vector float) {0.693359375f, 0.693359375f, 0.693359375f, 0.693359375f}), ((vector float) {0.0, 0.0, 0.0, 0.0}));
   x = vec_add(x, y);
-  x = vec_add(x, tmp);
+  x = vec_madd(e, vec_splat(constv, 3), x);
 
   /* IEEE754 cleanup: */
   x = vec_or(x, (vector float) invalid_mask);               /* log(x<0, including -0) = NaN  */
@@ -156,20 +158,21 @@ esl_vmx_logf(vector float x)
 vector float
 esl_vmx_expf(vector float x) 
 {
-  static float cephes_p[6] = { 1.9875691500E-4f, 1.3981999507E-3f, 8.3334519073E-3f, 
-			       4.1665795894E-2f, 1.6666665459E-1f, 5.0000001201E-1f };
-  static float cephes_c[2] = { 0.693359375f,    -2.12194440e-4f };
-  static float maxlogf     =   88.72283905206835;  /* log(2^128)  */
-  static float minlogf     = -103.27892990343185;  /* log(2^-149) */
+  static vector float cephesv_p1 = { 1.9875691500E-4f, 1.3981999507E-3f, 8.3334519073E-3f, 4.1665795894E-2f };
+  static vector float cephesv_p2 = { 1.6666665459E-1f, 5.0000001201E-1f, 0.693359375f,    -2.12194440E-4f   };
+
+  static vector float maxlogfv = {   88.72283905206835f,   88.72283905206835f,   88.72283905206835f,   88.72283905206835f };  /* log(2^128)  */
+  static vector float minlogfv = { -103.27892990343185f, -103.27892990343185f, -103.27892990343185f, -103.27892990343185f };  /* log(2^-149) */
+
   vector signed int k;
   vector bool int minmask, maxmask;
   vector float tmp, fx, y, z;
 
-  vector float zerov = { 0.0, 0.0, 0.0, 0.0};
+  vector float zerov = (vector float) vec_splat_u32(0);
   
   /* handle out-of-range and special conditions */
-  maxmask = vec_cmpgt(x, ((vector float) {maxlogf, maxlogf, maxlogf, maxlogf}));
-  minmask = vec_cmple(x, ((vector float) {minlogf, minlogf, minlogf, minlogf}));
+  maxmask = vec_cmpgt(x, maxlogfv);
+  minmask = vec_cmple(x, minlogfv);
 
   /* range reduction: exp(x) = 2^k e^f = exp(f + k log 2); k = floorf(0.5 + x / log2): */
   fx = vec_madd(x, ((vector float) {eslCONST_LOG2R, eslCONST_LOG2R, eslCONST_LOG2R, eslCONST_LOG2R}), zerov);
@@ -180,19 +183,19 @@ esl_vmx_expf(vector float x)
   k  = vec_cts(fx, 0);
   
   /* polynomial approx for e^f for f in range [-0.5, 0.5] */
-  tmp = vec_madd(fx, ((vector float) {cephes_c[0], cephes_c[0], cephes_c[0], cephes_c[0]}), zerov);
-  z   = vec_madd(fx, ((vector float) {cephes_c[1], cephes_c[1], cephes_c[1], cephes_c[1]}), zerov);
+  tmp = vec_madd(fx, vec_splat(cephesv_p2, 2), zerov);
+  z   = vec_madd(fx, vec_splat(cephesv_p2, 3), zerov);
   x   = vec_sub(x, tmp);
   x   = vec_sub(x, z);
   z   = vec_madd(x, x, zerov);
   
-  y =            ((vector float) {cephes_p[0], cephes_p[0], cephes_p[0], cephes_p[0]});    y = vec_madd(y, x, zerov);
-  y = vec_add(y, ((vector float) {cephes_p[1], cephes_p[1], cephes_p[1], cephes_p[1]}));   y = vec_madd(y, x, zerov);
-  y = vec_add(y, ((vector float) {cephes_p[2], cephes_p[2], cephes_p[2], cephes_p[2]}));   y = vec_madd(y, x, zerov);
-  y = vec_add(y, ((vector float) {cephes_p[3], cephes_p[3], cephes_p[3], cephes_p[3]}));   y = vec_madd(y, x, zerov);
-  y = vec_add(y, ((vector float) {cephes_p[4], cephes_p[4], cephes_p[4], cephes_p[4]}));   y = vec_madd(y, x, zerov);
-  y = vec_add(y, ((vector float) {cephes_p[5], cephes_p[5], cephes_p[5], cephes_p[5]}));   y = vec_madd(y, z, zerov);
-  y = vec_add(y, x);
+  y  =                vec_splat(cephesv_p1, 0);
+  y  = vec_madd(y, x, vec_splat(cephesv_p1, 1));
+  y  = vec_madd(y, x, vec_splat(cephesv_p1, 2));
+  y  = vec_madd(y, x, vec_splat(cephesv_p1, 3));
+  y  = vec_madd(y, x, vec_splat(cephesv_p2, 0));
+  y  = vec_madd(y, x, vec_splat(cephesv_p2, 1));
+  y  = vec_madd(y, z, x);
   y = vec_add(y, ((vector float) {1.0, 1.0, 1.0, 1.0}));
 
   /* build 2^k by hand, by creating a IEEE754 float */
@@ -227,7 +230,7 @@ esl_vmx_dump_vecfloat(FILE *fp, vector float v)
  *****************************************************************/
 #ifdef eslVMX_BENCHMARK
 
-/* gcc -msse2 -O3 -o benchmark-sse -I ~/src/hmmer/easel -L ~/src/hmmer/easel -DeslVMX_BENCHMARK -DHAVE_VMX esl_sse.c -leasel -lm
+/* gcc -maltivec -O3 -o vmx_benchmark -I . -L . -DeslVMX_BENCHMARK -DHAVE_VMX esl_vmx.c -leasel -lm
  */
 #include "esl_config.h"
 
@@ -255,7 +258,7 @@ main(int argc, char **argv)
   int             N       = esl_opt_GetInteger(go, "-N");
   float           origx   = 2.0;
   float           x       = origx;
-  vector float    xv      = (vector float) (x);
+  vector float    xv      = { 2.0f, 2.0f, 2.0f, 2.0f };
   int             i;
 
   /* First, serial time. */
@@ -422,7 +425,7 @@ utest_odds(ESL_GETOPTS *go, ESL_RANDOMNESS *r)
  *****************************************************************/
 
 #ifdef eslVMX_TESTDRIVE
-/* gcc -maltivec -g -Wall -o test -I. -L. -DeslVMX_TESTDRIVE -DHAVE_VMX esl_vmx.c -leasel -lm
+/* gcc -g -Wall -maltivec -o vmx_utest -I. -L. -DeslVMX_TESTDRIVE esl_vmx.c -leasel -lm
  */
 #include "esl_config.h"
 
@@ -438,7 +441,6 @@ static ESL_OPTIONS options[] = {
   /* name           type      default  env  range toggles reqs incomp  help                                       docgroup*/
   { "-h",        eslARG_NONE,   FALSE,  NULL, NULL,  NULL,  NULL, NULL, "show brief help on version and usage",             0 },
   { "-N",        eslARG_INT,  "10000",  NULL, NULL,  NULL,  NULL, NULL, "number of random test points",                     0 },
-  { "-r",        eslARG_NONE,   NULL,   NULL, NULL,  NULL,  NULL, NULL, "use arbitrary random number seed",                 0 },
   { "-s",        eslARG_INT,     "42",  NULL, NULL,  NULL,  NULL, NULL, "set random number seed to <n>",                    0 },
   { "-v",        eslARG_NONE,   FALSE,  NULL, NULL,  NULL,  NULL, NULL, "be verbose: show test report",                     0 },
   { "--vv",      eslARG_NONE,   FALSE,  NULL, NULL,  NULL,  NULL, NULL, "be very verbose: show individual test samples",    0 },
@@ -451,11 +453,8 @@ static char banner[] = "test driver for vmx module";
 int
 main(int argc, char **argv)
 {
-  ESL_GETOPTS    *go      = esl_getopts_CreateDefaultApp(options, 0, argc, argv, banner, usage);
-  ESL_RANDOMNESS *r;
-
-  if (esl_opt_GetBoolean(go, "-r")) r = esl_randomness_CreateTimeseeded();
-  else                              r = esl_randomness_Create(esl_opt_GetInteger(go, "-s"));
+  ESL_GETOPTS    *go = esl_getopts_CreateDefaultApp(options, 0, argc, argv, banner, usage);
+  ESL_RANDOMNESS *r  = esl_randomness_Create(esl_opt_GetInteger(go, "-s"));
 
   utest_logf(go);
   utest_expf(go);
@@ -475,8 +474,8 @@ main(int argc, char **argv)
  *****************************************************************/
 
 #ifdef eslVMX_EXAMPLE
-/*::cexcerpt::sse_example::begin::*/
-/* gcc -msse2 -g -Wall -o example -I. -L. -DeslVMX_EXAMPLE esl_sse.c -leasel -lm
+/*::cexcerpt::vmx_example::begin::*/
+/* gcc -msse2 -g -Wall -o vmx_example -I. -L. -DeslVMX_EXAMPLE esl_vmx.c -leasel -lm
  */
 #include "esl_config.h"
 
@@ -503,15 +502,36 @@ main(int argc, char **argv)
 
   return 0;
 }
-/*::cexcerpt::sse_example::end::*/
+/*::cexcerpt::vmx_example::end::*/
 #endif /*eslVMX_EXAMPLE*/
 #endif /*HAVE_VMX*/
 
+#ifndef HAVE_VMX
+
+/* If we don't have VMX compiled in, provide some nothingness to:
+ *   a. prevent Mac OS/X ranlib from bitching about .o file that "has no symbols" 
+ *   b. prevent compiler from bitching about "empty compilation unit"
+ *   c. automatically pass the automated tests.
+ */
+#include "easel.h"
+
+void esl_vmx_DoAbsolutelyNothing(void) { return; }
+#if defined eslVMX_TESTDRIVE || defined eslVMX_EXAMPLE || eslVMX_BENCHMARK
+int main(void) { return 0; }
+#endif
+#endif
 
 /*****************************************************************
- *    This copyrighted source code is freely distributed 
- *    under the terms of the GNU General Public License. See
- *    the files COPYRIGHT and LICENSE for details.
+ * Easel - a library of C functions for biological sequence analysis
+ * Version i1.1.1; July 2014
+ * Copyright (C) 2014 HHMI Janelia Farm Research Campus
+ * Other copyrights also apply. See the COPYRIGHT file for a full list.
+ * 
+ * Easel is distributed under the Janelia Farm Software License, a BSD
+ * license. See the LICENSE file for more details.
+ * 
+ * SVN $Id: esl_vmx.c 685 2011-05-23 14:27:52Z eddys $
+ * SVN $URL: https://svn.janelia.org/eddylab/eddys/easel/branches/infernal/1.1/esl_vmx.c $
  *****************************************************************/
 
 /* Additionally, esl_sse_logf() and esl_sse_expf() are 
