@@ -1,11 +1,11 @@
-/* esl_weibull.h
- * Weibull distributions.
+/* Weibull distributions.
  * 
  * SRE, Tue Aug  9 10:48:35 2005 [St. Louis]
- * SVN $Id: esl_weibull.h 83 2005-12-13 20:54:07Z eddy $
+ * SVN $Id: esl_weibull.h 664 2011-02-27 17:08:36Z eddys $
+ * SVN $URL: https://svn.janelia.org/eddylab/eddys/easel/branches/infernal/1.1/esl_weibull.h $
  */
-#ifndef ESL_WEI_INCLUDED
-#define ESL_WEI_INCLUDED
+#ifndef eslWEIBULL_INCLUDED
+#define eslWEIBULL_INCLUDED
 
 #ifdef eslAUGMENT_RANDOM
 #include <esl_random.h>
@@ -47,9 +47,13 @@ extern int esl_wei_FitCompleteBinned(ESL_HISTOGRAM *h, double *ret_mu,
 #endif /*eslAUGMENT_MINIMIZER*/
 
 
-#endif /*ESL_WEI_INCLUDED*/
+#endif /*eslWEIBULL_INCLUDED*/
 /*****************************************************************
- *    This copyrighted source code is freely distributed 
- *    under the terms of the GNU General Public License. See
- *    the files COPYRIGHT and LICENSE for details.
+ * Easel - a library of C functions for biological sequence analysis
+ * Version i1.1.1; July 2014
+ * Copyright (C) 2014 HHMI Janelia Farm Research Campus
+ * Other copyrights also apply. See the COPYRIGHT file for a full list.
+ * 
+ * Easel is distributed under the Janelia Farm Software License, a BSD
+ * license. See the LICENSE file for more details.
  *****************************************************************/
