@@ -1,12 +1,6 @@
-/* wuss.c
- * RNA secondary structure markup in WUSS notation.
- * 
- * SRE, Tue Feb 15 08:43:23 2005
- * SVN $Id: esl_wuss.c 241 2008-04-01 19:01:52Z eddys $
- * xref squid wuss.c.
+/* RNA secondary structure markup in WUSS notation.
  */
-
-#include <esl_config.h>
+#include "esl_config.h"
 
 #include <string.h>
 #include <ctype.h>
@@ -510,7 +504,14 @@ main(int argc, char **argv)
 
 
 /*****************************************************************
- *    This copyrighted source code is freely distributed 
- *    under the terms of the GNU General Public License. See
- *    the files COPYRIGHT and LICENSE for details.
+ * Easel - a library of C functions for biological sequence analysis
+ * Version i1.1.1; July 2014
+ * Copyright (C) 2014 HHMI Janelia Farm Research Campus
+ * Other copyrights also apply. See the COPYRIGHT file for a full list.
+ * 
+ * Easel is distributed under the Janelia Farm Software License, a BSD
+ * license. See the LICENSE file for more details.
+ * 
+ * SVN $Id: esl_wuss.c 685 2011-05-23 14:27:52Z eddys $
+ * SVN $URL: https://svn.janelia.org/eddylab/eddys/easel/branches/infernal/1.1/esl_wuss.c $
  *****************************************************************/
