@@ -1,7 +1,7 @@
 /* Select <n> random lines from a file and print them.
  *
  * SRE, Fri Jun 13 08:17:53 2008 [Janelia]
- * SVN $Id$
+ * SVN $Id: esl-selectn.c 526 2010-02-22 12:45:09Z eddys $
  */
 #include "esl_config.h"
 
@@ -16,7 +16,7 @@
 static ESL_OPTIONS options[] = {
   /* name           type      default  env  range toggles reqs incomp  help                                       docgroup*/
   { "-h",        eslARG_NONE,   FALSE,  NULL, NULL,  NULL,  NULL, NULL, "show brief help on version and usage",             1 },
-  { "--seed",    eslARG_INT,    FALSE,  NULL, NULL,  NULL,  NULL, NULL, "set random number generator's seed to <n>",        1 },
+  { "--seed",    eslARG_INT,      "0",  NULL, NULL,  NULL,  NULL, NULL, "set random number generator's seed to <n>",        1 },
   {  0, 0, 0, 0, 0, 0, 0, 0, 0, 0 },
 };
 static char usage[]  = "[-options] <n> <file>";
@@ -70,8 +70,7 @@ main(int argc, char **argv)
   nselect  = atoi(esl_opt_GetArg(go, 1));
   filename = esl_opt_GetArg(go, 2);
 
-  if (esl_opt_IsDefault(go, "--seed")) r = esl_randomness_CreateTimeseeded();
-  else                                 r = esl_randomness_Create(esl_opt_GetInteger(go, "--seed"));
+  r = esl_randomness_Create(esl_opt_GetInteger(go, "--seed"));
 
   if ((larr = malloc(sizeof(char *) * nselect)) == NULL) esl_fatal("allocation failed");
 
@@ -110,7 +109,11 @@ main(int argc, char **argv)
 }
 
 /*****************************************************************
- *    This copyrighted source code is freely distributed 
- *    under the terms of the GNU General Public License. See
- *    the files COPYRIGHT and LICENSE for details.
+ * Easel - a library of C functions for biological sequence analysis
+ * Version i1.1.1; July 2014
+ * Copyright (C) 2014 HHMI Janelia Farm Research Campus
+ * Other copyrights also apply. See the COPYRIGHT file for a full list.
+ * 
+ * Easel is distributed under the Janelia Farm Software License, a BSD
+ * license. See the LICENSE file for more details.
  *****************************************************************/ 
