@@ -2,7 +2,7 @@
  * 
  * From squid's sfetch and ffetch
  * SRE, Mon Mar 31 16:12:50 2008 [Janelia] 
- * SVN $Id: esl-sfetch.c 413 2009-10-21 17:17:30Z nawrockie $
+ * SVN $Id: esl-sfetch.c 769 2012-06-05 17:50:33Z nawrockie $
  */
 #include "esl_config.h"
 
@@ -77,10 +77,6 @@ static ESL_OPTIONS options[] = {
   { "-C",          eslARG_NONE,   FALSE,  NULL, NULL, NULL, "-f",              "--index",            "<namefile> in <f> contains subseq coords too",      2 },
 
   { "--informat",  eslARG_STRING, FALSE,  NULL, NULL, NULL, NULL,              NULL,                 "specify that input file is in format <s>",          3 },
-  { "--tabfile",   eslARG_NONE,   FALSE,  NULL, NULL, NULL, "-C,-f",           "--index",            "<namefile> in <f> is Infernal cmsearch tab file",   4 },
-  { "--shortname", eslARG_NONE,   FALSE,  NULL, NULL, NULL, "-C,-f,--tabfile", "--index",            "w/--tabfile, do not add bit score, E value, GC to name", 4 },
-  { "--Tmin",      eslARG_REAL,   NULL,   NULL, NULL, NULL, "-C,-f,--tabfile", "--index",            "w/--tabfile, only fetch sequences with bit scores above <x>", 4},
-  { "--Emax",      eslARG_REAL,   NULL,   NULL, "x>0.",NULL,"-C,-f,--tabfile", "--index",            "w/--tabfile, only fetch sequences with E-values below <x>", 4},
 
   /* undocumented as options, because they're documented as alternative invocations: */
   { "-f",          eslARG_NONE,  FALSE,   NULL, NULL, NULL, NULL,              "--index",           "second cmdline arg is a file of names to retrieve", 99 },
@@ -93,11 +89,9 @@ static void create_ssi_index(ESL_GETOPTS *go, ESL_SQFILE *sqfp);
 static void multifetch(ESL_GETOPTS *go, FILE *ofp, char *keyfile, ESL_SQFILE *sqfp);
 static void onefetch(ESL_GETOPTS *go, FILE *ofp, char *key, ESL_SQFILE *sqfp);
 static void multifetch_subseq(ESL_GETOPTS *go, FILE *ofp, char *keyfile, ESL_SQFILE *sqfp);
-static void multifetch_subseq_infernal(ESL_GETOPTS *go, FILE *ofp, char *tabfile, ESL_SQFILE *sqfp);
 static void onefetch_subseq(ESL_GETOPTS *go, FILE *ofp, ESL_SQFILE *sqfp, char *newname, 
 			    char *key, uint32_t given_start, uint32_t given_end);
 static int  parse_coord_string(const char *cstring, uint32_t *ret_start, uint32_t *ret_end);
-static void infernal_name_subseq(char **ret_name, const char *name, ...);
 
 int
 main(int argc, char **argv)
@@ -114,16 +108,16 @@ main(int argc, char **argv)
    ***********************************************/
 
   go = esl_getopts_Create(options);
-  if (esl_opt_ProcessCmdline(go, argc, argv)             != eslOK) cmdline_failure(argv[0], "Failed to parse command line: %s\n", go->errbuf);
-  if (esl_opt_VerifyConfig(go)                           != eslOK) cmdline_failure(argv[0], "Error in configuration: %s\n",       go->errbuf);
-  if (esl_opt_GetBoolean(go, "-h") )                               cmdline_help   (argv[0], go);
-  if (esl_opt_ArgNumber(go) < 1)                                   cmdline_failure(argv[0], "Incorrect number of command line arguments.\n");        
+  if (esl_opt_ProcessCmdline(go, argc, argv) != eslOK) cmdline_failure(argv[0], "Failed to parse command line: %s\n", go->errbuf);
+  if (esl_opt_VerifyConfig(go)               != eslOK) cmdline_failure(argv[0], "Error in configuration: %s\n",       go->errbuf);
+  if (esl_opt_GetBoolean(go, "-h") )                   cmdline_help   (argv[0], go);
+  if (esl_opt_ArgNumber(go) < 1)                       cmdline_failure(argv[0], "Incorrect number of command line arguments.\n");        
 
   /* Open the sequence file */
   seqfile = esl_opt_GetArg(go, 1);
   if (esl_opt_GetString(go, "--informat") != NULL) {
-    infmt = esl_sqio_FormatCode(esl_opt_GetString(go, "--informat"));
-    if (infmt == eslSQFILE_UNKNOWN) esl_fatal("%s is not a valid input sequence file format for --informat"); 
+    infmt = esl_sqio_EncodeFormat(esl_opt_GetString(go, "--informat"));
+    if (infmt == eslSQFILE_UNKNOWN) esl_fatal("%s is not a valid input sequence file format for --informat", esl_opt_GetString(go, "--informat")); 
   }
   status = esl_sqfile_Open(seqfile, infmt, NULL, &sqfp);
   if      (status == eslENOTFOUND) cmdline_failure(argv[0], "Sequence file %s not found.\n",     seqfile);
@@ -148,8 +142,8 @@ main(int argc, char **argv)
   if (esl_opt_GetBoolean(go, "--index")) 
     {
       if (esl_opt_ArgNumber(go) != 1) cmdline_failure(argv[0], "Incorrect number of command line arguments.\n");        
-      if (sqfp->do_gzip)  cmdline_failure(argv[0], "Can't index a .gz compressed file");
-      if (sqfp->do_stdin) cmdline_failure(argv[0], "Can't index a standard input pipe");
+      if (sqfp->data.ascii.do_gzip)  cmdline_failure(argv[0], "Can't index a .gz compressed file");
+      if (sqfp->data.ascii.do_stdin) cmdline_failure(argv[0], "Can't index a standard input pipe");
 
       create_ssi_index(go, sqfp);
     }
@@ -160,7 +154,7 @@ main(int argc, char **argv)
       if (esl_opt_ArgNumber(go) != 2) cmdline_failure(argv[0], "Incorrect number of command line arguments.\n");        
 
       /* Open the SSI index for retrieval */
-      if (! sqfp->do_gzip && ! sqfp->do_stdin &&  ! esl_sqio_IsAlignment(sqfp->format)) 
+      if (! sqfp->data.ascii.do_gzip && ! sqfp->data.ascii.do_stdin &&  ! esl_sqio_IsAlignment(sqfp->format)) 
 	{
 	  status = esl_sqfile_OpenSSI(sqfp, NULL);
 	  if      (status == eslEFORMAT)   cmdline_failure(argv[0], "SSI index is in incorrect format\n");
@@ -168,11 +162,8 @@ main(int argc, char **argv)
 	  else if (status != eslOK)        cmdline_failure(argv[0], "Failed to open SSI index\n");
 	}
 
-      if (esl_opt_GetBoolean(go, "-C")) { 
-	if(esl_opt_GetBoolean(go, "--tabfile")) multifetch_subseq_infernal(go, ofp, esl_opt_GetArg(go, 2), sqfp);
-	else                            	multifetch_subseq         (go, ofp, esl_opt_GetArg(go, 2), sqfp);
-      }
-      else                        	        multifetch       (go, ofp, esl_opt_GetArg(go, 2), sqfp);
+      if (esl_opt_GetBoolean(go, "-C")) multifetch_subseq(go, ofp, esl_opt_GetArg(go, 2), sqfp);
+      else              	        multifetch       (go, ofp, esl_opt_GetArg(go, 2), sqfp);
     }
 
   /* Single sequence retrieval mode */
@@ -184,7 +175,7 @@ main(int argc, char **argv)
       char *newname = esl_opt_GetString(go, "-n");
 
       /* Open the SSI index for retrieval */
-      if (! sqfp->do_gzip && ! sqfp->do_stdin &&  ! esl_sqio_IsAlignment(sqfp->format)) 
+      if (! sqfp->data.ascii.do_gzip && ! sqfp->data.ascii.do_stdin &&  ! esl_sqio_IsAlignment(sqfp->format)) 
 	{
 	  status = esl_sqfile_OpenSSI(sqfp, NULL);
 	  if      (status == eslEFORMAT)   cmdline_failure(argv[0], "SSI index is in incorrect format\n");
@@ -220,23 +211,24 @@ main(int argc, char **argv)
 static void
 create_ssi_index(ESL_GETOPTS *go, ESL_SQFILE *sqfp)
 {
-  int         status;
-  ESL_NEWSSI *ns      = esl_newssi_Create();
+  ESL_NEWSSI *ns      = NULL;
   ESL_SQ     *sq      = esl_sq_Create();
   int         nseq    = 0;
   char       *ssifile = NULL;
-  FILE       *sfp     = NULL;
   uint16_t    fh;
+  int         status;
 
   esl_strdup(sqfp->filename, -1, &ssifile);
   esl_strcat(&ssifile, -1, ".ssi", 4);
-  if (esl_FileExists(ssifile)                            == TRUE)   esl_fatal("SSI file %s already exists; delete or rename", ssifile);
-  if ((sfp = fopen(ssifile, "wb"))                       == NULL)   esl_fatal("Failed to open SSI file %s for writing\n",     ssifile);
+  status = esl_newssi_Open(ssifile, TRUE, &ns); /* TRUE is for allowing overwrite. */
+  if      (status == eslENOTFOUND)   esl_fatal("failed to open SSI index %s", ssifile);
+  else if (status == eslEOVERWRITE)  esl_fatal("SSI index %s already exists; delete or rename it", ssifile); /* won't happen, see TRUE above... */
+  else if (status != eslOK)          esl_fatal("failed to create a new SSI index");
 
   if (esl_newssi_AddFile(ns, sqfp->filename, sqfp->format, &fh) != eslOK)
     esl_fatal("Failed to add sequence file %s to new SSI index\n", sqfp->filename);
 
-  printf("Working...    "); 
+  printf("Creating SSI index for %s...    ", sqfp->filename); 
   fflush(stdout);
   
   while ((status = esl_sqio_ReadInfo(sqfp, sq)) == eslOK)
@@ -253,16 +245,19 @@ create_ssi_index(ESL_GETOPTS *go, ESL_SQFILE *sqfp)
       }
       esl_sq_Reuse(sq);
     }
-  if (status != eslEOF) esl_fatal("Parse failed, line %d, file %s: %s\n", sqfp->linenumber, sqfp->filename, sqfp->errbuf);
-  
+  if      (status == eslEFORMAT) esl_fatal("Parse failed (sequence file %s):\n%s\n",
+					   sqfp->filename, esl_sqfile_GetErrorBuf(sqfp));
+  else if (status != eslEOF)     esl_fatal("Unexpected error %d reading sequence file %s",
+					    status, sqfp->filename);
+
   /* Determine if the file was suitable for fast subseq lookup. */
-  if (sqfp->bpl > 0 && sqfp->rpl > 0) {
-    if ((status = esl_newssi_SetSubseq(ns, fh, sqfp->bpl, sqfp->rpl)) != eslOK) 
+  if (sqfp->data.ascii.bpl > 0 && sqfp->data.ascii.rpl > 0) {
+    if ((status = esl_newssi_SetSubseq(ns, fh, sqfp->data.ascii.bpl, sqfp->data.ascii.rpl)) != eslOK) 
       esl_fatal("Failed to set %s for fast subseq lookup.");
   }
 
   /* Save the SSI file to disk */
-  if (esl_newssi_Write(sfp, ns) != eslOK)  esl_fatal("Failed to write keys to ssi file %s\n", ssifile);
+  if (esl_newssi_Write(ns) != eslOK)  esl_fatal("Failed to write keys to ssi file %s\n", ssifile);
 
   /* Done - output and exit. */
   printf("done.\n");
@@ -272,10 +267,9 @@ create_ssi_index(ESL_GETOPTS *go, ESL_SQFILE *sqfp)
     printf("Indexed %d sequences (%ld names).\n", nseq, (long) ns->nprimary);
   printf("SSI index written to file %s\n", ssifile);
 
-  fclose(sfp);
   free(ssifile);
   esl_sq_Destroy(sq);
-  esl_newssi_Destroy(ns);
+  esl_newssi_Close(ns);
   return;
 }
 
@@ -305,7 +299,7 @@ multifetch(ESL_GETOPTS *go, FILE *ofp, char *keyfile, ESL_SQFILE *sqfp)
   int             status;
 
   
-  if (esl_fileparser_Open(keyfile, &efp) != eslOK)  esl_fatal("Failed to open key file %s\n", keyfile);
+  if (esl_fileparser_Open(keyfile, NULL, &efp) != eslOK)  esl_fatal("Failed to open key file %s\n", keyfile);
   esl_fileparser_SetCommentChar(efp, '#');
 
   while (esl_fileparser_NextLine(efp) == eslOK)
@@ -313,33 +307,36 @@ multifetch(ESL_GETOPTS *go, FILE *ofp, char *keyfile, ESL_SQFILE *sqfp)
       if (esl_fileparser_GetTokenOnLine(efp, &key, &keylen) != eslOK)
 	esl_fatal("Failed to read seq name on line %d of file %s\n", efp->linenumber, keyfile);
       
-      status = esl_key_Store(keys, key, &keyidx);
+      status = esl_keyhash_Store(keys, key, keylen, &keyidx);
       if (status == eslEDUP) esl_fatal("seq key %s occurs more than once in file %s\n", key, keyfile);
 	
       /* if we have an SSI index, just fetch them as we go. */
-      if (sqfp->ssi != NULL) { onefetch(go, ofp, key, sqfp);  nseq++; }
+      if (sqfp->data.ascii.ssi != NULL) { onefetch(go, ofp, key, sqfp);  nseq++; }
       nkeys++;
     }
 
   /* If we don't have an SSI index, we haven't fetched anything yet; do it now. */
-  if (sqfp->ssi == NULL) 
+  if (sqfp->data.ascii.ssi == NULL) 
     {
       ESL_SQ *sq     = esl_sq_Create();
 
       while ((status = esl_sqio_Read(sqfp, sq)) == eslOK)
 	{
-	  if ( (sq->name[0] != '\0' && esl_key_Lookup(keys, sq->name, NULL) == eslOK) ||
-	       (sq->acc[0]  != '\0' && esl_key_Lookup(keys, sq->acc,  NULL) == eslOK))
+	  if ( (sq->name[0] != '\0' && esl_keyhash_Lookup(keys, sq->name, -1, NULL) == eslOK) ||
+	       (sq->acc[0]  != '\0' && esl_keyhash_Lookup(keys, sq->acc,  -1, NULL) == eslOK))
 	    {
 	      if (esl_opt_GetBoolean(go, "-r") )
 		if (esl_sq_ReverseComplement(sq) != eslOK) 
 		  esl_fatal("Failed to reverse complement %s\n", sq->name);
-	      esl_sqio_Write(ofp, sq, eslSQFILE_FASTA);
+	      esl_sqio_Write(ofp, sq, eslSQFILE_FASTA, FALSE);
 	      nseq++;
 	    }
 	  esl_sq_Reuse(sq);
 	}
-      if (status != eslEOF) esl_fatal("Parse failed, line %d, file %s: %s\n", sqfp->linenumber, sqfp->filename, sqfp->errbuf);
+      if      (status == eslEFORMAT) esl_fatal("Parse failed (sequence file %s):\n%s\n",
+					       sqfp->filename, esl_sqfile_GetErrorBuf(sqfp));
+      else if (status != eslEOF)     esl_fatal("Unexpected error %d reading sequence file %s",
+					       status, sqfp->filename);
       esl_sq_Destroy(sq);
     }
   
@@ -370,22 +367,32 @@ onefetch(ESL_GETOPTS *go, FILE *ofp, char *key, ESL_SQFILE *sqfp)
   int      status;
 
   /* Try to position the file at the desired sequence with SSI. */
-  if (sqfp->ssi != NULL)	
+  if (sqfp->data.ascii.ssi != NULL)	
     {
       status = esl_sqfile_PositionByKey(sqfp, key);
       if      (status == eslENOTFOUND) esl_fatal("seq %s not found in SSI index for file %s\n", key, sqfp->filename);
       else if (status == eslEFORMAT)   esl_fatal("Failed to parse SSI index for %s\n", sqfp->filename);
       else if (status != eslOK)        esl_fatal("Failed to look up location of seq %s in SSI index of file %s\n", key, sqfp->filename);
 
-      if (esl_sqio_Read(sqfp, sq) != eslOK) 
-	esl_fatal("Parse failed, line %d, file %s: %s\n", sqfp->linenumber, sqfp->filename, sqfp->errbuf);
+      status = esl_sqio_Read(sqfp, sq);
+      if      (status == eslEFORMAT) esl_fatal("Parse failed (sequence file %s):\n%s\n",
+					       sqfp->filename, esl_sqfile_GetErrorBuf(sqfp));
+      else if (status == eslEOF)     esl_fatal("Unexpected EOF reading sequence file %s",
+					       status, sqfp->filename);
+      else if (status != eslOK)      esl_fatal("Unexpected error %d reading sequence file %s",
+					       status, sqfp->filename);
+
       if (strcmp(key, sq->name) != 0 && strcmp(key, sq->acc) != 0) 
 	esl_fatal("whoa, internal error; found the wrong sequence %s, not %s", sq->name, key);
     }  
   else 
     { /* Else, we have to read the whole damn file sequentially until we find the seq */
       while ((status = esl_sqio_Read(sqfp, sq)) != eslEOF) {
-	if (status != eslOK) esl_fatal("Parse failed, line %d, file %s: %s\n", sqfp->linenumber, sqfp->filename, sqfp->errbuf);
+	if      (status == eslEFORMAT) esl_fatal("Parse failed (sequence file %s):\n%s\n",
+						 sqfp->filename, esl_sqfile_GetErrorBuf(sqfp));
+	else if (status != eslOK)      esl_fatal("Unexpected error %d reading sequence file %s",
+						 status, sqfp->filename);
+
 	if (strcmp(key, sq->name) == 0 || strcmp(key, sq->acc) == 0) break;
 	esl_sq_Reuse(sq);
       }
@@ -395,13 +402,13 @@ onefetch(ESL_GETOPTS *go, FILE *ofp, char *key, ESL_SQFILE *sqfp)
 
   if (do_revcomp == FALSE && newname == NULL && ! esl_sqio_IsAlignment(sqfp->format)) 
     { /* If we're not manipulating the sequence in any way, and it's not from an alignment file, we can Echo() it. */
-      if (esl_sqio_Echo(sqfp, sq, ofp) != eslOK) esl_fatal("Echo failed: %s\n", sqfp->errbuf);
+      if (esl_sqio_Echo(sqfp, sq, ofp) != eslOK) esl_fatal("Echo failed: %s\n", esl_sqfile_GetErrorBuf(sqfp));
     }
   else
     { /* Otherwise we Write() the parsed version. */
       if (do_revcomp && esl_sq_ReverseComplement(sq) != eslOK) esl_fatal("Failed to reverse complement %s; is it a protein?\n", sq->name);
       if (newname != NULL) esl_sq_SetName(sq, newname);
-      esl_sqio_Write(ofp, sq, eslSQFILE_FASTA);
+      esl_sqio_Write(ofp, sq, eslSQFILE_FASTA, FALSE);
     }
 
   esl_sq_Destroy(sq);
@@ -417,7 +424,7 @@ multifetch_subseq(ESL_GETOPTS *go, FILE *ofp, char *gdffile, ESL_SQFILE *sqfp)
   int             start, end;
   char           *source;
  
-  if (esl_fileparser_Open(gdffile, &efp) != eslOK)  esl_fatal("Failed to open key file %s\n", gdffile);
+  if (esl_fileparser_Open(gdffile, NULL, &efp) != eslOK)  esl_fatal("Failed to open key file %s\n", gdffile);
   esl_fileparser_SetCommentChar(efp, '#');
 
   while (esl_fileparser_NextLine(efp) == eslOK)
@@ -445,7 +452,7 @@ onefetch_subseq(ESL_GETOPTS *go, FILE *ofp, ESL_SQFILE *sqfp, char *newname, cha
   int    do_revcomp;
   ESL_SQ *sq = esl_sq_Create();
 
-  if (sqfp->ssi == NULL) esl_fatal("no ssi index");
+  if (sqfp->data.ascii.ssi == NULL) esl_fatal("no ssi index");
 
   /* reverse complement indicated by coords. */
   /* -c 52: would be 52,0, so watch out for given_end = 0 case */
@@ -454,10 +461,10 @@ onefetch_subseq(ESL_GETOPTS *go, FILE *ofp, ESL_SQFILE *sqfp, char *newname, cha
   else
     { start = given_start; end = given_end;   do_revcomp = FALSE; }
 
-  if (esl_sqio_FetchSubseq(sqfp, key, start, end, sq) != eslOK) esl_fatal(sqfp->errbuf);
+  if (esl_sqio_FetchSubseq(sqfp, key, start, end, sq) != eslOK) esl_fatal(esl_sqfile_GetErrorBuf(sqfp));
 
   if      (newname != NULL) esl_sq_SetName(sq, newname);
-  else                      esl_sq_SetName(sq, "%s/%d-%d", key, given_start, (given_end == 0) ? sq->L : given_end);
+  else                      esl_sq_FormatName(sq, "%s/%d-%d", key, given_start, (given_end == 0) ? sq->L : given_end);
 
   /* Two ways we might have been asked to revcomp: by coord, or by -r option */
   /* (If both happen, they'll cancel each other out) */
@@ -466,7 +473,7 @@ onefetch_subseq(ESL_GETOPTS *go, FILE *ofp, ESL_SQFILE *sqfp, char *newname, cha
   if (esl_opt_GetBoolean(go, "-r"))
     if (esl_sq_ReverseComplement(sq) != eslOK) esl_fatal("Failed to reverse complement %s; is it a protein?\n", sq->name);
 
-  esl_sqio_Write(ofp, sq, eslSQFILE_FASTA);
+  esl_sqio_Write(ofp, sq, eslSQFILE_FASTA, FALSE);
   esl_sq_Destroy(sq);
 }
 
@@ -488,184 +495,3 @@ parse_coord_string(const char *cstring, uint32_t *ret_start, uint32_t *ret_end)
   esl_regexp_Destroy(re);
   return eslOK;
 }
-
-static void
-multifetch_subseq_infernal(ESL_GETOPTS *go, FILE *ofp, char *tabfile, ESL_SQFILE *sqfp)
-{
-  int status;
-  ESL_FILEPARSER *efp    = NULL;
-  char           *s;
-  int             start, end, n;
-  char           *tname = NULL;
-  char           *mname = NULL;
-  char           *newname = NULL;
-  double          bit, E;
-  int             gc;
-  int             has_E = FALSE;
-  char           *tok1 = NULL;
-  char           *tok2 = NULL;
-  char           *tok3 = NULL;
-  char           *tok4 = NULL;
-  char           *tok5 = NULL;
-  char           *tok6 = NULL;
-  char           *tok7 = NULL;
-  char           *tok8 = NULL;
-  char           *tok9 = NULL;
-  float           Tmin = -eslINFINITY;
-  float           Emax = eslINFINITY;
-  int             Emax_is_on = FALSE;
-  int             has_model_name = FALSE;
-
-  if (esl_fileparser_Open(tabfile, &efp) != eslOK)  esl_fatal("Failed to open Infernal tab file %s\n", tabfile);
-  esl_fileparser_SetCommentChar(efp, '#');
-  
-  if (! esl_opt_IsDefault(go, "--Tmin")) { Tmin = esl_opt_GetReal(go, "--Tmin"); }
-  if (! esl_opt_IsDefault(go, "--Emax")) { Emax = esl_opt_GetReal(go, "--Emax"); Emax_is_on = TRUE; }
-
-  while (esl_fileparser_NextLine(efp) == eslOK)
-    {
-      /* infernal 1.01 introduced a new tab file format.
-       * This function recognizes both formats (version 1.0 and 1.01+)
-       *
-       * Example line from infernal 1.0:
-       *se-1-2                1          24      1     24     18.93      0.234   54
-       *<target (t) name>     <t start>  <t end> <q s> <q e>  <bit sc>   <E-val> <GC content>
-       *
-       * Example line from infernal 1.01+:
-       *SEED          se-1-2                1          24      1     24     18.93      0.234   54
-       *<model name>  <target (t) name>     <t start>  <t end> <q s> <q e>  <bit sc>   <E-val> <GC content>
-       */
-      has_E = FALSE;
-      has_model_name = FALSE;
-
-      if (esl_fileparser_GetTokenOnLine(efp, &s, &n)  != eslOK) esl_fatal("Failed to read first token on line %d of Infernal tab file %s\n", efp->linenumber, tabfile);
-      if (esl_strdup(s, n, &tok1) != eslOK) goto ERROR;
-      if (esl_fileparser_GetTokenOnLine(efp, &s, &n)  != eslOK) esl_fatal("Failed to read token 2 on line %d of Infernal tab file %s\n", efp->linenumber, tabfile);
-      if (esl_strdup(s, n, &tok2) != eslOK) goto ERROR;
-      if (esl_fileparser_GetTokenOnLine(efp, &s, &n)  != eslOK) esl_fatal("Failed to read token 3 on line %d of Infernal tab file %s\n", efp->linenumber, tabfile);
-      if (esl_strdup(s, n, &tok3) != eslOK) goto ERROR;
-      if (esl_fileparser_GetTokenOnLine(efp, &s, &n)  != eslOK) esl_fatal("Failed to read token 4 on line %d of Infernal tab file %s\n", efp->linenumber, tabfile);
-      if (esl_strdup(s, n, &tok4) != eslOK) goto ERROR;
-      if (esl_fileparser_GetTokenOnLine(efp, &s, &n)  != eslOK) esl_fatal("Failed to read token 5 on line %d of Infernal tab file %s\n", efp->linenumber, tabfile);
-      if (esl_strdup(s, n, &tok5) != eslOK) goto ERROR;
-      if (esl_fileparser_GetTokenOnLine(efp, &s, &n)  != eslOK) esl_fatal("Failed to read token 6 on line %d of Infernal tab file %s\n", efp->linenumber, tabfile);
-      if (esl_strdup(s, n, &tok6) != eslOK) goto ERROR;
-      if (esl_fileparser_GetTokenOnLine(efp, &s, &n)  != eslOK) esl_fatal("Failed to read token 7 on line %d of Infernal tab file %s\n", efp->linenumber, tabfile);
-      if (esl_strdup(s, n, &tok7) != eslOK) goto ERROR;
-      if (esl_fileparser_GetTokenOnLine(efp, &s, &n)  != eslOK) esl_fatal("Failed to read token 8 on line %d of Infernal tab file %s\n", efp->linenumber, tabfile);
-      if (esl_strdup(s, n, &tok8) != eslOK) goto ERROR;
-      /* if we're in an infernal 1.0 file, we should be at the end of the line, we can tell if this is the case if the next esl_fileparser_GetTokenOnLine() call returns eslEOL. */
-      if ((status = esl_fileparser_GetTokenOnLine(efp, &s, &n)) == eslEOL) { 
-	/* we're in an infernal 1.0 cmsearch tabfile, parse the line */
-	tname = tok1;
-	start = atoi(tok2);
-	end   = atoi(tok3);
-	/* tok4, query start is not used */
-	/* tok5, query end   is not used */
-	bit   = atof(tok6);
-	if(strcmp(tok7, "-") != 0) { has_E = TRUE; E = atof(tok7); } 
-	else                       { has_E = FALSE; }
-	gc = atoi(tok8);
-      }
-      else if (status == eslOK) { /* 9th token was read */
-	/* we're in an infernal 1.01+ cmsearch tabfile, we have the model name */
-	if (esl_strdup(s, n, &tok9) != eslOK) goto ERROR;
-	mname = tok1;
-	has_model_name = TRUE;
-	tname = tok2;
-	start = atoi(tok3);
-	end   = atoi(tok4);
-	/* tok5, query start is not used */
-	/* tok6, query end   is not used */
-	bit   = atof(tok7);
-	if(strcmp(tok8, "-") != 0) { has_E = TRUE; E = atof(tok8); } 
-	else                       { has_E = FALSE; }
-	gc = atoi(tok9);
-	/* ensure next token is end of line */
-	if (esl_fileparser_GetTokenOnLine(efp, &s, NULL) == eslOK) { esl_fatal("Read more than 9 tokens on line %d of Infernal tab file %s\n", efp->linenumber, tabfile); }
-      }
-      else { /* this should never happen, esl_fileparser_GetTokenOnLine returns only either eslOK or eslEOL */
-	esl_fatal("Unexpected error status %d, reading line %d of Infernal tab file %s\n", status, efp->linenumber, tabfile); 
-      }
-
-      if(! esl_opt_GetBoolean(go, "--shortname")) { 
-	if(has_model_name) { 
-	  if(has_E) infernal_name_subseq(&newname, "%s/%d-%d/%s/B%.2f/E%.2g/GC%d", tname, start, end, mname, bit, E, gc);
-	  else      infernal_name_subseq(&newname, "%s/%d-%d/%s/B%.2f/GC%d",       tname, start, end, mname, bit, gc);
-	}
-	else { /* we don't have model name (infernal 1.0 file) */
-	  if(has_E) infernal_name_subseq(&newname, "%s/%d-%d/B%.2f/E%.2g/GC%d", tname, start, end, bit, E, gc);
-	  else      infernal_name_subseq(&newname, "%s/%d-%d/B%.2f/GC%d",       tname, start, end, bit, gc);
-	}
-      }
-      /* else use default esl-sfetch name style, pass newname as NULL to onefetch_subseq (note: --modelname and --shortname are incompatible)*/
-
-      /* make sure that if Emax is enabled, we've read an E-value */
-      if(Emax_is_on && (!has_E)) { esl_fatal("--Emax enabled, but no E-value read from line %d of Infernal tab file %s\n", efp->linenumber, tabfile); }
-
-      /* check if the hit meets our minimum score criteria (these are -inf bit score and inf E-value unless --Tmin and/or --Emax enabled) */
-      if(((has_E && E <= Emax) || (! has_E)) && (bit >= Tmin)) { 
-	onefetch_subseq(go, ofp, sqfp, newname, tname, start, end);
-      }
-
-      if(newname != NULL) { free(newname); newname = NULL; }
-      if(tok1    != NULL) { free (tok1);   tok1    = NULL; }
-      if(tok2    != NULL) { free (tok2);   tok2    = NULL; }
-      if(tok3    != NULL) { free (tok3);   tok3    = NULL; }
-      if(tok4    != NULL) { free (tok4);   tok4    = NULL; }
-      if(tok5    != NULL) { free (tok5);   tok5    = NULL; }
-      if(tok6    != NULL) { free (tok6);   tok6    = NULL; }
-      if(tok7    != NULL) { free (tok7);   tok7    = NULL; }
-      if(tok8    != NULL) { free (tok8);   tok8    = NULL; }
-      if(tok9    != NULL) { free (tok9);   tok9    = NULL; }
-    }
-  esl_fileparser_Close(efp);
-  return;
-  
- ERROR:
-  if(newname != NULL) { free(newname); newname = NULL; }
-  if(tok1    != NULL) { free (tok1);   tok1    = NULL; }
-  if(tok2    != NULL) { free (tok2);   tok2    = NULL; }
-  if(tok3    != NULL) { free (tok3);   tok3    = NULL; }
-  if(tok4    != NULL) { free (tok4);   tok4    = NULL; }
-  if(tok5    != NULL) { free (tok5);   tok5    = NULL; }
-  if(tok6    != NULL) { free (tok6);   tok6    = NULL; }
-  if(tok7    != NULL) { free (tok7);   tok7    = NULL; }
-  if(tok8    != NULL) { free (tok8);   tok8    = NULL; }
-  if(tok9    != NULL) { free (tok9);   tok9    = NULL; }
-  if(efp != NULL) esl_fileparser_Close(efp);
-  esl_fatal("Error, status code %d, probably out of memory.", status);
-  return;
-}
-
-
-static void
-infernal_name_subseq(char **ret_name, const char *name, ...)
-{
-  va_list argp;
-  va_list argp2;
-  int   n;
-  void *tmp;
-  int   status;
-  char *newname;
-  int nalloc = 1;
-  ESL_ALLOC(newname, sizeof(char) * nalloc);
-  if (name == NULL) { newname[0] = '\0'; *ret_name = newname; return; }
-
-  va_start(argp, name);
-  va_copy(argp2, argp);
-  if ((n = vsnprintf(newname, nalloc, name, argp)) >= nalloc)
-    {
-      ESL_RALLOC(newname, tmp, sizeof(char) * (n+1)); 
-      nalloc = n+1;
-      vsnprintf(newname, nalloc, name, argp2);
-    }
-  va_end(argp);
-  va_end(argp2);
-  *ret_name = newname;
-  return;
-
- ERROR:
-  esl_fatal("Memory allocation error in infernal_name_subseq().\n");
-  return;
-}
