@@ -1,7 +1,7 @@
 /* Shuffling or generating random sequences.
  * 
  * SRE, Wed Jan 16 15:30:05 2008 [UA5230 to New York]
- * SVN $Id: esl-shuffle.c 267 2008-06-18 12:19:44Z eddys $
+ * SVN $Id: esl-shuffle.c 715 2011-08-03 21:04:24Z eddys $
  * from squid's shuffle (1995)
  */
 #include "esl_config.h"
@@ -27,9 +27,9 @@ static char usage3[] = "-Q [options] <qrnafile> (shuffles QRNA pairwise alignmen
 static char usage4[] = "-G [options]            (generates random sequences)";
 
 
-#define MODE_OPTS "-S,-A,-G,-Q"	        /* toggle group, modes (seqfile, msafile, none) */
-#define SHUF_OPTS "-m,-d,-0,-1,-r,-w"   /* toggle group, seq shuffling options          */
-#define ALPH_OPTS "--rna,--dna,--amino" /* toggle group, alphabet type options          */
+#define MODE_OPTS "-S,-A,-G,-Q"	           /* toggle group, modes (seqfile, msafile, none) */
+#define SHUF_OPTS "-m,-d,-k,-0,-1,-r,-w"   /* toggle group, seq shuffling options          */
+#define ALPH_OPTS "--rna,--dna,--amino"    /* toggle group, alphabet type options          */
 
 static ESL_OPTIONS options[] = {
   /* name         type           default   env range      togs  reqs  incomp      help                                      docgroup */
@@ -41,6 +41,7 @@ static ESL_OPTIONS options[] = {
   /* Options for shuffling/generating based on input sequences */
   { "-m",         eslARG_NONE,"default", NULL, NULL, SHUF_OPTS, "-S", NULL, "shuffle preserving monoresidue composition",          2 },
   { "-d",         eslARG_NONE,    FALSE, NULL, NULL, SHUF_OPTS, "-S", NULL, "shuffle preserving mono- and di-residue composition", 2 },
+  { "-k",         eslARG_INT,     FALSE, NULL,"n>0", SHUF_OPTS, "-S", NULL, "shuffle nonoverlapping <n>-mers",                     2 },
   { "-0",         eslARG_NONE,    FALSE, NULL, NULL, SHUF_OPTS, "-S", NULL, "generate with 0th order Markov properties per input", 2 },
   { "-1",         eslARG_NONE,    FALSE, NULL, NULL, SHUF_OPTS, "-S", NULL, "generate with 1st order Markov properties per input", 2 },
   { "-r",         eslARG_NONE,    FALSE, NULL, NULL, SHUF_OPTS, "-S", NULL, "reverse each input",                                  2 },
@@ -55,7 +56,7 @@ static ESL_OPTIONS options[] = {
   { "--amino",    eslARG_NONE,    FALSE, NULL, NULL, ALPH_OPTS, "-G", NULL, "generate protein sequence",                           4 },
 
   /* Other "expert" options */
-  { "--seed",     eslARG_INT,     FALSE, NULL,"n>0",      NULL, NULL, NULL, "set random number generator's seed to <n>",           5 },
+  { "--seed",     eslARG_INT,       "0", NULL,"n>=0",     NULL, NULL, NULL, "set random number generator's seed to <n>",           5 },
   { "--informat", eslARG_STRING,  FALSE, NULL, NULL,      NULL, NULL, NULL, "specify that input file is in format <s>",            5 },
 
   /* "undocumented" options (these are documented w/ command line usage, and implemented as options) */
@@ -104,29 +105,28 @@ cmdline_help(char *argv0, ESL_GETOPTS *go)
 
 
 /* msa_shuffling()
- * SRE, Tue Jan 22 08:39:51 2008 [Market Street Cafe, Leesburg]
  * 
  * Shuffling multiple sequence alignments
  */
 static int
 msa_shuffling(ESL_GETOPTS *go, ESL_RANDOMNESS *r, FILE *ofp, int outfmt)
 {
-  char        *msafile = esl_opt_GetArg(go, 1);
-  int          infmt   = eslMSAFILE_UNKNOWN;
-  ESL_MSAFILE *afp     = NULL;
-  ESL_MSA     *msa     = NULL;
-  ESL_MSA     *shuf    = NULL;
-  int          N       = esl_opt_GetInteger(go, "-N");
-  int          i;
-  int          status, mstatus;
+  char         *msafile = esl_opt_GetArg(go, 1);
+  int           infmt   = eslMSAFILE_UNKNOWN;
+  ESLX_MSAFILE *afp     = NULL;
+  ESL_MSA      *msa     = NULL;
+  ESL_MSA      *shuf    = NULL;
+  int           N       = esl_opt_GetInteger(go, "-N");
+  int           i;
+  int           status;
 
-  status = esl_msafile_Open(msafile, infmt, NULL, &afp);
-  if (status == eslENOTFOUND)    esl_fatal("Alignment file %s isn't readable\n", msafile);
-  else if (status == eslEFORMAT) esl_fatal("Couldn't determine format of %s\n",  msafile);
-  else if (status != eslOK)      esl_fatal("Alignment file open failed (error %d)\n", status);
+  if ( (status = eslx_msafile_Open(NULL, msafile, NULL, infmt, NULL, &afp)) != eslOK)
+    eslx_msafile_OpenFailure(afp, status);
   
-  while ((mstatus = esl_msa_Read(afp, &msa)) == eslOK)
+  while ((status = eslx_msafile_Read(afp, &msa)) != eslEOF)
     {
+      if (status != eslOK) eslx_msafile_ReadFailure(afp, status);
+
       shuf = esl_msa_Clone(msa);
 
       for (i = 0; i < N; i++)
@@ -137,40 +137,35 @@ msa_shuffling(ESL_GETOPTS *go, ESL_RANDOMNESS *r, FILE *ofp, int outfmt)
 	  /* Set the name of the shuffled alignment */
 	  if (msa->name != NULL) {
 	    if (esl_opt_GetBoolean(go, "--boot")) {
-	      if (N > 1) esl_msa_SetName(shuf, "%s-sample-%d", msa->name, i);
-	      else       esl_msa_SetName(shuf, "%s-sample",    msa->name);
+	      if (N > 1) esl_msa_FormatName(shuf, "%s-sample-%d", msa->name, i);
+	      else       esl_msa_FormatName(shuf, "%s-sample",    msa->name);
 	    } else {
-	      if (N > 1) esl_msa_SetName(shuf, "%s-shuffle-%d", msa->name, i);
-	      else       esl_msa_SetName(shuf, "%s-shuffle",    msa->name);
+	      if (N > 1) esl_msa_FormatName(shuf, "%s-shuffle-%d", msa->name, i);
+	      else       esl_msa_FormatName(shuf, "%s-shuffle",    msa->name);
 	    }
 	  } else {
 	    if (esl_opt_GetBoolean(go, "--boot")) {
-	      if (N > 1) esl_msa_SetName(shuf, "sample-%d", i);
-	      else       esl_msa_SetName(shuf, "sample");
+	      if (N > 1) esl_msa_FormatName(shuf, "sample-%d", i);
+	      else       esl_msa_FormatName(shuf, "sample");
 	    } else {
-	      if (N > 1) esl_msa_SetName(shuf, "shuffle-%d", i);
-	      else       esl_msa_SetName(shuf, "shuffle");
+	      if (N > 1) esl_msa_FormatName(shuf, "shuffle-%d", i);
+	      else       esl_msa_FormatName(shuf, "shuffle");
 	    }
 	  }
 
-	  esl_msa_Write(ofp, shuf, outfmt);
+	  eslx_msafile_Write(ofp, shuf, afp->format);
 	}
 
       esl_msa_Destroy(shuf);
       esl_msa_Destroy(msa);
     }
-  if (mstatus == eslEFORMAT)
-    esl_fatal("Alignment file parse error, line %d of file %s:\n%s\nOffending line is:\n%s\n",
-	      afp->linenumber, afp->fname, afp->errbuf, afp->buf);
-  else if (mstatus != eslEOF)
-    esl_fatal("Alignment file read failed with error code %d\n", status);
 
+  eslx_msafile_Close(afp);
   return eslOK;
 }
 
 
 /* seq_generation()
- * SRE, Tue Jan 22 08:38:58 2008 [Market Street Cafe, Leesburg]
  *
  * Generating sequences.
  */
@@ -207,10 +202,10 @@ seq_generation(ESL_GETOPTS *go, ESL_RANDOMNESS *r, FILE *ofp, int outfmt)
   for (i = 0; i < N; i++)
     {
       esl_rsq_xIID(r, fq, abc->K, L, sq->dsq);
-      if (N > 1) esl_sq_SetName(sq, "random%d", i);
+      if (N > 1) esl_sq_FormatName(sq, "random%d", i);
       else       esl_sq_SetName(sq, "random");
       sq->n = L;
-      esl_sqio_Write(ofp, sq, outfmt);
+      esl_sqio_Write(ofp, sq, outfmt, FALSE);
     }
 
   free(fq);
@@ -251,14 +246,18 @@ seq_shuffling(ESL_GETOPTS *go, ESL_RANDOMNESS *r, FILE *ofp, int outfmt)
   char       *targ    = NULL;
   int         N       = esl_opt_GetInteger(go, "-N");
   int         L       = esl_opt_GetInteger(go, "-L"); /* L>0 means select random fixed-len subseqs */
+  int         kmers   = 0;
   int         i;
   int         status;
   
   if (esl_opt_GetString(go, "--informat") != NULL) {
-    infmt = esl_sqio_FormatCode(esl_opt_GetString(go, "--informat"));
+    infmt = esl_sqio_EncodeFormat(esl_opt_GetString(go, "--informat"));
     if (infmt == eslSQFILE_UNKNOWN) esl_fatal("%s is not a valid input sequence file format for --informat"); 
   }
 
+  if (esl_opt_IsOn(go, "-k")) kmers = esl_opt_GetInteger(go, "-k");
+
+
   status = esl_sqfile_Open(seqfile, infmt, NULL, &sqfp);
   if      (status == eslENOTFOUND) esl_fatal("No such file %s", seqfile);
   else if (status == eslEFORMAT)   esl_fatal("Format of seqfile %s unrecognized.", seqfile);
@@ -290,28 +289,31 @@ seq_shuffling(ESL_GETOPTS *go, ESL_RANDOMNESS *r, FILE *ofp, int outfmt)
 	  }
 
 	  /* Do the requested kind of shuffling */
-	  if      (esl_opt_GetBoolean(go, "-m"))  esl_rsq_CShuffle  (r, targ, shuff->seq);  /* monoresidue shuffling */
-	  else if (esl_opt_GetBoolean(go, "-d"))  esl_rsq_CShuffleDP(r, targ, shuff->seq);  /* diresidue shuffling */
-	  else if (esl_opt_GetBoolean(go, "-0"))  esl_rsq_CMarkov0  (r, targ, shuff->seq);  /* 0th order Markov */
-	  else if (esl_opt_GetBoolean(go, "-1"))  esl_rsq_CMarkov1  (r, targ, shuff->seq);  /* 1st order Markov */
-	  else if (esl_opt_GetBoolean(go, "-r"))  esl_rsq_CReverse  (   targ, shuff->seq);  /* reverse */
-	  else if (!esl_opt_IsDefault(go, "-w")) { /* regionally shuffle */	
+	  if      (esl_opt_GetBoolean(go, "-m"))  esl_rsq_CShuffle     (r, targ,        shuff->seq);  /* monoresidue shuffling */
+	  else if (esl_opt_GetBoolean(go, "-d"))  esl_rsq_CShuffleDP   (r, targ,        shuff->seq);  /* diresidue shuffling */
+	  else if (esl_opt_IsOn      (go, "-k"))  esl_rsq_CShuffleKmers(r, targ, kmers, shuff->seq);  /* diresidue shuffling */
+	  else if (esl_opt_GetBoolean(go, "-0"))  esl_rsq_CMarkov0     (r, targ,        shuff->seq);  /* 0th order Markov */
+	  else if (esl_opt_GetBoolean(go, "-1"))  esl_rsq_CMarkov1     (r, targ,        shuff->seq);  /* 1st order Markov */
+	  else if (esl_opt_GetBoolean(go, "-r"))  esl_rsq_CReverse     (   targ,        shuff->seq);  /* reverse */
+	  else if (esl_opt_IsOn      (go, "-w")) { /* regionally shuffle */	
 	    int W= esl_opt_GetInteger(go, "-w"); esl_rsq_CShuffleWindows(r, targ, W, shuff->seq);
 	  }
 
 	  /* Set the name of the shuffled sequence */
-	  if (N > 1) esl_sq_SetName(shuff, "%s-shuffled-%d", sq->name, i);
-	  else       esl_sq_SetName(shuff, "%s-shuffled", sq->name);
+	  if (N > 1) esl_sq_FormatName(shuff, "%s-shuffled-%d", sq->name, i);
+	  else       esl_sq_FormatName(shuff, "%s-shuffled", sq->name);
 
 	  /* Output the resulting sequence */
-	  esl_sqio_Write(ofp, shuff, outfmt);
+	  esl_sqio_Write(ofp, shuff, outfmt, FALSE);
+
+	  /* don't need to reuse the shuffled sequence: we will use exactly the same memory */
 	}
+      esl_sq_Reuse(sq);
     }
-  if      (status == eslEFORMAT)
-    esl_fatal("Sequence file parse error, line %d of file %s:\n%s\n", 
-	      sqfp->linenumber, seqfile, sqfp->errbuf);
-  else if (status != eslEOF) 
-    esl_fatal("Internal error reading sequence file %s\n", seqfile);
+  if      (status == eslEFORMAT) esl_fatal("Parse failed (sequence file %s):\n%s\n",
+					   sqfp->filename, esl_sqfile_GetErrorBuf(sqfp));
+  else if (status != eslEOF)     esl_fatal("Unexpected error %d reading sequence file %s",
+					    status, sqfp->filename);
 
   if (L>0) free(targ);
   esl_sq_Destroy(shuff);
@@ -354,8 +356,7 @@ main(int argc, char **argv)
   else ofp = stdout;
 
   /* Initialize */
-  if (esl_opt_IsDefault(go, "--seed")) r = esl_randomness_CreateTimeseeded();
-  else                                 r = esl_randomness_Create(esl_opt_GetInteger(go, "--seed"));
+  r = esl_randomness_Create(esl_opt_GetInteger(go, "--seed"));
 
   /* Hand off execution to one of the three modes */
   if (esl_opt_GetBoolean(go, "-A"))   /* Alignment shuffling */
@@ -381,13 +382,18 @@ main(int argc, char **argv)
     }
 
   if (esl_opt_GetString(go, "-o") != NULL) fclose(ofp);
+  esl_randomness_Destroy(r);
   esl_getopts_Destroy(go);
   return 0;
 } 
 
 
 /*****************************************************************
- *    This copyrighted source code is freely distributed 
- *    under the terms of the GNU General Public License. See
- *    the files COPYRIGHT and LICENSE for details.
+ * Easel - a library of C functions for biological sequence analysis
+ * Version i1.1.1; July 2014
+ * Copyright (C) 2014 HHMI Janelia Farm Research Campus
+ * Other copyrights also apply. See the COPYRIGHT file for a full list.
+ * 
+ * Easel is distributed under the Janelia Farm Software License, a BSD
+ * license. See the LICENSE file for more details.
  *****************************************************************/ 
