@@ -1,7 +1,11 @@
 /************************************************************
- *    This copyrighted source code is freely distributed 
- *    under the terms of the GNU General Public License. See
- *    the files COPYRIGHT and LICENSE for details.
+ * Infernal - inference of RNA secondary structure alignments
+ * Version 1.1.1; July 2014
+ * Copyright (C) 2014 Howard Hughes Medical Institute.
+ * Other copyrights also apply. See the COPYRIGHT file for a full list.
+ * 
+ * Infernal is distributed under the terms of the GNU General Public License
+ * (GPLv3). See the LICENSE file for details.
  ************************************************************/
 /* cp9_trace.c
  * EPN, Wed Dec  5 13:05:17 2007
@@ -14,6 +18,7 @@
  */
 
 #include "esl_config.h"
+#include "p7_config.h"
 #include "config.h"
 
 #include <stdio.h>
@@ -26,9 +31,9 @@
 #include "esl_msa.h"
 #include "esl_vectorops.h"
 
-#include "funcs.h"
-#include "structs.h"
+#include "hmmer.h"
 
+#include "infernal.h"
 
 /* Function: CP9AllocTrace(), CP9ReallocTrace(), CP9FreeTrace()
  * 
@@ -911,6 +916,7 @@ CP9ReverseTrace(CP9trace_t *tr)
  * 
  * Args:     cm         - the CM the CP9 was built from, needed to get emitmap,
  *                        so we know where to put EL transitions
+ *           cp9        - the CP9 used to determine the traces (cm->cp9loc or cm->cp9glb)   
  *           abc        - alphabet to use to create the return MSA
  *           sq         - sequences 
  *           wgt        - weights for seqs, NULL for none
@@ -924,7 +930,7 @@ CP9ReverseTrace(CP9trace_t *tr)
  *           MSA structure in ret_msa, caller responsible for freeing.
  */          
 int
-CP9Traces2Alignment(CM_t *cm, const ESL_ALPHABET *abc, ESL_SQ **sq, float *wgt, 
+CP9Traces2Alignment(CM_t *cm, CP9_t *cp9, const ESL_ALPHABET *abc, ESL_SQ **sq, float *wgt, 
 		    int nseq, CP9trace_t **tr, int do_full, int do_matchonly,
 		    ESL_MSA **ret_msa)
 {
@@ -957,24 +963,22 @@ CP9Traces2Alignment(CM_t *cm, const ESL_ALPHABET *abc, ESL_SQ **sq, float *wgt,
   int          max_ins_or_el[2];/* for regularizing (splitting) inserts */
   int          pass_offset[2];  /* for regularizing (splitting) inserts */
   int          pass;            /* for regularizing (splitting) inserts */
-  char         errbuf[cmERRBUFSIZE];
+  char         errbuf[eslERRBUFSIZE];
 
   /* Contract checks */
-  if(cm->cp9 == NULL)
-    cm_Fail("ERROR in CP9Traces2Alignment, cm->cp9 is NULL.\n");
+  if(cp9 == NULL)
+    cm_Fail("ERROR in CP9Traces2Alignment, cp9 is NULL.\n");
   if(cm->cp9map == NULL)
     cm_Fail("ERROR in CP9Traces2Alignment, cm->cp9map is NULL.\n");
-  if(!(cm->flags & CMH_CP9))
-     cm_Fail("ERROR in CP9Traces2Alignment, CMH_CP9 flag is down.");
   /* We allow the caller to specify the alphabet they want the 
    * resulting MSA in, but it has to make sense (see next few lines). */
   if(cm->abc->type == eslRNA)
     { 
       if(abc->type != eslRNA && abc->type != eslDNA)
-	cm_Fail("ERROR in Parsetrees2Alignment(), cm alphabet is RNA, but requested output alphabet is neither DNA nor RNA.");
+	cm_Fail("ERROR in CP9Traces2Alignment(), cm alphabet is RNA, but requested output alphabet is neither DNA nor RNA.");
     }
   else if(cm->abc->K != abc->K)
-    cm_Fail("ERROR in Parsetrees2Alignment(), cm alphabet size is %d, but requested output alphabet size is %d.", cm->abc->K, abc->K);
+    cm_Fail("ERROR in CP9Traces2Alignment(), cm alphabet size is %d, but requested output alphabet size is %d.", cm->abc->K, abc->K);
 
   /* create the emit map */
   emap = CreateEmitMap(cm);
@@ -1067,23 +1071,23 @@ CP9Traces2Alignment(CM_t *cm, const ESL_ALPHABET *abc, ESL_SQ **sq, float *wgt,
 	  statetype = tr[idx]->statetype[tpos]; /* just for clarity */
 	  cpos      = tr[idx]->nodeidx[tpos];      
 	  if(statetype == CSTM) next_match = cpos;
-	  if(statetype == CSTE) next_match = cm->cp9->M+1;
+	  if(statetype == CSTE) next_match = cp9->M+1;
 	  if(statetype == CSTEL) eposmap[idx][cpos] = next_match; /* this will be overwritten below */
 	}
       for (cpos = 0; cpos <= emap->clen; cpos++) 
 	{
 	  if(eposmap[idx][cpos] != -1)
 	    {
-	      /*printf("cpos: %d eposmap[idx][cpos]: %d ct: %d\n", cpos, eposmap[idx][cpos], cm->cp9->el_from_ct[eposmap[idx][cpos]]);*/
-	      /* determine the epos based on the CM emit map and cm->cp9->el* data structures */
-	      for(c = 0; c < cm->cp9->el_from_ct[eposmap[idx][cpos]]; c++)
+	      /*printf("cpos: %d eposmap[idx][cpos]: %d ct: %d\n", cpos, eposmap[idx][cpos], cp9->el_from_ct[eposmap[idx][cpos]]);*/
+	      /* determine the epos based on the CM emit map and cp9->el* data structures */
+	      for(c = 0; c < cp9->el_from_ct[eposmap[idx][cpos]]; c++)
 		{
-		  if(cm->cp9->el_from_idx[eposmap[idx][cpos]][c] == cpos)
+		  if(cp9->el_from_idx[eposmap[idx][cpos]][c] == cpos)
 		    {
-		      eposmap[idx][cpos] = emap->epos[cm->cp9->el_from_cmnd[eposmap[idx][cpos]][c]];
+		      eposmap[idx][cpos] = emap->epos[cp9->el_from_cmnd[eposmap[idx][cpos]][c]];
 		      break;
 		    }
-		  if(c == (cm->cp9->el_from_ct[eposmap[idx][cpos]] - 1))
+		  if(c == (cp9->el_from_ct[eposmap[idx][cpos]] - 1))
 		    cm_Fail("Couldn't determine epos for cpos: %d\n", cpos);
 		}
 	    }
@@ -1270,8 +1274,8 @@ CP9Traces2Alignment(CM_t *cm, const ESL_ALPHABET *abc, ESL_SQ **sq, float *wgt,
         
   msa->nseq = nseq;
   msa->alen = alen;
-  ESL_ALLOC(msa->au, sizeof(char) * (strlen(PACKAGE_VERSION)+10));
-  sprintf(msa->au, "Infernal %s", PACKAGE_VERSION);
+  ESL_ALLOC(msa->au, sizeof(char) * (strlen(INFERNAL_VERSION)+10));
+  sprintf(msa->au, "Infernal %s", INFERNAL_VERSION);
 
   /* copy names and weights */
   for (idx = 0; idx < nseq; idx++)
@@ -1290,7 +1294,7 @@ CP9Traces2Alignment(CM_t *cm, const ESL_ALPHABET *abc, ESL_SQ **sq, float *wgt,
    */
   ESL_ALLOC(msa->ss_cons, (sizeof(char) * (alen+1)));
   ESL_ALLOC(msa->rf,      (sizeof(char) * (alen+1)));
-  CreateCMConsensus(cm, abc, 3.0, 1.0, &con);
+  con = CreateCMConsensus(cm, abc);
 
   for (cpos = 0; cpos <= emap->clen; cpos++) 
     {
@@ -1381,7 +1385,7 @@ CP9Traces2Alignment(CM_t *cm, const ESL_ALPHABET *abc, ESL_SQ **sq, float *wgt,
  * Return:   the log_2-odds score correction.          
  */
 int
-CP9TraceScoreCorrectionNull2(CP9_t *hmm, char *errbuf, CP9trace_t *tr, ESL_DSQ *dsq, int start, float *ret_sc)
+CP9TraceScoreCorrectionNull2(CP9_t *hmm, char *errbuf, CP9trace_t *tr, ESL_DSQ *dsq, int start, float omega, float *ret_sc)
 {
   int status;
   float *p;		/* null2 model distribution */
@@ -1422,7 +1426,7 @@ CP9TraceScoreCorrectionNull2(CP9_t *hmm, char *errbuf, CP9trace_t *tr, ESL_DSQ *
     * interpreted as a prior, saying that the second null model is 
     * 1/2^8 (1/256) as likely as the standard null model
     */
-   score -= 8 * INTSCALE;	
+   score += ((int) (sreLOG2(omega))) * INTSCALE;
 
    /* Return the correction to the bit score.
     */
