@@ -1,18 +1,23 @@
 /* display.c
  * SRE, Thu May 23 08:18:05 2002 [St. Louis]
- * SVN $Id: display.c 2958 2009-10-19 21:28:28Z nawrockie $
+ * SVN $Id: display.c 3955 2012-03-20 10:17:48Z nawrockie $
  * 
  * Routines for formatting and displaying parse trees
  * for output.
  * 
  *****************************************************************
- *    This copyrighted source code is freely distributed 
- *    under the terms of the GNU General Public License. See
- *    the files COPYRIGHT and LICENSE for details.
+ * Infernal - inference of RNA secondary structure alignments
+ * Version 1.1.1; July 2014
+ * Copyright (C) 2014 Howard Hughes Medical Institute.
+ * Other copyrights also apply. See the COPYRIGHT file for a full list.
+ * 
+ * Infernal is distributed under the terms of the GNU General Public License
+ * (GPLv3). See the LICENSE file for details.
  *****************************************************************  
  */
 
 #include "esl_config.h"
+#include "p7_config.h"
 #include "config.h"
 
 #include <stdio.h>
@@ -27,8 +32,9 @@
 #include "esl_stack.h"
 #include "esl_vectorops.h"
 
-#include "funcs.h"
-#include "structs.h"
+#include "hmmer.h"
+
+#include "infernal.h"
 
 static int *createMultifurcationOrderChart(CM_t *cm);
 static int  bp_is_canonical(char lseq, char rseq);
@@ -48,8 +54,8 @@ static void createFaceCharts(CM_t *cm, int **ret_inface, int **ret_outface);
  *            dsq   - digitized sequence
  *            do_noncanonical - mark half-bps and negative scoring bps that are non-canonicals in top line with 'v'
  *                              (by default, all negative scoring and half-bps are marked with 'x')
- *            pcode1- posteriors, 'tens' place, '9' in '93', NULL if none
- *            pcode2- posteriors, 'ones' place, '3' in '93', NULL if none
+ *            pcode - posterior code
+ *            
  *
  * Returns:   fancy alignment structure.
  *            Caller frees, with FreeFancyAli(ali).
@@ -57,7 +63,7 @@ static void createFaceCharts(CM_t *cm, int **ret_inface, int **ret_outface);
  * Xref:      STL6 p.58
  */
 Fancyali_t *
-CreateFancyAli(const ESL_ALPHABET *abc, Parsetree_t *tr, CM_t *cm, CMConsensus_t *cons, ESL_DSQ *dsq, int do_noncanonical, char *pcode1, char *pcode2)
+CreateFancyAli(const ESL_ALPHABET *abc, Parsetree_t *tr, CM_t *cm, CMConsensus_t *cons, ESL_DSQ *dsq, int do_noncanonical, char *pcode)
 {
   int         status;
   Fancyali_t *ali;              /* alignment structure we're building        */
@@ -72,16 +78,15 @@ CreateFancyAli(const ESL_ALPHABET *abc, Parsetree_t *tr, CM_t *cm, CMConsensus_t
   int         lc, rc;		/* indices for left, right pos in consensus */
   int         symi, symj;
   int         d;
-  int         mode;
-  int         lannote, rannote; /* chars in annotation line; left, right     */
+  char        mode;
+  int         lrf, rrf;         /* chars in reference line; left, right     */
   int         lstr, rstr;	/* chars in structure line; left, right      */
   int         lcons, rcons;	/* chars in consensus line; left, right      */
   int         lmid, rmid;	/* chars in ali quality line; left, right    */
   int         ltop, rtop;	/* chars in optional noncompensatory line; left, right */
   int         lnegnc, rnegnc;	/* chars in optional noncanonical line; left, right */
   int         lseq, rseq;	/* chars in aligned target line; left, right */
-  int         lpost1, rpost1;	/* chars in aligned posteriors, left, right  */
-  int         lpost2, rpost2;	/* chars in aligned posteriors, left, right  */
+  int         lpost, rpost;	/* chars in aligned posteriors, left, right  */
   int         do_left, do_right;/* flags to generate left, right             */
   int cpos_l, cpos_r;   	/* positions in consensus (1..clen)          */
   int spos_l, spos_r;		/* positions in dsq (1..L)                   */
@@ -98,7 +103,7 @@ CreateFancyAli(const ESL_ALPHABET *abc, Parsetree_t *tr, CM_t *cm, CMConsensus_t
     cm_Fail("ERROR in CreateFancyAli(), cm alphabet size is %d, but requested output alphabet size is %d.", cm->abc->K, abc->K);
 
   ESL_ALLOC(ali, sizeof(Fancyali_t));
-  have_pcodes = (pcode1 != NULL && pcode2 != NULL) ? TRUE : FALSE;
+  have_pcodes = (pcode != NULL) ? TRUE : FALSE;
   
   /* Calculate length of the alignment display.
    *   MATP node        : +2
@@ -141,38 +146,35 @@ CreateFancyAli(const ESL_ALPHABET *abc, Parsetree_t *tr, CM_t *cm, CMConsensus_t
     }
 
   /* Allocate and initialize.
-   * Blank the annotation lines (memset calls) - only needed
+   * Blank the reference lines (memset calls) - only needed
    * because of the way we deal w/ EL. 
    */
-  if (cm->annote != NULL ) 
-    ESL_ALLOC(ali->annote, sizeof(char) * (ali->len+1));
+  if (cm->rf != NULL ) 
+    ESL_ALLOC(ali->rf, sizeof(char) * (ali->len+1));
   else                     
-    ali->annote = NULL;
+    ali->rf = NULL;
   ESL_ALLOC(ali->cstr, sizeof(char) * (ali->len+1));
   ESL_ALLOC(ali->cseq, sizeof(char) * (ali->len+1));
   ESL_ALLOC(ali->mid,  sizeof(char) * (ali->len+1));
   ESL_ALLOC(ali->top,  sizeof(char) * (ali->len+1));
   ESL_ALLOC(ali->aseq, sizeof(char) * (ali->len+1));
   if(have_pcodes) { 
-    ESL_ALLOC(ali->pcode1, sizeof(char) * (ali->len+1));
-    ESL_ALLOC(ali->pcode2, sizeof(char) * (ali->len+1));
+    ESL_ALLOC(ali->pcode, sizeof(char) * (ali->len+1));
   }
   else {
-    ali->pcode1 = NULL;
-    ali->pcode2 = NULL;
+    ali->pcode = NULL;
   }
   ESL_ALLOC(ali->scoord, sizeof(int)  * ali->len);
   ESL_ALLOC(ali->ccoord, sizeof(int)  * ali->len);
 
-  if (cm->annote != NULL) memset(ali->annote, ' ', ali->len);
+  if (cm->rf != NULL) memset(ali->rf, ' ', ali->len);
   memset(ali->cstr, ' ', ali->len);
   memset(ali->cseq, ' ', ali->len);
   memset(ali->mid,  ' ', ali->len);
   memset(ali->top,  ' ', ali->len);
   memset(ali->aseq, ' ', ali->len);
   if(have_pcodes) { 
-    memset(ali->pcode1, ' ', ali->len);
-    memset(ali->pcode2, ' ', ali->len);
+    memset(ali->pcode, ' ', ali->len);
   }
   for (pos = 0; pos < ali->len; pos++) 
     ali->ccoord[pos] = ali->scoord[pos] = 0;
@@ -187,9 +189,9 @@ CreateFancyAli(const ESL_ALPHABET *abc, Parsetree_t *tr, CM_t *cm, CMConsensus_t
   while (esl_stack_IPop(pda, &type) != eslEOD)
     {
       if (type == PDA_RESIDUE) {
-	if (cm->annote != NULL) { 
-	  esl_stack_IPop(pda, &rannote); 
-	  ali->annote[pos] = rannote;
+	if (cm->rf != NULL) { 
+	  esl_stack_IPop(pda, &rrf); 
+	  ali->rf[pos] = rrf;
 	}
 	esl_stack_IPop(pda, &rstr); 	  ali->cstr[pos]   = rstr;
 	esl_stack_IPop(pda, &rcons);	  ali->cseq[pos]   = rcons;
@@ -197,8 +199,7 @@ CreateFancyAli(const ESL_ALPHABET *abc, Parsetree_t *tr, CM_t *cm, CMConsensus_t
 	esl_stack_IPop(pda, &rtop);	  ali->top[pos]    = rtop;
 	esl_stack_IPop(pda, &rseq);       ali->aseq[pos]   = rseq;
 	if(have_pcodes) {
-	  esl_stack_IPop(pda, &rpost1);    ali->pcode1[pos] = rpost1;
-	  esl_stack_IPop(pda, &rpost2);    ali->pcode2[pos] = rpost2;
+	  esl_stack_IPop(pda, &rpost);    ali->pcode[pos] = rpost;
 	}
 	esl_stack_IPop(pda, &cpos_r);     ali->ccoord[pos] = cpos_r;
 	esl_stack_IPop(pda, &spos_r);     ali->scoord[pos] = spos_r;
@@ -238,69 +239,73 @@ CreateFancyAli(const ESL_ALPHABET *abc, Parsetree_t *tr, CM_t *cm, CMConsensus_t
       rc   = cons->rpos[nd];
       symi = dsq[tr->emitl[ti]];  /* residue indices that node is aligned to */
       symj = dsq[tr->emitr[ti]];
-      if(have_pcodes) { /* postal codes are indexed 0..alen-1, off-by-one w.r.t dsq */
-	lpost1 = pcode1[tr->emitl[ti]-1];
-	lpost2 = pcode2[tr->emitl[ti]-1];
-	rpost1 = pcode1[tr->emitr[ti]-1];
-	rpost2 = pcode2[tr->emitr[ti]-1];
+      if(have_pcodes) { /* posterior codes are indexed 0..alen-1, off-by-one w.r.t dsq */
+	lpost = '.'; /* init to gap, if it corresponds to a residue, we'll reset it below */
+	rpost = '.'; /* init to gap, if it corresponds to a residue, we'll reset it below */
       }
       d = tr->emitr[ti] - tr->emitl[ti] + 1;
       mode = tr->mode[ti];
 
-      /* Calculate four of the five lines: annote, str, cons, and seq.
+      /* Calculate four of the five lines: rf, str, cons, and seq.
        */
       do_left = do_right = FALSE;
       if (cm->sttype[v] == IL_st) {
 	do_left = TRUE;
-	if (cm->annote != NULL) lannote = '.';
+	if (cm->rf != NULL) lrf = '.';
 	lstr    = '.';
 	lcons   = '.';
-	if (mode == 3 || mode == 2) lseq = tolower((int) abc->sym[symi]);
+	if (mode == TRMODE_J || mode == TRMODE_L) lseq = tolower((int) abc->sym[symi]);
         else                        lseq = '~';
 	cpos_l  = 0;
 	spos_l  = tr->emitl[ti];
+	if(pcode != NULL) { lpost = pcode[tr->emitl[ti]-1]; } /* watch off-by-one w.r.t. dsq */
       } else if (cm->sttype[v] == IR_st) {
 	do_right = TRUE;
-	if (cm->annote != NULL) rannote = '.';
+	if (cm->rf != NULL) rrf = '.';
 	rstr    = '.';
 	rcons   = '.';
-	if (mode == 3 || mode == 1) rseq = tolower((int) abc->sym[symj]);
+	if (mode == TRMODE_J || TRMODE_R) rseq = tolower((int) abc->sym[symj]);
         else                        rseq = '~';
 	cpos_r  = 0;
 	spos_r  = tr->emitr[ti];
+	if(pcode != NULL) { rpost = pcode[tr->emitr[ti]-1]; } /* watch off-by-one w.r.t. dsq */
       } else {
 	if (cm->ndtype[nd] == MATP_nd || cm->ndtype[nd] == MATL_nd) {
 	  do_left = TRUE;
-	  if (cm->annote != NULL) lannote = cm->annote[lc];
+	  if (cm->rf != NULL) lrf = cm->rf[lc];
 	  lstr   = cons->cstr[lc];
 	  lcons  = cons->cseq[lc];
 	  cpos_l = lc+1;
 	  if (cm->sttype[v] == MP_st || cm->sttype[v] == ML_st) {
-	    if      (mode == 3)         lseq = abc->sym[symi];
-            else if (mode == 2 && d>0 ) lseq = abc->sym[symi];
+	    if      (mode == TRMODE_J)         lseq = abc->sym[symi];
+            else if (mode == TRMODE_L && d>0 ) lseq = abc->sym[symi];
             else                        lseq = '~';
 	    spos_l = tr->emitl[ti];
+	    if(pcode != NULL) { lpost = pcode[tr->emitl[ti]-1]; } /* watch off-by-one w.r.t. dsq */
 	  } else {
-	    if (mode == 3 || mode == 2) lseq = '-';
+	    if (mode == TRMODE_J || mode == TRMODE_L) lseq = '-';
             else                        lseq = '~';
 	    spos_l = 0;
+	    /* lpost remains as it was init'ed as a gap '.' */
 	  }
 	}
 	if (cm->ndtype[nd] == MATP_nd || cm->ndtype[nd] == MATR_nd) {
 	  do_right = TRUE;
-	  if (cm->annote != NULL) rannote = cm->annote[rc];
+	  if (cm->rf != NULL) rrf = cm->rf[rc];
 	  rstr   = cons->cstr[rc];
 	  rcons  = cons->cseq[rc];
 	  cpos_r = rc+1;
 	  if (cm->sttype[v] == MP_st || cm->sttype[v] == MR_st) {
-	    if      (mode == 3)         rseq = abc->sym[symj];
-            else if (mode == 1 && d>0 ) rseq = abc->sym[symj];
+	    if      (mode == TRMODE_J)         rseq = abc->sym[symj];
+            else if (TRMODE_R && d>0 ) rseq = abc->sym[symj];
             else                        rseq = '~';
 	    spos_r = tr->emitr[ti];
+	    if(pcode != NULL) { rpost = pcode[tr->emitr[ti]-1]; } /* watch off-by-one w.r.t. dsq */
 	  } else {
-	    if (mode == 3 || mode == 1) rseq = '-';
+	    if (mode == TRMODE_J || TRMODE_R) rseq = '-';
             else                        rseq = '~';
 	    spos_r = 0;
+	    /* rpost remains as it was init'ed as a gap '.' */
 	  }
 	}
       }
@@ -318,8 +323,8 @@ CreateFancyAli(const ESL_ALPHABET *abc, Parsetree_t *tr, CM_t *cm, CMConsensus_t
 	  }
         else if (mode != 3)
           {
-            if (mode == 2 && lseq == toupper(lcons)) lmid = lseq;
-            if (mode == 1 && rseq == toupper(rcons)) rmid = rseq;
+            if (mode == TRMODE_L && lseq == toupper(lcons)) lmid = lseq;
+            if (TRMODE_R && rseq == toupper(rcons)) rmid = rseq;
           }
 	else if (DegeneratePairScore(cm->abc, cm->esc[v], symi, symj) >= 0) 
 	  lmid = rmid = ':';
@@ -329,7 +334,7 @@ CreateFancyAli(const ESL_ALPHABET *abc, Parsetree_t *tr, CM_t *cm, CMConsensus_t
 	  ltop = rtop = 'x';
 
 	/* determine lnegnc, rnegnc for optional negative scoring non-canonical annotation, they are 'v' if lseq and rseq are a negative scoring non-canonical (not a AU,UA,GC,CG,GU,UG) pair */
-	if ((mode == 3) && (DegeneratePairScore(cm->abc, cm->esc[v], symi, symj) < 0) && (! bp_is_canonical(lseq, rseq))) {
+	if ((mode == TRMODE_J) && (DegeneratePairScore(cm->abc, cm->esc[v], symi, symj) < 0) && (! bp_is_canonical(lseq, rseq))) {
 	  lnegnc = rnegnc = 'v';
 	}
       } else if (cm->sttype[v] == ML_st || cm->sttype[v] == IL_st) {
@@ -348,7 +353,7 @@ CreateFancyAli(const ESL_ALPHABET *abc, Parsetree_t *tr, CM_t *cm, CMConsensus_t
 	  rmid = '+';
       }
       if(cm->stid[v] == MATP_ML || cm->stid[v] == MATP_MR) { 
-	if(mode == 3) { 
+	if(mode == TRMODE_J) { 
 	  ltop = rtop = 'x';     /* mark non-truncated half base-pairs (MATP_ML or MATP_MR) with 'x' */
 	  lnegnc = rnegnc = 'v'; /* mark non-truncated half base-pairs (MATP_ML or MATP_MR) with 'v' */
 	}
@@ -357,15 +362,14 @@ CreateFancyAli(const ESL_ALPHABET *abc, Parsetree_t *tr, CM_t *cm, CMConsensus_t
        * If rightwise - push it onto stack.
        */
       if (do_left) {
-	if (cm->annote != NULL) ali->annote[pos] = lannote;
+	if (cm->rf != NULL) ali->rf[pos] = lrf;
 	ali->cstr[pos]   = lstr;
 	ali->cseq[pos]   = lcons;
 	ali->mid[pos]    = lmid;
 	ali->top[pos]    = (do_noncanonical) ? lnegnc : ltop;
 	ali->aseq[pos]   = lseq;
 	if(have_pcodes) {
-	  ali->pcode1[pos] = lpost1;
-	  ali->pcode2[pos] = lpost2;
+	  ali->pcode[pos] = lpost;
 	}
 	ali->ccoord[pos] = cpos_l;
 	ali->scoord[pos] = spos_l;
@@ -375,8 +379,7 @@ CreateFancyAli(const ESL_ALPHABET *abc, Parsetree_t *tr, CM_t *cm, CMConsensus_t
 	if ((status = esl_stack_IPush(pda, spos_r)) != eslOK) goto ERROR;
 	if ((status = esl_stack_IPush(pda, cpos_r)) != eslOK) goto ERROR;
 	if(have_pcodes) {
-	  if ((status = esl_stack_IPush(pda, (int) rpost2)) != eslOK) goto ERROR;
-	  if ((status = esl_stack_IPush(pda, (int) rpost1)) != eslOK) goto ERROR;
+	  if ((status = esl_stack_IPush(pda, (int) rpost)) != eslOK) goto ERROR;
 	}
 	if ((status = esl_stack_IPush(pda, (int) rseq)) != eslOK) goto ERROR;
 	if (do_noncanonical) { if ((status = esl_stack_IPush(pda, (int) rnegnc)) != eslOK) goto ERROR; }
@@ -384,7 +387,7 @@ CreateFancyAli(const ESL_ALPHABET *abc, Parsetree_t *tr, CM_t *cm, CMConsensus_t
 	if ((status = esl_stack_IPush(pda, (int) rmid)) != eslOK) goto ERROR;
 	if ((status = esl_stack_IPush(pda, (int) rcons)) != eslOK) goto ERROR;
 	if ((status = esl_stack_IPush(pda, (int) rstr)) != eslOK) goto ERROR;
-	if (cm->annote != NULL) if ((status = esl_stack_IPush(pda, (int) rannote)) != eslOK) goto ERROR;
+	if (cm->rf != NULL) if ((status = esl_stack_IPush(pda, (int) rrf)) != eslOK) goto ERROR;
 	if ((status = esl_stack_IPush(pda, PDA_RESIDUE)) != eslOK) goto ERROR;
       }
 
@@ -417,15 +420,14 @@ CreateFancyAli(const ESL_ALPHABET *abc, Parsetree_t *tr, CM_t *cm, CMConsensus_t
       }
     } /* end loop over the PDA; PDA now empty */
 	 
-  if (cm->annote != NULL) ali->annote[ali->len] = '\0';
+  if (cm->rf != NULL) ali->rf[ali->len] = '\0';
   ali->cstr[ali->len] = '\0';
   ali->cseq[ali->len] = '\0';
   ali->mid[ali->len]  = '\0';
   ali->top[ali->len]  = '\0';
   ali->aseq[ali->len] = '\0';
   if(have_pcodes) { 
-    ali->pcode1[ali->len] = '\0';
-    ali->pcode2[ali->len] = '\0';
+    ali->pcode[ali->len] = '\0';
   }    
   /* Laboriously determine the maximum bounds.
    */
@@ -464,91 +466,107 @@ CreateFancyAli(const ESL_ALPHABET *abc, Parsetree_t *tr, CM_t *cm, CMConsensus_t
  *           but this could be changed. Modeled on HMMER's 
  *           eponymous function.
  *
- * Args:     fp  - where to print it (stdout or open FILE)
- *           ali - alignment structure to print.      
- *           offset- number of residues to add to target seq index,
- *                   to ease MPI search, all target hits start at posn 1
- *           in_revcomp- TRUE if hit we're printing an alignment for a
- *                       cmsearch hit on reverse complement strand.
- *           do_top  - TRUE to turn on optional 'top-line' annotation.
- *                     This was set in CreateFancyAli() as marking either:
- *                     - negative scoring (non-compensatories) and half-bps: 
- *                     - negative scoring *non-canonical* bps and half bps
- *                       (in this 2nd case negative scoring canonicals are unmarked)
+ *            Put at least <min_aliwidth> alignment characters per
+ *            line; try to make lines no longer than <linewidth>
+ *            characters, including name, coords, and spacing.  The
+ *            width of lines may exceed <linewidth>, if that's what it
+ *            takes to put a name, coords, and <min_aliwidth>
+ *            characters of alignment on a line.
+ *            
+ *            As a special case, if <linewidth> is negative or 0, then
+ *            alignments are formatted in a single block of unlimited
+ *            line length.
+ *
+ *
+ * Args:     fp              - where to print it (stdout or open FILE)
+ *           ali             - alignment structure to print.      
+ *           offset          - number of residues to add to target seq index,
+ *                             to ease MPI search, all target hits start at posn 1
+ *           in_revcomp      - TRUE if hit we're printing an alignment for a
+ *                             cmsearch hit on reverse complement strand.
+ *           do_top          - TRUE to turn on optional 'top-line' annotation.
+ *                             This was set in CreateFancyAli() as marking either:
+ *                             negative scoring (non-compensatories) and half-bps: 
+ *                             negative scoring *non-canonical* bps and half bps
+ *                             (in this 2nd case negative scoring canonicals are unmarked)
+ *           min_aliwidth    - min length for alignment block (see Purpose above)
+ *           linewidth       - preferred line length (see Purpose above)
+ *           show_accessions - TRUE to print seq/query accessions (if avail.), not names
  * Returns:  (void)
  */
 void
-PrintFancyAli(FILE *fp, Fancyali_t *ali, int offset, int in_revcomp, int do_top)
+PrintFancyAli(FILE *fp, Fancyali_t *ali, int64_t offset, int in_revcomp, int do_top, int linewidth)
 {
   int   status;
   char *buf;
   int   pos;
-  int   linelength;
   int   ci,  cj;		/* positions in CM consensus 1..clen */
   int   sqi, sqj;		/* positions in target seq 1..L      */
   int   i;
   int   i2print, j2print; /* i,j indices we'll print, used to deal with case of reverse complement */
-  int   have_pcodes;      /* TRUE if postal codes are valid */
+  int   have_pcodes;      /* TRUE if posterior codes are valid */
 
-  have_pcodes = (ali->pcode1 != NULL && ali->pcode2 != NULL) ? TRUE : FALSE;
-  linelength = 60;
-  ESL_ALLOC(buf, sizeof(char) * (linelength + 1));
-  buf[linelength] = '\0';
-  for (pos = 0; pos < ali->len; pos += linelength)
+  printf("in PrintFancyAli sqfrom..sqto %d..%d in_revcomp: %d offset: %" PRId64 "\n", ali->sqfrom, ali->sqto, in_revcomp, offset);
+
+  have_pcodes = (ali->pcode != NULL) ? TRUE : FALSE;
+
+  ESL_ALLOC(buf, sizeof(char) * (linewidth + 1));
+  buf[linewidth] = '\0';
+  for (pos = 0; pos < ali->len; pos += linewidth)
     {
       /* Laboriously determine our coord bounds on dsq
        * and consensus line for this alignment section.
        */
       sqi = 0;
-      for (i = pos; ali->aseq[i] != '\0' && i < pos + linelength; i++) {
+      for (i = pos; ali->aseq[i] != '\0' && i < pos + linewidth; i++) {
 	if (ali->scoord[i] != 0) {
 	  sqi = ali->scoord[i];
 	  break;
 	}
       }
       sqj = 0;
-      for (i = pos; ali->aseq[i] != '\0' && i < pos + linelength; i++) {
+      for (i = pos; ali->aseq[i] != '\0' && i < pos + linewidth; i++) {
 	if (ali->scoord[i] != 0) sqj = ali->scoord[i];
       }
       ci = 0; 
-      for (i = pos; ali->aseq[i] != '\0' && i < pos + linelength; i++) {
+      for (i = pos; ali->aseq[i] != '\0' && i < pos + linewidth; i++) {
 	if (ali->ccoord[i] != 0) {
 	  ci = ali->ccoord[i];
 	  break;
 	}
       }
       cj = 0;
-      for (i = pos; ali->aseq[i] != '\0' && i < pos + linelength; i++) {
+      for (i = pos; ali->aseq[i] != '\0' && i < pos + linewidth; i++) {
 	if (ali->ccoord[i] != 0) cj = ali->ccoord[i];
       }
 
       /* Formats and print the alignment section.
        */
-      if (ali->annote != NULL) {
-	strncpy(buf, ali->annote+pos, linelength);
+      if (ali->rf != NULL) {
+	strncpy(buf, ali->rf+pos, linewidth);
 	fprintf(fp, "  %8s %s\n", " ", buf);
       }
       if (do_top && ali->top != NULL) {
-	strncpy(buf, ali->top+pos, linelength);  
+	strncpy(buf, ali->top+pos, linewidth);  
 	fprintf(fp, "  %8s %s\n", " ", buf);
       }
       if (ali->cstr != NULL) {
-	strncpy(buf, ali->cstr+pos, linelength);  
+	strncpy(buf, ali->cstr+pos, linewidth);  
 	fprintf(fp, "  %8s %s\n", " ", buf);
       }
       if (ali->cseq != NULL) {
-	strncpy(buf, ali->cseq+pos, linelength);  
+	strncpy(buf, ali->cseq+pos, linewidth);  
 	if (ci && cj)
 	  fprintf(fp, "  %8d %s %-8d\n", ci, buf, cj);
 	else
 	  fprintf(fp, "  %8s %s %-8s\n", "-", buf, "-");
       }
       if (ali->mid != NULL) {
-	strncpy(buf, ali->mid+pos,  linelength);  
+	strncpy(buf, ali->mid+pos,  linewidth);  
 	fprintf(fp, "  %8s %s\n", " ", buf);
       }
       if (ali->aseq != NULL) {
-	strncpy(buf, ali->aseq+pos, linelength);  
+	strncpy(buf, ali->aseq+pos, linewidth);  
 	if (sqj && sqi) {
 	  if(in_revcomp) {
 	    i2print = offset - (sqi-1)    + 1;
@@ -564,13 +582,9 @@ PrintFancyAli(FILE *fp, Fancyali_t *ali, int offset, int in_revcomp, int do_top)
 	  fprintf(fp, "  %8s %s %-8s\n", "-", buf, "-");
 	}
       }
-      if (have_pcodes && ali->pcode1 != NULL) {
-	strncpy(buf, ali->pcode1+pos, linelength);  
-	fprintf(fp, "  %8s %s\n", "POSTX.", buf);
-      }
-      if (have_pcodes && ali->pcode2 != NULL) {
-	strncpy(buf, ali->pcode2+pos, linelength);  
-	fprintf(fp, "  %8s %s\n", "POST.X", buf);
+      if (have_pcodes && ali->pcode != NULL) {
+	strncpy(buf, ali->pcode+pos, linewidth);  
+	fprintf(fp, "  %8s %s\n", "PP", buf);
       }
       fprintf(fp, "\n");
     }
@@ -589,14 +603,13 @@ PrintFancyAli(FILE *fp, Fancyali_t *ali, int offset, int in_revcomp, int do_top)
 void
 FreeFancyAli(Fancyali_t *ali)
 {
-  if (ali->annote != NULL) free(ali->annote);
+  if (ali->rf != NULL) free(ali->rf);
   if (ali->cstr   != NULL) free(ali->cstr);
   if (ali->cseq   != NULL) free(ali->cseq);
   if (ali->mid    != NULL) free(ali->mid);
   if (ali->top    != NULL) free(ali->top);
   if (ali->aseq   != NULL) free(ali->aseq);
-  if (ali->pcode1 != NULL) free(ali->pcode1);
-  if (ali->pcode2 != NULL) free(ali->pcode2);
+  if (ali->pcode  != NULL) free(ali->pcode);
   if (ali->ccoord != NULL) free(ali->ccoord);
   if (ali->scoord != NULL) free(ali->scoord);
   free(ali);
@@ -614,7 +627,14 @@ FreeFancyAli(Fancyali_t *ali)
  *            for each emitting node. If score < pthresh (for pairs)
  *            or < sthresh (for singlets), the residue is shown
  *            in lower case. (That is, "strong" consensus residues
- *            are in upper case.)
+ *            are in upper case.) 
+ *
+ *            Currently, pthresh is hard-coded as 3.0 and sthresh as
+ *            1.0 (previously v1.0.2 and earlier had these passed in,
+ *            but they were always set as 3.0/1.0 so I modified the
+ *            function prototype - feel free to change it back if you
+ *            want different values, or want to let caller (user) set
+ *            them)).
  *            
  *            Consensus structure annotates
  *            base pairs according to "multifurcation order" (how
@@ -630,36 +650,25 @@ FreeFancyAli(Fancyali_t *ali)
  *                ::(((,,<<<__>>>,<<<__>>->,,)))::
  *                AAGGGAACCCTTGGGTGGGTTCCACAACCCAA   
  *
- * Args:      cm         - the model
- *            abc        - alphabet to create con->cseq with (often cm->abc)
- *            pthresh    - bit score threshold for base pairs to be lowercased
- *            sthresh    - bit score threshold for singlets to be lowercased
+ * Args:      cm  - the model
+ *            abc - alphabet to create con->cseq with (often cm->abc)
  *            
- * Returns:   <eslOK> on success, <eslEMEM> on memory error.
-8             CMConsensus_t structure in *ret_cons.
- *            Caller frees w/ FreeCMConsensus().
+ * Returns:   Newly created CMConsensus_t. 
+ *            Caller frees w/ FreeCMConsensus(). 
+ *            Returns NULL on any error. 
  *
  * Xref:      STL6 p.58.
  */
-int
-CreateCMConsensus(CM_t *cm, const ESL_ALPHABET *abc, float pthresh, float sthresh, CMConsensus_t **ret_cons)
+CMConsensus_t *
+CreateCMConsensus(CM_t *cm, const ESL_ALPHABET *abc)
 {
-  /* Contract check. We allow the caller to specify the alphabet they want the 
-   * resulting MSA in, but it has to make sense (see next few lines). */
-  if(cm->abc->type == eslRNA)
-    { 
-      if(abc->type != eslRNA && abc->type != eslDNA)
-	cm_Fail("ERROR in CreateCMConsensus(), cm alphabet is RNA, but requested output alphabet is neither DNA nor RNA.");
-    }
-  else if(cm->abc->K != abc->K)
-    cm_Fail("ERROR in CreateCMConsensus(), cm alphabet size is %d, but requested output alphabet size is %d.", cm->abc->K, abc->K);
-
   int       status;
-  CMConsensus_t *con;           /* growing consensus info */
-  char     *cseq;               /* growing consensus sequence display string   */
-  char     *cstr;               /* growing consensus structure display string  */
+  CMConsensus_t *con = NULL;    /* growing consensus info */
+  char     *cseq = NULL;        /* growing consensus sequence display string   */
+  char     *cstr = NULL;        /* growing consensus structure display string  */
   int      *ct;			/* growing ct Zuker pairing partnet string     */
-  int      *lpos, *rpos;        /* maps node->consensus position, [0..nodes-1] */
+  int      *lpos = NULL;        /* maps node->left consensus position, [0..nodes-1] */
+  int      *rpos = NULL;        /* maps node->right consensus position, [0..nodes-1] */
   int       cpos;		/* current position in cseq, cstr              */
   int       nalloc;		/* current allocated length of cseq, cstr      */
   ESL_STACK *pda;               /* pushdown automaton used to traverse model   */
@@ -673,6 +682,20 @@ CreateCMConsensus(CM_t *cm, const ESL_ALPHABET *abc, float pthresh, float sthres
   int       x;
   int       pairpartner;	/* coord of left pairing partner of a right base */
   void     *tmp;                /* for ESL_RALLOC */
+  /* bit score thresholds for base pairs (pthresh) and singlets
+   * (sthresh) to be lowercased. These are hard-coded. If you want
+   * to change them, consider changing the function prototype so they're
+   * chosen by caller and passed in.
+   */
+  float     pthresh = 3.0; 
+  float     sthresh = 1.0; 
+
+  /* Contract check. CM must have log odds scores. Alphabet must make sense. */
+  if(! (cm->flags & CMH_BITS)) return NULL;
+  if(cm->abc->type == eslRNA) {
+    if(abc->type != eslRNA && abc->type != eslDNA) return NULL;
+  }
+  else if(cm->abc->K != abc->K) return NULL;
 
   ESL_ALLOC(lpos, sizeof(int) * cm->nodes);
   ESL_ALLOC(rpos, sizeof(int) * cm->nodes);
@@ -814,11 +837,16 @@ CreateCMConsensus(CM_t *cm, const ESL_ALPHABET *abc, float pthresh, float sthres
   con->lpos = lpos;
   con->rpos = rpos;
   con->clen = cpos;
-  *ret_cons = con;
-  return eslOK;
+  return con;
 
  ERROR:
-  return status;
+  if(cseq != NULL) free(cseq);
+  if(cstr != NULL) free(cstr);
+  if(ct   != NULL) free(ct);
+  if(lpos != NULL) free(lpos);
+  if(rpos != NULL) free(rpos);
+  if(con  != NULL) free(con);
+  return NULL;
 }
 
 void
@@ -1001,41 +1029,6 @@ createFaceCharts(CM_t *cm, int **ret_inface, int **ret_outface)
  ERROR:
   cm_Fail("Memory allocation error.\n");
 }
-	
-
-
-/* Function: MainBanner()
- * Date:     SRE, Fri Sep 26 11:29:02 2003 [AA 886, from Salk Institute]
- *
- * Purpose:  Print a package version and copyright banner.
- *           Used by all the main()'s that use the squid library.
- *           
- *    Expects to be able to pick up defined preprocessor variables:
- *    variable          example
- *    --------           --------------  
- *    PACKAGE_NAME      "Infernal"
- *    PACKAGE_VERSION   "0.42"
- *    PACKAGE_DATE      "Sept 2003"
- *    PACKAGE_COPYRIGHT "Copyright (C) 2001-2003 Washington University School of Medicine"
- *    PACKAGE_LICENSE   "Freely distributed under the GNU General Public License (GPL)."
- *    This gives us a general mechanism to update release information
- *    without changing multiple points in the code.
- * 
- * Args:     fp     - where to print it
- *           banner - one-line program description, e.g.:
- *                    "foobar - make bars from foo with elan" 
- * Returns:  (void)
- */
-void
-MainBanner(FILE *fp, char *banner)
-{
-  fprintf(fp, "%s\n", banner);
-  fprintf(fp, "%s %s (%s)\n", PACKAGE_NAME, PACKAGE_VERSION, PACKAGE_DATE);
-  fprintf(fp, "%s\n", PACKAGE_COPYRIGHT);
-  fprintf(fp, "%s\n", PACKAGE_LICENSE);
-  fprintf(fp, "- - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -\n");
-}
-
 
 /* Function: IsCompensatory()
  * Date:     SRE, Sun Jun  2 10:16:59 2002 [Madison]
@@ -1080,7 +1073,7 @@ IsCompensatory(const ESL_ALPHABET *abc, float *pij, int symi, int symj)
 
 /* Implementation of CMEmitMap_t structure:
  * map of a CM's nodes onto consensus sequence positions.
- * Structure is declared in structs.h.
+ * Structure is declared in infernal.h.
  * 
  * Used for constructing multiple alignments.
  * 
@@ -1114,11 +1107,21 @@ IsCompensatory(const ESL_ALPHABET *abc, float *pij, int symi, int symj)
  * 
  */
 
+/* Function: CreateEmitMap()
+ * Date:     ? (SRE pre Infernal version 0.55).
+ *
+ * Purpose:  Create and fill an emit map for a given CM <cm>
+ *           and return it. 
+ *           If we run out of memory or we have a problem creating
+ *           the map, we return NULL.
+ *
+ */
+
 CMEmitMap_t *
 CreateEmitMap(CM_t *cm)
 {
   int          status;
-  CMEmitMap_t *map;
+  CMEmitMap_t *map = NULL;
   ESL_STACK   *pda;
   int          cpos;
   int          nd;
@@ -1192,13 +1195,44 @@ CreateEmitMap(CM_t *cm)
 
   map->clen = map->rpos[0]-1;
   esl_stack_Destroy(pda);
+
+  /* ensure we've filled in the map correctly */
+  for (nd = 0; nd < cm->nodes; nd++) { 
+    if(map->lpos[nd] == -1) goto ERROR;
+    if(map->rpos[nd] == -1) goto ERROR;
+    if(map->epos[nd] == -1) goto ERROR;
+  }
+
   return map;
 
  ERROR: 
-  cm_Fail("Memory allocation error.");
-  return NULL; /* never reached */
+  if(map != NULL) FreeEmitMap(map);
+  return NULL; 
 }
   
+
+/* Function: SizeofEmitMap()
+ * Date:     EPN, Wed Jan 18 10:15:46 2012
+ *
+ * Purpose:  Calculate and return the size of an emit map 
+ *           in Mb.
+ */
+
+float 
+SizeofEmitMap(CM_t *cm, CMEmitMap_t *emap)
+{
+  float bytes = 0.;
+
+  if(emap == NULL) return 0.;
+  
+  bytes = sizeof(CMEmitMap_t);
+  bytes += sizeof(int) * cm->nodes; /* lpos */
+  bytes += sizeof(int) * cm->nodes; /* rpos */
+  bytes += sizeof(int) * cm->nodes; /* epos */
+
+  return bytes / 1000000.;
+}
+
 void
 DumpEmitMap(FILE *fp, CMEmitMap_t *map, CM_t *cm)
 {
@@ -1206,13 +1240,13 @@ DumpEmitMap(FILE *fp, CMEmitMap_t *map, CM_t *cm)
 
   fprintf(fp, "CM to consensus emit map; consensus length = %d \n",
 	  map->clen);
-  fprintf(fp, "%4s %9s %4s %4s %4s\n", 
-	  "Node", "Node type", "lpos", "rpos", "epos");
-  fprintf(fp, "%4s %9s %4s %4s %4s\n", 
-	  "----", "---------", "----", "----", "----");
+  fprintf(fp, "%4s %7s %9s %4s %4s %4s\n", 
+	  "Node", "State 1", "Node type", "lpos", "rpos", "epos");
+  fprintf(fp, "%4s %7s %9s %4s %4s %4s\n", 
+	  "----", "-------", "---------", "----", "----", "----");
   for (nd = 0; nd < cm->nodes; nd++)
-    fprintf(fp, "%4d %9s %4d %4d %4d\n", 
-	    nd, Nodetype(cm->ndtype[nd]), 
+    fprintf(fp, "%4d %7d %9s %4d %4d %4d\n", 
+	    nd, cm->nodemap[nd], Nodetype(cm->ndtype[nd]), 
 	    map->lpos[nd], map->rpos[nd], map->epos[nd]);
 }
 
