@@ -1,7 +1,11 @@
 /************************************************************
- *    This copyrighted source code is freely distributed 
- *    under the terms of the GNU General Public License. See
- *    the files COPYRIGHT and LICENSE for details.
+ * Infernal - inference of RNA secondary structure alignments
+ * Version 1.1.1; July 2014
+ * Copyright (C) 2014 Howard Hughes Medical Institute.
+ * Other copyrights also apply. See the COPYRIGHT file for a full list.
+ * 
+ * Infernal is distributed under the terms of the GNU General Public License
+ * (GPLv3). See the LICENSE file for details.
  ************************************************************/
 
 /* hmmband.h 
@@ -27,7 +31,6 @@
 #include "esl_stopwatch.h" 
 
 #include "structs.h"		/* data structures, macros, #define's   */
-#include "cplan9.h"
 
 extern CP9Bands_t * AllocCP9Bands(CM_t *cm, CP9_t *hmm);
 extern void         FreeCP9Bands(CP9Bands_t *cp9bands);
